/*!
    @file  CDataGen.cpp
    @brief Fast Fourier transform

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <cmath>
#include <boost/format.hpp>
#include "CDataGen.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
FUJITSU::CDataGen::CDataGen(const boost::shared_ptr<C_param_wrap> param)
    : param_()
{
    std::string str;
    std::stringstream msg;
    boost::shared_ptr<Util_wrap> util = createUtil();
    
    /* パラメータの取得 */
    str = "WAVE_NUM";
    util->checkAndGetParam(param, str, &param_.WaveNum, C_param_wrap::INT);
    if (param_.WaveNum <= 0) {
        msg << "Parameter " << str << " is in valid. Must be 1 or more." << std::endl;
        g_err->appendMsg(msg.str()) ;
        msg.str("");
        msg.clear(std::stringstream::goodbit);
    }

    str = "WAVE_FREQ";
    util->checkAndGetParam(param, str, &param_.WaveFreq, C_param_wrap::DOUBLE_VECTOR);
    if (param_.WaveFreq.size() != param_.WaveNum) {
        msg << "Parameter " << str << " is in valid. Must be element of WAVE_FREQ equal WAVE_NUM" << std::endl;
        g_err->appendMsg(msg.str()) ;
        msg.str("");
        msg.clear(std::stringstream::goodbit);
    }
    
    str = "WAVE_OFFSET";
    util->checkAndGetParam(param, str, &param_.WaveOffset, C_param_wrap::DOUBLE_VECTOR);
    if (param_.WaveOffset.size() != param_.WaveNum) {
        msg << "Parameter " << str << " is in valid. Must be element of WAVE_OFFSET equal WAVE_NUM" << std::endl;
        g_err->appendMsg(msg.str()) ;
        msg.str("");
        msg.clear(std::stringstream::goodbit);
    }

    str = "WAVE_AMP";
    util->checkAndGetParam(param, str, &param_.WaveAmp, C_param_wrap::DOUBLE_VECTOR);
    if (param_.WaveAmp.size() != param_.WaveNum) {
        msg << "Parameter " << str << " is in valid. Must be element of WAVE_AMP equal WAVE_NUM" << std::endl;
        g_err->appendMsg(msg.str()) ;
        msg.str("");
        msg.clear(std::stringstream::goodbit);
    }

    str = "SAMPLING_FREQ";
    util->checkAndGetParam(param, str, &param_.SamplingFreq, C_param_wrap::DOUBLE);

    str = "FFT_POINT";
    util->checkAndGetParam(param, str, &param_.FftPoint, C_param_wrap::INT);
}

/*! デストラクタ 
*/
FUJITSU::CDataGen::~CDataGen()
{
}

/*! データ生成 */
void FUJITSU::CDataGen::Execute(std::vector<std::complex<double> >& OutputData)
{
    OutputData.resize(param_.FftPoint);
    for (size_t i=0; i < OutputData.size(); i++) {
        std::complex<double> sigma(0.0);
        for (size_t j=0; j < param_.WaveNum; j++) {
            double t =i *  1.0 / param_.SamplingFreq * pow(10, 3);  // ns
            double re, im;
            //re = param_.WaveAmp[j] * std::cos(2.0 * M_PI * (t + param_.WaveOffset[j]) * pow(10, -9)/(1.0/(param_.WaveFreq[j]*pow(10,3))));
            //im = param_.WaveAmp[j] * std::sin(2.0 * M_PI * (t + param_.WaveOffset[j]) * pow(10, -9)/(1.0/(param_.WaveFreq[j]*pow(10,3))));
            //sigma += std::complex<double>(re, im);
            re = param_.WaveAmp[j] * std::sin(2.0 * M_PI * (t + param_.WaveOffset[j]) * pow(10, -9)/(1.0/(param_.WaveFreq[j]*pow(10,3))));
            sigma += std::complex<double>(re, 0.0);
        }
        OutputData[i] = sigma;
    }

    // 浮動小数点誤差が邪魔なので消す(なくて良い処理)
    for (size_t i=0; i < OutputData.size(); i++) {
        double re, im;
        if (OutputData[i].real() > -0.000000001 && OutputData[i].real() < 0.000000001) re = 0;
        else                                                                           re = OutputData[i].real();
        if (OutputData[i].imag() > -0.000000001 && OutputData[i].imag() < 0.000000001) im = 0;
        else                                                                           im = OutputData[i].imag();
        OutputData[i] = std::complex<double>(re, im);
    }
}
