/*!
    @file  CFft16r16.cpp
    @brief FFTサイズ radix-16 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft16r16.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft16r16::CFft16r16(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 16) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子(半分でいいけどFFTサイズ分生成しておく)
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft16r16::~CFft16r16()
{
}

/* FFT Calculation */
void CFft16r16::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(nFft_);
    std::vector<std::complex<double> > x1(nFft_);
    std::vector<std::complex<double> > x2(nFft_);
    std::vector<std::complex<double> > x3(nFft_);
    std::vector<std::complex<double> > x4(nFft_);
    
    x0 = In;
    
    // loop1
    for (int i=0; i < 8; i++) {
        x1[i]   =  x0[i] + x0[i+8];
        x1[i+8] = (x0[i] - x0[i+8]) * W_[i];
    }
    
    // loop2
    for (int j=0; j < 2; j++) {
        for (int i=0; i < 4; i++) {
            x2[i+j*8]   =  x1[i+j*8] + x1[i+4+j*8];
            x2[i+4+j*8] = (x1[i+j*8] - x1[i+4+j*8]) * W_[i*2];
        }
    }

    // loop3
    for (int j=0; j < 4; j++) {
        for (int i=0; i < 2; i++) {
            x3[i+j*4]   =  x2[i+j*4] + x2[i+2+j*4];
            x3[i+2+j*4] = (x2[i+j*4] - x2[i+2+j*4]) * W_[i*4];
        }
    }

    // loop4
    for (int i=0; i < 8; i++) {
        x4[i*2]   = x3[i*2] + x3[i*2+1];
        x4[i*2+1] = x3[i*2] - x3[i*2+1];
    }    
    
    // reorder
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    xWork_[i*8+j*4+k*2+n] = x4[i+j*2+k*4+n*8];
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

/* result get */
const std::vector<std::complex<double> >& CFft16r16::get() const
{
    return xWork_;
}
