/*!
    @file  CFft16r4.cpp
    @brief FFTサイズ16 radix-4 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft16r4.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft16r4::CFft16r4(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 16) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        //CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft16r4::~CFft16r4()
{
}

/* FFT Calculation */
void CFft16r4::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(4, std::vector<std::complex<double> >(4));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j]   = In[4*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 4; i++) {
        RADIX_4(x[i], 0, i);
    }

    // reorder1
    std::vector<std::vector<std::complex<double> > > y(4, std::vector<std::complex<double> >(4));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 4; i++) {
        RADIX_4(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*4+j] = y[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    xWork_[i*8+j*4+k*2+n] = z[i+j*2+k*4+n*8];
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

void CFft16r4::RADIX_4(std::vector<std::complex<double> >& x, 
                       const unsigned int stage,
                       const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(4);
    std::vector<std::complex<double> > x1(4);
    std::vector<std::complex<double> > x2(4);
    
    x0 = x;
    
    // loop1
    for (int i=0; i < 2; i++) {
        x1[i]   =  x0[i] + x0[i+2];
        x1[i+2] = (x0[i] - x0[i+2]) * W_[i*4];
    }
    
    // loop2
    for (int i=0; i < 2; i++) {
        x2[i*2]   = x1[i*2] + x1[i*2+1];
        x2[i*2+1] = x1[i*2] - x1[i*2+1];
    }

    // final stageでなかったら回転因子乗算する
    if (stage == 0) {
        x2[1] *= W_[number*2];
        x2[2] *= W_[number*1];
        x2[3] *= W_[number*3];
    }

    x = x2;
}

/* result get */
const std::vector<std::complex<double> >& CFft16r4::get() const
{
    return xWork_;
}
