/*!
    @file  CFft256r16.cpp
    @brief FFTサイズ radix-16 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft256r16.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft256r16::CFft256r16(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 256) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        std::cout << "bf[" << i << "] : " << sgn * 2.0 * M_PI * j * double(i) / double(nfft) << std::endl;
        std::cout << "W[" << i << "] : "  << W_[i] << std::endl;

        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft256r16::~CFft256r16()
{
}

/* FFT Calculation */
void CFft256r16::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(16, std::vector<std::complex<double> >(16));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j] = In[16*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 16; i++) {
        RADIX_16(x[i], 0, i);
    }
    
    // reorder1
    std::vector<std::vector<std::complex<double> > > y(16, std::vector<std::complex<double> >(16));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 16; i++) {
        RADIX_16(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*16+j] = y[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            for (int q=0; q < 2; q++) {
                                for (int r=0; r < 2; r++) {
                                    xWork_[i*128+j*64+k*32+n*16+m*8+p*4+q*2+r] = z[i+j*2+k*4+n*8+m*16+p*32+q*64+r*128];
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

void CFft256r16::RADIX_16(std::vector<std::complex<double> >& x, 
                          const unsigned int stage,
                          const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(16);
    std::vector<std::complex<double> > x1(16);
    std::vector<std::complex<double> > x2(16);
    std::vector<std::complex<double> > x3(16);
    std::vector<std::complex<double> > x4(16);
    
    x0 = x;
    
    // loop1
    for (int i=0; i < 8; i++) {
        x1[i]   = x0[i] + x0[i+8];
        x1[i+8] = x0[i] - x0[i+8];
        if (i >= 4) {
            // x1 * (-j) : IQ交換で実施
            x1[i+8] = std::complex<double>(x1[i+8].imag(), -x1[i+8].real());
        }
    }
    
    // loop2
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 4; j++) {
            x2[i*8+j]   = x1[i*8+j] + x1[i*8+j+4];
            x2[i*8+j+4] = x1[i*8+j] - x1[i*8+j+4];
            if (i == 0) {
                if (j == 2) {
                    // x2 * (-j) : IQ交換で実施
                    x2[i*8+j+4] = std::complex<double>(x2[i*8+j+4].imag(), -x2[i*8+j+4].real());
                } else if (j == 1 || j == 3) {
                    // x2 * (-1/4π) or x2 * (-3/4π)
                    x2[i*8+j+4] *= W_[j*2*16];
                }
            } else {
                // i == 1
                // j=1:idx16(-1/8π),idx48(-3/8π),  
                // j=2:idx32(-1/4π),idx:96(-3/4π),  
                // j=3:idx48(-3/8π),idx144(-9/8π)
                x2[i*8+j]   *= W_[j*16];
                x2[i*8+j+4] *= W_[j*16*3];
            }
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        for (int j=0; j < 2; j++) {
            x3[i*4+j]   = x2[i*4+j] + x2[i*4+j+2];
            x3[i*4+j+2] = x2[i*4+j] - x2[i*4+j+2];
            if (j == 1) {
                // x3 * (-j) : IQ交換で実施
                x3[i*4+j+2] = std::complex<double>(x3[i*4+j+2].imag(), -x3[i*4+j+2].real());
            }
        }
    }

    // loop4
    for (int i=0; i < 8; i++) {
        x4[i*2]   = x3[i*2] + x3[i*2+1];
        x4[i*2+1] = x3[i*2] - x3[i*2+1];
    }

    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    if (stage == 0) {
                        // final stageでなかったら回転因子乗算する
                        x4[i*8+j*4+k*2+n] *= W_[number*(i+j*2+k*4+n*8)];
                    } else {
                        // final stageは規格化して出力
                        x4[i*8+j*4+k*2+n] /= 16.0;
                    }
                }
            }
        }
    }
    
    x = x4;
}

/* result get */
const std::vector<std::complex<double> >& CFft256r16::get() const
{
    return xWork_;
}
