/*!
    @file  CFft256r16P.cpp
    @brief FFTサイズ radix-16 固定(ハード検討確認用, 精度検討用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "prec_FFT.h"
#include "CFft256r16P.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft256r16P::CFft256r16P(const boost::shared_ptr<C_param_wrap> param,
                         const unsigned int nfft, 
                         const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft),
      p_fft_exec_times_()
{
    // チェック
    if (nfft != 256) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    /* ビット制限clip回数 */
    clip_p_.resize(FFT_PREC_NAME_END, 0);
    clip_n_.resize(FFT_PREC_NAME_END, 0);

    // パラメータ関連
    std::string str;
    std::stringstream msg;
    boost::shared_ptr<Util_wrap> util = createUtil();    
    
    // パラメータ取得 (演算精度関連)
    str = "FFT_PRECISION_W";                    /* 回転因子テーブル精度 */
    util->checkAndGetParam(param, str, &pparam_.p_w, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_IN";                   /* 入力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_in, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP1";              /* stage1 loop1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp1, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP2";              /* stage1 loop2出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp2, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP3";              /* stage1 loop3出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp3, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP4";              /* stage1 loop4出力精度(回転因子乗算前 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp4, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_OUT";              /* stage1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_out, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP1";              /* stage2 loop1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp1, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP2";              /* stage2 loop2出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp2, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP3";              /* stage2 loop3出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp3, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP4";              /* stage2 loop4出力精度(規格化前) */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp4, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_OUT";                  /* 出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_out, C_param_wrap::INT_VECTOR);    
    
    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        W_[i] = prec_pos_round_clip_FFT(W_[i], pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft256r16P::~CFft256r16P()
{
}

/* FFT Calculation */
void CFft256r16P::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(16, std::vector<std::complex<double> >(16));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j] = In[16*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 16; i++) {
        RADIX_16(x[i], 0, i);
    }
    
    // reorder1
    std::vector<std::vector<std::complex<double> > > y(16, std::vector<std::complex<double> >(16));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 16; i++) {
        RADIX_16(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*16+j] = y[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            for (int q=0; q < 2; q++) {
                                for (int r=0; r < 2; r++) {
                                    xWork_[i*128+j*64+k*32+n*16+m*8+p*4+q*2+r] = z[i+j*2+k*4+n*8+m*16+p*32+q*64+r*128];
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }

    p_fft_exec_times_++; // ビット精度ログ出力用
}

void CFft256r16P::RADIX_16(std::vector<std::complex<double> >& x, 
                          const unsigned int stage,
                          const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(16);
    std::vector<std::complex<double> > x1(16);
    std::vector<std::complex<double> > x2(16);
    std::vector<std::complex<double> > x3(16);
    std::vector<std::complex<double> > x4(16);
    
    x0 = x;

    // 量子化
    for (size_t i=0; i < x0.size(); i++) {
        if (stage == 0) {
            // stage1
            x0[i] = prec_pos_round_clip_FFT(x0[i], pparam_.p_in[0], pparam_.p_in[1], &clip_p_[FFT_PRECISION_IN], &clip_n_[FFT_PRECISION_IN]);
        }
    }

    // loop1
    for (int i=0; i < 8; i++) {
        x1[i]   = x0[i] + x0[i+8];
        x1[i+8] = x0[i] - x0[i+8];
        if (i >= 4) {
            // x1 * (-j) : IQ交換で実施
            x1[i+8] = std::complex<double>(x1[i+8].imag(), -x1[i+8].real());
        }
        
        // 量子化
        if (stage == 0) {
            // stage1
            x1[i]   = prec_pos_round_clip_FFT(x1[i],   pparam_.p_st1_lp1[0], pparam_.p_st1_lp1[1], &clip_p_[FFT_PRECISION_ST1_LP1], &clip_n_[FFT_PRECISION_ST1_LP1]);
            x1[i+8] = prec_pos_round_clip_FFT(x1[i+8], pparam_.p_st1_lp1[0], pparam_.p_st1_lp1[1], &clip_p_[FFT_PRECISION_ST1_LP1], &clip_n_[FFT_PRECISION_ST1_LP1]);
        } else {
            // stage2
            x1[i]   = prec_pos_round_clip_FFT(x1[i],   pparam_.p_st2_lp1[0], pparam_.p_st2_lp1[1], &clip_p_[FFT_PRECISION_ST2_LP1], &clip_n_[FFT_PRECISION_ST2_LP1]);
            x1[i+8] = prec_pos_round_clip_FFT(x1[i+8], pparam_.p_st2_lp1[0], pparam_.p_st2_lp1[1], &clip_p_[FFT_PRECISION_ST2_LP1], &clip_n_[FFT_PRECISION_ST2_LP1]);
        }
        
    }
    
    // loop2
    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    //std::complex<double> rot1p4PIm = std::exp(-M_PI * j * 1.0 / 4.0);  // -1/4・π
    //std::complex<double> rot3p4PIm = std::exp(-M_PI * j * 3.0 / 4.0);  // -3/4・π
    std::complex<double> rot1p8PIm = std::exp(-M_PI * j * 1.0 / 8.0);  // -1/8・π
    std::complex<double> rot3p8PIm = std::exp(-M_PI * j * 3.0 / 8.0);  // -3/8・π
    std::complex<double> rot9p8PIm = std::exp(-M_PI * j * 9.0 / 8.0);  // -9/8・π
    double root0p5 = std::sqrt(0.5);

    // 量子化
    rot1p8PIm = prec_pos_round_clip_FFT(rot1p8PIm, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot3p8PIm = prec_pos_round_clip_FFT(rot3p8PIm, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot9p8PIm = prec_pos_round_clip_FFT(rot9p8PIm, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    root0p5   = prec_pos_round_clip_FFT(root0p5, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);

    for (int i=0; i < 2; i++) {
        for (int j=0; j < 4; j++) {
            x2[i*8+j]   = x1[i*8+j] + x1[i*8+j+4];
            x2[i*8+j+4] = x1[i*8+j] - x1[i*8+j+4];
            if (i == 0) {
                if (j == 1) {
                    // x2 * -1/4・π
                    //x2[i*8+j+4] *= rot1p4PIm;
                    std::complex<double> tmp = std::complex<double>(x2[i*8+j+4].real()+x2[i*8+j+4].imag(), x2[i*8+j+4].imag()-x2[i*8+j+4].real());
                    x2[i*8+j+4] = tmp * root0p5;
                } else if (j == 2) {
                    // x2 * (-j) : IQ交換で実施
                    x2[i*8+j+4] = std::complex<double>(x2[i*8+j+4].imag(), -x2[i*8+j+4].real());
                } else if (j == 3) {
                    // x2 * -3/4・π
                    //x2[i*8+j+4] *= rot3p4PIm;
                    std::complex<double> tmp = std::complex<double>(x2[i*8+j+4].imag()-x2[i*8+j+4].real(), -x2[i*8+j+4].real()-x2[i*8+j+4].imag());
                    x2[i*8+j+4] = tmp * root0p5;

                }
            } else {
                // i == 1
                // j=1:idx16(-1/8π),idx48(-3/8π), 
                // j=2:idx32(-1/4π),idx:96(-3/4π), 
                // j=3:idx48(-3/8π),idx144(-9/8π)
                if (j == 1) {
                    x2[i*8|i]   *= rot1p8PIm;
                    x2[i*8+j+4] *= rot3p8PIm;
                } else if (j == 2) {
                    //x2[i*8+j]   *= rot1p4PIm;
                    std::complex<double> tmp = std::complex<double>(x2[i*8+j].real()+x2[i*8+j].imag(), x2[i*8+j].imag()-x2[i*8+j].real());
                    x2[i*8+j] = tmp * root0p5;
                    //x2[i*8+j+4] *= rot3p4PIm;
                    tmp = std::complex<double>(x2[i*8+j+4].imag()-x2[i*8+j+4].real(), -x2[i*8+j+4].real()-x2[i*8+j+4].imag());
                    x2[i*8+j+4] = tmp * root0p5;
                } else if (j == 3) {
                    x2[i*8+j]   *= rot3p8PIm;
                    x2[i*8+j+4] *= rot9p8PIm;
                }    
            }
            
            // 量子化
            if (stage == 0) {
                // stage1
                x2[i*8+j]   = prec_pos_round_clip_FFT(x2[i*8+j],     pparam_.p_st1_lp2[0], pparam_.p_st1_lp2[1], &clip_p_[FFT_PRECISION_ST1_LP2], &clip_n_[FFT_PRECISION_ST1_LP2]);
                x2[i*8+j+4] = prec_pos_round_clip_FFT(x2[i*8+j+4],   pparam_.p_st1_lp2[0], pparam_.p_st1_lp2[1], &clip_p_[FFT_PRECISION_ST1_LP2], &clip_n_[FFT_PRECISION_ST1_LP2]);
            } else {
                // stage2
                x2[i*8+j]   = prec_pos_round_clip_FFT(x2[i*8+j],     pparam_.p_st2_lp2[0], pparam_.p_st2_lp2[1], &clip_p_[FFT_PRECISION_ST2_LP2], &clip_n_[FFT_PRECISION_ST2_LP2]);
                x2[i*8+j+4] = prec_pos_round_clip_FFT(x2[i*8+j+4],   pparam_.p_st2_lp2[0], pparam_.p_st2_lp2[1], &clip_p_[FFT_PRECISION_ST2_LP2], &clip_n_[FFT_PRECISION_ST2_LP2]);
            }
            
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        for (int j=0; j < 2; j++) {
            x3[i*4+j]   = x2[i*4+j] + x2[i*4+j+2];
            x3[i*4+j+2] = x2[i*4+j] - x2[i*4+j+2];
            if (j == 1) {
                // x3 * (-j) : IQ交換で実施
                x3[i*4+j+2] = std::complex<double>(x3[i*4+j+2].imag(), -x3[i*4+j+2].real());
            }

            // 量子化
            if (stage == 0) {
                // stage1
                x3[i*4+j]   = prec_pos_round_clip_FFT(x3[i*4+j],   pparam_.p_st1_lp3[0], pparam_.p_st1_lp3[1], &clip_p_[FFT_PRECISION_ST1_LP3], &clip_n_[FFT_PRECISION_ST1_LP3]);
                x3[i*4+j+2] = prec_pos_round_clip_FFT(x3[i*4+j+2], pparam_.p_st1_lp3[0], pparam_.p_st1_lp3[1], &clip_p_[FFT_PRECISION_ST1_LP3], &clip_n_[FFT_PRECISION_ST1_LP3]);
            } else {
                // stage2
                x3[i*4+j]   = prec_pos_round_clip_FFT(x3[i*4+j],   pparam_.p_st2_lp3[0], pparam_.p_st2_lp3[1], &clip_p_[FFT_PRECISION_ST2_LP3], &clip_n_[FFT_PRECISION_ST2_LP3]);
                x3[i*4+j+2] = prec_pos_round_clip_FFT(x3[i*4+j+2], pparam_.p_st2_lp3[0], pparam_.p_st2_lp3[1], &clip_p_[FFT_PRECISION_ST2_LP3], &clip_n_[FFT_PRECISION_ST2_LP3]);
            }
            
        }
    }

    // loop4
    for (int i=0; i < 8; i++) {
        x4[i*2]   = x3[i*2] + x3[i*2+1];
        x4[i*2+1] = x3[i*2] - x3[i*2+1];
    }

    // 量子化
    for (size_t i=0; i < x4.size(); i++) {
        if (stage == 0) {
            // stage1
            x4[i] = prec_pos_round_clip_FFT(x4[i], pparam_.p_st1_lp4[0], pparam_.p_st1_lp4[1], &clip_p_[FFT_PRECISION_ST1_LP4], &clip_n_[FFT_PRECISION_ST1_LP4]);
        } else {
            // stage2
            x4[i] = prec_pos_round_clip_FFT(x4[i], pparam_.p_st2_lp4[0], pparam_.p_st2_lp4[1], &clip_p_[FFT_PRECISION_ST2_LP4], &clip_n_[FFT_PRECISION_ST2_LP4]);
        }
    }

    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    if (stage == 0) {
                        // final stageでなかったら回転因子乗算する
                        x4[i*8+j*4+k*2+n] *= W_[number*(i+j*2+k*4+n*8)];
                    } else {
                        // final stageは規格化して出力
                        x4[i*8+j*4+k*2+n] /= 16.0;
                    }
                }
            }
        }
    }
    
    // 量子化
    for (size_t i=0; i < x4.size(); i++) {
        if (stage == 0) {
            // stage1出力
            x4[i] = prec_pos_round_clip_FFT(x4[i], pparam_.p_st1_out[0], pparam_.p_st1_out[1], &clip_p_[FFT_PRECISION_ST1_OUT], &clip_n_[FFT_PRECISION_ST1_OUT]);
        } else {
            // stage2出力(FFT出力)
            x4[i] = prec_pos_round_clip_FFT(x4[i], pparam_.p_out[0], pparam_.p_out[1], &clip_p_[FFT_PRECISION_OUT], &clip_n_[FFT_PRECISION_OUT]);
        }
    }
    
    x = x4;
}

/* result get */
const std::vector<std::complex<double> >& CFft256r16P::get() const
{
    return xWork_;
}

/* Report出力 */
void CFft256r16P::ReportStatisticalData(std::ostream& os, const double NowTime)
{
    int active_count = 0;  // とりあえず

    if (!ifft_) {
    /* ビット精度最終結果ログ出力 */
        os << boost::format("\n----- FEQ FFT256 radix-16 Block Clip Result -----  active_count = %.0f  DSPTOP_NowTime = %.0f") % p_fft_exec_times_ % NowTime << std::endl;
        std::string prec_name;
        for (int i=0; i < FFT_PREC_NAME_END; i++) {
            int total_count;
            switch (i) {
            case FFT_PRECISION_W :
                prec_name = "FFT_PRECISION_W                ";
                total_count = nFft_ * 2;
                break;
            case FFT_PRECISION_IN :
                prec_name = "FFT_PRECISION_IN               ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP1 :
                prec_name = "FFT_PRECISION_ST1_LP1          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP2 :
                prec_name = "FFT_PRECISION_ST1_LP2          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP3 :
                prec_name = "FFT_PRECISION_ST1_LP3          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP4 :
                prec_name = "FFT_PRECISION_ST1_LP4          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_OUT :
                prec_name = "FFT_PRECISION_ST1_OUT          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP1 :
                prec_name = "FFT_PRECISION_ST2_LP1          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP2 :
                prec_name = "FFT_PRECISION_ST2_LP2          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP3 :
                prec_name = "FFT_PRECISION_ST2_LP3          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP4 :
                prec_name = "FFT_PRECISION_ST2_LP4          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_OUT :
                prec_name = "FFT_PRECISION_OUT              ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            }
            os << boost::format(" %s clip times :  (+) %d , (-) %d  / %d") % prec_name % clip_p_[i] % clip_n_[i] % total_count << std::endl;
        }
        os << std::endl;
    } else {
        os << "---- Yet Supported! IFFT256 radix-16 Block Clip Result ----" 
           << std::endl;
    }
}

