/*!
    @file  CFft4096r64.cpp
    @brief FFTサイズ4096 radix-64 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft4096r64.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft4096r64::CFft4096r64(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 4096) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft4096r64::~CFft4096r64()
{
}

/* FFT Calculation */
void CFft4096r64::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(64, std::vector<std::complex<double> >(64));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j] = In[64*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 64; i++) {
        RADIX_64(x[i], 0, i);
    }
    
    // reorder1
    std::vector<std::vector<std::complex<double> > > y(64, std::vector<std::complex<double> >(64));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 64; i++) {
        RADIX_64(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*64+j] = y[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            for (int q=0; q < 2; q++) {
                                for (int r=0; r < 2; r++) {
                                    for (int s=0; s < 2; s++) {
                                        for (int t=0; t < 2; t++) {
                                            for (int u=0; u < 2; u++) {
                                                for (int v=0; v < 2; v++) {
                                                    xWork_[i*2048+j*1024+k*512+n*256+m*128+p*64+q*32+r*16+s*8+t*4+u*2+v] = z[i+j*2+k*4+n*8+m*16+p*32+q*64+r*128+s*256+t*512+u*1024+v*2048];
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

void CFft4096r64::RADIX_64(std::vector<std::complex<double> >& x, 
                          const unsigned int stage,
                          const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(64);
    std::vector<std::complex<double> > x1(64);
    std::vector<std::complex<double> > x2(64);
    std::vector<std::complex<double> > x3(64);
    std::vector<std::complex<double> > x4(64);
    std::vector<std::complex<double> > x5(64);
    std::vector<std::complex<double> > x6(64);

    for (int i=0; i < 64; i++) {
        if (stage == 0) {
            std::cout << "num:" << number << ", in[" << i << "]:" << x[i] << std::endl;
        }
    }

    x0 = x;
    
    // loop1
    for (int i=0; i < 32; i++) {
        x1[i]    = x0[i] + x0[i+32];
        x1[i+32] = x0[i] - x0[i+32];
        if (i >= 16) {
            // x1 * (-j) : IQ交換で実施
            x1[i+32] = std::complex<double>(x1[i+32].imag(), -x1[i+32].real());
        }
    }
    
    // loop2
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 16; j++) {
            x2[i*32+j]    = x1[i*32+j] + x1[i*32+j+16];
            x2[i*32+j+16] = x1[i*32+j] - x1[i*32+j+16];
            if (i == 0) {
                // -n/16・π回転
                x2[i*32+j+16] *= W_[j*2*64];
            } else {
                // -n/32・π回転
                x2[i*32+j]    *= W_[j*64];
                x2[i*32+j+16] *= W_[j*3*64];
            }
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        for (int j=0; j < 8; j++) {
            x3[i*16+j]   = x2[i*16+j] + x2[i*16+j+8];
            x3[i*16+j+8] = x2[i*16+j] - x2[i*16+j+8];
            if (j >= 4) {
                // x3 * (-j) : IQ交換で実施
                x3[i*16+j+8] = std::complex<double>(x3[i*16+j+8].imag(), -x3[i*16+j+8].real());    
            }
        }
    }

    // loop4
    for (int i=0; i < 8; i++) {
        for (int j=0; j < 4; j++) {
            x4[i*8+j]   = x3[i*8+j] + x3[i*8+j+4];
            x4[i*8+j+4] = x3[i*8+j] - x3[i*8+j+4];
            if (i % 2 == 0) {
                // -n/4・π回転
                x4[i*8+j+4] *= W_[j*8*64];
            } else {
                // -n/8・π回転
                x4[i*8+j]   *= W_[j*4*64];
                x4[i*8+j+4] *= W_[j*4*3*64];
            }
        }
    }
    
    // loop5
    for (int i=0; i < 16; i++) {
        for (int j=0; j < 2; j++) {
            x5[i*4+j]   = x4[i*4+j] + x4[i*4+j+2];
            x5[i*4+j+2] = x4[i*4+j] - x4[i*4+j+2];
            if (j == 1) {
                // x5 * (-j) : IQ交換で実施
                x5[i*4+j+2] = std::complex<double>(x5[i*4+j+2].imag(), -x5[i*4+j+2].real());    
            }
        }
    }

    // loop6
    for (int i=0; i < 32; i++) {
        x6[i*2]   = x5[i*2] + x5[i*2+1];
        x6[i*2+1] = x5[i*2] - x5[i*2+1];
    }    

    // final stageでなかったら回転因子乗算する
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            if (stage == 0) {
                                // final stageでなかったら回転因子乗算する
                                x6[i*32+j*16+k*8+n*4+m*2+p] *= W_[number*(i+j*2+k*4+n*8+m*16+p*32)];
                            } else {
                                // final stageは規格化して出力
                                x6[i*32+j*16+k*8+n*4+m*2+p] /= 64.0;
                            }
                        }
                    }
                }
            }
        }
    }
    x = x6;
}

/* result get */
const std::vector<std::complex<double> >& CFft4096r64::get() const
{
    return xWork_;
}
