/*!
    @file  CFft4096r64P.cpp
    @brief FFTサイズ4096 radix-64 固定(ハード検討確認用, 精度検討用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "prec_FFT.h"
#include "CFft4096r64P.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft4096r64P::CFft4096r64P(const boost::shared_ptr<C_param_wrap> param,
                           const unsigned int nfft, 
                           const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft),
      p_fft_exec_times_()
{
    // チェック
    if (nfft != 4096) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    /* ビット制限clip回数 */
    clip_p_.resize(FFT_PREC_NAME_END, 0);
    clip_n_.resize(FFT_PREC_NAME_END, 0);

    // パラメータ関連
    std::string str;
    std::stringstream msg;
    boost::shared_ptr<Util_wrap> util = createUtil();    
    
    // パラメータ取得 (演算精度関連)
    str = "FFT_PRECISION_W";                    /* 回転因子テーブル精度 */
    util->checkAndGetParam(param, str, &pparam_.p_w, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_IN";                   /* 入力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_in, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP1";              /* stage1 loop1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp1, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP2";              /* stage1 loop2出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp2, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP3";              /* stage1 loop3出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp3, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP4";              /* stage1 loop4出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp4, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP5";              /* stage1 loop5出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp5, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_LP6";              /* stage1 loop6出力精度(回転因子乗算前) */
    util->checkAndGetParam(param, str, &pparam_.p_st1_lp6, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST1_OUT";              /* stage1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st1_out, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP1";              /* stage2 loop1出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp1, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP2";              /* stage2 loop2出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp2, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP3";              /* stage2 loop3出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp3, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP4";              /* stage2 loop4出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp4, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP5";              /* stage2 loop5出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp5, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_ST2_LP6";              /* stage2 loop6出力精度(規格化前) */
    util->checkAndGetParam(param, str, &pparam_.p_st2_lp6, C_param_wrap::INT_VECTOR);
    str = "FFT_PRECISION_OUT";                  /* 出力精度 */
    util->checkAndGetParam(param, str, &pparam_.p_out, C_param_wrap::INT_VECTOR);    
    
    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        W_[i] = prec_pos_round_clip_FFT(W_[i], pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }

    // バタフライ間の回転因子(HW化向け)
    root0p5_     = std::sqrt(0.5); // -n/4・π用
    rot1d8PIm_   = std::exp(-M_PI * j * 1.0 / 8.0);   // -1/8・π
    rot3d8PIm_   = std::exp(-M_PI * j * 3.0 / 8.0);   // -3/8・π
    rot1d16PIm_  = std::exp(-M_PI * j * 1.0 / 16.0);  // -1/16・π
    rot3d16PIm_  = std::exp(-M_PI * j * 3.0 / 16.0);  // -3/16・π
    rot5d16PIm_  = std::exp(-M_PI * j * 5.0 / 16.0);  // -5/16・π
    rot7d16PIm_  = std::exp(-M_PI * j * 7.0 / 16.0);  // -7/16・π
    rot1d32PIm_  = std::exp(-M_PI * j * 1.0 / 32.0);  // -1/32・π
    rot3d32PIm_  = std::exp(-M_PI * j * 3.0 / 32.0);  // -3/32・π
    rot5d32PIm_  = std::exp(-M_PI * j * 5.0 / 32.0);  // -5/32・π
    rot7d32PIm_  = std::exp(-M_PI * j * 7.0 / 32.0);  // -7/32・π
    rot9d32PIm_  = std::exp(-M_PI * j * 9.0 / 32.0);  // -9/32・π
    rot11d32PIm_ = std::exp(-M_PI * j * 11.0 / 32.0); // -11/32・π
    rot13d32PIm_ = std::exp(-M_PI * j * 13.0 / 32.0); // -13/32・π
    rot15d32PIm_ = std::exp(-M_PI * j * 15.0 / 32.0); // -15/32・π
    // 量子化
    root0p5_     = prec_pos_round_clip_FFT(root0p5_,     pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot1d8PIm_   = prec_pos_round_clip_FFT(rot1d8PIm_,   pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot3d8PIm_   = prec_pos_round_clip_FFT(rot3d8PIm_,   pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot1d16PIm_  = prec_pos_round_clip_FFT(rot1d16PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot3d16PIm_  = prec_pos_round_clip_FFT(rot3d16PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot5d16PIm_  = prec_pos_round_clip_FFT(rot5d16PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot7d16PIm_  = prec_pos_round_clip_FFT(rot7d16PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot1d32PIm_  = prec_pos_round_clip_FFT(rot1d32PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot3d32PIm_  = prec_pos_round_clip_FFT(rot3d32PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot5d32PIm_  = prec_pos_round_clip_FFT(rot5d32PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot7d32PIm_  = prec_pos_round_clip_FFT(rot7d32PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot9d32PIm_  = prec_pos_round_clip_FFT(rot9d32PIm_,  pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot11d32PIm_ = prec_pos_round_clip_FFT(rot11d32PIm_, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot13d32PIm_ = prec_pos_round_clip_FFT(rot13d32PIm_, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
    rot15d32PIm_ = prec_pos_round_clip_FFT(rot15d32PIm_, pparam_.p_w[0], pparam_.p_w[1], &clip_p_[FFT_PRECISION_W], &clip_n_[FFT_PRECISION_W]);
}

/*! デストラクタ */
CFft4096r64P::~CFft4096r64P()
{
}

/* FFT Calculation */
void CFft4096r64P::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(64, std::vector<std::complex<double> >(64));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j] = In[64*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 64; i++) {
        RADIX_64(x[i], 0, i);
    }
    
    // reorder1
    std::vector<std::vector<std::complex<double> > > y(64, std::vector<std::complex<double> >(64));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 64; i++) {
        RADIX_64(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*64+j] = y[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            for (int q=0; q < 2; q++) {
                                for (int r=0; r < 2; r++) {
                                    for (int s=0; s < 2; s++) {
                                        for (int t=0; t < 2; t++) {
                                            for (int u=0; u < 2; u++) {
                                                for (int v=0; v < 2; v++) {
                                                    xWork_[i*2048+j*1024+k*512+n*256+m*128+p*64+q*32+r*16+s*8+t*4+u*2+v] = z[i+j*2+k*4+n*8+m*16+p*32+q*64+r*128+s*256+t*512+u*1024+v*2048];
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }

     p_fft_exec_times_++; // ビット精度ログ出力用
}

void CFft4096r64P::RADIX_64(std::vector<std::complex<double> >& x, 
                            const unsigned int stage,
                            const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(64);
    std::vector<std::complex<double> > x1(64);
    std::vector<std::complex<double> > x2(64);
    std::vector<std::complex<double> > x3(64);
    std::vector<std::complex<double> > x4(64);
    std::vector<std::complex<double> > x5(64);
    std::vector<std::complex<double> > x6(64);

    x0 = x;

    // 量子化
    for (size_t i=0; i < x0.size(); i++) {
        if (stage == 0) {
            // stage1
            x0[i] = prec_pos_round_clip_FFT(x0[i], pparam_.p_in[0], pparam_.p_in[1], &clip_p_[FFT_PRECISION_IN], &clip_n_[FFT_PRECISION_IN]);
        }
    }
    
    // loop1
    for (int i=0; i < 32; i++) {
        x1[i]    = x0[i] + x0[i+32];
        x1[i+32] = x0[i] - x0[i+32];
        if (i >= 16) {
            // x1 * (-j) : IQ交換で実施
            x1[i+32] = std::complex<double>(x1[i+32].imag(), -x1[i+32].real());
        }
    }

    // 量子化
    for (int i=0; i < x1.size(); i++) {
        if (stage == 0) {
            x1[i] = prec_pos_round_clip_FFT(x1[i],   pparam_.p_st1_lp1[0], pparam_.p_st1_lp1[1], &clip_p_[FFT_PRECISION_ST1_LP1], &clip_n_[FFT_PRECISION_ST1_LP1]);
        } else {
            x1[i] = prec_pos_round_clip_FFT(x1[i],   pparam_.p_st2_lp1[0], pparam_.p_st2_lp1[1], &clip_p_[FFT_PRECISION_ST2_LP1], &clip_n_[FFT_PRECISION_ST2_LP1]);
        }
    }

    // loop2
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 16; j++) {
            x2[i*32+j]    = x1[i*32+j] + x1[i*32+j+16];
            x2[i*32+j+16] = x1[i*32+j] - x1[i*32+j+16];
            if (i == 0) {
                // -n/16・π回転
                //x2[i*32+j+16] *= W_[j*2*64];

                /* HW乗算モジュールを意識してベタ書き */
                std::complex<double> tmp;
                switch (j) {
                case  1: x2[i*32+j+16] *= rot1d16PIm_; break;  // -1/16・π
                case  2: x2[i*32+j+16] *= rot1d8PIm_;  break;  // -2/16・π = -1/8・π
                case  3: x2[i*32+j+16] *= rot3d16PIm_; break;  // -3/16・π
                case  4:                                       // -4/16・π = -1/4・π
                    tmp = std::complex<double>(x2[i*32+j+16].real()+x2[i*32+j+16].imag(), x2[i*32+j+16].imag()-x2[i*32+j+16].real());
                    x2[i*32+j+16] = tmp * root0p5_;
                    break;
                case  5: x2[i*32+j+16] *= rot5d16PIm_; break;  // -5/16・π
                case  6: x2[i*32+j+16] *= rot3d8PIm_;  break;  // -6/16・π = -3/8・π
                case  7: x2[i*32+j+16] *= rot7d16PIm_; break;  // -7/16・π
                case  8:                                       // -8/16・π = -1/2・π : IQ交換で実施
                    x2[i*32+j+16] =  std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case  9:                                       // -9/16・π : -1/16・πの90度回転
                    x2[i*32+j+16] *= rot1d16PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 10:                                       // -10/16・π : -2/16・πの90度回転
                    x2[i*32+j+16] *= rot1d8PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 11:                                       // -11/16・π : -3/16・πの90度回転
                    x2[i*32+j+16] *= rot3d16PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                    
                case 12:                                       // -12/16・π : -4/16・πの90度回転
                    tmp = std::complex<double>(x2[i*32+j+16].real()+x2[i*32+j+16].imag(), x2[i*32+j+16].imag()-x2[i*32+j+16].real());
                    x2[i*32+j+16] = tmp * root0p5_; 
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 13:                                       // -13/16・π : -5/16・πの90度回転
                    x2[i*32+j+16] *= rot5d16PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 14:                                       // -14/16・π : -6/16・πの90度回転
                    x2[i*32+j+16] *= rot3d8PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 15:                                       // -15/16・π : -7/16・πの90度回転
                    x2[i*32+j+16] *= rot7d16PIm_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                default : /* no operation */ ;
                }
                  
            } else {
                // -n/32・π回転
                //x2[i*32+j]    *= W_[j*64];
                //x2[i*32+j+16] *= W_[j*3*64];
                /* HW乗算モジュールを意識してベタ書き */
                std::complex<double> tmp;
                switch (j) {
                case  1: 
                    x2[i*32+j]    *= rot1d32PIm_;   // -1/32・π
                    x2[i*32+j+16] *= rot3d32PIm_;   // -1/32・π
                    break;
                case 2:
                    x2[i*32+j]    *= rot1d16PIm_;   // -2/32・π = -1/16・π
                    x2[i*32+j+16] *= rot3d16PIm_;   // -6/32・π = -3/16・π
                    break;
                case  3: 
                    x2[i*32+j]    *= rot3d32PIm_;   // -3/32・π
                    x2[i*32+j+16] *= rot9d32PIm_;   // -9/32・π
                    break;
                case 4:
                    x2[i*32+j]    *= rot1d8PIm_;    // -4/32・π = -1/8・π
                    x2[i*32+j+16] *= rot3d8PIm_;    // -12/32・π = -3/8・π
                    break;
                case  5: 
                    x2[i*32+j]    *= rot5d32PIm_;   // -5/32・π
                    x2[i*32+j+16] *= rot15d32PIm_;  // -15/32・π
                    break;
                case 6:
                    x2[i*32+j]    *= rot3d16PIm_;   // -6/32・π = -3/16・π
                    x2[i*32+j+16] *= rot1d16PIm_;   // -18/32・π = -9/16・π : -1/16・πの90度回転
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case  7: 
                    x2[i*32+j]    *= rot7d32PIm_;   // -7/32・π
                    x2[i*32+j+16] *= rot5d32PIm_;  // -21/32・π : -5/32・πの90度回転
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case  8: 
                    // -8/32・π = -1/4・π
                    tmp = std::complex<double>(x2[i*32+j].real()+x2[i*32+j].imag(), x2[i*32+j].imag()-x2[i*32+j].real());
                    x2[i*32+j]    = tmp * root0p5_;
                    // -24/32・π : -3/4・π  : -1/4・πの90度回転
                    tmp = std::complex<double>(x2[i*32+j+16].real()+x2[i*32+j+16].imag(), x2[i*32+j+16].imag()-x2[i*32+j+16].real());
                    x2[i*32+j+16] = tmp * root0p5_;
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case  9: 
                    x2[i*32+j]    *= rot9d32PIm_;   // -9/32・π
                    x2[i*32+j+16] *= rot11d32PIm_;  // -27/32・π : -11/32・πの90度回転
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case 10:
                    x2[i*32+j]    *= rot5d16PIm_;   // -10/32・π = -5/16・π
                    x2[i*32+j+16] *= rot7d16PIm_;   // -30/32・π = -15/16・π : -7/16・πの90度回転
                    x2[i*32+j+16] = std::complex<double>(x2[i*32+j+16].imag(), -x2[i*32+j+16].real());
                    break;
                case  11: 
                    x2[i*32+j]    *= rot11d32PIm_;  // -11/32・π
                    x2[i*32+j+16] *= rot1d32PIm_;   // -33/32・π : -1/32・πの180度回転
                    x2[i*32+j+16] = -x2[i*32+j+16];
                    break;
                case 12:
                    x2[i*32+j]    *= rot3d8PIm_;    // -12/32・π = -3/8・π
                    x2[i*32+j+16] *= rot1d8PIm_;    // -36/32・π = -9/8・π : -1/8・πの180度回転
                    x2[i*32+j+16] = -x2[i*32+j+16];
                    break;
                case  13: 
                    x2[i*32+j]    *= rot13d32PIm_;  // -13/32・π
                    x2[i*32+j+16] *= rot7d32PIm_;   // -39/32・π : -7/32・πの180度回転
                    x2[i*32+j+16] = -x2[i*32+j+16];
                    break;
                case 14:
                    x2[i*32+j]    *= rot7d16PIm_;   // -14/32・π = -7/16・π
                    x2[i*32+j+16] *= rot5d16PIm_;   // -42/32・π = -21/16・π : -5/16・πの180度回転
                    x2[i*32+j+16] = -x2[i*32+j+16];
                    break;
                case  15: 
                    x2[i*32+j]    *= rot15d32PIm_;  // -15/32・π
                    x2[i*32+j+16] *= rot13d32PIm_;   // -45/32・π : -13/32・πの180度回転
                    x2[i*32+j+16] = -x2[i*32+j+16];
                    break;
                default: /* no operation */ ;
                }
            }
        }
    }

    // 量子化
    for (int i=0; i < x2.size(); i++) {
        if (stage == 0) {
            x2[i] = prec_pos_round_clip_FFT(x2[i],   pparam_.p_st1_lp2[0], pparam_.p_st1_lp2[1], &clip_p_[FFT_PRECISION_ST1_LP2], &clip_n_[FFT_PRECISION_ST1_LP2]);
        } else {
            x2[i] = prec_pos_round_clip_FFT(x2[i],   pparam_.p_st2_lp2[0], pparam_.p_st2_lp2[1], &clip_p_[FFT_PRECISION_ST2_LP2], &clip_n_[FFT_PRECISION_ST2_LP2]);
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        for (int j=0; j < 8; j++) {
            x3[i*16+j]   = x2[i*16+j] + x2[i*16+j+8];
            x3[i*16+j+8] = x2[i*16+j] - x2[i*16+j+8];
            if (j >= 4) {
                // x3 * (-j) : IQ交換で実施
                x3[i*16+j+8] = std::complex<double>(x3[i*16+j+8].imag(), -x3[i*16+j+8].real());    
            }
        }
    }

    // 量子化
    for (int i=0; i < x3.size(); i++) {
        if (stage == 0) {
            x3[i] = prec_pos_round_clip_FFT(x3[i],   pparam_.p_st1_lp3[0], pparam_.p_st1_lp3[1], &clip_p_[FFT_PRECISION_ST1_LP3], &clip_n_[FFT_PRECISION_ST1_LP3]);
        } else {
            x3[i] = prec_pos_round_clip_FFT(x3[i],   pparam_.p_st2_lp3[0], pparam_.p_st2_lp3[1], &clip_p_[FFT_PRECISION_ST2_LP3], &clip_n_[FFT_PRECISION_ST2_LP3]);
        }
    }

    // loop4
    for (int i=0; i < 8; i++) {
        for (int j=0; j < 4; j++) {
            x4[i*8+j]   = x3[i*8+j] + x3[i*8+j+4];
            x4[i*8+j+4] = x3[i*8+j] - x3[i*8+j+4];
            if (i % 2 == 0) {
                // -n/4・π回転
                //x4[i*8+j+4] *= W_[j*8*64];
                /* HW乗算モジュールを意識してベタ書き */
                std::complex<double> tmp;
                switch (j) {
                case 1: // -1/4・π
                    tmp = std::complex<double>(x4[i*8+j+4].real()+x4[i*8+j+4].imag(), x4[i*8+j+4].imag()-x4[i*8+j+4].real());
                    x4[i*8+j+4] = tmp * root0p5_;
                    break;
                case 2: // -1/2・π : IQ交換で実施
                    x4[i*8+j+4] = std::complex<double>(x4[i*8+j+4].imag(), -x4[i*8+j+4].real());
                    break;
                case 3: // -3/4・π : -1/4・πの90度回転
                    tmp = std::complex<double>(x4[i*8+j+4].real()+x4[i*8+j+4].imag(), x4[i*8+j+4].imag()-x4[i*8+j+4].real());
                    x4[i*8+j+4] = tmp * root0p5_;
                    x4[i*8+j+4] = std::complex<double>(x4[i*8+j+4].imag(), -x4[i*8+j+4].real());
                    break;    
                default: /* no operation */ ;
                }
            } else {
                // -n/8・π回転
                //x4[i*8+j]   *= W_[j*4*64];
                //x4[i*8+j+4] *= W_[j*4*3*64];
                /* HW乗算モジュールを意識してベタ書き */
                std::complex<double> tmp;
                switch (j) {
                case 1: 
                    x4[i*8+j]   *= rot1d8PIm_;  // -1/8・π
                    x4[i*8+j+4] *= rot3d8PIm_;  // -3/8・π
                    break;
                case 2: 
                    // -2/8・π = -1/4・π
                    tmp = std::complex<double>(x4[i*8+j].real()+x4[i*8+j].imag(), x4[i*8+j].imag()-x4[i*8+j].real());
                    x4[i*8+j] = tmp * root0p5_;
                    // -6/8・π = -3/4・π : -1/4・πの90度回転
                    tmp = std::complex<double>(x4[i*8+j+4].real()+x4[i*8+j+4].imag(), x4[i*8+j+4].imag()-x4[i*8+j+4].real());
                    x4[i*8+j+4] = tmp * root0p5_;
                    x4[i*8+j+4] = std::complex<double>(x4[i*8+j+4].imag(), -x4[i*8+j+4].real());
                    break;
                case 3:
                    x4[i*8+j]   *= rot3d8PIm_;  // -3/8・π
                    x4[i*8+j+4] *= rot1d8PIm_;  // -9/8・π : -1/8・πの180度回転 
                    x4[i*8+j+4] = -x4[i*8+j+4];
                    break;
                default: /* no operation */ ;
                }
            }
        }
    }
    
    // 量子化
    for (int i=0; i < x4.size(); i++) {
        if (stage == 0) {
            x4[i] = prec_pos_round_clip_FFT(x4[i],   pparam_.p_st1_lp4[0], pparam_.p_st1_lp4[1], &clip_p_[FFT_PRECISION_ST1_LP4], &clip_n_[FFT_PRECISION_ST1_LP4]);
        } else {
            x4[i] = prec_pos_round_clip_FFT(x4[i],   pparam_.p_st2_lp4[0], pparam_.p_st2_lp4[1], &clip_p_[FFT_PRECISION_ST2_LP4], &clip_n_[FFT_PRECISION_ST2_LP4]);
        }
    }

    // loop5
    for (int i=0; i < 16; i++) {
        for (int j=0; j < 2; j++) {
            x5[i*4+j]   = x4[i*4+j] + x4[i*4+j+2];
            x5[i*4+j+2] = x4[i*4+j] - x4[i*4+j+2];
            if (j == 1) {
                // x5 * (-j) : IQ交換で実施
                x5[i*4+j+2] = std::complex<double>(x5[i*4+j+2].imag(), -x5[i*4+j+2].real());    
            }
        }
    }

    // 量子化
    for (int i=0; i < x5.size(); i++) {
        if (stage == 0) {
            x5[i] = prec_pos_round_clip_FFT(x5[i],   pparam_.p_st1_lp5[0], pparam_.p_st1_lp5[1], &clip_p_[FFT_PRECISION_ST1_LP5], &clip_n_[FFT_PRECISION_ST1_LP5]);
        } else {
            x5[i] = prec_pos_round_clip_FFT(x5[i],   pparam_.p_st2_lp5[0], pparam_.p_st2_lp5[1], &clip_p_[FFT_PRECISION_ST2_LP5], &clip_n_[FFT_PRECISION_ST2_LP5]);
        }
    }

    // loop6
    for (int i=0; i < 32; i++) {
        x6[i*2]   = x5[i*2] + x5[i*2+1];
        x6[i*2+1] = x5[i*2] - x5[i*2+1];
    }

    // 量子化
    for (int i=0; i < x6.size(); i++) {
        if (stage == 0) {
            x6[i] = prec_pos_round_clip_FFT(x6[i],   pparam_.p_st1_lp6[0], pparam_.p_st1_lp6[1], &clip_p_[FFT_PRECISION_ST1_LP6], &clip_n_[FFT_PRECISION_ST1_LP6]);
        } else {
            x6[i] = prec_pos_round_clip_FFT(x6[i],   pparam_.p_st2_lp6[0], pparam_.p_st2_lp6[1], &clip_p_[FFT_PRECISION_ST2_LP6], &clip_n_[FFT_PRECISION_ST2_LP6]);
        }
    }

    // final stageでなかったら回転因子乗算する
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            if (stage == 0) {
                                // final stageでなかったら回転因子乗算する
                                x6[i*32+j*16+k*8+n*4+m*2+p] *= W_[number*(i+j*2+k*4+n*8+m*16+p*32)];
                            } else {
                                // final stageは規格化して出力
                                x6[i*32+j*16+k*8+n*4+m*2+p] /= 64.0;
                            }
                        }
                    }
                }
            }
        }
    }

    // 量子化
    for (size_t i=0; i < x6.size(); i++) {
        if (stage == 0) {
            // stage1出力
            x6[i] = prec_pos_round_clip_FFT(x6[i], pparam_.p_st1_out[0], pparam_.p_st1_out[1], &clip_p_[FFT_PRECISION_ST1_OUT], &clip_n_[FFT_PRECISION_ST1_OUT]);
        } else {
            // stage2出力(FFT出力)
            x6[i] = prec_pos_round_clip_FFT(x6[i], pparam_.p_out[0], pparam_.p_out[1], &clip_p_[FFT_PRECISION_OUT], &clip_n_[FFT_PRECISION_OUT]);
        }
    }
    x = x6;
}

/* result get */
const std::vector<std::complex<double> >& CFft4096r64P::get() const
{
    return xWork_;
}

/* Report出力 */
void CFft4096r64P::ReportStatisticalData(std::ostream& os, const double NowTime)
{
    int active_count = 0;  // とりあえず

    if (!ifft_) {
    /* ビット精度最終結果ログ出力 */
        os << boost::format("\n----- FEQ FFT4096 radix-64 Block Clip Result -----  active_count = %.0f  DSPTOP_NowTime = %.0f") % p_fft_exec_times_ % NowTime << std::endl;
        std::string prec_name;
        for (int i=0; i < FFT_PREC_NAME_END; i++) {
            int total_count;
            switch (i) {
            case FFT_PRECISION_W :
                prec_name = "FFT_PRECISION_W                ";
                total_count = nFft_ * 2;
                break;
            case FFT_PRECISION_IN :
                prec_name = "FFT_PRECISION_IN               ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP1 :
                prec_name = "FFT_PRECISION_ST1_LP1          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP2 :
                prec_name = "FFT_PRECISION_ST1_LP2          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP3 :
                prec_name = "FFT_PRECISION_ST1_LP3          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP4 :
                prec_name = "FFT_PRECISION_ST1_LP4          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP5 :
                prec_name = "FFT_PRECISION_ST1_LP5          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_LP6 :
                prec_name = "FFT_PRECISION_ST1_LP6          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST1_OUT :
                prec_name = "FFT_PRECISION_ST1_OUT          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP1 :
                prec_name = "FFT_PRECISION_ST2_LP1          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP2 :
                prec_name = "FFT_PRECISION_ST2_LP2          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP3 :
                prec_name = "FFT_PRECISION_ST2_LP3          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP4 :
                prec_name = "FFT_PRECISION_ST2_LP4          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP5 :
                prec_name = "FFT_PRECISION_ST2_LP5          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_ST2_LP6 :
                prec_name = "FFT_PRECISION_ST2_LP6          ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            case FFT_PRECISION_OUT :
                prec_name = "FFT_PRECISION_OUT              ";
                total_count = p_fft_exec_times_ * nFft_ * 2;
                break;
            }
            os << boost::format(" %s clip times :  (+) %d , (-) %d  / %d") % prec_name % clip_p_[i] % clip_n_[i] % total_count << std::endl;
        }
        os << std::endl;
    } else {
        os << "---- Yet Supported! IFFT4096 radix-64 Block Clip Result ----" 
           << std::endl;
    }
}

