/*!
    @file  CFft4096r64.h
    @brief FFTサイズ4096 radix-64 固定(ハード検討確認用, 精度検討用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFft4096r64P_h
#define CFft4096r64P_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "Util_wrap.h"
#include "CFftIF.h"

class CFft4096r64P : public CFftIF {
public:
    /* コンストラクタ */
    CFft4096r64P(const boost::shared_ptr<C_param_wrap> param,
                 const unsigned int nfft,
                 const bool ifft);
    
    /* デストラクタ */
    virtual ~CFft4096r64P();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    /*!Report出力 */
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime);
   
private:
    struct SPrecisionParam {
        std::vector<int>     p_in;                /*!< 入力精度 */
        std::vector<int>     p_w;                 /*!< 回転因子テーブル精度 */
        std::vector<int>     p_st1_lp1;           /*!< stage1 loop1出力精度 */
        std::vector<int>     p_st1_lp2;           /*!< stage1 loop2出力精度 */
        std::vector<int>     p_st1_lp3;           /*!< stage1 loop3出力精度 */
        std::vector<int>     p_st1_lp4;           /*!< stage1 loop4出力精度 */
        std::vector<int>     p_st1_lp5;           /*!< stage1 loop5出力精度 */
        std::vector<int>     p_st1_lp6;           /*!< stage1 loop6出力精度(回転因子乗算前) */
        std::vector<int>     p_st1_out;           /*!< stage1出力精度 */
        std::vector<int>     p_st2_lp1;           /*!< stage2 loop1出力精度 */
        std::vector<int>     p_st2_lp2;           /*!< stage2 loop2出力精度 */
        std::vector<int>     p_st2_lp3;           /*!< stage2 loop3出力精度 */
        std::vector<int>     p_st2_lp4;           /*!< stage2 loop4出力精度 */
        std::vector<int>     p_st2_lp5;           /*!< stage1 loop5出力精度 */
        std::vector<int>     p_st2_lp6;           /*!< stage1 loop6出力精度(規格化前) */
        std::vector<int>     p_out;               /*!< 出力精度 */
    };

    enum FFT_BIT_PREC_NAME {
        FFT_PRECISION_W,
        FFT_PRECISION_IN,
        FFT_PRECISION_ST1_LP1,
        FFT_PRECISION_ST1_LP2,
        FFT_PRECISION_ST1_LP3,
        FFT_PRECISION_ST1_LP4,
        FFT_PRECISION_ST1_LP5,
        FFT_PRECISION_ST1_LP6,
        FFT_PRECISION_ST1_OUT,
        FFT_PRECISION_ST2_LP1,
        FFT_PRECISION_ST2_LP2,
        FFT_PRECISION_ST2_LP3,
        FFT_PRECISION_ST2_LP4,
        FFT_PRECISION_ST2_LP5,
        FFT_PRECISION_ST2_LP6,
        FFT_PRECISION_OUT,
        FFT_PREC_NAME_END
    };


    unsigned int nFft_;                        /* FFT point Num */
    bool ifft_;                                /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_; /* FFT work      */
    std::vector<std::complex<double> > W_;     /* 回転因子      */
    /* バタフライ間回転因子(HW化向け) */
    double root0p5_;                    // -n/4・π用
    std::complex<double> rot1d8PIm_;    // -1/8・π用
    std::complex<double> rot3d8PIm_;    // -3/8・π
    std::complex<double> rot1d16PIm_;   // -1/16・π
    std::complex<double> rot3d16PIm_;   // -3/16・π
    std::complex<double> rot5d16PIm_;   // -5/16・π
    std::complex<double> rot7d16PIm_;   // -7/16・π
    std::complex<double> rot1d32PIm_;   // -1/32・π
    std::complex<double> rot3d32PIm_;   // -3/32・π
    std::complex<double> rot5d32PIm_;   // -5/32・π
    std::complex<double> rot7d32PIm_;   // -7/32・π
    std::complex<double> rot9d32PIm_;   // -9/32・π
    std::complex<double> rot11d32PIm_;  // -11/32・π
    std::complex<double> rot13d32PIm_;  // -13/32・π
    std::complex<double> rot15d32PIm_;  // -15/32・π


    /* ビット制限関連 */
    unsigned int                       p_fft_exec_times_;
    SPrecisionParam                    pparam_;/* 量子化パラメータ */
    std::vector<unsigned int>          clip_p_;
    std::vector<unsigned int>          clip_n_;
    
    void RADIX_64(std::vector<std::complex<double> >& x, 
                  const unsigned int stage,
                  const unsigned int number);
};

#endif /* CFft4096r64P_h */
