/*!
    @file  CFft4r4.cpp
    @brief FFTサイズ4 radix-4 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft4r4.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft4r4::CFft4r4(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 4) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;
    
    // 回転因子(半分でいいけどFFTサイズ分生成しておく)
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft4r4::~CFft4r4()
{
}

/* FFT Calculation */
void CFft4r4::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(nFft_);
    std::vector<std::complex<double> > x1(nFft_);
    std::vector<std::complex<double> > x2(nFft_);
    
    
    x0 = In;
    
    // loop1
    for (int i=0; i < 2; i++) {
        x1[i]   =  x0[i] + x0[i+2];
        x1[i+2] = (x0[i] - x0[i+2]) * W_[i];
    }
    
    // loop2
    for (int i=0; i < 2; i++) {
        x2[i*2]   = x1[i*2] + x1[i*2+1];
        x2[i*2+1] = x1[i*2] - x1[i*2+1];
    }

    // reorder
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            xWork_[i*2+j] = x2[i+j*2];
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

/* result get */
const std::vector<std::complex<double> >& CFft4r4::get() const
{
    return xWork_;
}
