/*!
    @file  CFft512r8.cpp
    @brief FFTサイズ512 radix-8 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft512r8.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft512r8::CFft512r8(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 512) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft512r8::~CFft512r8()
{
}

/* FFT Calculation */
void CFft512r8::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(64, std::vector<std::complex<double> >(8));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j] = In[64*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 64; i++) {
        RADIX_8(x[i], 0, i);
    }
    
    // reorder1
    std::vector<std::vector<std::complex<double> > > y1(64, std::vector<std::complex<double> >(8));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y1[i][j] = x[i%8+j*8][i/8];
        }
    }
    
    // stage2
    for (int i=0; i < 64; i++) {
        RADIX_8(y1[i], 1, i);
    }

    // reorder2
    std::vector<std::vector<std::complex<double> > > y2(64, std::vector<std::complex<double> >(8));
    for (size_t i=0; i < y1.size(); i++) {
        for (size_t j=0; j < y1[i].size(); j++) {
            y2[i][j] = y1[(i/8)*8+j][i%8];
        }
    }
    
    // stage3
    for (int i=0; i < 64; i++) {
        RADIX_8(y2[i], 2, i);
    }

    // reorder3
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y2.size(); i++) {
        for (size_t j=0; j < y2[i].size(); j++) {
            z[i*8+j] = y2[i][j];
        }
    }
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            for (int q=0; q < 2; q++) {
                                for (int r=0; r < 2; r++) {
                                    for (int s=0; s < 2; s++) {
                                        xWork_[i*256+j*128+k*64+n*32+m*16+p*8+q*4+r*2+s] = z[i+j*2+k*4+n*8+m*16+p*32+q*64+r*128+s*256];
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

void CFft512r8::RADIX_8(std::vector<std::complex<double> >& x, 
                          const unsigned int stage,
                          const unsigned int number)
{
    //std::cout << "RADIX_8 stage:" << stage << ", " << number << std::endl;
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(8);
    std::vector<std::complex<double> > x1(8);
    std::vector<std::complex<double> > x2(8);
    std::vector<std::complex<double> > x3(8);
    
    x0 = x;
    
    // loop1
    for (int i=0; i < 4; i++) {
        x1[i]   =  x0[i] + x0[i+4];
        x1[i+4] = (x0[i] - x0[i+4]) * W_[i*64];
    }
    
    // loop2
    for (int j=0; j < 2; j++) {
        for (int i=0; i < 2; i++) {
            x2[i+j*4]   =  x1[i+j*4] + x1[i+2+j*4];
            x2[i+2+j*4] = (x1[i+j*4] - x1[i+2+j*4]) * W_[i*2*64];
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        x3[i*2]   = x2[i*2] + x2[i*2+1];
        x3[i*2+1] = x2[i*2] - x2[i*2+1];
    }

    // 回転因子乗算
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                if (stage == 0) {
                    // stage1 回転因子乗算 W512
                    x3[i*4+j*2+k] *= W_[number*(i+j*2+k*4)];
                } else if (stage == 1) {
                    // stage2 回転因子乗算 W64
                    x3[i*4+j*2+k] *= W_[(number%8)*(i+j*2+k*4)*8];
                }
            }
        }
    }

    x = x3;
}

/* result get */
const std::vector<std::complex<double> >& CFft512r8::get() const
{
    return xWork_;
}
