/*!
    @file  CFft64r64.cpp
    @brief FFTサイズ64 radix-64 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft64r64.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft64r64::CFft64r64(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 64) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子(半分でいいけどFFTサイズ分生成しておく)
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft64r64::~CFft64r64()
{
}

/* FFT Calculation */
void CFft64r64::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(nFft_);
    std::vector<std::complex<double> > x1(nFft_);
    std::vector<std::complex<double> > x2(nFft_);
    std::vector<std::complex<double> > x3(nFft_);
    std::vector<std::complex<double> > x4(nFft_);
    std::vector<std::complex<double> > x5(nFft_);
    std::vector<std::complex<double> > x6(nFft_);

    x0 = In;
    
    // loop1
    for (int i=0; i < 32; i++) {
        x1[i]    =  x0[i] + x0[i+32];
        x1[i+32] = (x0[i] - x0[i+32]) * W_[i];
    }
    
    // loop2
    for (int j=0; j < 2; j++) {
        for (int i=0; i < 16; i++) {
            x2[i+j*32]    =  x1[i+j*32] + x1[i+16+j*32];
            x2[i+16+j*32] = (x1[i+j*32] - x1[i+16+j*32]) * W_[i*2];
        }
    }

    // loop3
    for (int j=0; j < 4; j++) {
        for (int i=0; i < 8; i++) {
            x3[i+j*16]   =  x2[i+j*16] + x2[i+8+j*16];
            x3[i+8+j*16] = (x2[i+j*16] - x2[i+8+j*16]) * W_[i*4];
        }
    }

    // loop4
    for (int j=0; j < 8; j++) {
        for (int i=0; i < 4; i++) {
            x4[i+j*8]   =  x3[i+j*8] + x3[i+4+j*8];
            x4[i+4+j*8] = (x3[i+j*8] - x3[i+4+j*8]) * W_[i*8];
        }
    }
    
    // loop5
    for (int j=0; j < 16; j++) {
        for (int i=0; i < 2; i++) {
            x5[i+j*4]   =  x4[i+j*4] + x4[i+2+j*4];
            x5[i+2+j*4] = (x4[i+j*4] - x4[i+2+j*4]) * W_[i*16];
        }
    }

    // loop6
    for (int i=0; i < 32; i++) {
        x6[i*2]   = x5[i*2] + x5[i*2+1];
        x6[i*2+1] = x5[i*2] - x5[i*2+1];
    }    
    
    
    // reorder
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            xWork_[i*32+j*16+k*8+n*4+m*2+p] = x6[i+j*2+k*4+n*8+m*16+p*32];
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

/* result get */
const std::vector<std::complex<double> >& CFft64r64::get() const
{
    return xWork_;
}
