/*!
    @file  CFft64r8.cpp
    @brief FFTサイズ64 radix-8 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFft64r8.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFft64r8::CFft64r8(const unsigned int nfft, 
                 const bool ifft)
    : nFft_(nfft),
      ifft_(ifft),
      xWork_(nfft),
      W_(nfft)
{
    // チェック
    if (nfft != 64) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(0);
    }

    double sgn = -1.0;
    if (ifft) sgn = 1.0;

    // 回転因子
    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    for (size_t i=0; i < W_.size(); i++) {
        W_[i] = std::exp(sgn * 2.0 * M_PI * j * double(i) / double(nfft));
        // debug時に見やすいように0付近の値を丸める(なくてもよい処理)
        CFftIF::ComplexForLog(W_);
        //std::cout << "W[" << i << "]:" << W_[i] << std::endl;
    }
}

/*! デストラクタ */
CFft64r8::~CFft64r8()
{
}

/* FFT Calculation */
void CFft64r8::fft(const std::vector<std::complex<double> >& In)
{
    // わかりやすいようにベタ書き
    std::vector<std::vector<std::complex<double> > > x(8, std::vector<std::complex<double> >(8));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            x[i][j]   = In[8*j+i];
        }
    }
    
    // stage1
    for (int i=0; i < 8; i++) {
        RADIX_8(x[i], 0, i);
    }

    // reorder1
    std::vector<std::vector<std::complex<double> > > y(8, std::vector<std::complex<double> >(8));
    for (size_t i=0; i < x.size(); i++) {
        for (size_t j=0; j < x[i].size(); j++) {
            y[i][j] = x[j][i];
        }
    }
    
    // stage2
    for (int i=0; i < 8; i++) {
        RADIX_8(y[i], 1, i);
    }

    // reorder2
    std::vector<std::complex<double> > z(nFft_);
    for (size_t i=0; i < y.size(); i++) {
        for (size_t j=0; j < y[i].size(); j++) {
            z[i*8+j] = y[i][j];
        }
    }
    
    for (int i=0; i < 2; i++) {
        for (int j=0; j < 2; j++) {
            for (int k=0; k < 2; k++) {
                for (int n=0; n < 2; n++) {
                    for (int m=0; m < 2; m++) {
                        for (int p=0; p < 2; p++) {
                            xWork_[i*32+j*16+k*8+n*4+m*2+p] = z[i+j*2+k*4+n*8+m*16+p*32];
                        }
                    }
                }
            }
        }
    }

    if (ifft_) {
        for (int i=0; i < xWork_.size(); i++) {
            xWork_[i] /= static_cast<double>(nFft_);
        }
    }
}

void CFft64r8::RADIX_8(std::vector<std::complex<double> >& x, 
                       const unsigned int stage,
                       const unsigned int number)
{
    // わかりやすいようにベタ書き
    std::vector<std::complex<double> > x0(8);
    std::vector<std::complex<double> > x1(8);
    std::vector<std::complex<double> > x2(8);
    std::vector<std::complex<double> > x3(8);
    
    x0 = x;
    
    // loop1
    for (int i=0; i < 4; i++) {
        x1[i]   =  x0[i] + x0[i+4];
        x1[i+4] = (x0[i] - x0[i+4]) * W_[i*8];
    }
    
    // loop2
    for (int j=0; j < 2; j++) {
        for (int i=0; i < 2; i++) {
            x2[i+j*4]   =  x1[i+j*4] + x1[i+2+j*4];
            x2[i+2+j*4] = (x1[i+j*4] - x1[i+2+j*4]) * W_[i*16];
        }
    }

    // loop3
    for (int i=0; i < 4; i++) {
        x3[i*2]   = x2[i*2] + x2[i*2+1];
        x3[i*2+1] = x2[i*2] - x2[i*2+1];
    }

    // final stageでなかったら回転因子乗算する
    if (stage == 0) {
        x3[1] *= W_[number*4];
        x3[2] *= W_[number*2];
        x3[3] *= W_[number*6];
        x3[4] *= W_[number*1];
        x3[5] *= W_[number*5];
        x3[6] *= W_[number*3];
        x3[7] *= W_[number*7];
    }
    
    x = x3;
}

/* result get */
const std::vector<std::complex<double> >& CFft64r8::get() const
{
    return xWork_;
}
