/*!
    @file  CFftRef.cpp
    @brief Fast Fourier transform

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFftRef.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
CFftRef::CFftRef(const unsigned int nfft, 
                 const unsigned int radix,
                 const bool ifft)
    : nFft_(nfft),
      Radix_(radix),
      ifft_(ifft),
      xWork_(nfft)
{
}

/*! デストラクタ */
CFftRef::~CFftRef()
{
}

/* FFT Calculation */
void CFftRef::fft(const std::vector<std::complex<double> >& In)
{
     xWork_ = In;
     fft_exec(xWork_, nFft_, Radix_, ifft_);
     if (ifft_) {
         for (int i=0; i < xWork_.size(); i++) {
             xWork_[i] /= static_cast<double>(nFft_);
         }
     }
}

/* result get */
const std::vector<std::complex<double> >& CFftRef::get() const
{
    return xWork_;
}



// ----------------------------------------------------------------------
// FFTエンジン
// -----------------------------------------------------------------------
void CFftRef::fft_exec(std::vector<std::complex<double> >& x, const int length, const int radix, const bool ifft)
{
    if (length <= radix-1) return;

    double sgn;
    if (ifft) sgn =  1.0;
    else      sgn = -1.0;

    const std::complex<double> j = std::complex<double>(0.0, 1.0);
    std::vector<std::complex<double> > rot(radix);
    for (int i=0; i < rot.size(); i++) {
        rot[i] = std::conj(std::exp(2.0 * j * M_PI * double(i) / double(radix)));
    }
    
    int dip = length / radix;
    std::vector<std::vector<std::complex<double> > > X(radix, std::vector<std::complex<double> >(dip, 0.0));
    for (int i = 0; i < dip; i++) {
        std::vector<std::complex<double> > W(radix);
        for (int nn=0; nn < radix; nn++) {
            // 加算
            for (int mm=0; mm < radix; mm++) {
                X[nn][i] += x[i+mm*dip] * rot[nn*mm % radix];
            }
            // 回転因子
            W[nn] = std::exp(sgn * 2.0 * M_PI * j * double(i) * double(nn) / double(length));
            // 乗算
            X[nn][i] = X[nn][i] * W[nn];
        }
    }

    
    for (size_t nn=0; nn < X.size(); nn++) {
        fft_exec(X[nn], dip, radix, ifft);
    }
    
    for (int i=0; i < dip; i++) {
        for (size_t nn=0; nn < X.size(); nn++) {
            x[i*radix+nn] = X[nn][i];
        }
    }
}

