/*!
    @file  CFftWrap.cpp
    @brief Fast Fourier transform

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "CFftRef.h"
#include "CFft4r4.h"
#include "CFft8r8.h"
#include "CFft16r16.h"
#include "CFft64r64.h"
#include "CFft16r4.h"
#include "CFft64r4.h"
#include "CFft64r8.h"
#include "CFft256r16.h"
#include "CFft256r16P.h"
#include "CFft512r8.h"
#include "CFft4096r64.h"
#include "CFft4096r64P.h"
#include "CFftWrap.h"
#include "prec_FFT.h"


/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
FUJITSU::CFftWrap::CFftWrap(const boost::shared_ptr<C_param_wrap> param)
{
    
    
    std::string str;
    std::stringstream msg;
    boost::shared_ptr<Util_wrap> util = createUtil();

    /* とりあえず FFTサイズ と radixを取得する (CFftRef用) */
    str = "FFT_POINT";
    util->checkAndGetParam(param, str, &param_.FftPoint, C_param_wrap::INT);
    str = "FFT_RADIX";
    util->checkAndGetParam(param, str, &param_.FftRadix, C_param_wrap::INT);    

    /* アルゴリズム選択 */
    str = "FFT_Algorithm";
    std::string algo_name;
    if (util->checkAndGetParam(param, str, &algo_name, C_param_wrap::STRING)) {
        if (algo_name == "FFT") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFftRef(param_.FftPoint, param_.FftRadix, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFftRef(param_.FftPoint, param_.FftRadix, true));
        } else if (algo_name == "FFT4_RADIX4") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft4r4(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft4r4(param_.FftPoint, true));
        } else if (algo_name == "FFT8_RADIX8") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft8r8(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft8r8(param_.FftPoint, true));
        } else if (algo_name == "FFT16_RADIX16") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft16r16(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft16r16(param_.FftPoint, true));
        } else if (algo_name == "FFT64_RADIX64") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft64r64(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft64r64(param_.FftPoint, true));
        } else if (algo_name == "FFT16_RADIX4") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft16r4(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft16r4(param_.FftPoint, true));
        } else if (algo_name == "FFT64_RADIX4") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft64r4(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft64r4(param_.FftPoint, true));
        } else if (algo_name == "FFT64_RADIX8") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft64r8(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft64r8(param_.FftPoint, true));
        } else if (algo_name == "FFT256_RADIX16") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft256r16(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft256r16(param_.FftPoint, true));
        } else if (algo_name == "FFT256_RADIX16P") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft256r16P(param, param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft256r16P(param, param_.FftPoint, true));
        } else if (algo_name == "FFT512_RADIX8") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft512r8(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft512r8(param_.FftPoint, true));
        } else if (algo_name == "FFT4096_RADIX64") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft4096r64(param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft4096r64(param_.FftPoint, true));
        } else if (algo_name == "FFT4096_RADIX64P") {
            fft_operator_  = boost::shared_ptr<CFftIF>(new CFft4096r64P(param, param_.FftPoint, false));
            ifft_operator_ = boost::shared_ptr<CFftIF>(new CFft4096r64P(param, param_.FftPoint, true));
        } else {
            g_err->appendMsg("FFT_Algorithm is invalid name");
            msg.str("");
            msg.clear(std::stringstream::goodbit);
        }
    }

    if (algo_name == "FFT") {
        // FFTサイズ, RADIXパラメータのチェック
        // CFftRef時のみ有効
        if (param_.FftPoint <= 1 || param_.FftPoint > 4096  || !isPowerOfTwo(param_.FftPoint)) {
            msg << "Parameter " << str << " is in valid. 2〜4096 かつ 2のべき乗を設定してください." << std::endl;
            g_err->appendMsg(msg.str()) ;
            msg.str("");
            msg.clear(std::stringstream::goodbit);
        }
        
        if (param_.FftRadix <= 1 || param_.FftRadix > 64  || !isPowerOfTwo(param_.FftRadix)) {
            msg << "Parameter " << str << " is in valid. 2〜64 かつ 2のべき乗を設定してください." << std::endl;
            g_err->appendMsg(msg.str()) ;
            msg.str("");
            msg.clear(std::stringstream::goodbit);
        }
        
        // FFT sizeと基数のチェック
        //   FFT sizeを最大 4096 に制限
        //   基数を最大 64に制限したので ... とりあえずべた書きする
        bool result = false;
        if (param_.FftRadix == 2) {
            result = true;
        } else if (param_.FftRadix == 4) {
            if (param_.FftPoint == 4   || param_.FftPoint == 16   || param_.FftPoint == 64 || 
                param_.FftPoint == 256 || param_.FftPoint == 1024 || param_.FftPoint == 4096) {
                result = true;
            }
        } else if (param_.FftRadix == 8) {
            if (param_.FftPoint == 8   || param_.FftPoint == 64   || param_.FftPoint == 512 || 
                param_.FftPoint == 4096) {
                result = true;
            }
        } else if (param_.FftRadix == 16) {
            if (param_.FftPoint == 16   || param_.FftPoint == 256   || param_.FftPoint == 4096) {
                result = true;
            }
        } else if (param_.FftRadix == 32) {
            if (param_.FftPoint == 32   || param_.FftPoint == 1024) {
                result = true;
            }
        } else if (param_.FftRadix == 64) {
            if (param_.FftPoint == 64   || param_.FftPoint == 4096) {
                result = true;
            }
        }
        
        if (!result) {
            msg << "FFT_POINT と FFT_RADIXの対応がおかしいです．" << std::endl;
            g_err->appendMsg(msg.str()) ;
            msg.str("");
            msg.clear(std::stringstream::goodbit);
        }
    }  // FFTサイズ, RADIXパラメータの取得/チェック
        
    /* サンプリング周波数(log */
    str = "SAMPLING_FREQ";
    util->checkAndGetParam(param, str, &param_.SamplingFreq, C_param_wrap::DOUBLE);  // log出力に使用

    // log出力関連
    std::string str_fft_in_onoff;
    std::string str_fft_in_file;
    str = "FFT_IN_LOG";
    util->checkAndGetParam(param, str, &str_fft_in_onoff, C_param_wrap::STRING);
    if (str_fft_in_onoff == "on" || str_fft_in_onoff == "ON") {
        str = "FFT_IN_LOG_FILE_NAME";
        if (util->checkAndGetParam(param, str, &str_fft_in_file, C_param_wrap::STRING)) {
            fp_fft_in_ = boost::shared_ptr<std::ofstream>(new std::ofstream);
            util->openAndCheck(fp_fft_in_, str_fft_in_file);
        }
    }
    
    std::string str_fft_out_onoff;
    std::string str_fft_out_file;
    str = "FFT_OUT_LOG";
    util->checkAndGetParam(param, str, &str_fft_out_onoff, C_param_wrap::STRING);
    if (str_fft_out_onoff == "on" || str_fft_out_onoff == "ON") {
        str = "FFT_OUT_LOG_FILE_NAME";
        if (util->checkAndGetParam(param, str, &str_fft_out_file, C_param_wrap::STRING)) {
            fp_fft_out_ = boost::shared_ptr<std::ofstream>(new std::ofstream);
            util->openAndCheck(fp_fft_out_, str_fft_out_file);
        }
    }

    std::string str_ifft_in_onoff;
    std::string str_ifft_in_file;
    str = "IFFT_IN_LOG";
    util->checkAndGetParam(param, str, &str_ifft_in_onoff, C_param_wrap::STRING);
    if (str_ifft_in_onoff == "on" || str_ifft_in_onoff == "ON") {
        str = "IFFT_IN_LOG_FILE_NAME";
        if (util->checkAndGetParam(param, str, &str_ifft_in_file, C_param_wrap::STRING)) {
            fp_ifft_in_ = boost::shared_ptr<std::ofstream>(new std::ofstream);
            util->openAndCheck(fp_ifft_in_, str_ifft_in_file);
        }
    }
    
    std::string str_ifft_out_onoff;
    std::string str_ifft_out_file;
    str = "FFT_OUT_LOG";
    util->checkAndGetParam(param, str, &str_ifft_out_onoff, C_param_wrap::STRING);
    if (str_ifft_out_onoff == "on" || str_ifft_out_onoff == "ON") {
        str = "IFFT_OUT_LOG_FILE_NAME";
        if (util->checkAndGetParam(param, str, &str_ifft_out_file, C_param_wrap::STRING)) {
            fp_ifft_out_ = boost::shared_ptr<std::ofstream>(new std::ofstream);
            util->openAndCheck(fp_ifft_out_, str_ifft_out_file);
        }
    }
}

/*! デストラクタ */
FUJITSU::CFftWrap::~CFftWrap()
{
}

/*! FFT/IFFT処理実行 */
void FUJITSU::CFftWrap::Execute(const std::vector<std::complex<double> >& InputData,
                            std::vector<std::complex<double> >& OutputData,
                            const bool ifft)
{
    if (!ifft) {
        fft_operator_->fft(InputData);
        OutputData = fft_operator_->get();
        
        // log用に絶対値10-E7 以下を0に丸める
        //ComplexForLog(OutputData);
        // 見やすいように四捨五入
        for (size_t i=0; i < OutputData.size(); i++) {
            OutputData[i] = prec_pos_round_FFT(OutputData[i], 20, 32);
        }

        // Log出力
        if (fp_fft_in_) {
            *fp_fft_in_ << "No, 時間[ns], real, imag, amp" << std::endl;
            for (size_t i=0; i < InputData.size(); i++) {
                double t = i * (1.0 / param_.SamplingFreq) * 1000.0;
                *fp_fft_in_ << i << ", " << t << ", " 
                            << InputData[i].real() << ", " << InputData[i].imag() << ", "
                            << std::abs(InputData[i])
                            << std::endl;
            }
        }
        if (fp_fft_out_) {
            *fp_fft_out_ << "周波数[MHz], FFT出力絶対値" << std::endl;
            for (size_t i=0; i < OutputData.size(); i++) {
                double freq = i * param_.SamplingFreq / static_cast<double>(OutputData.size());
                *fp_fft_out_ << freq << ", " << std::abs(OutputData[i]) << std::endl;
                std::cout << freq << ", " << OutputData[i] << std::endl;
            }
        }
    } else {
        // IFFT
        ifft_operator_->fft(InputData);
        OutputData = ifft_operator_->get();
        
        // log用に絶対値10-E7 以下を0に丸める
        ComplexForLog(OutputData);
        // Log出力
        if (fp_ifft_in_) {
            *fp_ifft_in_ << "周波数[MHz], FFT出力絶対値" << std::endl;
            for (size_t i=0; i < InputData.size(); i++) {
                double freq = i * param_.SamplingFreq / static_cast<double>(InputData.size());
                *fp_ifft_in_ << freq << ", " << std::abs(InputData[i]) << std::endl;
            }
        }
        
        if (fp_ifft_out_) {
            *fp_ifft_out_ << "No, 時間[ns], real, imag, amp" << std::endl;
            for (size_t i=0; i < OutputData.size(); i++) {
                double t = i * (1.0 / param_.SamplingFreq) * 1000.0;
                *fp_ifft_out_ << i << ", " << t << ", " 
                              << OutputData[i].real() << ", " << OutputData[i].imag() << ", "
                              << std::abs(OutputData[i])
                              << std::endl;
            }
        }
    }
}

/* Report出力 */
void FUJITSU::CFftWrap::ReportStatisticalData(std::ostream& os, const double NowTime)
{
    fft_operator_->ReportStatisticalData(os, NowTime);
}


// ---- log用に絶対値10-E7 以下を0に丸める
void FUJITSU::CFftWrap::ComplexForLog(std::vector<std::complex<double> >& x)
{
    // 浮動小数点誤差が見にくいので消す(なくて良い処理)
    for (size_t i=0; i < x.size(); i++) {
        double re, im;
        if (x[i].real() > -0.00000001 && x[i].real() < 0.00000001) re = 0;
        else                                                         re = x[i].real();
        if (x[i].imag() > -0.00000001 && x[i].imag() < 0.00000001) im = 0;
        else                                                         im = x[i].imag();
        x[i] = std::complex<double>(re, im);
    }
}

// ---- paramチェック用関数
bool FUJITSU::CFftWrap::isPowerOfTwo(int a)
{
    return (!(a & (a-1)));
}
