/*!
    @file  prec_pos_FFT.h
    @brief 演算精度制限関数 (FFT, 値域指定付)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef prec_FFT_h
#define prec_FFT_h

#include <cmath>
#include <complex>

    /*!
      @brief 値域の制限と切り捨て・四捨五入を行う共通関数
      @param[in] value 入力値
      @param[in] pos 値域設定  例) 0: (-1.0, +1.0), -1: (-0.5, 0.5)
      @param[in] bits 精度bit数(符号bitを含む)。0の場合は四捨五入しない(値域制限は行う)
      @param[in] round 切り捨て時はfalse、四捨五入時はtrue
      @param[in] saturation 値域制限する時はtrue、しないときはfalse
      @param[in] high_prec 32bit以上のシフト量(bits-pos-1またはbits-1, pos)に対応が必要なときはtrue
      @param[inout] p_cliped 上限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @param[inout] n_cliped 下限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @return 値域制限と切り捨て・四捨五入の値
    */
    inline
    double prec_pos_common_round_clip_FFT(const double value,
                                          const int pos,
                                          const unsigned int bits,
                                          const bool round,
                                          const bool saturation,
                                          bool high_prec,
                                          unsigned int* p_cliped,
                                          unsigned int* n_cliped)
    {
        if (bits > 0) {
            const int shift = bits - pos;
            const double range =
                high_prec ? std::ldexp(1.0, shift-1) :
                shift>0 ? static_cast<double>(1U<<(shift-1)) :
                1.0/static_cast<double>(1U<<(-shift+1));
            double shifted_value = std::floor(value * range + (round ? 0.5 : 0.0));
            if (saturation) {
                const double max_value = high_prec ? std::ldexp(1.0, bits-1) : static_cast<double>(1U<<(bits-1));
                if (shifted_value >= max_value) {
                    shifted_value = max_value - 1.0;
                    if(p_cliped) (*p_cliped)++;
                } else if(-max_value >= shifted_value) {
                    shifted_value = -max_value + 1.0;
                    if(n_cliped) (*n_cliped)++;
                }
            }
            return shifted_value / range;
        } else if (saturation) {
            const double max_value = high_prec ? std::ldexp(1.0, pos) : static_cast<double>(1U<<pos);
            double out;
            if (value > max_value) {
                out = max_value;
                if(p_cliped)
                    (*p_cliped)++;
            } else if(-max_value > value) {
                out = -max_value;
                if(n_cliped)
                    (*n_cliped)++;
            } else {
                out = value;
            }
            return out;
        } else {
            return value;
        }
    }

    /*!
      @brief 四捨五入
      @param[in] value 入力値
      @param[in] pos 四捨五入位置 小数点以下(bits-1)-posを残す
      @param[in] bits 四捨五入位置 小数点以下(bits-1)-posを残す。0の場合は四捨五入しない。
      @return 四捨五入後の値
      @note posとbitsはprec_pos_round_clip_FFT()と同じ設定にすると、値域制限なしの四捨五入となる。
    */
    inline
    double prec_pos_round_FFT(const double value, const int pos, const unsigned int bits)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, true, false, false, NULL, NULL);
    }

    /*!
      @brief 四捨五入 (32bit以上シフト対応版)
      @param[in] value 入力値
      @param[in] pos 四捨五入位置 小数点以下(bits-1)-posを残す
      @param[in] bits 四捨五入位置 小数点以下(bits-1)-posを残す。0の場合は四捨五入しない。
      @return 四捨五入後の値
      @note posとbitsはprec_pos_round_clip_FFT()と同じ設定にすると、値域制限なしの四捨五入となる。
    */
    inline
    double prec_pos_round_FFT_d(const double value, const int pos, const unsigned int bits)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, true, false, true, NULL, NULL);
    }

    /*!
      @brief 値域の制限と四捨五入を行う
      @param[in] value 入力値
      @param[in] pos 値域設定  例) 0: (-1.0, +1.0), -1: (-0.5, 0.5)
      @param[in] bits 精度bit数(符号bitを含む)。0の場合は四捨五入しない(値域制限は行う)
      @param[inout] p_cliped 上限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @param[inout] n_cliped 下限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @return 値域制限と四捨五入後の値
    */
    inline
    double prec_pos_round_clip_FFT(const double value, const int pos, const unsigned int bits, unsigned int* p_cliped = 0, unsigned int* n_cliped = 0)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, true, true, false, p_cliped, n_cliped);
    }

    /*!
      @brief 値域の制限と四捨五入を行う (32bit以上シフト対応版)
      @param[in] value 入力値
      @param[in] pos 値域設定  例) 0: (-1.0, +1.0), -1: (-0.5, 0.5)
      @param[in] bits 精度bit数(符号bitを含む)。0の場合は四捨五入しない(値域制限は行う)
      @param[inout] p_cliped 上限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @param[inout] n_cliped 下限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @return 値域制限と四捨五入後の値
    */
    inline
    double prec_pos_round_clip_FFT_d(const double value, const int pos, const unsigned int bits, unsigned int* p_cliped = 0, unsigned int* n_cliped = 0)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, true, true, true, p_cliped, n_cliped);
    }

    /*!
      @brief 四捨五入
      @param[in] value 入力値
      @param[in] pos 四捨五入位置 小数点以下(bits-1)-posを残す
      @param[in] bits 四捨五入位置 小数点以下(bits-1)-posを残す。0の場合は四捨五入しない。
      @return 四捨五入後の値
      @note posとbitsはprec_pos_round_clip_FFT()と同じ設定にすると、値域制限なしの四捨五入となる。
    */
    inline
    std::complex<double>
    prec_pos_round_FFT(const std::complex<double>& value, const int pos, const unsigned int bits)
    {
        return std::complex<double>(prec_pos_round_FFT(value.real(), pos, bits),
                                    prec_pos_round_FFT(value.imag(), pos, bits));
    }

    /*!
      @brief 値域の制限と四捨五入を行う
      @param[in] value 入力値
      @param[in] pos 値域設定  例) 0: (-1.0, +1.0), -1: (-0.5, 0.5)
      @param[in] bits 精度bit数(符号bitを含む)。0の場合は四捨五入しない(値域制限は行う)
      @param[inout] p_cliped 上限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @param[inout] n_cliped 下限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @return 値域制限と四捨五入後の値
    */
    inline
    std::complex<double>
    prec_pos_round_clip_FFT(const std::complex<double>& value, const int pos, const unsigned int bits, unsigned int* p_cliped = 0, unsigned int* n_cliped = 0)
    {
        return std::complex<double>(prec_pos_round_clip_FFT(value.real(), pos, bits, p_cliped, n_cliped),
                                    prec_pos_round_clip_FFT(value.imag(), pos, bits, p_cliped, n_cliped));
    }

    /*!
      @brief 切り捨て(負の無限大方向)
      @param[in] value 入力値
      @param[in] pos 切り捨て位置 小数点以下(bits-1)-posを残す
      @param[in] bits 切り捨て位置 小数点以下(bits-1)-posを残す。0の場合は切り捨てしない。
      @return 切り捨て後の値
      @note posとbitsはprec_pos_cut_clip_FFT()と同じ設定にすると、値域制限なしの切り捨てとなる。
    */
    inline
    double prec_pos_cut_FFT(const double value, const int pos, const unsigned int bits)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, false, false, false, NULL, NULL);
    }

    /*!
      @brief 切り捨て(負の無限大方向)
      @param[in] value 入力値
      @param[in] pos 切り捨て位置 小数点以下(bits-1)-posを残す
      @param[in] bits 切り捨て位置 小数点以下(bits-1)-posを残す。0の場合は切り捨てしない。
      @return 切り捨て後の値
      @note posとbitsはprec_pos_cut_clip_FFT()と同じ設定にすると、値域制限なしの切り捨てとなる。
    */
    inline
    std::complex<double> prec_pos_cut_FFT(const std::complex<double>& value, const int pos, const unsigned int bits)
    {
        return std::complex<double>(prec_pos_cut_FFT(value.real(), pos, bits),
                                    prec_pos_cut_FFT(value.imag(), pos, bits));
    }

    /*!
      @brief 値域の制限と切り捨てを行う
      @param[in] value 入力値
      @param[in] pos 値域設定  例) 0: (-1.0, +1.0), -1: (-0.5, 0.5)
      @param[in] bits 精度bit数(符号bitを含む)。0の場合は切り捨てしない(値域制限は行う)
      @param[inout] p_cliped 上限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @param[inout] n_cliped 下限値に制限した場合にその値を1増やすカウンタへのポインタ。NULLの場合はなにもしない。
      @return 値域制限と切り捨ての値
    */
    inline
    double prec_pos_cut_clip_FFT(const double value, const int pos, const unsigned int bits, unsigned int* p_cliped = 0, unsigned int* n_cliped = 0)
    {
        return prec_pos_common_round_clip_FFT(value, pos, bits, false, true, false, p_cliped, n_cliped);
    }

#endif /* prec_FFT_h */

