/*!
    @file  CRxFde.cpp
    @brief DSP Frequency Domain Equlizer Top

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2020 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <sstream>
#include <boost/format.hpp>
#include "Util_wrap.h"
#include "CRxFde.h"

/*! @brief  コンストラクタ 
    @param  param : C_param_wrapクラスへのポインタ 
    @return なし 
*/
FUJITSU::CRxFde::CRxFde(const boost::shared_ptr<C_param_wrap> param)
    : rep_foff_(0.0),
      rep_foff_fft_grid_(0),
      rep_sample_time_(0.0),
      rep_skew_(4, 0.0),
      rep_skew_samp_(4, 0.0)
{
    std::string str, msg;
    boost::shared_ptr<Util_wrap> util = createUtil();

    /* パラメータの取得 */
    unsigned int subc_n;       /* サブキャリア数 */
    str = "COM_SubCarrier_N";
    util->checkAndGetParam(param, str, &subc_n, C_param_wrap::INT);

    int fft_size;           /* FFTサイズ */
    str = "DSP_RXFDE_FFT_SIZE";
    util->checkAndGetParam(param, str, &fft_size, C_param_wrap::INT);
    /* point数check  */
    int bits;
    for(bits=0; ((fft_size>>bits)&1)!=1 && (fft_size>>bits)!=0; bits++);
    if((fft_size >> bits) != 1) {
        g_err->appendMsg("DSP_RXFDE_FFT_SIZE is not 2^N");
    }

    double fs_mode = 2.0;      /* オーバーサンプリング率設定 */
    str = "DSP_ATS_FEQ_fractional_sampling_mode";
    std::string fs_str;
    util->checkAndGetParam(param, str, &fs_str, C_param_wrap::STRING);
    if(fs_str == "4/2") {
        fs_mode = 2.0;
    } else if(fs_str == "3/2") {
        fs_mode = 1.5;
    } else if(fs_str == "4/3") {
        fs_mode = 4.0/3.0;
    } else if(fs_str == "5/4") {
        fs_mode = 1.25;
    } else if(fs_str == "6/5") {
        fs_mode = 1.2;
    } else {
        g_err->appendMsg("DSP_ATS_FEQ_fractional_sampling_mode must be 4/2 or 3/2 or 4/3 or 5/4 or 6/5");
    }
//    std::string tx_fs_str;      /* 送信側オーバーサンプリング率設定 */
//    str = "TXDSP_fractional_sampling_mode";
//    util->checkAndGetParam(param, str, &tx_fs_str, C_param_wrap::STRING);
//    if(fs_str != tx_fs_str) {
//        g_err->appendMsg("DSP_ATS_FEQ_fractional_sampling_mode must be equal TXDSP_fractional_sampling_mode");
//    }

    int tx_ifft_size;   /* 送信側IFFTサイズ */
    str = "COM_TxFDE2_IFFT_size";
    util->checkAndGetParam(param, str, &tx_ifft_size, C_param_wrap::INT);

    std::vector<int> subc_bin_shift;   /* サブキャリアのbinシフト量(受信FFTサイズに換算する) */
    str = "COM_TxFDE2_bin_shift";
    util->checkAndGetParam(param, str, &subc_bin_shift, C_param_wrap::INT_VECTOR);
    for(size_t i=0; i<subc_bin_shift.size(); i++) {
        subc_bin_shift[i] *= (fft_size / tx_ifft_size);
    }

    bool subc_lpf_rrc = true;
    std::string lpf_str;      /* サブキャリア分離LPFの選択 */
    str = "DSP_RXFDE_SUBC_LPF_method";
    util->checkAndGetParam(param, str, &lpf_str, C_param_wrap::STRING);
    if(lpf_str == "RootRollOff") {
        subc_lpf_rrc = true;
    } else if(lpf_str == "Gaussian") {
        subc_lpf_rrc = false;
    } else {
        g_err->appendMsg("DSP_RXFDE_SUBC_LPF_method must be RootRollOff or Gaussian");
    }

    double subc_rrc_factor;      /* サブキャリア分離LPF ロールオフ率 */
    str = "DSP_RXFDE_SUBC_LPF_roll_off_factor";
    util->checkAndGetParam(param, str, &subc_rrc_factor, C_param_wrap::DOUBLE);

    double subc_rrc_width;      /* サブキャリア分離LPF RRC帯域幅 */
    str = "DSP_RXFDE_SUBC_LPF_RRC_width";
    util->checkAndGetParam(param, str, &subc_rrc_width, C_param_wrap::DOUBLE);

    std::vector<double> subc_gauss_factor;      /* サブキャリア分離LPF gaussian filter係数 */
    str = "DSP_RXFDE_SUBC_LPF_gaussian_params";
    util->checkAndGetParam(param, str, &subc_gauss_factor, C_param_wrap::DOUBLE_VECTOR);
    if(subc_gauss_factor.size() != 2) {
        g_err->appendMsg("DSP_RXFDE_SUBC_LPF_gaussian_params : parameter num must be equal 2");
    }

    bool subc_norm_auto = true;
    std::string norm_str;      /* サブキャリア振幅正規化方法の選択 */
    str = "DSP_RXFDE_SUBC_NORM_method";
    util->checkAndGetParam(param, str, &norm_str, C_param_wrap::STRING);
    if(norm_str == "auto") {
        subc_norm_auto = true;
    } else if(norm_str == "ext") {
        subc_norm_auto = false;
    } else {
        g_err->appendMsg("DSP_RXFDE_SUBC_NORM_method must be auto or ext");
    }

    std::vector<double> subc_norm_ext_coef;      /* サブキャリア振幅正規化 ext時の正規化係数(サブキャリア毎) */
    str = "DSP_RXFDE_SUBC_NORM_ext_coeff";
    util->checkAndGetParam(param, str, &subc_norm_ext_coef, C_param_wrap::DOUBLE_VECTOR);
    if(!subc_norm_auto && subc_norm_ext_coef.size() != subc_n) {
        g_err->appendMsg("DSP_RXFDE_SUBC_NORM_ext_coeff : parameter num must be equal COM_SubCarrier_N");
    }

    double subc_norm_rms;      /* サブキャリア振幅正規化 CH当たり目標RMS値 */
    str = "DSP_RXFDE_SUBC_NORM_ExpectedAmplitude";
    util->checkAndGetParam(param, str, &subc_norm_rms, C_param_wrap::DOUBLE);

    std::vector<int> subc_norm_update_sample;   /* サブキャリア振幅正規化 auto時の正規化係数update sample数(x10^3) 何個でも可 */
    str = "DSP_RXFDE_SUBC_NORM_update_samples";
    util->checkAndGetParam(param, str, &subc_norm_update_sample, C_param_wrap::INT_VECTOR);
    for(size_t i=0; i<subc_norm_update_sample.size(); i++) subc_norm_update_sample[i] *= 1000;

    std::vector<int> subc_delay_sample;   /* サブキャリア毎delay値 (サンプル) */
    str = "DSP_RXFDE_SUBC_delay_sample";
    util->checkAndGetParam(param, str, &subc_delay_sample, C_param_wrap::INT_VECTOR);
    if(subc_n > 1 && subc_delay_sample.size() != subc_n) {
        g_err->appendMsg("DSP_RXFDE_SUBC_delay_sample : parameter num must be equal COM_SubCarrier_N");
    }

    std::vector<double>  iq_comp;            /* I-Qクロストーク補償量 X,Y */
    str = "DSP_RXFDE_IQ_COMP";
    util->checkAndGetParam(param, str, &iq_comp, C_param_wrap::DOUBLE_VECTOR);
    if(iq_comp.size() != 2) {
        msg = "Parameter " + str + "is in valid. Must 2 value.";
        g_err->appendMsg(msg);
    }

    std::vector<double>  norm_coef_x(2);     /* ノーマライゼーション レーン個別調整係数X */
    std::vector<double>  norm_coef_y(2);     /* ノーマライゼーション レーン個別調整係数Y */
    str = "DSP_RXFDE_NORM_COEF";
    std::vector<double> tmp_norm_coef;
    util->checkAndGetParam(param, str, &tmp_norm_coef, C_param_wrap::DOUBLE_VECTOR);
    if(tmp_norm_coef.size() != 4) {
        msg = "Parameter " + str + "is in valid. Must 4 value.";
        g_err->appendMsg(msg);
    }
    norm_coef_x[0] = tmp_norm_coef[0];
    norm_coef_x[1] = tmp_norm_coef[1];
    norm_coef_y[0] = tmp_norm_coef[2];
    norm_coef_y[1] = tmp_norm_coef[3];

    double               norm_coef_all;      /* ノーマライゼーション 4レーン総合値調整係数 */
    str = "DSP_RXFDE_NORM_COEF_ALL";
    util->checkAndGetParam(param, str, &norm_coef_all, C_param_wrap::DOUBLE);

    int                  norm_idv;           /* ノーマライゼーション 個別/総合モード選択 0:総合, 1:個別 */
    str = "DSP_RXFDE_NORM_IDV";
    util->checkAndGetParam(param, str, &norm_idv, C_param_wrap::INT);

    double               norm_rms;           /* ノーマライゼーション ターゲットRMS */
    str = "DSP_RXFDE_NORM_RMS";
    util->checkAndGetParam(param, str, &norm_rms, C_param_wrap::DOUBLE);

    int                  norm_int;           /* ノーマライゼーション 外部・内部設定選択 0:外部, 1:内部 */
    str = "DSP_RXFDE_NORM_INT";
    util->checkAndGetParam(param, str, &norm_int, C_param_wrap::INT);

    std::vector<double>  fres_tap_xi;        /* 周波数応答用 フィルタ係数 XI */
    std::vector<double>  fres_tap_xq;        /* 周波数応答用 フィルタ係数 XQ */
    std::vector<double>  fres_tap_yi;        /* 周波数応答用 フィルタ係数 YI */
    std::vector<double>  fres_tap_yq;        /* 周波数応答用 フィルタ係数 YQ */
    str = "DSP_RXFDE_FRES_XI_TAP";
    util->checkAndGetParam(param, str, &fres_tap_xi, C_param_wrap::DOUBLE_VECTOR);
    str = "DSP_RXFDE_FRES_XQ_TAP";
    util->checkAndGetParam(param, str, &fres_tap_xq, C_param_wrap::DOUBLE_VECTOR);
    str = "DSP_RXFDE_FRES_YI_TAP";
    util->checkAndGetParam(param, str, &fres_tap_yi, C_param_wrap::DOUBLE_VECTOR);
    str = "DSP_RXFDE_FRES_YQ_TAP";
    util->checkAndGetParam(param, str, &fres_tap_yq, C_param_wrap::DOUBLE_VECTOR);

    int                  cd_offset_on;       /* 周波数オフセット補償と波長分散補償の実施順番 1:周波数オフセット補償を先に実施，0:波長分散補償を先に実施 */
    str = "DSP_RXFDE_CD_OFFSET_ON";
    int tmp_cd_offset_on;
    util->checkAndGetParam(param, str, &tmp_cd_offset_on, C_param_wrap::INT);
    cd_offset_on = (tmp_cd_offset_on == 1)? true : false;

    if(tmp_cd_offset_on != 0 && tmp_cd_offset_on != 1) {
        msg = "Parameter " + str + "is in valid. Must be 0 or 1.";
        g_err->appendMsg(msg);
    }

    /* 波長分散補償関連 */
    double               fc_laser;
    str = "COM_Fc_Laser";
    util->checkAndGetParam(param, str, &fc_laser, C_param_wrap::DOUBLE);
    str = "COM_Bit_Rate";
    double bit_rate;
    util->checkAndGetParam(param, str, &bit_rate, C_param_wrap::DOUBLE);
    double sampling_rate = bit_rate * fs_mode / 4.0;
    /* スケールの変換 */
    fc_laser *= 1E+12;
    sampling_rate *= 1.0E9;    
    double*              dispersion;
    str = "DSP_RXFDE_DISPERSION";
    util->checkAndSetParam(param, str, &dispersion, C_param_wrap::DOUBLE);    

    int                  spec_offset_on;     /* 周波数オフセット補償とスペクトル整形の実施順番 1:周波数オフセット補償を先に実施，0:スペクトル整形を先に実施 */
    str = "DSP_RXFDE_SPEC_OFFSET_ON";
    int tmp_spec_offset_on;
    util->checkAndGetParam(param, str, &tmp_spec_offset_on, C_param_wrap::INT);
    spec_offset_on = (tmp_spec_offset_on == 1)? true : false;
    if(tmp_spec_offset_on != 0 && tmp_spec_offset_on != 1) {
        msg = "Parameter " + str + "is in valid. Must be 0 or 1.";
        g_err->appendMsg(msg);
    }

    bool                 spec_on;            /* スペクトル整形 ON/OFF */
    str = "DSP_RXFDE_SPEC_ON";
    int tmp_spec_on;
    util->checkAndGetParam(param, str, &tmp_spec_on, C_param_wrap::INT);
    spec_on = (tmp_spec_on == 1)? true : false;
    if(tmp_spec_on != 0 && tmp_spec_on != 1) {
        msg = "Parameter " + str + "is in valid. Must be 0 or 1.";
        g_err->appendMsg(msg);
    }
    double               rolloff;            /* スペクトル整形のロールオフ率設定 */
    str = "DSP_RXFDE_SPEC_ROLLOFF";
    util->checkAndGetParam(param, str, &rolloff, C_param_wrap::DOUBLE);

    /* 周波数オフセット */
    int                  offset_val;         /* 周波数オフセット補償量 (グリッド単位) */
    str = "DSP_RXFDE_FREQ_OFFSET";
    double freq_offset;  /* GHz単位 */
    util->checkAndGetParam(param, str, &freq_offset, C_param_wrap::DOUBLE);
    /* FFTグリッド単位に変換(1/4 Overlap時は4グリットの倍数) */
//    offset_val = -4 * static_cast<int>(freq_offset * 1.0e9 / (sampling_rate / static_cast<double>(fft_size))/4.0);
    /* FFTグリッド単位に変換(1/2 Overlap時は2グリットの倍数) */
    offset_val = -2 * static_cast<int>(freq_offset * 1.0e9 / (sampling_rate / static_cast<double>(fft_size))/2.0);
    /* レポート用にメンバにコピー */
    rep_foff_ = freq_offset;
    rep_foff_fft_grid_ = offset_val;
    
    /* skew */
    std::vector<double>  skew_x(2);          /* スキュー補償量X */
    std::vector<double>  skew_y(2);          /* スキュー補償量Y */
    /* ps単位からsample単位に変換して設定する */
    str = "DSP_RXFDE_SKEW";
    std::vector<double> tmp_skew;
    util->checkAndGetParam(param, str, &tmp_skew, C_param_wrap::DOUBLE_VECTOR);
    if(tmp_skew.size() != 4) {
        msg = "Parameter " + str + "is in valid. Must 4 value.";
        g_err->appendMsg(msg);
    }
    double sample_time = 1.0 / sampling_rate * 1.0E12; /* ps */
    /* sample単位に変換 */
    skew_x[0] = tmp_skew[0] / sample_time;
    skew_x[1] = tmp_skew[1] / sample_time;
    skew_y[0] = tmp_skew[2] / sample_time;
    skew_y[1] = tmp_skew[3] / sample_time;
    /* レポート用メンバにコピー */
    rep_sample_time_ = sample_time;
    for(int i=0; i < 4; i++) {
        rep_skew_[i] = tmp_skew[i];
    }
    for(int i=0; i < 2; i++) {
        rep_skew_samp_[i]   = skew_x[i];
        rep_skew_samp_[i+2] = skew_y[i];
    }

    /* ■ DCオフセット補償関連 ■ */
    int                  dc_ext;             /* DCオフセット補償 外部・内部設定選択 0:内部, 1:外部 */
    std::vector<double>  dc_x(2);            /* DCオフセット補償 レーン個別補償値 X */
    std::vector<double>  dc_y(2);            /* DCオフセット補償 レーン個別補償値 Y */
    int                  dc_fb;              /* DCオフセット補償 FeedForward/FeedBack選択 0:FF, 1:FB */
    int                  dc_snum;            /* DCオフセット補償 Sampling周期設定 */
    int                  dc_cnum;            /* DCオフセット補償 FB定常係数移行回数設定 */
    double               dc_initcoef;        /* DCオフセット補償 FB初期引き込み係数 */
    double               dc_normcoef;        /* DCオフセット補償 FB定常係数 */
    str = "DSP_RXFDE_DC_EXT";
    util->checkAndGetParam(param, str, &dc_ext, C_param_wrap::INT);

    str = "DSP_RXFDE_DC";
    std::vector<double> tmp_dc;
    util->checkAndGetParam(param, str, &tmp_dc, C_param_wrap::DOUBLE_VECTOR);
    if(tmp_dc.size() != 4) {
        msg = "Parameter " + str + "is in valid. Must 4 value.";
        g_err->appendMsg(msg) ;
    }
    
    dc_x[0] = tmp_dc[0];
    dc_x[1] = tmp_dc[1];
    dc_y[0] = tmp_dc[2];
    dc_y[1] = tmp_dc[3];

    str = "DSP_RXFDE_DC_FB";
    util->checkAndGetParam(param, str, &dc_fb, C_param_wrap::INT);

    str = "DSP_RXFDE_DC_SNUM";
    util->checkAndGetParam(param, str, &dc_snum, C_param_wrap::INT);

    str = "DSP_RXFDE_DC_CNUM";
    util->checkAndGetParam(param, str, &dc_cnum, C_param_wrap::INT);
    
    str = "DSP_RXFDE_DC_INIT_COEF";
    util->checkAndGetParam(param, str, &dc_initcoef, C_param_wrap::DOUBLE);

    str = "DSP_RXFDE_DC_NORM_COEF";
    util->checkAndGetParam(param, str, &dc_normcoef, C_param_wrap::DOUBLE);

    /* ■ Monitor関連 ■ */
    double               mon_clip_th;        /* Monitor Clip閾値設定 */
    int                  mon_clip_snum;      /* Monitor Clip数算出 Sample数 */
    int                  mon_rms_snum;       /* Monitor RMS算出 Sample数 */
    str = "DSP_RXFDE_MON_CLIP_TH";
    util->checkAndGetParam(param, str, &mon_clip_th, C_param_wrap::DOUBLE);
    
    str = "DSP_RXFDE_MON_CLIP_SNUM";
    util->checkAndGetParam(param, str, &mon_clip_snum, C_param_wrap::INT);

    str = "DSP_RXFDE_MON_RMS_SNUM";
    util->checkAndGetParam(param, str, &mon_rms_snum, C_param_wrap::INT);

    /* ■ 周波数領域線形補償係数のパラメータ設定関連 ■ */
    int                  fc_ext;         /* 周波数領域線形補償係数パラメータ設定 ON/OFF*/
    std::vector<std::complex<double> > rhi, rhq, rvi, rvq;  /* 周波数領域線形補償係数パラメータ */
    
    str = "DSP_RXFDE_Fchar_ext_input";
    util->checkAndGetParam(param, str, &fc_ext, C_param_wrap::INT);

    if(fc_ext == 1){
        str = "DSP_RXFDE_Fchar_xi";
        util->checkAndGetParam(param, str, &rhi, C_param_wrap::COMPLEX_VECTOR);
        if(static_cast<int>(rhi.size()) != fft_size) {
            msg = "Parameter " + str + " : Number of data is not equal DSP_RXFDE_FFT_SIZE";
            g_err->appendMsg(msg);
        }
        str = "DSP_RXFDE_Fchar_xq";
        util->checkAndGetParam(param, str, &rhq, C_param_wrap::COMPLEX_VECTOR);
        if(static_cast<int>(rhq.size()) != fft_size) {
            msg = "Parameter " + str + " : Number of data is not equal DSP_RXFDE_FFT_SIZE";
            g_err->appendMsg(msg);
        }
        str = "DSP_RXFDE_Fchar_yi";
        util->checkAndGetParam(param, str, &rvi, C_param_wrap::COMPLEX_VECTOR);
        if(static_cast<int>(rvi.size()) != fft_size) {
            msg = "Parameter " + str + " : Number of data is not equal DSP_RXFDE_FFT_SIZE";
            g_err->appendMsg(msg);
        }
        str = "DSP_RXFDE_Fchar_yq";
        util->checkAndGetParam(param, str, &rvq, C_param_wrap::COMPLEX_VECTOR);
        if(static_cast<int>(rvq.size()) != fft_size) {
            msg = "Parameter " + str + " : Number of data is not equal DSP_RXFDE_FFT_SIZE";
            g_err->appendMsg(msg) ;
        }
    }

    bool phase_adust_on = false;
    std::string on_off;
    str = "DSP_RXFDE_phase_adust_on";
    util->checkAndGetParam(param, str, &on_off, C_param_wrap::STRING);
    phase_adust_on = (on_off == "ON");

    /* CRxFdeCoreの生成 */
    RxFdeCoreX_ = boost::shared_ptr<FUJITSU::CRxFdeCore>(new CRxFdeCore(CRxFdeCore::H,
                                                                        subc_n,
                                                                        fft_size,
                                                                        fs_mode,
                                                                        subc_bin_shift,
                                                                        subc_lpf_rrc,
                                                                        subc_rrc_factor,
                                                                        subc_rrc_width,
                                                                        subc_gauss_factor,
                                                                        subc_norm_auto,
                                                                        subc_norm_ext_coef,
                                                                        subc_norm_rms,
                                                                        subc_norm_update_sample,
                                                                        subc_delay_sample,
                                                                        spec_on,
                                                                        rolloff,
                                                                        fres_tap_xi,
                                                                        fres_tap_xq,
                                                                        skew_x,
                                                                        iq_comp[0],
                                                                        norm_coef_x,
                                                                        norm_coef_all,
                                                                        norm_idv,
                                                                        norm_rms,
                                                                        norm_int,
                                                                        offset_val,
                                                                        cd_offset_on,
                                                                        spec_offset_on,
                                                                        fc_laser,
                                                                        sampling_rate,
                                                                        dispersion,
                                                                        dc_ext,
                                                                        dc_x,
                                                                        dc_snum,
                                                                        dc_fb,
                                                                        dc_cnum,
                                                                        dc_initcoef,
                                                                        dc_normcoef,
                                                                        mon_clip_th,
                                                                        mon_clip_snum,
                                                                        mon_rms_snum,
                                                                        rhi,
                                                                        rhq,
                                                                        fc_ext,
                                                                        phase_adust_on));
                                                                       
    
    RxFdeCoreY_ = boost::shared_ptr<FUJITSU::CRxFdeCore>(new CRxFdeCore(CRxFdeCore::V,
                                                                        subc_n,
                                                                        fft_size,
                                                                        fs_mode,
                                                                        subc_bin_shift,
                                                                        subc_lpf_rrc,
                                                                        subc_rrc_factor,
                                                                        subc_rrc_width,
                                                                        subc_gauss_factor,
                                                                        subc_norm_auto,
                                                                        subc_norm_ext_coef,
                                                                        subc_norm_rms,
                                                                        subc_norm_update_sample,
                                                                        subc_delay_sample,
                                                                        spec_on,
                                                                        rolloff,
                                                                        fres_tap_yi,
                                                                        fres_tap_yq,
                                                                        skew_y,
                                                                        iq_comp[1],
                                                                        norm_coef_y,
                                                                        norm_coef_all,
                                                                        norm_idv,
                                                                        norm_rms,
                                                                        norm_int,
                                                                        offset_val,
                                                                        cd_offset_on,
                                                                        spec_offset_on,
                                                                        fc_laser,
                                                                        sampling_rate,
                                                                        dispersion,
                                                                        dc_ext,
                                                                        dc_y,
                                                                        dc_snum,
                                                                        dc_fb,
                                                                        dc_cnum,
                                                                        dc_initcoef,
                                                                        dc_normcoef,
                                                                        mon_clip_th,
                                                                        mon_clip_snum,
                                                                        mon_rms_snum,
                                                                        rvi,
                                                                        rvq,
                                                                        fc_ext,
                                                                        phase_adust_on));
}

/*! デストラクタ 
*/
FUJITSU::CRxFde::~CRxFde()
{
}

/*! DSP 周波数ドメインイコライザ処理実行
  @param[in] dsptm          シミュレーション時刻
  @param[in] InputData_H    H 入力データ
  @param[in] InputData_V    V 入力データ
  @param[out] OutputData_H  H 出力データ
  @param[out] OutputData_V  V 出力データ
*/
void FUJITSU::CRxFde::Execute(const double dsptm,
                              const double adjust_phase,
                              const std::complex<double>& InputData_H,
                              const std::complex<double>& InputData_V,
                              bool&                               OutputData_valid,
                              std::vector<std::complex<double> >& OutputData_H,
                              std::vector<std::complex<double> >& OutputData_V)
{
    bool valid_h, valid_v;
    RxFdeCoreX_->Execute(dsptm, adjust_phase, InputData_H, valid_h, OutputData_H);
    RxFdeCoreY_->Execute(dsptm, adjust_phase, InputData_V, valid_v, OutputData_V);
    OutputData_valid = valid_h && valid_v;
}

/* 残留分散値設定処理 */
void FUJITSU::CRxFde::SetResidualDispersion(const double rcd, const unsigned int scn)
{
    RxFdeCoreX_->SetResidualDispersion(rcd, scn);
    RxFdeCoreY_->SetResidualDispersion(rcd, scn);
}

/* 残留周波数オフセット値設定処理 */
void FUJITSU::CRxFde::SetResidualFrequencyOffset(const double rofs, const unsigned int scn)
{
    RxFdeCoreX_->SetResidualFrequencyOffset(rofs, scn);
    RxFdeCoreY_->SetResidualFrequencyOffset(rofs, scn);
}

void FUJITSU::CRxFde::ReportStatisticalData(std::ostream& os, const double NowTime)
{
    os << "----- RXFDE Param Report -----" << std::endl;
    os << " Freq Offset compensate              : " << rep_foff_ << " GHz" << std::endl;
    os << " Freq Offset compensate FFT Grid     : " << rep_foff_fft_grid_ << " Grid" << std::endl;
    os << " Skew compensate XI, XQ, YI, YQ      : ";
    for (int i=0; i < 4; i++) {
        os << rep_skew_[i];
        if (i !=3) os << ", ";
        else       os << " ps" << std::endl;
    }
    
    os << " Skew compensate XI, XQ, YI, YQ sample: ";
    for (int i=0; i < 4; i++) {
        os << rep_skew_samp_[i];
        if (i !=3) os << ", ";
        else       os << " sample (1sample:" << rep_sample_time_ << "ps)" << std::endl;
    }
    
    os << std::endl;
    
    RxFdeCoreX_->ReportStatisticalData(os, NowTime);
    RxFdeCoreY_->ReportStatisticalData(os, NowTime);
    
}
