/*!
    @file  CRxFdeCore.cpp
    @brief DSP Frequency Domain Equlizer Core

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

/* 係数パラメータのDynamic設定は後回し */
/*
#define OPT_ENV_DEBUG_FREQ_RXFDE
*/
#include <iostream>
#include "common.h"
#include "CRxFdeCore.h"

#ifdef OPT_ENV_DEBUG_FREQ_RXFDE
#include <iomanip>
#endif


const int FUJITSU::CRxFdeCore::c_RMS_SAMPLE_NUM = 4194304;  /* 32768 * 128para */

/*! @brief  コンストラクタ 
    @param  HV            : 偏波面識別
    @param  FftSize       : FFTサイズ
    @param  FsMode        : オーバーサンプリング率設定
    @param  SpecOn        : スペクトル整形 ON/OFF
    @param  RoolOff       : Rool off率
    @param  FresTapI      : 周波数応答調整用 FIRフィルタ係数 I
    @param  FresTapQ      : 周波数応答調整用 FIRフィルタ係数 Q
    @param  Skew          : スキュー補償量 I,Q
    @param  Scale         : 振幅調整用係数
    @return なし 
*/
FUJITSU::CRxFdeCore::CRxFdeCore(
               const PolarizationT        HV,
               const int                  subc_n,         /* サブキャリア数 */
               const int                  FftSize,        /* FFTサイズ */
               const double               FsMode,         /* オーバーサンプリング率設定 */
               const std::vector<int>     subc_bin_shift, /* サブキャリアのbinシフト量(受信FFTサイズに換算する) */
               const bool                 subc_lpf_rrc,   /* サブキャリア分離LPF is RRC */
               const double               subc_rrc_factor,   /* サブキャリア分離LPF ロールオフ率 */
               const double               subc_rrc_width,    /* サブキャリア分離LPF RRC帯域幅 */
               const std::vector<double>  subc_gauss_factor, /* サブキャリア分離LPF gaussian filter係数 */
               const bool                 subc_norm_auto,    /* サブキャリア振幅正規化 is auto */
               const std::vector<double>  subc_norm_ext_coef,/* サブキャリア振幅正規化 ext時の正規化係数(サブキャリア毎) */
               const double               subc_norm_rms,     /* サブキャリア振幅正規化 CH当たり目標RMS値 */
               const std::vector<int>     subc_norm_update_sample,  /* サブキャリア振幅正規化 auto時の正規化係数update sample数 */
               const std::vector<int>     subc_delay_sample,        /* サブキャリア毎delay値 (サンプル) */
               const bool                 SpecOn,
               const double               RoolOff,
               const std::vector<double>& FresTapI,
               const std::vector<double>& FresTapQ,
               const std::vector<double>& Skew,
               const double               IqComp,
               const std::vector<double>& NormCoefIdv,
               const double               NormCoefAll,
               const int                  NormIdv,
               const double               NormRms,
               const int                  NormInt,
               const int                  OffsetVal,
               const bool                 CdOffsetOn,
               const bool                 SpecOffsetOn,
               const double               FcLaser,
               const double               SamplingRate,
               double*                    prm_dispersion,
               const int                  DcExt,
               const std::vector<double>& Dc,
               const int                  DcSnum,
               const int                  DcFb,
               const int                  DcCnum,
               const double               DcInitCoef,
               const double               DcNormCoef,
               const double               MonClipTh,
               const int                  MonClipSnum,
               const int                  MonRmsSnum,
               const std::vector<std::complex<double> >& PramCoefI,
               const std::vector<std::complex<double> >& PramCoefQ,
               const int                  FcPramOn,
               const bool                 phase_adjust_on)                                
    : HV_str_(HV==H ? "H" : "V"),
      subc_n_(subc_n),
      FftSize_(FftSize),
      IFftSize_(FftSize/subc_n),
      FsMode_(FsMode),
      subc_bin_shift_(subc_bin_shift),
      subc_lpf_coef_(IFftSize_, 1.0),
      subc_norm_auto_(subc_norm_auto),
      subc_norm_rms_(subc_norm_rms),
      subc_norm_update_sample_(subc_norm_update_sample),
      subc_norm_update_index_(subc_n, 0),
      subc_norm_samples_(subc_n, 0),
      subc_norm_exec_num_(0),
      subc_norm_pow_(subc_n, 0.0),
      SpecOn_(SpecOn),
      RoolOff_(RoolOff),
      FresTapI_(FresTapI),
      FresTapQ_(FresTapQ),
      Skew_(Skew),
      IqComp_(IqComp),
      NormCoefIdv_(NormCoefIdv),
      NormCoefAll_(NormCoefAll),
      NormIdv_(NormIdv),
      NormRms_(NormRms),
      NormInt_(NormInt),
      OffsetVal_(OffsetVal),
      CdOffsetOn_(CdOffsetOn),
      SpecOffsetOn_(SpecOffsetOn),
      FcLaser_(FcLaser),
      SamplingRate_(SamplingRate),
      p_dispersion_(prm_dispersion),
      ParamDispersion_(*prm_dispersion),
      Dispersion_(*prm_dispersion),
      DcExt_(DcExt),
      Dc_(Dc),
      DcSnum_(DcSnum),
      DcFb_(DcFb),
      DcCnum_(DcCnum),
      DcInitCoef_(DcInitCoef),
      DcNormCoef_(DcNormCoef),
      MonClipTh_(MonClipTh),
      MonClipSnum_(MonClipSnum),
      MonRmsSnum_(MonRmsSnum),
      PramCoefI_(PramCoefI),
      PramCoefQ_(PramCoefQ),
      FcPramOn_(FcPramOn),
      phase_adjust_on_(phase_adjust_on),

      CoefI_(FftSize),
      CoefQ_(FftSize),
      CoefChangeDone_(true),
      input_buff_(FftSize, std::complex<double>(0.0, 0.0)),
      output_buff_(subc_n, std::vector<std::complex<double> >(IFftSize_/2, std::complex<double>(0.0, 0.0))),
      rofs_feedback_done_(subc_n, false),
      rofs_feedback_value_(subc_n, 0.0),
      rcd_feedback_done_(subc_n, false),
      rcd_feedback_value_(subc_n, 0.0),
      sample_num_(0),
      out_sample_num_(IFftSize_),
      coef_update_(true), // 初期値
      rms_update_(true),  // 初期値
      rms_(2, 0.0),
      rms_before_(2, 0.0),
      CoefCd_(FftSize, std::complex<double>(1.0, 0.0)),
      CoefSpec_(FftSize, std::complex<double>(1.0, 0.0)),
      CoefFres_(2, std::vector<std::complex<double> >(FftSize, std::complex<double>(1.0, 0.0))),
      CoefSkew_(2, std::vector<std::complex<double> >(FftSize, std::complex<double>(1.0, 0.0))),
      CoefIqComp_(4),  // II, IQ, QI, QQ
      CoefNorm_(2, 1.0),  // IQ
      
      dcoffset_outdt_(0.0, 0.0),
      dc_calc_ff_(2, 0.0),
      dc_calc_fb_(2, 0.0),
      dc_rtn_ff_(2, 0.0),
      dc_rtn_fb_(2, 0.0),
      dc_ctr_(0),
      dc_val_(2, 0.0),
      dc_fbchg_ctr_(0),
      mon_clip_count_(0),
      mon_clip_bf_(2, 0),
      mon_clip_bf_reg_(2, 0),
      mon_rms_count_(0),
      mon_rms_bf_(2, 0.0),
      mon_rms_af_(2, 0.0),
      mon_rms_bf_reg_(2, 0.0),
      mon_rms_af_reg_(2, 0.0),
      fft_operator_(boost::shared_ptr<CRxFdeFFT>(new CRxFdeFFT(FftSize, 1))),
      ifft_operator_(boost::shared_ptr<CRxFdeFFT>(new CRxFdeFFT(IFftSize_, -1)))
{
    /* サブキャリア分離LPF係数 */
    if(subc_n>1) {
        if(subc_lpf_rrc) { /* root raised cosine filter */
            const int c_a = static_cast<int>(IFftSize_ * subc_rrc_width * (1.0 - subc_rrc_factor) / (2.0 * FsMode_));
            const int c_b = static_cast<int>(IFftSize_ * subc_rrc_width * (1.0 + subc_rrc_factor) / (2.0 * FsMode_));
            for(int i=0; i<IFftSize_/2+1; i++) {
                if(i<c_a) {
                    subc_lpf_coef_[i] = 1.0;
                } else if (i>=c_a && i<c_b) {
                    subc_lpf_coef_[i] = sqrt((1.0 - sin(2.0*M_PI*(i*FsMode_/(2.0*IFftSize_*subc_rrc_width)-0.25)/subc_rrc_factor)) / 2.0);
                } else {
                    subc_lpf_coef_[i] = 0.0;
                }
                //std::cout << i << "\t" << subc_lpf_coef_[i] << std::endl;
            }
            //exit(0);
        } else {           /* gaussian filter */
            /* gaussian : exp(-ln(sqrt(2)) * { |fn|/fc }^2p )   */
            const double comm = IFftSize_ / (2.0 * FsMode_) * subc_gauss_factor[1];
            for(int i=0; i<IFftSize_/2+1; i++) {
                subc_lpf_coef_[i] = std::exp( -std::log(std::sqrt(2.0)) * std::pow(comm * i, 2.0*subc_gauss_factor[0]) );
            }
        }
        /* 正の係数の折り返し */
        for(int i=1; i<IFftSize_/2; i++) {
            subc_lpf_coef_[IFftSize_-i] = subc_lpf_coef_[i];
        }
    }
    /* サブキャリア正規化係数 */
    if(subc_norm_auto_) {
        subc_norm_coef_.resize(subc_n);
        for(int i=0; i<subc_n; i++) subc_norm_coef_[i] = 1.0;
    } else {
        subc_norm_coef_ = subc_norm_ext_coef;
    }

    /* サブキャリア毎delay buffer */
    subc_delay_buf_.resize(subc_n);
    for(int i=0; i<subc_n; i++) {
        subc_delay_buf_[i].resize(subc_delay_sample[i]);
    }

    /* ------------------------ */
    /* 以下 通常FEQ             */
    if(FcPramOn_ == 0){
        /* 周波数領域線形補償係数のパラメータ内部設定 */
        /* フィルタ係数ネタ+フィルタ係数を算出しておく */
        CalcCdCoef();
        if (SpecOn_) CalcSpecCoef();
        CalcFresCoef(FresTapI_, CoefFres_[0]);
        CalcFresCoef(FresTapQ_, CoefFres_[1]);
        CalcSkew();
        CalcIqComp();
        CalcNorm();

        CalculateCoef(0.0);

        #ifdef OPT_ENV_DEBUG_FREQ_RXFDE
            std::cout << "FcPramOn_ = OFF " << std::endl;
        #endif
    } else {
        /* 周波数領域線形補償係数のパラメータ外部設定 */
        CoefI_ = PramCoefI_;
        CoefQ_ = PramCoefQ_;

        #ifdef OPT_ENV_DEBUG_FREQ_RXFDE
            std::cout << "FcPramOn_ = ON " << std::endl;
        #endif
    }
    #ifdef OPT_ENV_DEBUG_FREQ_RXFDE
    /* 周波数領域線形補償係数Dump */
    if(HV_str_ == "H"){
        for(unsigned int n=0; n<FftSize_; n++) {
            std::cout << "H_CoefI , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefI_[n].real() << " , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefI_[n].imag() << " , "
                      << " // " << n << std::endl;
            std::cout << "H_CoefQ , "
                      << std::setw(25) << std::setprecision(17) << std::left << CoefQ_[n].real() << " , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefQ_[n].imag() << " , "
                      << " // " << n << std::endl;
        }
    } else {
        for(unsigned int n=0; n<FftSize_; n++) {
            std::cout << "V_CoefI , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefI_[n].real() << " , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefI_[n].imag() << " , "
                      << " // " << n << std::endl;
            std::cout << "V_CoefQ , "
                      << std::setw(25) << std::setprecision(17) << std::left << CoefQ_[n].real() << " , " 
                      << std::setw(25) << std::setprecision(17) << std::left << CoefQ_[n].imag() << " , "
                      << " // " << n << std::endl;
        }
    }
    #endif
}

/*! デストラクタ 
*/
FUJITSU::CRxFdeCore::~CRxFdeCore()
{
}

/*! DSP 周波数ドメインイコライザ処理実行
  @param[in] dsptm          シミュレーション時刻
  @param[in] InputData      入力データ
  @param[out] OutputData    出力データ
*/
void FUJITSU::CRxFdeCore::Execute(const double dsptm,
                                  const double adjust_phase,
                                  const std::complex<double>& InputData,
                                  bool&                               OutputData_valid,
                                  std::vector<std::complex<double> >& OutputData)
{
    /* DCオフセット補償 */
    DcOffset(InputData);
    Execute_Dc(InputData, dcoffset_outdt_);

    /* ノーマライゼーション */
    if(NormInt_) {
        CalcRms(dcoffset_outdt_);
        CalcNormInternal();
    }
   
    /* 入力データ生成 1/2 overlap */
    input_buff_[sample_num_] = input_buff_[sample_num_ + FftSize_ / 2];
    input_buff_[sample_num_ + FftSize_ / 2] = dcoffset_outdt_;
    sample_num_++;

    /*  半分溜まった所で、周波数領域等化処理開始(1/2 Overlap FFT) */   
    if(sample_num_ == FftSize_ / 2) {
        sample_num_ = 0;
        out_sample_num_ = 0;

        /* 周波数領域線形補償係数のパラメータ内部設定 */
        if(FcPramOn_ == 0){
            /* DynamicParameter(シナリオ)で変更される場合は、シナリオ設定を優先 */
            if(ParamDispersion_ != *p_dispersion_){
                ParamDispersion_ = *p_dispersion_;
                Dispersion_ = *p_dispersion_;
                CalcCdCoef();
            }
            /* フィルタ係数更新(ATSからのフィードバックを含めてupdateがあれば更新する) */
            if(coef_update_ || phase_adjust_on_) {
                CalculateCoef(adjust_phase);
            }
        }

        /* FFT */
        fft_operator_->fft(input_buff_);
        const std::vector<std::complex<double> >& ffto = fft_operator_->get();
        std::vector<std::complex<double> > fft_out(FftSize_);
        fft_out = ffto;

        /* IQ分離, 係数乗算 */
        std::vector<std::complex<double> > inv_out(FftSize_);
        inv_out[0] = std::conj(fft_out[0]);
        for(int i=1; i < FftSize_; i++) {
            inv_out[i] = conj(fft_out[FftSize_ - i]);
        }
        std::vector<std::complex<double> > mlt_out(FftSize_);
        for(int i=0; i < FftSize_; i++) {
            mlt_out[i] = ((fft_out[i] + inv_out[i]) / 2.0) * CoefI_[i] +
                         ((fft_out[i] - inv_out[i]) / 2.0) * CoefQ_[i];
        }
        
        /* 周波数Offset補償 */
        FreqOffset(OffsetVal_, mlt_out);

        /* subcarrier DMUX */
        for(int scn=0; scn<subc_n_; scn++) {
            /* subcarrier shift */
            int sub_shift = subc_bin_shift_[scn] < 0 ? subc_bin_shift_[scn]+FftSize_ : subc_bin_shift_[scn];
            std::vector<std::complex<double> > sft_out = mlt_out;
            std::rotate(sft_out.begin(), sft_out.begin() + sub_shift, sft_out.end());

            /* LPF & down-sample */
            std::vector<std::complex<double> > lpf_out(IFftSize_);
            for(int i=0; i<IFftSize_; i++) {
                int index = i > IFftSize_/2 ? (FftSize_ - IFftSize_) + i : i;
                lpf_out[i] = subc_lpf_coef_[i] * sft_out[index];
            }

            /* IFFT , 1/2 Overlap除去 & FFT/IFFTの正規化 */
            ifft_operator_->fft(lpf_out);
            const std::vector<std::complex<double> >& iffto = ifft_operator_->get();
            std::vector<std::complex<double> > ifft_out(IFftSize_/2);
            for(int i=0; i < IFftSize_/2; i++) {
                ifft_out[i] = iffto[i + IFftSize_/4] / static_cast<double>(IFftSize_);
            }

            /* 振幅正規化 & 出力バッファ格納 */
            for(int i=0; i<IFftSize_/2; i++) {
                output_buff_[scn][i] = ifft_out[i] * subc_norm_coef_[scn];
            }
            if(subc_norm_auto_ && subc_norm_exec_num_>1) calc_subc_norm(scn, ifft_out);
        }
        subc_norm_exec_num_++;
    }
    /* サブキャリア毎遅延印加 & 出力 */
    int out_num = out_sample_num_ / subc_n_;
    bool out_valid = (out_sample_num_ % subc_n_ == 0) && (out_num < IFftSize_/2);
    if(out_valid) {
        OutputData_valid = true;
        for(int scn=0; scn<subc_n_; scn++) {
            subc_delay_buf_[scn].push_back(output_buff_[scn][out_num]);
            OutputData[scn] = subc_delay_buf_[scn][0];
            subc_delay_buf_[scn].pop_front();
        }
        /* Monitor */
        Execute_MonClip(InputData);
        Execute_MonRms(dcoffset_outdt_, OutputData[0]);
    } else {
        OutputData_valid = false;
        for(int scn=0; scn<subc_n_; scn++) {
            OutputData[scn] = std::complex<double>(0.0, 0.0);
        }
    }
    out_sample_num_++;
}

/* FFTサイズ分の補償係数を生成する */
void FUJITSU::CRxFdeCore::CalculateCoef(const double adjust_phase)
{
    coef_update_ = false;
    for(int i=0; i<FftSize_; i++) {
        const std::complex<double> j(0.0, 1.0);
        std::complex <double> tmp_i, tmp_q;
        /* X(e^jw) への周波数応答 */
        tmp_i = 0.5 * ( CoefNorm_[0] * CoefFres_[0][i] * (CoefIqComp_[0] * CoefSkew_[0][i] - j * CoefIqComp_[2] * CoefSkew_[1][i])
                  + j * CoefNorm_[1] * CoefFres_[1][i] * (CoefIqComp_[1] * CoefSkew_[0][i] - j * CoefIqComp_[3] * CoefSkew_[1][i]) )
               * CoefCd_[i] * CoefSpec_[i];
        /* X*(e^-jw) への周波数応答 */
        tmp_q = 0.5 * ( CoefNorm_[0] * CoefFres_[0][i] * (CoefIqComp_[0] * CoefSkew_[0][i] + j * CoefIqComp_[2] * CoefSkew_[1][i]) 
                  + j * CoefNorm_[1] * CoefFres_[1][i] * (CoefIqComp_[1] * CoefSkew_[0][i] + j * CoefIqComp_[3] * CoefSkew_[1][i]) )
               * CoefCd_[i] * CoefSpec_[i];
        /* Re/Imにブロックする */
        if(phase_adjust_on_) {
            int fn = (i <= FftSize_/2) ? i : i - FftSize_;
            std::complex<double> rotor = std::exp(std::complex<double>(0.0, -2.0 * M_PI * fn * adjust_phase / static_cast<double>(FftSize_)));
            CoefI_[i] = rotor * (tmp_i + tmp_q);
            CoefQ_[i] = rotor * (tmp_i - tmp_q);
        } else {
            CoefI_[i] = tmp_i + tmp_q;
            CoefQ_[i] = tmp_i - tmp_q;
        }
    }
}

/* 波長分散補償係数生成 */
void FUJITSU::CRxFdeCore::CalcCdCoef()
{
    coef_update_ = true;
    std::complex<double> exp_coef_cd(0.0,
                                     M_PI * LIGHT_SPEED * Dispersion_ *1E-12/1E-9
                                      * SamplingRate_ * SamplingRate_ 
                                      / (static_cast<double>(FftSize_) * FftSize_ * FcLaser_ * FcLaser_) );
    for(int i=0; i<FftSize_; i++) {
        int fn = (i < FftSize_/2) ? i : i - FftSize_;
        CoefCd_[i] = std::exp( - static_cast<double>(fn) * fn * exp_coef_cd);
    }
    /* 周波数オフセット処理 */
    if(CdOffsetOn_) {
        FreqOffset(-OffsetVal_, CoefCd_);
    }
}

/* スペクトル整形係数生成 */
void FUJITSU::CRxFdeCore::CalcSpecCoef()
{
    coef_update_ = true;

    const int c_a = static_cast<int>(FftSize_ * (1.0 - RoolOff_) / (2.0 * FsMode_));
    const int c_b = static_cast<int>(FftSize_ * (1.0 + RoolOff_) / (2.0 * FsMode_));
    
    for(int i=0; i<FftSize_/2+1; i++) {
        if(i >= 0 && i < c_a) {
            CoefSpec_[i] = std::complex<double>(1.0, 0.0);
        } else if(i >= c_a && i < c_b) {
            CoefSpec_[i] = std::complex<double>(sqrt((1.0 - sin(2.0 * M_PI * (i * FsMode_ / (2.0 * FftSize_) -0.25) / RoolOff_)) / 2.0), 0.0);
        } else {
            CoefSpec_[i] = std::complex<double>(0.0, 0.0);
        }
    }
    /* 折り返し */
    for(int i=1; i<FftSize_/2; i++) {
        CoefSpec_[FftSize_-i] = CoefSpec_[i];
    }
    /* 周波数オフセット処理 */
    if(SpecOffsetOn_) {
        FreqOffset(-OffsetVal_, CoefSpec_);
    }
}

/* 周波数応答調整係数生成 */
void FUJITSU::CRxFdeCore::CalcFresCoef(const std::vector<double>& tap_coef, std::vector<std::complex<double> >& fres_coef)
{
    coef_update_ = true;
    for(int i=0; i<FftSize_; i++) {
        fres_coef[i] = std::complex<double>(0.0, 0.0);
        int tap_size = static_cast<int>(tap_coef.size());
        for(int j=0; j<tap_size; j++) {
            double theta = 2.0 * M_PI * i * (j-(tap_size-1)/2)  / FftSize_;
            fres_coef[i] += std::complex<double>((tap_coef[j] * cos(theta)), -(tap_coef[j] * sin(theta)));
        }
    }
}

/* スキュー調整係数生成 */
void FUJITSU::CRxFdeCore::CalcSkew()
{
    coef_update_ = true;
    for(int iq=0; iq<2; iq++) {
        for(int i=0; i<FftSize_; i++) {
            int fn = (i < FftSize_/2) ? i : i - FftSize_;
            double theta = 2.0 * M_PI * Skew_[iq] * fn / FftSize_;
            CoefSkew_[iq][i] = std::complex<double>(cos(theta), -sin(theta));
        }
    }
}

/* I-Qクロストーク補償係数生成 */
void FUJITSU::CRxFdeCore::CalcIqComp()
{
    coef_update_ = true;
    double theta2 = M_PI / 180.0 * IqComp_; // 2*theta
    double c = cos(theta2/2)/cos(theta2);
    double s = sin(theta2/2)/cos(theta2);
    CoefIqComp_[0] = c;
    CoefIqComp_[1] = s;
    CoefIqComp_[2] = s;
    CoefIqComp_[3] = c;
}

/* ノーマライゼーション係数生成 */
void FUJITSU::CRxFdeCore::CalcNorm()
{
    if(!NormInt_) {
        /* 外部設定 */
        if(NormIdv_ == 1) {
            /* 個別設定 */
            CoefNorm_[0] = NormCoefIdv_[0];
            CoefNorm_[1] = NormCoefIdv_[1];
        } else {
            /* 総合設定 */
            CoefNorm_[0] = NormCoefAll_;
            CoefNorm_[1] = NormCoefAll_;
        }
    } else {
        /* 内部設定 */
        CalcNormInternal();
    }
}

/* 内部設定時のノーマライゼーション係数生成 */
void FUJITSU::CRxFdeCore::CalcNormInternal()
{
    if(rms_update_) {
        rms_update_  = false;
        coef_update_ = true;
        for(int i=0; i<2; i++) { /* IQ loop */
            if(rms_[i] < 1e-10 && rms_[i] > -1e-10) {
                CoefNorm_[i] = 4.0;  /* 上限は4倍 */
            } else {
                CoefNorm_[i] = NormRms_ / rms_[i];
                if(CoefNorm_[i] > 4.0) {
                    CoefNorm_[i] = 4.0;
                }
            }
        }
    }
}

/* RMS生成 */
void FUJITSU::CRxFdeCore::CalcRms(std::complex<double> indt)
{
    rms_ctr_++;
    rms_before_[0] += indt.real() * indt.real();
    rms_before_[1] += indt.imag() * indt.imag();

    if(rms_ctr_ == c_RMS_SAMPLE_NUM) {
        rms_ctr_ = 0;
        rms_update_ = true;
        for(int i=0; i<2; i++) {  /* I,Q loop */
            rms_[i] = sqrt(rms_before_[i] / c_RMS_SAMPLE_NUM);
            rms_before_[i] = 0.0;
        }
    }
}

/* 周波数オフセット補償処理 */
void FUJITSU::CRxFdeCore::FreqOffset(const int freq_offset, std::vector<std::complex<double> >& inout)
{
    if(freq_offset>0) {
        std::rotate(inout.begin(), inout.begin() + freq_offset, inout.end());
        for(int i = FftSize_ /2 - freq_offset; i < FftSize_ / 2; i++) {
            inout[i] = std::complex<double>(0.0, 0.0);
        }
    } else if(freq_offset<0) {
        std::rotate(inout.begin(), inout.begin() + FftSize_ + freq_offset, inout.end());
        for(int i = FftSize_ / 2; i < FftSize_ / 2 - freq_offset; i++) {
            inout[i] = std::complex<double>(0.0, 0.0);
        }
    }
}

/* DC オフセット補償処理 */
void FUJITSU::CRxFdeCore::DcOffset(const std::complex<double> indt)
{
    /* DC Offset補償値内部/外部選択 */
    if(DcExt_ == 0) {                   /* DC Offset = 内部補償 */
        /* DC Offset補償 FeedForward/FeedBack 選択 */
        if(DcFb_ == 0) {
            dc_val_ = dc_rtn_ff_;           /* FeedForward補償 */
        } else {
            dc_val_ = dc_rtn_fb_;           /* FeedBack補償 */
        }
    } else {                               /* DC Offset = 外部補償 */
        dc_val_ = Dc_;
    }
    /* DC Offset補償処理 */
    dcoffset_outdt_ = std::complex<double>((indt.real() - dc_val_[0]), (indt.imag() - dc_val_[1]));
}

/* DC Offset Execute */
void FUJITSU::CRxFdeCore::Execute_Dc(std::complex<double> indt, std::complex<double> outdt)
{
    dc_ctr_++;
    CalcDc(dc_calc_ff_, indt);   
    CalcDc(dc_calc_fb_, outdt);  

    if(dc_ctr_ == DcSnum_) {
        for(int i=0; i<2; i++) {                       /* I,Q loop */
            dc_rtn_ff_[i] = dc_calc_ff_[i] / DcSnum_;  /* DcSnum_ = 128*2^n */
            dc_calc_ff_[i] = 0.0;
            if(dc_fbchg_ctr_ < DcCnum_) {
                dc_rtn_fb_[i] += (dc_calc_fb_[i] / DcSnum_) * DcInitCoef_ ;
            } else {
                dc_rtn_fb_[i] += (dc_calc_fb_[i] / DcSnum_) * DcNormCoef_ ;
            }
            dc_calc_fb_[i] = 0.0;
        }
        dc_ctr_ = 0;
        dc_fbchg_ctr_++; 
    }
}

/* DC Offset Calc */
void FUJITSU::CRxFdeCore::CalcDc(std::vector<double> &rtn, std::complex<double> indt)
{
    rtn[0] += indt.real();
    rtn[1] += indt.imag();
}


/* Mon Clip Execute */
void FUJITSU::CRxFdeCore::Execute_MonClip(std::complex<double> indt)
{
    mon_clip_count_++;                        /* Sample Timing Count  */
    MonClipCalc(mon_clip_bf_, indt);          /* Monitor Execute (in) */

    /* Sample Timing Monitor Latch&Clear */
    if(mon_clip_count_ == MonClipSnum_) { /* MonClipSnum_ = 128*2^n */
        mon_clip_count_ = 0 ;
        for(int i=0; i<2; i++) {
            mon_clip_bf_reg_[i] = mon_clip_bf_[i];
            mon_clip_bf_[i] = 0;
        }
    }
}

/* MonClip Calc */
void FUJITSU::CRxFdeCore::MonClipCalc(std::vector<int> &rtn, std::complex<double> indt)
{
     std::vector<double> data(2, 0.0);
     data[0] = fabs(indt.real());
     data[1] = fabs(indt.imag());

     for(int i=0; i<2; i++) {
         if(data[i] > MonClipTh_)              /* MonClipTh_ = xx  */
             rtn[i] += 1;
     }
}

/* Mon RMS Execute */
void FUJITSU::CRxFdeCore::Execute_MonRms(std::complex<double> indt, std::complex<double> outdt)
{
    mon_rms_count_++;                          /* Sample Timing Count   */
    MonRmsCalc(mon_rms_bf_, indt);             /* Monitor Execute (in)  */
    MonRmsCalc(mon_rms_af_, outdt);            /* Monitor Execute (out) */

    /* Sample Timing Monitor Latch&Clear */
    if(mon_rms_count_ == MonRmsSnum_) {        /* MonRmsSnum_ = 128*2^n */
        mon_rms_count_ = 0 ;
        for(int i=0; i<2; i++) {
            mon_rms_bf_reg_[i] = sqrt(mon_rms_bf_[i] / MonRmsSnum_);
            mon_rms_af_reg_[i] = sqrt(mon_rms_af_[i] / MonRmsSnum_);
            mon_rms_bf_[i] = 0.0;
            mon_rms_af_[i] = 0.0;
        }
    }
}

/* MonRms Calc */
void FUJITSU::CRxFdeCore::MonRmsCalc(std::vector<double> &rtn, std::complex<double> indt)
{
    rtn[0] += indt.real() * indt.real();
    rtn[1] += indt.imag() * indt.imag();
}

/* サブキャリア時のauto正規化係数算出 */
void FUJITSU::CRxFdeCore::calc_subc_norm(const int scn, const std::vector<std::complex<double> >& dt)
{
    /* update指定時刻まで実施 */
    if(subc_norm_update_index_[scn] < static_cast<int>(subc_norm_update_sample_.size())) {
        int n = static_cast<int>(dt.size());
        for(int i=0; i<n; i++) {
            subc_norm_pow_[scn] += (dt[i].real() * dt[i].real() + dt[i].imag() * dt[i].imag());
        }
        subc_norm_samples_[scn] += n;
        if(subc_norm_samples_[scn] >= subc_norm_update_sample_[subc_norm_update_index_[scn]]) { /* update timming */
            double now_rms = std::sqrt(subc_norm_pow_[scn] / 2.0 / static_cast<double>(subc_norm_samples_[scn]));
            subc_norm_coef_[scn] = subc_norm_rms_ / now_rms;
            //std::cout << "scn=" << scn << ", " << subc_norm_update_index_[scn] << ", " << subc_norm_samples_[scn] << ", " << subc_norm_coef_[scn] << std::endl;
            subc_norm_update_index_[scn]++;
        }
    }
}

/* 残留分散値設定処理 */
void FUJITSU::CRxFdeCore::SetResidualDispersion(const double rcd, const unsigned int scn)
{
    rcd_feedback_done_[scn] = true;
    rcd_feedback_value_[scn] = rcd;

    /* 全サブキャリアフィードバックが終わったら 分散補償値を更新 */
    bool all_subcarrier_done = true;
    for(int i=0; i<subc_n_; i++) all_subcarrier_done &= rcd_feedback_done_[i];
    if(all_subcarrier_done) {
        double ave_rcd = 0.0;
        for(int i=0; i<subc_n_; i++) {
            ave_rcd += rcd_feedback_value_[i];
        }
        ave_rcd /= static_cast<double>(subc_n_);
        Dispersion_ += ave_rcd;
        CalcCdCoef();
    }
}

/* 残留周波数オフセット値設定処理 */
void FUJITSU::CRxFdeCore::SetResidualFrequencyOffset(const double rofs, const unsigned int scn)
{
    rofs_feedback_done_[scn] = true;
    /* ATSは、サンプリング速度1/Nで動作しているため、1サンプル当たり回転量がN倍となっている */
    rofs_feedback_value_[scn] = rofs / static_cast<double>(subc_n_);

    /* 全サブキャリアフィードバックが終わったら 分散補償値を更新 */
    bool all_subcarrier_done = true;
    for(int i=0; i<subc_n_; i++) all_subcarrier_done &= rofs_feedback_done_[i];
    if(all_subcarrier_done) {
        double ave_rofs = 0.0;
        for(int i=0; i<subc_n_; i++) {
            ave_rofs += rofs_feedback_value_[i];
        }
        ave_rofs /= static_cast<double>(subc_n_);
        /* FFTグリッド単位に変換(1/2 Overlap時は2グリットの倍数) */
        OffsetVal_ += 2 * static_cast<int>(ave_rofs * FftSize_ /2.0);
        CalcCdCoef();
        if(SpecOn_) CalcSpecCoef();
    }
}

/* シミュレーション結果report */
void FUJITSU::CRxFdeCore::ReportStatisticalData(std::ostream& os, const double NowTime)
{
    double MonClipSnum_tmp_     = static_cast<double>(MonClipSnum_) ;
    double mon_clip_bf_reg0_tmp = static_cast<double>(mon_clip_bf_reg_[0]) ; 
    double mon_clip_bf_reg1_tmp = static_cast<double>(mon_clip_bf_reg_[1]) ; 
      
    double clip_ratio_i = (  mon_clip_bf_reg0_tmp / MonClipSnum_tmp_ ) * 100.0;
    double clip_ratio_q = (  mon_clip_bf_reg1_tmp / MonClipSnum_tmp_ ) * 100.0;

    os << "----- RXFDE(" << HV_str_ << ") monitor Report -----" << std::endl;
    os << " DC offset compensate          I, Q  : " << dc_val_[0]          << ", " << dc_val_[1]          << std::endl;
    os << " Monitor Threshold over rate   I, Q  : " << clip_ratio_i  << "% (" << mon_clip_bf_reg_[0] << "), " << clip_ratio_q << "% (" << mon_clip_bf_reg_[1] << ")" << std::endl;
    os << " Monitor RMS(after DC_offset)  I, Q  : " << mon_rms_bf_reg_[0]  << ", " << mon_rms_bf_reg_[1]  << std::endl;
    os << " Monitor RMS(after IFFT)       I, Q  : " << mon_rms_af_reg_[0]  << ", " << mon_rms_af_reg_[1]  << std::endl;
    os << " Output Normalize coeff  #scn : ";
    for(int scn=0; scn<subc_n_; scn++) os << subc_norm_coef_[scn] << ", ";
    os << std::endl;
    os << " Conpensating last dispersion : " << Dispersion_ << " (ps/nm)\n";
    double ofs = static_cast<double>(OffsetVal_) * SamplingRate_ / static_cast<double>(FftSize_) / 1.0e9;
    os << " Conpensating last freq offset: " << ofs << " (GHz)\n";

}
