/*!
    @file  CRxFdeCore.h
    @brief DSP Frequency Domain Equlizer Core(ヘッダ)

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2020 Fujitsu Limited. All rights reserved.
*/

/* 係数パラメータのDynamic設定は後回し */

#ifndef CRxFdeCore_h
#define CRxFdeCore_h

#include <deque>
#include <boost/shared_ptr.hpp>
#include "common.h"
#include "CRxFdeFFT.h"

namespace FUJITSU {
/*! DSP 周波数ドメインイコライザ処理 */
class CRxFdeCore {
public:
    enum PolarizationT {
        H,
        V
    };

    /* コンストラクタ */
    CRxFdeCore(const PolarizationT        HV,
               const int                  subc_n,         /* サブキャリア数 */
               const int                  FftSize,        /* FFTサイズ */
               const double               FsMode,         /* オーバーサンプリング率設定 */
               const std::vector<int>     subc_bin_shift, /* サブキャリアのbinシフト量(受信FFTサイズに換算する) */
               const bool                 subc_lpf_rrc,   /* サブキャリア分離LPF is RRC */
               const double               subc_rrc_factor,   /* サブキャリア分離LPF ロールオフ率 */
               const double               subc_rrc_width,    /* サブキャリア分離LPF RRC帯域幅 */
               const std::vector<double>  subc_gauss_factor, /* サブキャリア分離LPF gaussian filter係数 */
               const bool                 subc_norm_auto,    /* サブキャリア振幅正規化 is auto */
               const std::vector<double>  subc_norm_ext_coef,/* サブキャリア振幅正規化 ext時の正規化係数(サブキャリア毎) */
               const double               subc_norm_rms,      /* サブキャリア振幅正規化 CH当たり目標RMS値 */
               const std::vector<int>     subc_norm_update_sample,  /* サブキャリア振幅正規化 auto時の正規化係数update sample数 */
               const std::vector<int>     subc_delay_sample,        /* サブキャリア毎delay値 (サンプル) */
               const bool                 SpecOn,
               const double               RoolOff,
               const std::vector<double>& FresTapI,
               const std::vector<double>& FresTapQ,
               const std::vector<double>& Skew,
               const double               IqComp,
               const std::vector<double>& NormCoefIdv,
               const double               NormCoefAll,
               const int                  NormIdv,
               const double               NormRms,
               const int                  NormInt,
               const int                  OffsetVal,
               const bool                 CdOffsetOn,
               const bool                 SpecOffsetOn,
               const double               FcLaser,
               const double               SamplingRate,
               double*                    prm_dispersion,
               const int                  DcExt,
               const std::vector<double>& Dc,
               const int                  DcSnum,
               const int                  DcFb,
               const int                  DcCnum,
               const double               DcInitCoef,
               const double               DcNormCoef,
               const double               MonClipTh,
               const int                  MonClipSnum,
               const int                  MonRmsSnum,
               const std::vector<std::complex<double> >& PramCoefI,
               const std::vector<std::complex<double> >& PramCoefQ,
               const int                  FcPramOn,
               const bool                 phase_adjust_on);
    
    /* デストラクタ */
    virtual ~CRxFdeCore();
    
    /* 周波数ドメインイコライザ処理実行 */
    virtual void Execute(const double dsptm,
                         const double adjust_phase,
                         const std::complex<double>& InputData,
                         bool&                               OutputData_valid,
                         std::vector<std::complex<double> >& OutputData);

    /* 残留分散値設定処理 */
    virtual void SetResidualDispersion(const double rcd, const unsigned int scn);

    /* 残留周波数オフセット値設定処理 */
    virtual void SetResidualFrequencyOffset(const double rofs, const unsigned int scn);

    /* シミュレーション結果report */
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime);

private:

    /*--------------
      メンバ変数
      --------------*/
    std::string                        HV_str_;         /* 偏波面識別(デバッグ用) */
    int                                subc_n_;         /* サブキャリア数 */
    int                                FftSize_;        /* FFTサイズ */
    int                                IFftSize_;       /* IFFTサイズ */
    double                             FsMode_;         /* オーバーサンプリング率設定 */
    std::vector<int>                   subc_bin_shift_; /* サブキャリアのbinシフト量(受信FFTサイズに換算する) */
    std::vector<double>                subc_lpf_coef_;  /* サブキャリア分離LPF係数 */
    bool                               subc_norm_auto_;     /* サブキャリア振幅正規化 ext時の正規化係数(サブキャリア毎) */
    std::vector<double>                subc_norm_coef_; /* サブキャリア振幅正規化 正規化係数 */
    double                             subc_norm_rms_;      /* サブキャリア振幅正規化 CH当たり目標RMS値 */
    std::vector<int>                   subc_norm_update_sample_;  /* サブキャリア振幅正規化 auto時の正規化係数update sample数 */
    std::vector<int>                   subc_norm_update_index_;   /* subc_norm_update_sample_のindex */
    std::vector<int>                   subc_norm_samples_;         /* サブキャリア振幅正規化sample数 */
    int                                subc_norm_exec_num_;         /* サブキャリア振幅正規化sample数 */
    std::vector<double>                subc_norm_pow_; /* サブキャリア振幅正規化 正規化係数 */
    std::vector<std::deque<std::complex<double> > >  subc_delay_buf_;   /* サブキャリア毎delay buffer */

    bool                               SpecOn_;         /* スペクトル整形 ON/OFF */
    double                             RoolOff_;        /* Rool off率 */
    std::vector<double>                FresTapI_;       /* 周波数応答調整用 FIRフィルタ係数 I */
    std::vector<double>                FresTapQ_;       /* 周波数応答調整用 FIRフィルタ係数 Q */
    std::vector<double>                Skew_;           /* スキュー補償量 I,Q */
    double                             IqComp_;         /* I-Qクロストーク補償量 */
    std::vector<double>                NormCoefIdv_;    /* ノーマライゼーション レーン個別調整係数(I, Q) */
    double                             NormCoefAll_;    /* ノーマライゼーション 総合値調整係数 */
    int                                NormIdv_;        /* ノーマライゼーション 個別/総合モード選択 0:総合, 1:個別 */
    double                             NormRms_;        /* ノーマライゼーション ターゲットRMS */
    int                                NormInt_;        /* ノーマライゼーション 外部・内部設定選択 0:外部, 1:内部 */
    int                                OffsetVal_;      /* 周波数オフセット補償量 (グリッド単位指定) */
    bool                               CdOffsetOn_;     /* 周波数オフセット補償と波長分散補償の実施順番 1:周波数オフセット補償を先に実施，0:波長分散補償を先に実施 */
    bool                               SpecOffsetOn_;   /* 周波数オフセット補償とスペクトル整形の実施順番 1:周波数オフセット補償を先に実施，0:スペクトル整形を先に実施 */
    double                             FcLaser_;        /* レーザー光の基準周波数fc  (THz) */
    double                             SamplingRate_;   /* サンプリングレート */
    double*                            p_dispersion_;   /* 波長分散補償量 */
    double                             ParamDispersion_;/* 波長分散補償量(dynamic parameter設定保持値) */
    double                             Dispersion_;     /* 波長分散補償量(current) */
    int                                DcExt_;          /* DCオフセット補償 外部/内部選択 0:内部, 1:外部 [追加] */
    std::vector<double>                Dc_;             /* DCオフセット補償量 [追加] */
    int                                DcSnum_;         /* DCオフセット補償 Sampling周期設定 */
    int                                DcFb_;           /* DCオフセット補償 FeedForward/FeedBack選択 0:FF, 1:FB */
    int                                DcCnum_;         /* DCオフセット補償 FB定常係数移行回数設定 */
    double                             DcInitCoef_;     /* DCオフセット補償 FB初期引き込み係数(1.0以下) */
    double                             DcNormCoef_;     /* DCオフセット補償 FB定常係数(1.0以下) */
    double                             MonClipTh_;      /* Monitor Clip閾値設定 */
    int                                MonClipSnum_;    /* Monitor Clip数算出 Sample数 */
    int                                MonRmsSnum_;     /* Monitor RMS算出 Sample数 */
    std::vector<std::complex<double> > PramCoefI_;      /* 周波数領域線形補償係数パラメータ */
    std::vector<std::complex<double> > PramCoefQ_;      /* 周波数領域線形補償係数パラメータ */
    int                                FcPramOn_;       /* 周波数領域線形補償係数パラメータ設定 */
    const bool                         phase_adjust_on_;

    //std::vector<std::complex<double> > CoefI_bf_;       /* 補償係数I ラッチ前 */
    //std::vector<std::complex<double> > CoefQ_bf_;       /* 補償係数Q ラッチ前 */
    std::vector<std::complex<double> > CoefI_;          /* 補償係数I */
    std::vector<std::complex<double> > CoefQ_;          /* 補償係数Q */
    bool                               CoefChangeDone_; /* 補償係数更新済みフラグ */
    std::vector<std::complex<double> > input_buff_;     /* 入力バッファ */
    std::vector<std::vector<std::complex<double> > > output_buff_;    /* 出力バッファ */

    std::vector<bool>   rofs_feedback_done_;    /* 残留周波数オフセットフィードバック完了フラグ(サブキャリア分) */
    std::vector<double> rofs_feedback_value_;   /* 残留周波数オフセットフィードバック値(サブキャリア分) */
    std::vector<bool>   rcd_feedback_done_;     /* 残留分散フィードバック完了フラグ(サブキャリア分) */
    std::vector<double> rcd_feedback_value_;    /* 残留分散フィードバック値(サブキャリア分) */

    int                                sample_num_;     /* sample番号 */
    int                                out_sample_num_; /* tmp番号 */
    bool                               coef_update_;    /* 係数更新フラグ */
    bool                               rms_update_;     /* RMS更新フラグ */
    std::vector<double>                rms_;            /* RMS */
    std::vector<double>                rms_before_;     /* RMS算出中間値 */
    int                                rms_ctr_;        /* RMS算出カウンタ */
    
    std::vector<std::complex<double> > CoefCd_;         /* 波長分散補償係数 */
    std::vector<std::complex<double> > CoefSpec_;       /* スペクトル整形係数 */
    std::vector<std::vector<std::complex<double> > >  CoefFres_;       /* 周波数応答調整係数[IQ][FFT_SIZE] */
    std::vector<std::vector<std::complex<double> > >  CoefSkew_;       /* スキュー調整係数[IQ][FFT_SIZE] */
    std::vector<double>                CoefIqComp_;     /* I-Qクロストーク補償係数 */
    std::vector<double>                CoefNorm_;       /* ノーマライゼーション係数 */

    std::complex<double>               dcoffset_outdt_; /* DCオフセット補償後出力データ */
    std::vector<double>                dc_calc_ff_;     /* DCオフセット補償 加算(feedforward用) */
    std::vector<double>                dc_calc_fb_;     /* DCオフセット補償 加算(feedback用) */
    std::vector<double>                dc_rtn_ff_;      /* DCオフセット補償 加算平均結果(feedforward用) */
    std::vector<double>                dc_rtn_fb_;      /* DCオフセット補償 加算平均結果(feedback用) */
    int                                dc_ctr_;         /* DCオフセット補償用 サンプル数カウンタ */
    std::vector<double>                dc_val_;         /* DCオフセット補償 加算平均結果 内部/外部選択後 */
    int                                dc_fbchg_ctr_;   /* DCオフセット補償 更新数カウンタ */

    int                                mon_clip_count_;     /* モニタ用clip数算出 サンプル数カウンタ */
    std::vector<int>                   mon_clip_bf_;        /* モニタ用clip カウント数 */
    std::vector<int>                   mon_clip_bf_reg_;    /* モニタ用clip カウント数 通知*/
    int                                mon_rms_count_;      /* モニタ用RMS数算出 サンプル数カウンタ */
    std::vector<double>                mon_rms_bf_;         /* モニタ用pow算出(DC_offset出力) */
    std::vector<double>                mon_rms_af_;         /* モニタ用pow算出(IFFT出力) */
    std::vector<double>                mon_rms_bf_reg_;     /* モニタ用pow算出(DC_offset出力)通知 */
    std::vector<double>                mon_rms_af_reg_;     /* モニタ用pow算出(DC_offset出力)通知 */

    boost::shared_ptr<CRxFdeFFT>       fft_operator_;   /* FFTオブジェクト */
    boost::shared_ptr<CRxFdeFFT>       ifft_operator_;  /* IFFTオブジェクト */


    /*--------------
      定数
      --------------*/
    static const int c_RMS_SAMPLE_NUM;                  /* RMS算出サンプル数 */

    /*--------------
      メンバ関数
      --------------*/
    /* FFTサイズ分の補償係数を生成する */
    void CalculateCoef(const double adjust_phase);

    /* 波長分散補償係数算出 */
    void CalcCdCoef();
    /* スペクトル整形係数生成 */
    void CalcSpecCoef();
    /* 周波数応答調整係数生成 */
    void CalcFresCoef(const std::vector<double>& tap_coef, std::vector<std::complex<double> >& fres_coef);
    /* スキュー調整係数生成 */
    void CalcSkew();
    /* I-Qクロストーク補償係数生成 */
    void CalcIqComp();
    /* ノーマライゼーション係数生成 */
    void CalcNorm();
    /* 内部設定時のノーマライゼーション係数生成 */
    void CalcNormInternal();
    /* RMS生成 */
    void CalcRms(std::complex<double> indt);
    /* 周波数オフセット補償処理 */
    void FreqOffset(const int freq_offset, std::vector<std::complex<double> >& inout);
    
    /* DCオフセット補償処理 */
    void DcOffset(std::complex<double> indt);
    /* 内部設定時のDCオフセット補償値生成 */
    void Execute_Dc(std::complex<double> indt, std::complex<double> outdt);
    /* DCオフセット用加算処理 */
    void CalcDc(std::vector<double> &rtn, std::complex<double> indt);
    
    /* モニタ用 clip数通知 */
    void Execute_MonClip(std::complex<double> indt);
    /* モニタ用 clip数カウント */
    void MonClipCalc(std::vector<int> &rtn, std::complex<double> indt);
    /* モニタ用 RMS通知 */
    void Execute_MonRms(std::complex<double> indt, std::complex<double> outdt);
    /* モニタ用 RMS生成 */
    void MonRmsCalc(std::vector<double> &rtn, std::complex<double> indt);
    /* サブキャリア時のauto正規化係数算出 */
    void calc_subc_norm(const int scn, const std::vector<std::complex<double> >& dt);
};
}

#endif /* CRxFdeCore_h */
