/*!
    @file  CRxFdeFFT.cpp 
    @brief Radix2 FFT/IFFT 

    @date     $Date:: 2022-01-21 10:47:24 #$
    @revision $Revision: 2 $

    Copyright (C) 2017-2020 Fujitsu Limited. All rights reserved.
*/

#include <iostream>
#include <cstdlib>
#include <cmath>
#include "CRxFdeFFT.h"

/*---------------------------*/
/*   コンストラクタ          */
/*  dir=+1:FFT, dir=-1:IFFT  */
/*---------------------------*/
FUJITSU::CRxFdeFFT::CRxFdeFFT(const unsigned int nfft, const int dir)
    : nFft_(nfft) 
{
    /* point数check  */
    for(iBit_=0; ((nfft>>iBit_)&1)!=1 && (nfft>>iBit_)!=0; iBit_++);
    if((nfft >> iBit_) != 1) {
        std::cerr << "FFT point Num Error (" << nfft << ")\n";
        exit(1);
    }
    if(!(dir==1 || dir==-1)) {
        std::cerr << "FFT direction error (dir=" << dir << ")\n";
        exit(1);
    }
    /* メモリ領域確保  */
    bsw_.resize(nfft);
    xCoef_.resize(nfft/2);
    xWork_.resize(nfft);
    /* 回転子テーブル作成  */
    const double dPai = std::acos(-1.0);
    const double dph = -dir * 2.0 * dPai / static_cast<double>(nfft);
    for(unsigned int i=0; i<nfft/2; i++) {
        double rad = static_cast<double>(i) * dph;
        xCoef_[i] = std::complex<double>(std::cos(rad), std::sin(rad));
    }
    /* ビットスワップテーブルの作成  */
    for(unsigned int i=0; i<nfft; i++) {
        unsigned int iT = 1 << (iBit_-1);    /* MSB='1'  */
        unsigned int iL = 1;                 /* LSB='1'  */
        unsigned int iSw = 0;                /* clear iSw  */
        for(unsigned int k=0; k<iBit_; k++) {
            if(i & iL) {
                iSw = iSw | iT;  /* detect '1' from LSB to MSB & If detect set iSw '1'   */
            }
            iT = iT >> 1;
            iL = iL << 1;
        }
        bsw_[i] = iSw;
    }
}
/*---------------------------*/
/*   デストラクタ            */
/*---------------------------*/
FUJITSU::CRxFdeFFT::~CRxFdeFFT()
{
}
/*---------------------------*/
/*   FFT Calculation         */
/*---------------------------*/
void FUJITSU::CRxFdeFFT::fft(const std::vector<std::complex<double> >& In)
{
    if(nFft_ != In.size()) {
        std::cerr << "FFT input data size is invalid!\n";
        exit(1);
    }

    xWork_ = In;
    /*  FFT start  */
    unsigned int i,j,k;
    unsigned int i2 = 1;
    /* ***** column *****  */
    for(i=1; i<iBit_; i++) {
        const unsigned int k2 = nFft_/2/i2;
        for(j=0; j<i2; j++) {
            for(k=0; k<k2; k++) {
                const unsigned int ix=k*i2;     /* Wcoef (回転子アドレス)  */
                const unsigned int ka=k+2*k2*j; /* pair of butterfly operation  */
                const unsigned int kb=ka+k2;
                const std::complex<double> xbuf = xWork_[ka];
                xWork_[ka] += xWork_[kb];       /* butterfly operation  */
                xWork_[kb] = xbuf - xWork_[kb];
                xWork_[kb] *= xCoef_[ix];
            }
        }
        i2 *= 2;
    }
    /* ***** last column *****  */
    /* サブキャリアDMUXを考慮して、正規化係数はFFT/IFFTの、外側でかける */
    const double morm_coef = 1.0;
    for(j=0; j<nFft_/2; j++) {
        const unsigned int ka=2*j;                     /* pair of butterfly operation   */
        const unsigned int kb=ka+1;
        const std::complex<double> xbuf = xWork_[ka];  /* butterfly operation   */
        xWork_[ka] = (xbuf + xWork_[kb]) * morm_coef;
        xWork_[kb] = (xbuf - xWork_[kb]) * morm_coef;
    }
    /* ***** output (出力並び替え) *****  */
    for(j=0; j<nFft_; j++) {
        const unsigned int m = bsw_[j];
        if(m > j) {
            const std::complex<double> xbuf = xWork_[j];
            xWork_[j] = xWork_[m];
            xWork_[m] = xbuf;
        }
    }
}
/*---------------------------*/
/* result get                */
/*---------------------------*/
const std::vector<std::complex<double> >& FUJITSU::CRxFdeFFT::get() const
{
    return xWork_;
}
