/*!
    @file  c_param_wrap.h
    @brief 共通クラス公開インターフェイス定義 (gen2-green wrapper) 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_PARAM_WRAP_H
#define C_PARAM_WRAP_H

#include <complex>
#include <vector>

/*!
  @brief  GEN2 GREEN互換用 パラメータクラスのインターフェースクラス 
 */
class C_param_wrap
{
public:
    /*! @brief 値の型宣言 
     */
    enum EParamType {
        NONE = 0,
        INT, DOUBLE, STRING, COMPLEX, ICOMPLEX,
        INT_VECTOR, DOUBLE_VECTOR, STRING_VECTOR, COMPLEX_VECTOR, ICOMPLEX_VECTOR,
        BOOL,
        OTHER
    };

    /*! @brief モード宣言 
     */
    enum EParamMode {
        ABSOLUTE=1, RELATIVE
    };

    /*! @brief 内部変数の型 
     */
    struct SParamElement {
        std::string str;    /*!< パラメータ文字列 */
        void* ptr;          /*!< 変数へのポインタ */
        EParamType ptype;   /*!< パラメータのタイプ */
        int rad;            /*!< 整数時の基数 */
        EParamMode mode;    /*!< 絶対／相対指定モード */
        int used;           /*!< 使用／未使用フラグ */
        SParamElement():str(""),ptr(NULL),ptype(NONE),rad(10),mode(ABSOLUTE),used(0) {};
    };

public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    C_param_wrap() {}

    /*! @brief デコンストラクタ 
     */
    virtual ~C_param_wrap() {}

    /*! @brief 初期化(未サポート) 
     */
    virtual int init( const std::string& filename) = 0;

    /*! @brief ファイル読み込み(未サポート) 
     */
    virtual int read( const std::string& file) = 0;

    /*! @brief 値の取得 静的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @param   ptype    値の型（未使用） 
     *  @param   rad      値の基数（未使用） 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual bool getValue(const std::string &var_name, int* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, unsigned int* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, double* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::complex<double>* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::string* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::vector<int>* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::vector<unsigned int>* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::vector<double>* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::vector<std::complex<double> >* val, EParamType ptype, int rad=10) = 0;
    virtual bool getValue(const std::string &var_name, std::vector<std::string>* val, EParamType ptype, int rad=10) = 0;

    /*! @brief パラメタの設定 動的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @param   ptype    値の型（未使用） 
     *  @param   rad      値の基数（未使用） 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual int setparam( const std::string& name, int* val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, unsigned int* val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, double* val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::complex<double>* val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::string* val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::vector<int>*& val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::vector<unsigned int>*& val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::vector<double>*& val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::vector<std::complex<double> >*& val, EParamType ptype, int rad=10) = 0;
    virtual int setparam( const std::string& name, std::vector<std::string>*& val, EParamType ptype, int rad=10) = 0;

    /*! @brief パラメータの追加 シミュレータ内部変数の追加用 
     *  @param var_name パラメータ名 
     *  @param val      string型の値 
     *  @param mode     （未使用） 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual int setvalue( const std::string& var, const std::string& val, EParamMode mode=ABSOLUTE) = 0;
    
    /*! @brief パラメータの追加 シミュレータ内部変数の取得用 
     *  未サポート 
     */
    virtual SParamElement* getparam( const std::string& name) = 0;
    
    /*! @brief  ベンダー名を返す。 
     *  @return ベンダー名 
     */
    virtual const std::string getVender() = 0;
    
    /*! @brief  時間の文字列を作成する。 
     *  @return 時間の文字列 
     */
    virtual const std::string getMakeTime() = 0;

    /*! @brief  dumpをする 
     *  @param  os 出力ストリーム 
     */
    virtual void param_dump(std::ostream& os) = 0;
};

#endif /* C_PARAM_WRAP_H */
