/*!
    @file  I_Util.h
    @brief ユーティリティクラスインタフェース 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef I_UTIL_H
#define I_UTIL_H

#include <vector>
#include <string>
#include <complex>
#include <boost/shared_ptr.hpp>

#include "Util_wrap.h"
#include "I_param.h"
#include "C_ErrMng.h"

class C_SimTime;
class C_param_parser;
class C_sc_parser;
class C_scenario;

/*!
  ユーティリティクラスインタフェース 
  @brief  ユーティリティクラスのインターフェースクラス 
 */
class I_Util : public Util_wrap
{
public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    I_Util() {}

    /*! @brief デストラクタ 
     */
    virtual ~I_Util() {}

    /*! @brief  ベンダー名を返す。 
     *  @return ベンダー名 
     */
    virtual const std::string getVender() = 0;

    /*! @brief  時間の文字列を作成する。 
     *  @return 時間の文字列 
     */
    virtual const std::string getMakeTime() = 0;

    /*! @brief パスからファイル名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ファイル名 
     */
    virtual const std::string getFileName(const std::string& path) = 0;

    /*! @brief パスからディレクトリ名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ディレクトリ名 
     */
    virtual const std::string getDirName(const std::string& path) = 0;

    /*! @brief プログラムの実行時間を出力します。 
     *         特定期間の実行時間を計測したい場合は、計測を開始したい 
     *         ポイントで別途本クラスをインスタンスして下さい。 
     *  @param os     出力stream（標準出力に出力したい場合は、省略可です） 
     *  @param prefix 出力する際のprefixを指定してください。 
     *                省略可です（省略時には"SIM"を指定した事になります）。 
     *                ["XXX"を渡した場合の出力例] 
     *                  XXX START : Mon Jul  6 14:28:27 2009 
     *                  XXX END   : Mon Jul  6 14:30:24 2009 
     *                  XXX TOTAL : 116 sec. (CPU TIME : 100.00) 
     */
    virtual void outputProcessingTime(std::ostream& os, const std::string& prefix = std::string("SIM")) = 0;

    /*! @brief パラメータの設定とチェックを行います。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータに設定する値 (int型,double型等) 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 成功, false 失敗 
     */
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, int*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, unsigned int*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, double*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::string*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::complex<double>*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<int>*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<unsigned int>*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<double>*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::string>*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::complex<double> >*& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;

    /*! @brief パラメータの取得とチェックを行います。
     *  @param  param   I_paramクラスのshared_ptr
     *  @param  name    パラメータ名
     *  @param  value   パラメータ名から取得する値 (int型)
     *  @param  e_level チェックでエラーとなった場合のSEVERITY
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。
     *                    省略可(省略時はERROR_MSG)
     *  @return true 成功, false 失敗
     */
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, int& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, unsigned int& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, double& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::string& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::complex<double>& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<int>& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<unsigned int>& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<double>& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::string>& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::complex<double> >& value, ERROR_SEVERITY e_level = ERROR_MSG) = 0;

    /*! @brief 出力ファイルストリームを開きます。 
     *  @param  param   出力ファイルストリームのshared_ptr 
     *  @param  name    ファイル名 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                   省略可 (省略時はERROR_MSG) 
     */
    virtual void openAndCheck(boost::shared_ptr<std::ofstream> file,
                              const std::string&               name,
                              ERROR_SEVERITY                   e_level = ERROR_MSG) = 0;

    /*! @brief 使用禁止パラメータのチェックを行います。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 使用禁止パラメータの発見, false 使用禁止パラメータ発見なし 
     */
    virtual bool checkAndAbortParam(boost::shared_ptr<I_param> param,
                                    const std::string&         name, 
                                    ERROR_SEVERITY             e_level = ERROR_MSG) = 0;

    /*! @briefモードによっては有効なパラメータのアクセス履歴を残す。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @return true 成功, false 失敗 
     */
    virtual bool touchParam(boost::shared_ptr<I_param> param,
                            const std::string&         name) = 0;

    /*! @brief シナリオで参照する名前と内部変数をリンクする。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    シナリオで使用する名前 (文字列) 
     *  @param  *val    内部変数 (型は int型のポインタ限定) 
     *  @return true 設定成功, false 設定失敗 
     */
    virtual bool createMonitorSig (boost::shared_ptr<I_param> param,
                                   const std::string&         name,
                                   int*&                      val) = 0;

    /*! @brief パラメータファイルを読み込む。 
     *  @param  param        I_paramクラスのshared_ptr 
     *  @param  param_parser C_param_parserクラスのshared_ptr 
     *  @param  filename     パラメータファイル名 
     *  @param  errmsg       パラメータ解析時エラーメッセージ 
     *  @return true パラメータ読み込み成功, false パラメータ読み込み失敗 
     */
    virtual bool readParamFile(boost::shared_ptr<I_param>        param,
                               boost::shared_ptr<C_param_parser> param_parser,
                               std::vector<std::string>&         filename,
                               std::vector<std::string>&         errmsg) = 0;

    /*! @brief シナリオファイルを読み込む。 
     *  @param  param     I_paramクラスのshared_ptr 
     *  @param  sc_parser C_sc_parserクラスのshared_ptr 
     *  @param  filename  シナリオファイル名 
     *  @param  sc_table  vector<C_scenario>のshared_ptr (パース結果)  
     *  @param  errmsg    エラーメッセージ 
     *  @return true シナリオ読み込み成功, false シナリオ読み込み失敗 
     */
    virtual bool readScenarioFile(boost::shared_ptr<I_param>                  param,
                                  boost::shared_ptr<C_sc_parser>              sc_parser,
                                  std::string                                 filename,
                                  boost::shared_ptr<std::vector<C_scenario> > sc_table,
                                  std::vector<std::string>&                   errmsg) = 0;

    /*! @brief シナリオを実行する。 
     *  @param  scenario_table シナリオのvector 
     *  @param  time シミュレーション時間 
     *  @return true シナリオ正常実行, false シナリオ停止の通知 
     */
    virtual bool execScenario(boost::shared_ptr<std::vector<C_scenario> >   scenario_table,
                              boost::shared_ptr<C_SimTime>                  time) = 0;

    /*! @brief  時間を初期化する 
     *  @param  time  シミュレーション時間クラス 
     */
    virtual void clearTime(boost::shared_ptr<C_SimTime> time) = 0;

    /*! @brief  時刻を設定する 
     *  @param  time  シミュレーション時間クラス 
     *  @param  frame frame数 
     *  @param  tick  tick数 
     */
    virtual void setSimTime(boost::shared_ptr<C_SimTime> time, int frame, int tick) = 0;

    /*! @brief  1FRAME数を設定する 
     *  @param  time         シミュレーション時間クラス 
     *  @param  tickperframe 1frame辺りのtick数 
     */
    virtual void setTickPerFrame(boost::shared_ptr<C_SimTime> time, int tickperframe) = 0;

    /*! @brief  frame数を取得する 
     *  @param  time  シミュレーション時間クラス 
     *  @param  frame frame数 
     */
    virtual int getFrame(boost::shared_ptr<C_SimTime> time) = 0;

    /*! @brief  tick数を取得する 
     *  @param  time  シミュレーション時間クラス 
     *  @param  tick  tick数 
     */
    virtual int getTick(boost::shared_ptr<C_SimTime> time) = 0;

    /*! @brief  内部の時間を取得する 
     *  @param  time  シミュレーション時間クラス 
     *  @param  tick  tick数 
     */
    virtual double getDoubleTick(boost::shared_ptr<C_SimTime> time) = 0;

    /*! @brief  シミュレーション時間を比較する 
     *  @param  lhs  シミュレーション時間クラス(左辺) 
     *  @param  rhs  シミュレーション時間クラス(右辺) 
     */
    virtual bool timeLT(boost::shared_ptr<C_SimTime> lhs, boost::shared_ptr<C_SimTime> rhs) = 0;

    /*! @brief  時刻を進める。 
     *  @param  time シミュレーション時間 
     */
    virtual void nextSimTime(boost::shared_ptr<C_SimTime> time) = 0;

    /*! @brief 引数の処理を行う。 
     *  @param argc          プログラムの引数 
     *  @param argv[]        プログラムの引数 
     *  @param param_files   パラメータファイル 
     *  @param scenario_file シナリオファイル 
     *  @param errmsg        ファイルオープン時エラーメッセージ 
     *  @return true 取得成功, false 取得失敗 
     */
    virtual bool readArgs(int argc, char *argv[],
                          std::vector<std::string> &param_files,
                          std::string              &scenario_file,
                          std::vector<std::string> &errmsg) = 0;

    /*! @brief シナリオのトレース情報を出力 
     *  @param os             出力ファイルストリーム 
     *  @param scenario_table シナリオテーブル 
     */
    virtual void outputTraceInfo(std::ostream                                   &os, 
                                 boost::shared_ptr<std::vector<C_scenario> >    scenario_table) = 0;

    /*! @brief シナリオのエラー情報を出力 
     *  @param os             出力ファイルストリーム 
     *  @param scenario_table シナリオテーブル 
     */
    virtual void outputScenarioErr(std::ostream                                   &os, 
                                   boost::shared_ptr<std::vector<C_scenario> >    scenario_table) = 0;

    /*! @brief パラメータクラスの未使用パラメータをファイルに出力 
     *  @param os             出力ファイルストリーム 
     *  @param param          パラメータクラス 
     */
    virtual void outputUnusedParam(std::ostream               &os, 
                                   boost::shared_ptr<I_param> param) = 0;

private:
    /* private メンバ関数 */
};

/* @brief I_Util取得関数 */
boost::shared_ptr<I_Util> createI_Util();
/* @brief C_SimTime取得関数 */
boost::shared_ptr<C_SimTime> createC_SimTime();
boost::shared_ptr<C_SimTime> createC_SimTime(int frame, int tick);
boost::shared_ptr<C_SimTime> createC_SimTime(const boost::shared_ptr<C_SimTime> tim);
/* @brief C_param_parser取得関数 */
boost::shared_ptr<C_param_parser> createC_param_parser();
/* @brief C_sc_parser取得関数 */
boost::shared_ptr<C_sc_parser> createC_sc_parser();
/* @brief C_scenario取得関数 */
boost::shared_ptr<std::vector<C_scenario> > createC_scenario();

#endif
