/*!
    @file  I_param.h
    @brief パラメータクラスインタフェース 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef I_PARAM_H
#define I_PARAM_H

#include <vector>
#include <string>
#include <complex>
#include <boost/shared_ptr.hpp>
#include "C_param_wrap.h"

/*!
  パラメータクラスインタフェース 
  @brief  パラメータクラスのインターフェースクラス 
 */
class I_param : public C_param_wrap
{
public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    I_param() {}

    /*! @brief デコンストラクタ 
     */
    virtual ~I_param() {}

    /*! @brief 存在確認を行う 
     *  @param   var_name パラメータ名 
     *  @return  true: 存在, false: 存在しない 
     */
    virtual bool exist(const std::string &var_name) = 0;


    /*! @brief 参照カウンタ更新 
     *  @param   var_name パラメータ名 
     *  @return  true: 追加成功, false: 追加失敗 
     */
    virtual bool touchParam(const std::string &var_name) = 0;


    /*! @brief パラメータの追加 静的パラメータ用 
     *  @param var_name パラメータ名 
     *  @param val_str  パラメータの値の文字列 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual bool addParam(const std::string &var_name,const std::string &var_str) = 0;

    /*! @brief パラメータの追加 シミュレータ内部変数の追加用 
     *  @param var_name パラメータ名 
     *  @param val      int *型の値 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual bool addParam(const std::string &var_name, int *var) = 0;

    /*! @brief 値の取得 静的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual bool getValue(const std::string &var_name,int &val) = 0;
    virtual bool getValue(const std::string &var_name,unsigned int &val) = 0;
    virtual bool getValue(const std::string &var_name,double &val) = 0;
    virtual bool getValue(const std::string &var_name,std::complex<double> &val) = 0;
    virtual bool getValue(const std::string &var_name,std::string &val) = 0;
    virtual bool getValue(const std::string &var_name,std::vector<int> &val) = 0;
    virtual bool getValue(const std::string &var_name,std::vector<unsigned int> &val) = 0;
    virtual bool getValue(const std::string &var_name,std::vector<double> &val) = 0;
    virtual bool getValue(const std::string &var_name,std::vector<std::complex<double> > &val) = 0;
    virtual bool getValue(const std::string &var_name,std::vector<std::string> &val) = 0;

    /*! @brief パラメタの設定 動的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual bool setParam(const std::string &var_name,int* &val) = 0;
    virtual bool setParam(const std::string &var_name,unsigned int* &val) = 0;
    virtual bool setParam(const std::string &var_name,double* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::complex<double>* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::string* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::vector<int>* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::vector<unsigned int>* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::vector<double>* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::vector<std::complex<double> >* &val) = 0;
    virtual bool setParam(const std::string &var_name,std::vector<std::string>* &val) = 0;

    /*! @brief 使用されなかったパラメータの取得 
     *  @return  使用されなかったパラメータ 
     */
    virtual std::vector<std::string> getUnusedParam() = 0;

    /*! @brief  静的パラメータかどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素が静的パラメータ, false: 静的パラメータではないorパラメータが存在しない 
     */
    virtual bool isStatic(const std::string &var_name) = 0;

    /*! @brief  動的パラメータかどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素が動的パラメータ, false: 動的パラメータではないorパラメータが存在しない 
     */
    virtual bool isDynamic(const std::string &var_name) = 0;

    /*! @brief  シミュレータ内部変数かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素がシミュレータ内部変数, false: シミュレータ内部変数ではないorパラメータが存在しない 
     */
    virtual bool isMonitor(const std::string &var_name) = 0;

    /*! @brief  int型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: int型, false: int型でないorパラメータが存在しない 
     */
    virtual bool isInt(const std::string &var_name) = 0;
    
    /*! @brief  unsigned int型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: unsigned int型, false: unsigned int型でないorパラメータが存在しない 
     */
    virtual bool isUnsignedInt(const std::string &var_name) = 0;

    /*! @brief  double型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: double型, false: double型でないorパラメータが存在しない 
     */
    virtual bool isDouble(const std::string &var_name) = 0;

    /*! @brief  string型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: string型, false: string型でないorパラメータが存在しない 
     */
    virtual bool isString(const std::string &var_name) = 0;

    /*! @brief  Complex型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: Complex型, false: Complex型でないorパラメータが存在しない 
     */
    virtual bool isComplex(const std::string &var_name) = 0;

    /*! @brief  vector<int>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<int>型, false: vector<int>型でないorパラメータが存在しない 
     */
    virtual bool isIntVector(const std::string &var_name) = 0;
    
    /*! @brief  vector<unsigned int>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<unsigned int>型, false: vector<unsigned int>型でないorパラメータが存在しない 
     */
    virtual bool isUnsignedIntVector(const std::string &var_name) = 0;

    /*! @brief  vector<double>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<double>型, false: vector<double>型でないorパラメータが存在しない 
     */
    virtual bool isDoubleVector(const std::string &var_name) = 0;

    /*! @brief  vector<string>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<string>型, false: vector<string>型でないorパラメータが存在しない 
     */
    virtual bool isStringVector(const std::string &var_name) = 0;

    /*! @brief  vector<complex>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<complex>型, false: vector<complex>型でないorパラメータが存在しない 
     */
    virtual bool isComplexVector(const std::string &var_name) = 0;

    /*! @brief  エラーメッセージを取得する 
     *  @return エラーメッセージ 
     */
    virtual std::vector<std::string> getErrMsg() = 0;

    /*! @brief  ワーニングメッセージを取得する 
     *  @return ワーニングメッセージ 
     */
    virtual std::vector<std::string> getWarnMsg() = 0;

    /*! @brief  インフォメーションメッセージを取得する 
     *  @return インフォメーションメッセージ 
     */
    virtual std::vector<std::string> getInfoMsg() = 0;

    /*! @brief  メッセージをクリアする 
     */
    virtual void clearMsg() = 0;

    /*! @brief  dumpをする 
     *  @param  os 出力ストリーム 
     */
    virtual void dump(std::ostream& os) = 0;

private:
    /* private メンバ関数 */
};

/* @brief インタフェース取得関数 */
boost::shared_ptr<I_param> createI_param();

#endif
