/*!
    @file  Util_wrap.h
    @brief ユーティリティクラス 

    @date     $Date:: 2016-01-29 16:40:19 #$
    @revision $Revision: 8 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef UTIL_WRAP_H
#define UTIL_WRAP_H

#include <string>
#include <iostream>
#include <fstream>
#include <boost/shared_ptr.hpp>

#include "C_param_wrap.h"
#include "C_ErrMng.h"
#include "C_OneBlockBase.h"

/*!
  @brief  ユーティリティクラスのインターフェースクラス(gen2 green互換用) 
 */
class Util_wrap {
public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    Util_wrap() { ; }
    /*! @brief デストラクタ 
     */
    virtual ~Util_wrap() { ; }
  
    /*! @brief  ベンダー名を返す。 
     *  @return ベンダー名 
     */
    virtual const std::string getVender() = 0 ;

    /*! @brief  時間の文字列を作成する。 
     *  @return 時間の文字列 
     */
    virtual const std::string getMakeTime() = 0 ;
  
    /*! @brief パスからファイル名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ファイル名 
     */
    virtual const std::string getFileName(const std::string& path) = 0 ;
  
    /*! @brief パスからディレクトリ名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ディレクトリ名 
     */
    virtual const std::string getDirName(const std::string& path) = 0 ;
  
    /*! @brief プログラムの実行時間を出力します。 
     *         特定期間の実行時間を計測したい場合は、計測を開始したい 
     *         ポイントで別途本クラスをインスタンスして下さい。 
     *  @param os     出力stream（標準出力に出力したい場合は、省略可です） 
     *  @param prefix 出力する際のprefixを指定してください。 
     *                省略可です（省略時には"SIM"を指定した事になります）。 
     *                ["XXX"を渡した場合の出力例] 
     *                  XXX START : Mon Jul  6 14:28:27 2009 
     *                  XXX END   : Mon Jul  6 14:30:24 2009 
     *                  XXX TOTAL : 116 sec. (CPU TIME : 100.00) 
     */
    virtual void outputProcessingTime(std::ostream& os, const std::string& prefix = std::string("SIM")) = 0 ;

    /*! @brief パラメータの設定とチェックを行います。 
     *  @param  param   C_param_wrapクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータに設定する値 
     *  @param  type    パラメータに設定する値の種別 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     */
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, int** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, unsigned int** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, double** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::string** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::complex<double>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<int>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<unsigned int>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<double>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::string>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::complex<double> >** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;

    /*! @brief パラメータの取得とチェックを行います。 
     *  @param  param   C_param_wrapクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータ名から取得する値 
     *  @param  type    取得する値の種別 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     */
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, int* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, unsigned int* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, double* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::string* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::complex<double>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<int>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<unsigned int>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<double>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::string>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;
    virtual bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::complex<double> >* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;

    /*! @brief 出力ファイルストリームを開きます。 
     *  @param  param   出力ファイルストリームのshared_ptr 
     *  @param  name    ファイル名 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                   省略可 (省略時はERROR_MSG) 
     */
    virtual void openAndCheck(boost::shared_ptr<std::ofstream> file, const std::string& name, ERROR_SEVERITY e_level = ERROR_MSG) = 0 ;

#ifndef rel_NTT
#ifdef FW_CMD_MODE
    /*! @brief 引数の処理を行う。（未サポート） 
     *  @param argc          プログラムの引数 
     *  @param argv[]        プログラムの引数 
     *  @param param_files   パラメータファイル 
     *  @param scenario_file シナリオファイル 
     *  @param errmsg        ファイルオープン時エラーメッセージ 
     *  @return true 取得成功, false 取得失敗 
     */
    virtual void readArgs(int argc, char* argv[], boost::shared_ptr<C_param_wrap> param, boost::shared_ptr<C_scenario_wrap> sc, int& do_scenario, std::string& pfiles) = 0 ;

    /*
     * @brief 文字を分割する関数（未サポート） 
     * @param str   文字列 
     * @param delim 区切り文字 
     * @return 分割された文字列 
     */
    virtual std::vector<std::string > str_split(std::string str, std::string delim) = 0 ;
#endif
#endif
} ;

/*! @brief Util_wrapクラスをインスタンスします 
 *  @return Util_wrapクラスのshared_ptr 
 */
boost::shared_ptr<Util_wrap> createUtil() ;

#endif
