/*!
    @file  C_SimTime.h
    @brief SIM時間クラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SIMTIME_H
#define C_SIMTIME_H

#include <stdio.h>
#include <stdlib.h>

/*---------------------------------------------------------------------------
**  1.日本語名 
**    時間クラス 
** 
**  2.概要 
**    シミュレーション時間 
** 
**  3.機能説明 
**    シミュレーション時間を扱うクラス 
**    32bitのintの範囲を超える値も扱える 
** 
**  4.備考 
**---------------------------------------------------------------------------*/
class C_SimTime {
public:
  C_SimTime();
  C_SimTime( int frame, int tick);
  C_SimTime( const C_SimTime& tim);
  ~C_SimTime();

  /*********************************************************************
   代入演算子 
  **********************************************************************/
  C_SimTime& operator=( const C_SimTime& tim) {
    m_frame = tim.m_frame;
    m_tick = tim.m_tick;
    return *this;
  }

  /*********************************************************************
   代入（TICK） 
  **********************************************************************/
  C_SimTime& operator=( int tick) {
    m_frame = 0;
    m_tick = tick;
    normalize();
    return *this;
  }
  
  /*********************************************************************
   加算 
  **********************************************************************/
  C_SimTime operator+( C_SimTime& tim) {
    C_SimTime tmp_time;
    tmp_time.m_frame = m_frame + tim.m_frame;
    tmp_time.m_tick  = m_tick + tim.m_tick;
    tmp_time.normalize();
    return tmp_time;
  }
  
  /*********************************************************************
   減算 
  **********************************************************************/
  C_SimTime operator-( C_SimTime& tim) {
    C_SimTime tmp_time;
    tmp_time.m_frame = m_frame - tim.m_frame;
    tmp_time.m_tick  = m_tick - tim.m_tick;
    tmp_time.normalize();
    return tmp_time;
  }
  
  /*********************************************************************
   加算代入 
  **********************************************************************/
  C_SimTime& operator+=( C_SimTime& tim) {
    m_frame += tim.m_frame;
    m_tick += tim.m_tick;
    normalize();
    return *this;
  }
  
  /*********************************************************************
   乗算 （整数倍） 
  **********************************************************************/
  C_SimTime operator*( int mul) {
    C_SimTime tmp_time;
    tmp_time.m_frame = m_frame * mul;
    tmp_time.m_tick  = m_tick * mul;
    tmp_time.normalize();
    return tmp_time;
  }
  
  /*********************************************************************
   整数加算（TICKに加算） 
  **********************************************************************/
  C_SimTime operator+( int tick) {
    C_SimTime tmp_time;
    tmp_time.m_tick  = m_tick + tick;
    tmp_time.normalize();
    return tmp_time;
  }
  
  /*********************************************************************
   整数加算代入 
  **********************************************************************/
  C_SimTime& operator+=( int tick) {
    m_tick += tick;
    normalize();
    return *this;
  }
  
  /*********************************************************************
   乗算代入 （整数倍） 
  **********************************************************************/
  C_SimTime& operator*=( int mul) {
    m_frame *= mul;
    m_tick *= mul;
    normalize();
    return *this;
  }
  
  /*********************************************************************
   インクリメント（前置） 
  **********************************************************************/
  C_SimTime operator++() {
    m_tick++;
    normalize();
    return *this;
  }
  
  /*********************************************************************
   インクリメント（後置） 
  **********************************************************************/
  C_SimTime operator++(int) {
    C_SimTime tmp_time(*this);
    m_tick++;
    normalize();
    return tmp_time;
  }
  
  /*********************************************************************
   比較(==) 
  **********************************************************************/
  bool operator==( C_SimTime& tmp) {
    bool ret;
    if ((m_tick == tmp.m_tick) && (m_frame == tmp.m_frame)) {
      ret = true;
    } else {
      ret = false;
    }
    return ret;
  }
  
  /*********************************************************************
   比較(<) 
  **********************************************************************/
  bool operator<(C_SimTime& tmp) {
    bool ret;
    if (m_frame < tmp.m_frame) {
      ret = true;
    } else if ((m_frame == tmp.m_frame) && (m_tick < tmp.m_tick)) {
      ret = true;
    } else {
      ret = false;
    }
    return ret;
  }
  
  /*********************************************************************
   比較(<=) 
  **********************************************************************/
  bool operator<=( C_SimTime& tim) {
    bool ret;
    if (m_frame < tim.m_frame) {
      ret = true;
    } else if ((m_frame == tim.m_frame) && (m_tick <= tim.m_tick)) {
      ret = true;
    } else {
      ret = false;
    }
    return ret;
  }
  
  /*********************************************************************
   比較(>) 
  **********************************************************************/
  bool operator>( C_SimTime& tim) { return !(*this <= tim);}
  
  /*********************************************************************
   比較(>=) 
  **********************************************************************/
  bool operator>=( C_SimTime& tim) { return !(*this < tim);}
  
  /*********************************************************************
   比較(!=) 
  **********************************************************************/
  bool operator!=( C_SimTime& tim) { return !(*this == tim);}
  
  /*********************************************************************
   整数加算（TICKに加算：正規化なし） 
  **********************************************************************/
  C_SimTime& addTick( int tick) {
    m_tick += tick;
    return *this;
  }

  /*********************************************************************
   整数加算（FRAMEに加算） 
  **********************************************************************/
  C_SimTime& addFrame( int frame) {
    m_frame += frame;
    return *this;
  }
  
  /*********************************************************************
   クリア 
  **********************************************************************/
  void clear() {
    m_frame = 0;
    m_tick = 0;
  }

  /*********************************************************************
   整数のペアで代入 
  **********************************************************************/
  void set(int frame, int tick) {
    m_frame = frame;
    m_tick = tick;
  }
  
  /*********************************************************************
   正規化 
  **********************************************************************/
  void normalize( void) {
    if (m_tick >= C_SIMTIME_TICK_PER_FRAME) {
      m_frame += m_tick / C_SIMTIME_TICK_PER_FRAME;
      m_tick = m_tick % C_SIMTIME_TICK_PER_FRAME;
    } else if (m_tick < 0) {
      m_frame -= m_tick / C_SIMTIME_TICK_PER_FRAME + 1;
      m_tick = m_tick % C_SIMTIME_TICK_PER_FRAME + C_SIMTIME_TICK_PER_FRAME;
    } else {
      /* 0 <= m_tick < C_SIMTIME_TICK_PER_FRAME の時は何もしない */
    }
  }

  void set_tickperframe( int tpf) {
    C_SIMTIME_TICK_PER_FRAME = tpf;
  }

  /* 内部データの取得 */
  int getTick( void) { return m_tick; };
  int getFrame( void) { return m_frame; };
  double getDoubleTick( void) {
    return (double)C_SIMTIME_TICK_PER_FRAME * (double)m_frame + (double) m_tick;
  }

private:
  int m_frame;            /*!< フレーム番号 */
  int m_tick;             /*!< フレーム内のTICKカウンタ */
  static int C_SIMTIME_TICK_PER_FRAME;
};


#endif /* C_SIMTIME_H */
