/*!
    @file  C_Util.h
    @brief ユーティリティクラス

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include <fstream>
#include <sstream>
#include <boost/tokenizer.hpp>
#include <boost/algorithm/string/trim.hpp>

#include "C_Util.h"
#include "C_sc_parser.h"
#include "C_err_msg.h"
#include "C_param_parser.h"
#include "C_SimTime.h"

/*! @brief  ベンダー名を返す。 
 *  @return ベンダー名 
 */
const std::string C_Util::getVender(){
    return std::string("FUJITSU");
}

/*! @brief  時間の文字列を作成する。 
 *  @return 時間の文字列 
 */
const std::string C_Util::getMakeTime(){
    return std::string(__DATE__) + std::string(" ") + std::string(__TIME__);
}

/*! @brief パスからファイル名を取り出します。 
 *  @param path  パス(文字列) 
 *  @return      ファイル名 
 */
const std::string C_Util::getFileName(const std::string& path){
    std::string::size_type  rfind;
    std::string             ret_str;

    /* セパレータを後ろから検索 */
    rfind = path.find_last_of("\\/");
    
    if ( rfind != std::string::npos ){
        /* セパレータがある場合 */
        ret_str.assign(path.substr(rfind+1));
    }
    else{
        /* セパレータがない場合 */
        ret_str = path;
    }

    return ret_str;
}

/*! @brief パスからディレクトリ名を取り出します。 
 *  @param path  パス(文字列) 
 *  @return      ディレクトリ名 
 */
const std::string C_Util::getDirName(const std::string& path){
    std::string::size_type  rfind;
    std::string             ret_str;

    /* セパレータを後ろから検索 */
    rfind = path.find_last_of("\\/");
    
    if ( rfind != std::string::npos ){
        /* セパレータがある場合 */
        ret_str.assign(path.substr(0, rfind+1));
    }
    else{
        /* セパレータがない場合 */
    }

    return ret_str;
}

/*! @brief プログラムの実行時間を出力します。 
 *         特定期間の実行時間を計測したい場合は、計測を開始したい 
 *         ポイントで別途本クラスをインスタンスして下さい。 
 *  @param os     出力stream（標準出力に出力したい場合は、省略可です） 
 *  @param prefix 出力する際のprefixを指定してください。 
 *                省略可です（省略時には"SIM"を指定した事になります）。 
 *                ["XXX"を渡した場合の出力例] 
 *                  XXX START : Mon Jul  6 14:28:27 2009 
 *                  XXX END   : Mon Jul  6 14:30:24 2009 
 *                  XXX TOTAL : 116 sec. (CPU TIME : 100.00) 
 */
void C_Util::outputProcessingTime(std::ostream& os, const std::string& prefix){
    time_t  current_time     = time(NULL);
    clock_t current_cpu_time = clock();
    os << std::endl;
    os << prefix << " START: " << asctime(localtime(&m_start_real_time));
    os << prefix << " END  : " << asctime(localtime(&current_time));
    os << prefix << " TOTAL: " << current_time - m_start_real_time
       << " sec. (CPU TIME : " << (current_cpu_time - m_start_cpu_time)/CLOCKS_PER_SEC  << " sec.)" << std::endl << std::endl ;
}

/*! @brief パラメータの設定とチェックを行います。 
 *  @param  param   I_paramクラスのshared_ptr 
 *  @param  name    パラメータ名 
 *  @param  value   パラメータに設定する値 (int型,double型等) 
 *  @param  e_level チェックでエラーとなった場合のSEVERITY 
 *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
 *                    省略可(省略時はERROR_MSG) 
 *  @return true 成功, false 失敗 
 */
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              int*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              unsigned int*& value,
                              ERROR_SEVERITY             e_level){
    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              double*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::string*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param, 
                              const std::string& name,
                              std::complex<double>*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<int>*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<unsigned int>*& value,
                              ERROR_SEVERITY             e_level){
    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<double>*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<std::string>*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndSetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<std::complex<double> >*& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->setParam(name, value) ){
        /* パラメータ設定成功 */
        return true;
    }
    else{
        /* パラメータ設定失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}

/*! @brief パラメータの取得とチェックを行います。 
 *  @param  param   I_paramクラスのshared_ptr 
 *  @param  name    パラメータ名 
 *  @param  value   パラメータ名から取得する値 (int型,double型等) 
 *  @param  e_level チェックでエラーとなった場合のSEVERITY 
 *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
 *                    省略可(省略時はERROR_MSG) 
 *  @return true 成功, false 失敗 
 */
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              int& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              unsigned int& value,
                              ERROR_SEVERITY             e_level){
    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              double& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::string& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::complex<double>& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<int>& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<unsigned int>& value,
                              ERROR_SEVERITY             e_level){
    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<double>& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<std::string>& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}
bool C_Util::checkAndGetParam(const boost::shared_ptr<I_param>& param,
                              const std::string& name,
                              std::vector<std::complex<double> >& value,
                              ERROR_SEVERITY             e_level){
    /* パラメータクラスのメッセージをクリアしておく */
    param->clearMsg();

    if ( param->getValue(name, value) ){
        /* パラメータ取得成功 */
        return true;
    }
    else{
        /* パラメータ取得失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_PARAM_ACQUIRED, 
                   C_err_msg::UTIL_WARN_PARAM_ACQUIRED,
                   C_err_msg::UTIL_INFO_PARAM_ACQUIRED,
                   name).c_str(),
            e_level);

        /* パラメータクラスからのメッセージを追加 */
        addParamMsg(param,e_level);

        return false;
    }
}

/*! @brief 出力ファイルストリームを開きます。 
 *  @param  param   出力ファイルストリームのshared_ptr 
 *  @param  name    ファイル名 
 *  @param  e_level チェックでエラーとなった場合のSEVERITY 
 *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
 *                   省略可 (省略時はERROR_MSG) 
 */
void C_Util::openAndCheck(boost::shared_ptr<std::ofstream> file,
                          const std::string&               name,
                          ERROR_SEVERITY                   e_level){
    /* ファイルオープン */
    file->open(name.c_str());

    if (!file->is_open()) {
        /* オープン失敗 */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_OUTPUTFILE_OPEN, 
                   C_err_msg::UTIL_WARN_OUTPUTFILE_OPEN,
                   C_err_msg::UTIL_INFO_OUTPUTFILE_OPEN,
                   name).c_str(),
            e_level);
    }
}

/*! @brief 使用禁止パラメータのチェックを行います。 
 *  @param  param   I_paramクラスのshared_ptr 
 *  @param  name    パラメータ名 
 *  @param  e_level チェックでエラーとなった場合のSEVERITY 
 *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
 *                    省略可(省略時はERROR_MSG) 
 *  @return true 使用禁止パラメータの発見, false 使用禁止パラメータ発見なし 
 */
bool C_Util::checkAndAbortParam(boost::shared_ptr<I_param> param,
                                const std::string&         name, 
                                ERROR_SEVERITY             e_level){
    if ( param->exist(name) ){
        /* パラメータが存在する */
        g_err->appendMsg(
            getMsg(e_level,
                   C_err_msg::UTIL_ERR_ABORT_PARAM, 
                   C_err_msg::UTIL_WARN_ABORT_PARAM,
                   C_err_msg::UTIL_INFO_ABORT_PARAM,
                   name).c_str(),
            e_level);
        return true;
    }
    else{
        /* パラメータは存在しない */
        return false;
    }
}

/*! @brief モードによっては有効なパラメータのアクセス履歴を残す。 
 *  @param  param   I_paramクラスのshared_ptr 
 *  @param  name    パラメータ名 
 *  @return true 成功, false 失敗 
 */
bool C_Util::touchParam(boost::shared_ptr<I_param> param,
                        const std::string&         name){
    /* I_param::touchParamを呼び出す */
    return param->touchParam(name);
}

/*! @brief シナリオで参照する名前と内部変数をリンクする。 
 *  @param  param   I_paramクラスのshared_ptr 
 *  @param  name    シナリオで使用する名前 (文字列) 
 *  @param  *val    内部変数 (型は int型のポインタ限定) 
 *  @return true 設定成功, false 設定失敗 
 */
bool C_Util::createMonitorSig (boost::shared_ptr<I_param> param,
                               const std::string&         name,
                               int*&                      val){
    /* パラメータの存在をチェック */
    if ( !param->exist(name) ){
        /* パラメータが存在しない場合 */
        /* パラメータにint型ポインタをセット */
        return param->addParam(name, val);
    }
    else{
        /* パラメータが存在する場合 */
        return false;
    }
}

/*! @brief パラメータファイルを読み込む。 
 *  @param  param        I_paramクラスのshared_ptr 
 *  @param  param_parser C_param_parserクラスのshared_ptr 
 *  @param  filename     パラメータファイル名 
 *  @param  errmsg       パラメータ解析時エラーメッセージ 
 *  @return true パラメータ読み込み成功, false パラメータ読み込み失敗 
 */
bool C_Util::readParamFile(boost::shared_ptr<I_param>        param,
                           boost::shared_ptr<C_param_parser> param_parser,
                           std::vector<std::string>&         filename,
                           std::vector<std::string>&         errmsg){
    bool result = true;

    std::vector<std::string>::iterator ite = filename.begin();

    errmsg.clear();
    for (; ite != filename.end(); ++ite) {
        /* parserクラスにファイル名とパラメータクラスを設定 */
        param_parser->setData(*ite, param);
        /* パラメータファイル解析+読み込み結果出力 */
        result &= param_parser->parse();
        /* エラーメッセージの取得 */
/*      errmsg = param_parser->getErrMsg();
*/
        const std::vector<std::string>& geterrmsg = param_parser->getErrMsg();
        errmsg.insert(errmsg.end(), geterrmsg.begin(), geterrmsg.end());
    }

    return result;
}

/*! @brief シナリオファイルを読み込む。 
 *  @param  param     I_paramクラスのshared_ptr 
 *  @param  sc_parser C_sc_parserクラスのshared_ptr 
 *  @param  filename  シナリオファイル名 
 *  @param  sc_table  vector<C_scenario>のshared_ptr (パース結果)  
 *  @param  errmsg    エラーメッセージ 
 *  @return true シナリオ読み込み成功, false シナリオ読み込み失敗 
 */
bool C_Util::readScenarioFile(boost::shared_ptr<I_param>                  param,
                              boost::shared_ptr<C_sc_parser>              sc_parser,
                              std::string                                 filename,
                              boost::shared_ptr<std::vector<C_scenario> > sc_table,
                              std::vector<std::string>&                   errmsg){

    /* parserクラスにファイル名とパラメータクラスを設定 */
    sc_parser->setData(filename, param);

    /* シナリオファイル解析+読み込み結果出力 */
    bool result = sc_parser->parse();

    /* シナリオテーブルの取得 */
    if (sc_table == NULL ){
        result = false;
    }
    else if (result){
        *sc_table = sc_parser->getResult();
    }

    /* エラーメッセージの取得 */
    errmsg = sc_parser->getErrMsg();

    return result;
}

/*! @brief シナリオを実行する。 
 *  @param  scenario_table シナリオのvector 
 *  @param  time シミュレーション時間 
 *  @return true シナリオ正常実行, false シナリオ停止の通知 
 */
bool C_Util::execScenario(boost::shared_ptr<std::vector<C_scenario> >   scenario_table,
                          boost::shared_ptr<C_SimTime>                  time){

    /* シナリオテーブルが空の時はシナリオ停止を通知 */
    if( scenario_table == NULL)         return false;
    else if( scenario_table->empty())   return false;
    
    std::vector<C_scenario>::iterator it = scenario_table->begin();
    
    /* シナリオをすべて実行 */
    while( it != scenario_table->end() ){
        /* 各シナリオの実行 */
        if ( !it->execute(time) ){
            /* シナリオ停止の通知がある場合 */
            return false;
        }
        ++it;
    }
    /* シナリオ正常実行 */
    return true;
}

/*! @brief  時間を初期化する 
 *  @param  time  シミュレーション時間クラス 
 */
void C_Util::clearTime(boost::shared_ptr<C_SimTime> time){
    time->clear();
}

/*! @brief  時刻を設定する 
 *  @param  time  シミュレーション時間クラス 
 *  @param  frame frame数 
 *  @param  tick  tick数 
 */
void C_Util::setSimTime(boost::shared_ptr<C_SimTime> time, int frame, int tick){
    time->set(frame, tick);
}

/*! @brief  1FRAME数を設定する 
 *  @param  time         シミュレーション時間クラス 
 *  @param  tickperframe 1frame辺りのtick数 
 */
void C_Util::setTickPerFrame(boost::shared_ptr<C_SimTime> time, int tickperframe){
    time->set_tickperframe(tickperframe);
}

/*! @brief  frame数を取得する 
 *  @param  time  シミュレーション時間クラス 
 */
int C_Util::getFrame(boost::shared_ptr<C_SimTime> time){
    return time->getFrame();
}

/*! @brief  tick数を取得する 
 *  @param  time  シミュレーション時間クラス 
 */
int C_Util::getTick(boost::shared_ptr<C_SimTime> time){
    return time->getTick();
}

/*! @brief  内部の時間を取得する 
 *  @param  time  シミュレーション時間クラス 
 */
double C_Util::getDoubleTick(boost::shared_ptr<C_SimTime> time){
    return time->getDoubleTick();
}

/*! @brief  シミュレーション時間の比較をする 
 *  @param  lhs  シミュレーション時間クラス 
 *  @param  rhs  シミュレーション時間クラス 
 */
bool C_Util::timeLT(boost::shared_ptr<C_SimTime> lhs, boost::shared_ptr<C_SimTime> rhs){
    return (*lhs < *rhs);
}

/*! @brief  時刻を進める。 
 *  @param  time シミュレーション時間 
 */
void C_Util::nextSimTime(boost::shared_ptr<C_SimTime> time){
    /* インクリメント */
    ++(*time);
}

/*! @brief 引数の処理を行う。 
 *  @param argc          プログラムの引数 
 *  @param argv[]        プログラムの引数 
 *  @param param_files   パラメータファイル 
 *  @param scenario_file シナリオファイル 
 *  @param errmsg        ファイルオープン時エラーメッセージ 
 *  @return true 取得成功, false 取得失敗 
 */
bool C_Util::readArgs(int argc, char *argv[],
                      std::vector<std::string> &param_files,
                      std::string              &scenario_file,
                      std::vector<std::string> &errmsg){
    bool result = true;
    errmsg.clear();
    /* パラメータ・シナリオファイル判別ワード */
    const std::string   param_check_code    = "<PARAM_FILE>";
    const std::string   scenario_check_code = "<SCENARIO_FILE>";
    std::ifstream       ifs;
    std::string         linebuf;

    /* 引数がない場合 */
    if( argc == 1 ){
        return false;
    }
    /* 引数がある場合 */
    for(int i=1; i<argc; ++i){
        /* ファイルオープン */
        ifs.open( argv[i] );

        if( ifs.is_open() ){
            /* ファイルオープン可能 */
            bool exist_header = false;

            /* 1行ずつ読み込む */
            while( std::getline(ifs, linebuf) ){

                /* コメント削除 */
                linebuf = linebuf.substr(0, linebuf.find("//"));

                /* 文字列前後の空白を削除 */
                boost::algorithm::trim(linebuf);
                
                /* 文字がある場合 */
                if( linebuf.compare(param_check_code) == 0){
                    /* パラメータファイルの場合 */
                    param_files.push_back( argv[i] );
                    exist_header = true;
                    break;
                }
                else if( linebuf.compare(scenario_check_code) == 0){
                    /* シナリオファイルの場合 */
                    scenario_file = argv[i];
                    exist_header = true;
                    break;
                }
                else{
                    /* ヘッダ(<PARAM_FILE>、<SCENARIO_FILE>)が先頭にあることが仕様なので 
                    ** 先頭にヘッダ、コメント、半角スペース、空行以外の文字列がある場合はエラー 
                    */
                    if(!linebuf.empty()) { break; }
                }
            }
            
            if(!exist_header) {
                /* 識別不可 */
                std::string msginfo = getMsg(ERROR_MSG,
                                                C_err_msg::PARSER_BASE_ERR_NO_EXIST_HEADER,
                                                C_err_msg::UTIL_WARN_ABORT_PARAM,
                                                C_err_msg::UTIL_INFO_ABORT_PARAM, "");
                errmsg.push_back(std::string(argv[i]) + ":" + msginfo);
                result = false;
                break;
            }

            /* ファイルを閉じる */
            ifs.close();
        }
        else{
            /* ファイルオープン不可 */
            std::string msginfo = getMsg(ERROR_MSG,
                                         C_err_msg::PARSER_BASE_ERR_OPEN_FILE,
                                         C_err_msg::UTIL_WARN_ABORT_PARAM,
                                         C_err_msg::UTIL_INFO_ABORT_PARAM, "");
            errmsg.push_back(std::string(argv[i]) + ":" + msginfo);
            result = false;
        }
    }
    return result;
}

/*! @brief シナリオのトレース情報をファイルに出力 
 *  @param os             出力ファイルストリーム 
 *  @param scenario_table シナリオテーブル 
 */
void C_Util::outputTraceInfo(std::ostream                                   &os, 
                             boost::shared_ptr<std::vector<C_scenario> >    scenario_table){
    if ( os == NULL ) return;

    std::vector<C_scenario>::iterator next  = scenario_table->begin();
    std::vector<C_scenario>::iterator end   = scenario_table->end();
    
    for(; next != end; ++next){
        /* シナリオ毎にトレース情報を取得 */
        std::vector<std::string> trace = next->getTraceInfo();

        /* トレース情報を書き出し */
        for(std::vector<std::string>::iterator it = trace.begin(); it != trace.end(); ++it){
            os << *it << std::endl;
        }
    }
}

/*! @brief シナリオのエラー情報を出力 
 *  @param os             出力ファイルストリーム 
 *  @param scenario_table シナリオテーブル 
 */
void C_Util::outputScenarioErr(std::ostream                                   &os, 
                               boost::shared_ptr<std::vector<C_scenario> >    scenario_table){
    if ( os == NULL ) return;

    std::vector<C_scenario>::iterator next  = scenario_table->begin();
    std::vector<C_scenario>::iterator end   = scenario_table->end();

    for(; next != end; ++next){
        /* シナリオ毎にエラー情報を取得 */
        std::vector<std::string> errmsg = next->getMsg();

        /* トレース情報を書き出し */
        for(std::vector<std::string>::iterator it = errmsg.begin(); it != errmsg.end(); ++it){
            os << *it << std::endl;
        }
    }
}

/*! @brief パラメータクラスの未使用パラメータをファイルに出力 
 *  @param os             出力ファイルストリーム 
 *  @param param          パラメータクラス 
 */
void C_Util::outputUnusedParam(std::ostream               &os, 
                               boost::shared_ptr<I_param> param){
    if ( os == NULL ) return;
    /* 未使用パラメータを取得 */
    std::vector<std::string> unuse = param->getUnusedParam();
    std::vector<std::string>::iterator next = unuse.begin();
    std::vector<std::string>::iterator end  = unuse.end();

    os << "------------------------------" << std::endl;
    os << "Unused parameter list         " << std::endl;
    os << "------------------------------" << std::endl;
    /* 未使用パラメータを書き出し */
    for(; next != end; ++next){
        os << *next << std::endl;
    }
    os << "------------------------------" << std::endl;
}

/*! @brief エラー、ワーニング、インフォメッセージの取得 
 *  @param e_level   エラー時のSEVERITY 
 *  @param err_code  エラー時のメッセージコード 
 *  @param warn_code ワーニング時のメッセージコード 
 *  @param info_code インフォ時のメッセージコード 
 *  @param chgstr    メッセージ内で使用する置換用文字列 
 *  @return エラー、ワーニング、インフォメッセージ 
 */
std::string C_Util::getMsg( ERROR_SEVERITY e_level,
                            C_err_msg::E_err_type  err_code,
                            C_err_msg::E_warn_type warn_code,
                            C_err_msg::E_info_type info_code,
                            std::string chgstr){
    C_err_msg   msg;
    std::string msgbuf;

    /* メッセージの選択と取得 */
    switch( e_level ){
        case ERROR_MSG :
            msgbuf = msg.getErrMsg( err_code );
            break;
        case WARNING_MSG :
            msgbuf = msg.getWarnMsg( warn_code );
            break;
        case INFO_MSG :
            msgbuf = msg.getInfoMsg( info_code );
            break;
        default:
            break;
    }

    /* 置換文字列があれば置換して返す */
    return msg.changeString(msgbuf, chgstr);
}

/*! @brief エラー、ワーニング、インフォメッセージの取得 
 *  @param param   パラメータクラス 
 *  @param e_level エラー時のSEVERITY 
 *  @return エラー、ワーニング、インフォメッセージ 
 */
void C_Util::addParamMsg( boost::shared_ptr<I_param> param,
                          ERROR_SEVERITY e_level) {

    std::vector<std::string> msgs;
    
    /* メッセージの選択と取得 */
    switch( e_level ){
        case ERROR_MSG :
            msgs = param->getErrMsg();
            break;
        case WARNING_MSG :
            msgs = param->getWarnMsg();
            break;
        case INFO_MSG :
            msgs = param->getInfoMsg();
            break;
        default:
            break;
    }
    param->clearMsg();

    std::vector<std::string>::iterator ite = msgs.begin();
    for (;ite != msgs.end(); ++ite) {
        g_err->appendMsg(*ite,e_level);
    }
}

/* @brief I_Util取得関数 */
boost::shared_ptr<I_Util> createI_Util(){
    return boost::shared_ptr<I_Util>(new C_Util());
}
/* @brief Util_wrap取得関数(green互換用) */
boost::shared_ptr<Util_wrap> createUtil()
{
    return boost::shared_ptr<Util_wrap>(new C_Util());
}

/* @brief C_SimTime取得関数 */
boost::shared_ptr<C_SimTime> createC_SimTime(){
    return boost::shared_ptr<C_SimTime>(new C_SimTime());
}
boost::shared_ptr<C_SimTime> createC_SimTime(int frame, int tick){
    return boost::shared_ptr<C_SimTime>(new C_SimTime(frame, tick));
}
boost::shared_ptr<C_SimTime> createC_SimTime(const boost::shared_ptr<C_SimTime> tim){
    return boost::shared_ptr<C_SimTime>(new C_SimTime(*tim));
}

/* @brief C_param_parser取得関数 */
boost::shared_ptr<C_param_parser> createC_param_parser(){
    return boost::shared_ptr<C_param_parser>(new C_param_parser());
}

/* @brief C_sc_parser取得関数 */
boost::shared_ptr<C_sc_parser> createC_sc_parser(){
    return boost::shared_ptr<C_sc_parser>(new C_sc_parser());
}

/* @brief C_scenario取得関数 */
boost::shared_ptr<std::vector<C_scenario> > createC_scenario(){
    return boost::shared_ptr<std::vector<C_scenario> >(new std::vector<C_scenario>());
}
