/*!
    @file  C_Util.h
    @brief ユーティリティクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_UTIL_H
#define C_UTIL_H

#include <time.h>

#include "I_param.h"
#include "C_param_parser.h"
#include "C_scenario.h"
#include "C_SimTime.h"
#include "C_err_msg.h"

#include "Util_wrap.h"
#include "I_Util.h"

/*!
  ユーティリティクラス 
  @brief  シナリオクラス、パラメータクラス、シナリオ時刻クラスを操作するクラス 
 */
class C_Util : public I_Util {
private:
    /* private メンバ変数 */
    time_t  m_start_real_time;  /*!< プログラムのスタート時間 */
    clock_t m_start_cpu_time;   /*!< CPUのスタート時間 */

public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    C_Util(){
        m_start_real_time = time(NULL);
        m_start_cpu_time  = clock();
    }

    /*! @brief デストラクタ 
     */
    virtual ~C_Util() {}

    /*! @brief  ベンダー名を返す。 
     *  @return ベンダー名 
     */
    virtual const std::string getVender();

    /*! @brief  時間の文字列を作成する。 
     *  @return 時間の文字列 
     */
    virtual const std::string getMakeTime();

    /*! @brief パスからファイル名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ファイル名 
     */
    virtual const std::string getFileName(const std::string& path);

    /*! @brief パスからディレクトリ名を取り出します。 
     *  @param path  パス(文字列) 
     *  @return      ディレクトリ名 
     */
    virtual const std::string getDirName(const std::string& path);

    /*! @brief プログラムの実行時間を出力します。 
     *         特定期間の実行時間を計測したい場合は、計測を開始したい 
     *         ポイントで別途本クラスをインスタンスして下さい。 
     *  @param os     出力stream（標準出力に出力したい場合は、省略可です） 
     *  @param prefix 出力する際のprefixを指定してください。 
     *                省略可です（省略時には"SIM"を指定した事になります）。 
     *                ["XXX"を渡した場合の出力例] 
     *                  XXX START : Mon Jul  6 14:28:27 2009 
     *                  XXX END   : Mon Jul  6 14:30:24 2009 
     *                  XXX TOTAL : 116 sec. (CPU TIME : 100.00) 
     */
    virtual void outputProcessingTime(std::ostream& os, const std::string& prefix = std::string("SIM"));

    /*! @brief パラメータの設定とチェックを行います。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータに設定する値 (int型,double型等) 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 成功, false 失敗 
     */
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, int*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, unsigned int*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, double*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::string*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::complex<double>*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<int>*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<unsigned int>*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<double>*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::string>*& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndSetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::complex<double> >*& value, ERROR_SEVERITY e_level = ERROR_MSG);

    /*! @brief パラメータの取得とチェックを行います。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータ名から取得する値 (int型) 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 成功, false 失敗 
     */
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, int& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, unsigned int& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, double& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::string& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::complex<double>& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<int>& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<unsigned int>& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<double>& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::string>& value, ERROR_SEVERITY e_level = ERROR_MSG);
    virtual bool checkAndGetParam(const boost::shared_ptr<I_param>& param, const std::string& name, std::vector<std::complex<double> >& value, ERROR_SEVERITY e_level = ERROR_MSG);

    /*! @brief 出力ファイルストリームを開きます。 
     *  @param  param   出力ファイルストリームのshared_ptr 
     *  @param  name    ファイル名 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                   省略可 (省略時はERROR_MSG) 
     */
    virtual void openAndCheck(boost::shared_ptr<std::ofstream> file,
                              const std::string&               name,
                              ERROR_SEVERITY                   e_level = ERROR_MSG);

    /*! @brief 使用禁止パラメータのチェックを行います。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 使用禁止パラメータの発見, false 使用禁止パラメータ発見なし 
     */
    virtual bool checkAndAbortParam(boost::shared_ptr<I_param> param,
                                    const std::string&         name, 
                                    ERROR_SEVERITY             e_level = ERROR_MSG);

    /*! @briefモードによっては有効なパラメータのアクセス履歴を残す。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @return true 成功, false 失敗 
     */
    virtual bool touchParam(boost::shared_ptr<I_param> param,
                            const std::string&         name);

    /*! @brief シナリオで参照する名前と内部変数をリンクする。 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    シナリオで使用する名前 (文字列) 
     *  @param  *val    内部変数 (型は int型のポインタ限定) 
     *  @return true 設定成功, false 設定失敗 
     */
    virtual bool createMonitorSig (boost::shared_ptr<I_param> param,
                                   const std::string&         name,
                                   int*&                      val);

    /*! @brief パラメータファイルを読み込む。 
     *  @param  param        I_paramクラスのshared_ptr 
     *  @param  param_parser C_param_parserクラスのshared_ptr 
     *  @param  filename     パラメータファイル名 
     *  @param  errmsg       パラメータ解析時エラーメッセージ 
     *  @return true パラメータ読み込み成功, false パラメータ読み込み失敗 
     */
    virtual bool readParamFile(boost::shared_ptr<I_param>        param,
                               boost::shared_ptr<C_param_parser> param_parser,
                               std::vector<std::string>&         filename,
                               std::vector<std::string>&         errmsg);

    /*! @brief シナリオファイルを読み込む。 
     *  @param  param     I_paramクラスのshared_ptr 
     *  @param  sc_parser C_sc_parserクラスのshared_ptr 
     *  @param  filename  シナリオファイル名 
     *  @param  sc_table  vector<C_scenario>のshared_ptr (パース結果)  
     *  @param  errmsg    エラーメッセージ 
     *  @return true シナリオ読み込み成功, false シナリオ読み込み失敗 
     */
    virtual bool readScenarioFile(boost::shared_ptr<I_param>                  param,
                                  boost::shared_ptr<C_sc_parser>              sc_parser,
                                  std::string                                 filename,
                                  boost::shared_ptr<std::vector<C_scenario> > sc_table,
                                  std::vector<std::string>&                   errmsg);

    /*! @brief シナリオを実行する。 
     *  @param  scenario_table シナリオのvector 
     *  @param  time シミュレーション時間 
     *  @return true シナリオ正常実行, false シナリオ停止の通知 
     */
    virtual bool execScenario(boost::shared_ptr<std::vector<C_scenario> >   scenario_table,
                              boost::shared_ptr<C_SimTime>                  time);

    /*! @brief  時間を初期化する 
     *  @param  time  シミュレーション時間クラス 
     */
    virtual void clearTime(boost::shared_ptr<C_SimTime> time);

    /*! @brief  時刻を設定する 
     *  @param  time  シミュレーション時間クラス 
     *  @param  frame frame数 
     *  @param  tick  tick数 
     */
    virtual void setSimTime(boost::shared_ptr<C_SimTime> time, int frame, int tick);

    /*! @brief  1FRAME数を設定する 
     *  @param  time         シミュレーション時間クラス 
     *  @param  tickperframe 1frame辺りのtick数 
     */
    virtual void setTickPerFrame(boost::shared_ptr<C_SimTime> time, int tickperframe);

    /*! @brief  frame数を取得する 
     *  @param  time  シミュレーション時間クラス 
     */
    virtual int getFrame(boost::shared_ptr<C_SimTime> time);

    /*! @brief  tick数を取得する 
     *  @param  time  シミュレーション時間クラス 
     */
    virtual int getTick(boost::shared_ptr<C_SimTime> time);

    virtual bool timeLT(boost::shared_ptr<C_SimTime> lhs, boost::shared_ptr<C_SimTime> rhs);

    /*! @brief  内部の時間を取得する 
     *  @param  time  シミュレーション時間クラス 
     */
    virtual double getDoubleTick(boost::shared_ptr<C_SimTime> time);

    /*! @brief  時刻を進める。 
     *  @param  time シミュレーション時間 
     */
    virtual void nextSimTime(boost::shared_ptr<C_SimTime> time);

    /*! @brief 引数の処理を行う。 
     *  @param argc          プログラムの引数 
     *  @param argv[]        プログラムの引数 
     *  @param param_files   パラメータファイル 
     *  @param scenario_file シナリオファイル 
     *  @param errmsg        ファイルオープン時エラーメッセージ 
     *  @return true 取得成功, false 取得失敗 
     */
    virtual bool readArgs(int argc, char *argv[],
                          std::vector<std::string> &param_files,
                          std::string              &scenario_file,
                          std::vector<std::string> &errmsg);

    /*! @brief シナリオのトレース情報をファイルに出力 
     *  @param os             出力ファイルストリーム 
     *  @param scenario_table シナリオテーブル 
     */
    virtual void outputTraceInfo(std::ostream                                   &os, 
                                 boost::shared_ptr<std::vector<C_scenario> >    scenario_table);

    /*! @brief シナリオのエラー情報を出力 
     *  @param os             出力ファイルストリーム 
     *  @param scenario_table シナリオテーブル 
     */
    virtual void outputScenarioErr(std::ostream                                   &os, 
                                   boost::shared_ptr<std::vector<C_scenario> >    scenario_table);

    /*! @brief パラメータクラスの未使用パラメータをファイルに出力 
     *  @param os             出力ファイルストリーム 
     *  @param param          パラメータクラス 
     */
    virtual void outputUnusedParam(std::ostream               &os, 
                                   boost::shared_ptr<I_param> param);

private:
    /* private メンバ関数 */

    /*! @brief エラー、ワーニング、インフォメッセージの取得 
     *  @param e_level   エラー時のSEVERITY 
     *  @param err_code  エラー時のメッセージコード 
     *  @param warn_code ワーニング時のメッセージコード 
     *  @param info_code インフォ時のメッセージコード 
     *  @param chgstr    メッセージ内で使用する置換用文字列 
     *  @return エラー、ワーニング、インフォメッセージ 
     */
    std::string getMsg( ERROR_SEVERITY e_level,
                        C_err_msg::E_err_type  err_code,
                        C_err_msg::E_warn_type warn_code,
                        C_err_msg::E_info_type info_code,
                        std::string chgstr);

    /*! @brief エラー、ワーニング、インフォメッセージの取得 
     *  @param param   パラメータクラス 
     *  @param e_level エラー時のSEVERITY 
     */
    void addParamMsg( boost::shared_ptr<I_param> param,
                      ERROR_SEVERITY e_level);

/* GEN2 GREEN互換用 */
private:
    inline bool check(std::string name, C_param_wrap::EParamType type, C_param_wrap::EParamType correct_type)
    {
        if (type != correct_type) {
            std::string msg = "GreenWrapper: EParamType mismatch. name=" + name; 
            g_err->appendMsg(msg, ERROR_MSG);
            return false;
        }
        return true;
    }

public:
/* gold用をそのまま利用
    virtual const std::string getVender();
    virtual const std::string getMakeTime();
    virtual const std::string getFileName(const std::string& path);
    virtual const std::string getDirName(const std::string& path);
    virtual void outputProcessingTime(std::ostream& os, const std::string& prefix);
    virtual void openAndCheck(boost::shared_ptr<std::ofstream> file, const std::string& name, ERROR_SEVERITY e_level);
*/

    /*! @brief パラメータの設定とチェックを行います。(green互換用) 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータに設定する値 
     *  @param  type    パラメータ型(未使用) 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 成功, false 失敗 
     */
    template<class T> bool CHECKANDSETPARAM(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, T val, ERROR_SEVERITY e_level)
    {
        return checkAndSetParam(boost::dynamic_pointer_cast<I_param>(param), name, *val, e_level);
    }

    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, int** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, unsigned int** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, double** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::DOUBLE);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::string** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::STRING);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::complex<double>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::COMPLEX);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<int>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT_VECTOR);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<unsigned int>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT_VECTOR);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<double>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::DOUBLE_VECTOR);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::string>** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::STRING_VECTOR);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };
    virtual inline bool checkAndSetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::complex<double> >** val, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::COMPLEX_VECTOR);
        return CHECKANDSETPARAM(param, name, val, e_level);
    };

    /*! @brief パラメータの設定とチェックを行います。(green互換用) 
     *  @param  param   I_paramクラスのshared_ptr 
     *  @param  name    パラメータ名 
     *  @param  value   パラメータに設定する値 (int型,double型等) 
     *  @param  type    パラメータ型(未使用) 
     *  @param  e_level チェックでエラーとなった場合のSEVERITY 
     *                  ※ERROR_MSG、WARINIG_MSG、INFO_MSGが指定可能です。 
     *                    省略可(省略時はERROR_MSG) 
     *  @return true 成功, false 失敗 
     */
    template<class T> bool CHECKANDGETPARAM(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, T val, ERROR_SEVERITY e_level)
    {
        return checkAndGetParam(boost::dynamic_pointer_cast<I_param>(param), name, *val, e_level);
    }
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, int* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, unsigned int* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, double* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::DOUBLE);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::string* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::STRING);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::complex<double>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::DOUBLE);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<int>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT_VECTOR);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<unsigned int>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::INT_VECTOR);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<double>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::DOUBLE_VECTOR);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::string>* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::STRING_VECTOR);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };
    virtual inline bool checkAndGetParam(const boost::shared_ptr<C_param_wrap>& param, const std::string& name, std::vector<std::complex<double> >* value, C_param_wrap::EParamType type, ERROR_SEVERITY e_level)
    {
        check(name, type, C_param_wrap::COMPLEX_VECTOR);
        return CHECKANDGETPARAM(param, name, value, e_level);
    };

#ifndef rel_NTT
#ifdef FW_CMD_MODE
    /*! @brief 引数の処理を行う。（未サポート） 
     */
    virtual void readArgs(int argc, char* argv[], boost::shared_ptr<C_param_wrap> param, boost::shared_ptr<C_scenario_wrap> sc, int& do_scenario, std::string& pfiles) { assert(0); };
    /*
     * @brief 文字を分割する関数（未サポート） 
     */
    virtual std::vector<std::string > str_split(std::string str, std::string delim) { assert(0);};
#endif
#endif
};

#endif
