/*!
    @file  C_call_cmd_interpreter.h
    @brief CALLコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_call_cmd_interpreter.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_call_cmd_interpreter::
C_call_cmd_interpreter(boost::shared_ptr<I_param> param,
                       boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
}
    

/*! @brief   インタプリタの状態をリセットする。 
 */
void C_call_cmd_interpreter::reset() {
    m_check = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_call_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                         std::vector<std::string>::iterator end) {

    /* CALLコマンドのトークン列のサイズをチェック */
    bool size_check = ((end - begin) == 2);

    bool call_token_check = false;
    if (size_check) {
        /* CALLコマンドのチェック */
        call_token_check = (*begin == "@CALL");
    }

    m_check = (size_check && call_token_check);

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     call_token_check);

    return m_check;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_call_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                                  std::vector<std::string>::iterator end,
                                                  int line_num) {
    C_scenario_elem sc_elem;
    if (m_check) {
        /* サブルーチン位置情報用ポインタ */
        int *sub_pos = new int(0);

        /* ポインタの登録 */
        getScPtrTablePtr()->addPtr(sub_pos);

        /* コマンド設定 */
        sc_elem.setCallCmd(sub_pos,line_num);
    }
    return sc_elem;
}

/*! @brief   トークン列からサブルーチン名を取得する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  サブルーチン名 
 */
std::string C_call_cmd_interpreter::getSubName(std::vector<std::string>::iterator begin,
                                               std::vector<std::string>::iterator end) {
    if (!m_check) return "";
    std::vector<std::string>::iterator sub = begin+1;
    return *sub;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_call_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   call_token_check         CALLコマンドトークン存在チェック 
 */
void C_call_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                bool call_token_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* CALLコマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::CALL_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* "@CALL"トークンが不正ときにエラーメッセージを追加 */
    if (!call_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::CALL_CMD_INTERPRETER_ERR_CALL_CMD_POS);
        addErrMsg(msg);
    }
}
