/*! 
    @file  C_err_msg.cpp 
    @brief エラーメッセージの定義 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_err_msg.h"
#include <sstream>

bool C_err_msg::m_init_flag = false;
std::map<C_err_msg::E_err_type,std::string> C_err_msg::m_err_msg_map;
std::map<C_err_msg::E_warn_type,std::string> C_err_msg::m_warn_msg_map;
std::map<C_err_msg::E_info_type,std::string> C_err_msg::m_info_msg_map;

/*! @brief   コンストラクタ 
 */
C_err_msg::C_err_msg() {
    if (!m_init_flag) {
        createErrMsgMap();
        createWarnMsgMap();
        createInfoMsgMap();
        m_init_flag = true;
    }
}

/*! @brief   エラーメッセージ取得関数 
 *  @param   err_code エラーコード (C_err_msg::E_err_type) 
 *  @return  エラーメッセージ 
 */
std::string C_err_msg::getErrMsg(C_err_msg::E_err_type err_code) {
    std::ostringstream oss("");
    oss << "[E" << err_code << "] " << m_err_msg_map[err_code];
    return oss.str();
}

/*! @brief   ワーニングメッセージ取得関数 
 *  @param   warn_code ワーニングコード (C_err_msg::E_warn_type) 
 *  @return  ワーニングメッセージ 
 */
std::string C_err_msg::getWarnMsg(C_err_msg::E_warn_type warn_code) {
    std::ostringstream oss("");
    oss << "[W" << warn_code << "] " << m_warn_msg_map[warn_code];
    return oss.str();
}

/*! @brief   インフォメーションメッセージ取得関数 
 *  @param   info_code インフォメーションコード (C_err_msg::E_info_type) 
 *  @return  インフォメーションメッセージ 
 */
std::string C_err_msg::getInfoMsg(C_err_msg::E_info_type info_code) {
    std::ostringstream oss("");
    oss << "[I" << info_code << "] " << m_info_msg_map[info_code];
    return oss.str();
}

/*! @brief   エラーメッセージマップ作成関数 
 */
void C_err_msg::createErrMsgMap() {
    /* C_token_list 内エラー */
    m_err_msg_map[TOKEN_LIST_ERR_ARRAY_IDX] 
        = "array index is not integer.";
    m_err_msg_map[TOKEN_LIST_ERR_PARAM_NAME] 
        = "param name is not single string or array format.";
    m_err_msg_map[TOKEN_LIST_ERR_VAL_STR] 
        = "value string is not single string or vector format.";
    m_err_msg_map[TOKEN_LIST_ERR_ARRAY_RANGE]
        = "The starting position of the array is larger than end position.";

    /* C_parser_base 内エラー */
    m_err_msg_map[PARSER_BASE_ERR_OPEN_FILE] 
        = "can not open file.";
    m_err_msg_map[PARSER_BASE_ERR_EMPTY_FILE] 
        = "file data is empty.";
    m_err_msg_map[PARSER_BASE_ERR_ZENAKKU]
        = "line include 2 byte char.";
    m_err_msg_map[PARSER_BASE_ERR_NO_EXIST_HEADER]
        = "not found param-header or scenario-header.";

    /* C_param_parser 内エラー */
    m_err_msg_map[PARAM_PARSER_ERR_NO_EXIST_CODE_END] 
        = "not found code end \";\".";
    m_err_msg_map[PARAM_PARSER_ERR_NO_EXIST_PARAM_DEF_END] 
        = "not found param def delimiter \":\".";
    m_err_msg_map[PARAM_PARSER_ERR_EXIST_OTHER_PARAM_DEF_END] 
        = "found some param def delimiters \":\".";

    /* C_sc_parser 内エラー */
    m_err_msg_map[SC_PARSER_ERR_NO_EXIST_CODE_END]
        = "not found code end \";\".";
    m_err_msg_map[SC_PARSER_ERR_UNDEF_CMD]
        = "found undefined command.";
    m_err_msg_map[SC_PARSER_ERR_CMD_FORMAT]
        = "command is illegal format.";
    m_err_msg_map[SC_PARSER_ERR_INSIDE_DEF]
        = "A different definition started in the definition.";
    m_err_msg_map[SC_PARSER_ERR_OUTSIDE_DEF_END]
        = "There is a end command outside the definition.";
    m_err_msg_map[SC_PARSER_ERR_ASSIGN_TYPE_ERROR]
        = "a left-hand operand type and a right-hand operand type are different in assign expression.";
    m_err_msg_map[SC_PARSER_ERR_COMP_TYPE_ERROR]
        = "a left-hand operand type and a right-hand operand type are different in conditional expression.";
    m_err_msg_map[SC_PARSER_ERR_SC_REDEF]
        = "found redefined scenario name.";
    m_err_msg_map[SC_PARSER_ERR_SC_UNDEF]
        = "found undefined sceanrio name.";
    m_err_msg_map[SC_PARSER_ERR_SUB_REDEF]
        = "found redefined subroutine name.";
    m_err_msg_map[SC_PARSER_ERR_SUB_UNDEF]
        = "found undefined subroutine name.";
    m_err_msg_map[SC_PARSER_ERR_LABEL_REDEF]
        = "found redefined label name.";
    m_err_msg_map[SC_PARSER_ERR_LABEL_UNDEF]
        = "found undefined label name.";
    m_err_msg_map[SC_PARSER_ERR_ILLEGAL_JUMP]
        = "illegal jump.(scenario -> subroutin) or (subroutin -> scenario) or (subroutin -> other subroutin) or (scenario -> other scenario)";
    m_err_msg_map[SC_PARSER_ERR_NOT_FOUND_LOOP_END]
        = "not found loop end label.";
    m_err_msg_map[SC_PARSER_ERR_OUTSIDE_DEF_END]
        = "@SCENARIO or @SUB is not found.";
    m_err_msg_map[SC_PARSER_ERR_OUTSIDE_DEF_CMD]
        = "@SCENARIO or @SUB is not found.";

    /* C_dset_cmd_interpreter 内エラー */
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_NO_EXIST_EQUAL]
        = "not found equal token \"=\".";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_PARAM_NAME]
        = "found undefined parameter.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_VAL]
        = "found illegal value string.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_TYPE_OF_PARAM]
        = "parameter type and value type are different.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_TYPE_OF_ARRAY]
        = "all array parameter type are not same type.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_SIZE_OF_VECTOR]
        = "parameter vector size and value vector size are different.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_STATE_OF_STATIC_PARAM]
        = "static parameter is read only.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_STATE_OF_MONITOR_PARAM]
        = "monitor parameter is read only.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_STATE_OF_NONSTATE_PARAM]
        = "the parameter is unused parameter.";
    m_err_msg_map[DSET_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
   

    /* C_rset_cmd_interpreter 内エラー */
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_NO_EXIST_PLUS_EQUAL]
        = "not found plus equal token \"+=\".";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_PARAM_NAME]
        = "found undefined parameter.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_VAL]
        = "found illegal value string.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_TYPE_OF_PARAM]
        = "parameter type and value type are different.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_TYPE_OF_ARRAY]
        = "all array parameter type are not same type.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_SIZE_OF_VECTOR]
        = "parameter vector size and value vector size are different.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_STATE_OF_STATIC_PARAM]
        = "static parameter is read only.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_STATE_OF_MONITOR_PARAM]
        = "monitor parameter is read only.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_STATE_OF_NONSTATE_PARAM]
        = "the parameter is unused parameter.";
    m_err_msg_map[RSET_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";

    /* C_tdset_cmd_interpreter 内エラー */
    m_err_msg_map[TDSET_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[TDSET_CMD_INTERPRETER_ERR_TIME_VAR_POS]
        = "@TIME position is illegal or @TIME is not exist.";
    m_err_msg_map[TDSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG]
        = "time value is not integer.";

    /* C_trset_cmd_interpreter 内エラー */
    m_err_msg_map[TRSET_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[TRSET_CMD_INTERPRETER_ERR_TIME_VAR_POS]
        = "@TIME position is illegal or @TIME is not exist.";
    m_err_msg_map[TRSET_CMD_INTERPRETER_ERR_WAIT_CMD_POS]
        = "@WAIT position is illegal or @WAIT is not exist.";
    m_err_msg_map[TRSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG]
        = "the time value is no integer.";
    m_err_msg_map[TRSET_CMD_INTERPRETER_ERR_PLUS_POS]
        = "not found plus token \"+\".";

    /* C_for_cmd_interpreter 内エラー */
    m_err_msg_map[FOR_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[FOR_CMD_INTERPRETER_ERR_FOR_CMD_POS]
        = "@FOR position is illegal or @FOR is not exist.";
    m_err_msg_map[FOR_CMD_INTERPRETER_ERR_NO_INTEGER_ARG]
        = "the loop number is not integer.";

    /* C_wait_cmd_interpreter 内エラー */
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_WAIT_CMD_POS]
        = "@WAIT position is illegal or @WAIT is not exist.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC]
        = "\"(\" position is illegal or \"(\" is not exist.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC]
        = "\")\" position is illegal or \")\" is not exist.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_COND_LEFT]
        = "left-hand operand in conditional expression is illegal.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_COND_RIGHT]
        = "right-hand operand in conditional expression is illegal.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_COND_OP]
        = "operator in conditional expression is illegal.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_NO_INTEGER_ARG]
        = "the timeout time is not integer.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_TIMEOUT_TYPE]
        = "the timeout type is illegal.";
    m_err_msg_map[WAIT_CMD_INTERPRETER_ERR_COND_TYPE]
        = "a left-hand operand type and a right-hand operand type are different in conditional expression.";

    /* C_jump_cmd_interpreter 内エラー */
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_JUMP_CMD_POS]
        = "@JUMP position is illegal or @JUMP is not exist.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC]
        = "\"(\" position is illegal or \"(\" is not exist.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC]
        = "\")\" position is illegal or \")\" is not exist.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_COND_LEFT]
        = "left-hand operand in conditional expression is illegal.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_COND_RIGHT]
        = "right-hand operand in conditional expression is illegal.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_COND_OP]
        = "operator in conditional expression is illegal.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_NO_INTEGER_ARG]
        = "the jump number is not integer.";
    m_err_msg_map[JUMP_CMD_INTERPRETER_ERR_COND_TYPE]
        = "a left-hand operand type and a right-hand operand type are different in conditional expression.";

    /* C_call_cmd_interpreter 内エラー */
    m_err_msg_map[CALL_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[CALL_CMD_INTERPRETER_ERR_CALL_CMD_POS]
        = "@CALL position is illegal or @CALL is not exist.";

    /* C_scdef_cmd_interpreter 内エラー */
    m_err_msg_map[SCDEF_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[SCDEF_CMD_INTERPRETER_ERR_SCDEF_CMD_POS]
        = "@SCENARIO position is illegal or @SCENARIO is not exist.";
    m_err_msg_map[SCDEF_CMD_INTERPRETER_ERR_REDEF_SCENARIO]
        = "the scenario name has already been defined.";

    /* C_scend_cmd_interpreter 内エラー */
    m_err_msg_map[SCEND_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[SCEND_CMD_INTERPRETER_ERR_SCEND_CMD_POS]
        = "@END position is illegal or @END is not exist.";

    /* C_subdef_cmd_interpreter 内エラー */
    m_err_msg_map[SUBDEF_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[SUBDEF_CMD_INTERPRETER_ERR_SUBDEF_CMD_POS]
        = "@SUB position is illegal or @SUB is not exist.";
    m_err_msg_map[SUBDEF_CMD_INTERPRETER_ERR_REDEF_SUB]
        = "the subroutine name has already been defined.";

    /* C_subend_cmd_interpreter 内エラー */
    m_err_msg_map[SUBEND_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[SUBEND_CMD_INTERPRETER_ERR_SUBEND_CMD_POS]
        = "@SUBEND position is illegal or @SUBEND is not exist.";

    /* C_labeldef_cmd_interpreter 内エラー */
    m_err_msg_map[LABELDEF_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[LABELDEF_CMD_INTERPRETER_ERR_LABELDEF_CMD_POS]
        = "\":\" position is illegal or \":\" is not exist.";
    m_err_msg_map[LABELDEF_CMD_INTERPRETER_ERR_REDEF_LABEL]
        = "the label name has already been defined.";

    /* C_simstop_cmd_interpreter 内エラー */
    m_err_msg_map[SIMSTOP_CMD_INTERPRETER_ERR_ARGS_NUM]
        = "the number of command args is illegal.";
    m_err_msg_map[SIMSTOP_CMD_INTERPRETER_ERR_SIMSTOP_CMD_POS]
        = "@SIM_STOP position is illegal or @SIM_STOP is not exist.";

    /* C_scenairo 内エラー */
    m_err_msg_map[SCENARIO_ERR_SUBSTACK_EMPTY]
        = "subroutine stack is empty.";
    m_err_msg_map[SCENARIO_ERR_SUBSTACK_OVERFLOW]
        = "subroutine stack is overflow.";

    /* C_scenairo_elem 内エラー */
    m_err_msg_map[SCENARIO_ELEM_ERR_UNDEF_CMD]
        = "execute undefined command.";
    m_err_msg_map[SCENARIO_ELEM_ERR_CMD_EXECUTE]
        = "a command execute error.";
    m_err_msg_map[SCENARIO_ELEM_ERR_ASSIGN_TYPE]
        = "execute assign operation which type illegal.";
    m_err_msg_map[SCENARIO_ELEM_ERR_COMP_TYPE]
        = "execute compair operation which type illegal.";
    m_err_msg_map[SCENARIO_ELEM_ERR_WAIT_TIME_OVER]
        = "wait timeout.";
    m_err_msg_map[SCENARIO_ELEM_ERR_JUMP_COUNT_OVER]
        = "jump times is more than max jump times.";

    /* C_param 内エラー */
    m_err_msg_map[PARAM_ERR_FIND_NAME]
        = "can not find parameter.";
    m_err_msg_map[PARAM_ERR_TRANS_INT]
        = "can not translate to INT.";
    m_err_msg_map[PARAM_ERR_TRANS_UNSIGNED_INT]
        = "can not translate to UNSIGNED INT.";
    m_err_msg_map[PARAM_ERR_TRANS_DOUBLE]
        = "can not translate to DOUBLE.";
    m_err_msg_map[PARAM_ERR_TRANS_STRING]
        = "can not translate to STRING.";
    m_err_msg_map[PARAM_ERR_TRANS_COMPLEX]
        = "can not translate to COMPLEX.";
    m_err_msg_map[PARAM_ERR_TRANS_INT_VEC]
        = "can not translate to INT VECTOR.";
    m_err_msg_map[PARAM_ERR_TRANS_UNSIGNED_INT_VEC]
        = "can not translate to UNSIGNED INT VECTOR.";
    m_err_msg_map[PARAM_ERR_TRANS_DOUBLE_VEC]
        = "can not translate to DOUBLE VECTOR.";
    m_err_msg_map[PARAM_ERR_TRANS_STRING_VEC]
        = "can not translate to STRING VECTOR.";
    m_err_msg_map[PARAM_ERR_TRANS_COMPLEX_VEC]
        = "can not translate to COMPLEX VECTOR.";
    m_err_msg_map[PARAM_ERR_TYPE_DECIDED]
        = "The param type already has been decided.";


    /* C_Util 内エラー */
    m_err_msg_map[UTIL_ERR_PARAM_ACQUIRED]
        = "\"%s\" was not able to be acquired.";
    m_err_msg_map[UTIL_ERR_OUTPUTFILE_OPEN]
        = "\"%s\" was not able to be opened.";
    m_err_msg_map[UTIL_ERR_ABORT_PARAM]
        = "use prohibition parameter \"%s\" is exist.";
}

/*! @brief   ワーニングメッセージマップ作成関数 
 */
void C_err_msg::createWarnMsgMap() {
    /* C_scenairo 内ワーニング */
    m_warn_msg_map[SCENARIO_WARN_NEXT_TIME]
        = "The command that the execution time passes is executed.";

    /* C_scenairo_elem 内ワーニング */
    m_warn_msg_map[SCENARIO_ELEM_WARN_WAIT_TIME_OVER]
        = "wait timeout.";

    /* C_sc_parser 内ワーニング */
    m_warn_msg_map[SC_PARSER_WARN_MULTI_DEF_SC]
        = "Two or more scenarios are defined.";

    /* C_param 内ワーニング */
    m_warn_msg_map[PARAM_WARN_FIND_NAME]
        = "can not find parameter.";
    m_warn_msg_map[PARAM_WARN_TRANS_INT]
        = "can not translate to INT.";
    m_warn_msg_map[PARAM_WARN_TRANS_UNSIGNED_INT]
        = "can not translate to UNSIGNED INT.";
    m_warn_msg_map[PARAM_WARN_TRANS_DOUBLE]
        = "can not translate to DOUBLE.";
    m_warn_msg_map[PARAM_WARN_TRANS_STRING]
        = "can not translate to STRING.";
    m_warn_msg_map[PARAM_WARN_TRANS_COMPLEX]
        = "can not translate to COMPLEX.";
    m_warn_msg_map[PARAM_WARN_TRANS_INT_VEC]
        = "can not translate to INT VECTOR.";
    m_warn_msg_map[PARAM_WARN_TRANS_UNSIGNED_INT_VEC]
        = "can not translate to UNSIGNED INT VECTOR.";
    m_warn_msg_map[PARAM_WARN_TRANS_DOUBLE_VEC]
        = "can not translate to DOUBLE VECTOR.";
    m_warn_msg_map[PARAM_WARN_TRANS_STRING_VEC]
        = "can not translate to STRING VECTOR.";
    m_warn_msg_map[PARAM_WARN_TRANS_COMPLEX_VEC]
        = "can not translate to COMPLEX VECTOR.";
    m_warn_msg_map[PARAM_WARN_TYPE_DECIDED]
        = "The param type already has been decided.";

    /* C_Util 内ワーニング */
    m_warn_msg_map[UTIL_WARN_PARAM_ACQUIRED]
        = "\"%s\" was not able to be acquired.";
    m_warn_msg_map[UTIL_WARN_OUTPUTFILE_OPEN]
        = "\"%s\" was not able to be opened.";
    m_warn_msg_map[UTIL_WARN_ABORT_PARAM]
        = "use prohibition parameter \"%s\" is exist.";
}

/*! @brief   インフォメーションメッセージマップ作成関数 
 */
void C_err_msg::createInfoMsgMap() {
    /* C_scenairo内インフォメーション */
    m_info_msg_map[SCENARIO_INFO_STOP]
        = "scenario execute stop by @SIM_STOP";
    m_info_msg_map[SCENARIO_INFO_END]
        = "scenario execute end by @END";

    /* C_param 内インフォメーション */
    m_info_msg_map[PARAM_INFO_FIND_NAME]
        = "can not find parameter.";
    m_info_msg_map[PARAM_INFO_TRANS_INT]
        = "can not translate to INT.";
    m_info_msg_map[PARAM_INFO_TRANS_UNSIGNED_INT]
        = "can not translate to UNSIGNED INT.";
    m_info_msg_map[PARAM_INFO_TRANS_DOUBLE]
        = "can not translate to DOUBLE.";
    m_info_msg_map[PARAM_INFO_TRANS_STRING]
        = "can not translate to STRING.";
    m_info_msg_map[PARAM_INFO_TRANS_COMPLEX]
        = "can not translate to COMPLEX.";
    m_info_msg_map[PARAM_INFO_TRANS_INT_VEC]
        = "can not translate to INT VECTOR.";
    m_info_msg_map[PARAM_INFO_TRANS_UNSIGNED_INT_VEC]
        = "can not translate to UNSIGNED INT VECTOR.";
    m_info_msg_map[PARAM_INFO_TRANS_DOUBLE_VEC]
        = "can not translate to DOUBLE VECTOR.";
    m_info_msg_map[PARAM_INFO_TRANS_STRING_VEC]
        = "can not translate to STRING VECTOR.";
    m_info_msg_map[PARAM_INFO_TRANS_COMPLEX_VEC]
        = "can not translate to COMPLEX VECTOR.";
    m_info_msg_map[PARAM_INFO_TYPE_DECIDED]
        = "The param type already has been decided.";

    /* C_Util 内インフォメーション */
    m_info_msg_map[UTIL_INFO_PARAM_ACQUIRED]
        = "\"%s\" was not able to be acquired.";
    m_info_msg_map[UTIL_INFO_OUTPUTFILE_OPEN]
        = "\"%s\" was not able to be opened.";
    m_info_msg_map[UTIL_INFO_ABORT_PARAM]
        = "use prohibition parameter \"%s\" is exist.";
}

/*! @brief   メッセージの%sをstrに変換 
 *  @param str     置換対象文字列 
 *  @param chgstr  置換文字列 
 *  @return 置換後文字列 
 */
std::string C_err_msg::changeString(std::string str, std::string chgstr){
    std::string::size_type  pos;
    const std::string       repstr = "%s";

    /* %sがある場合 */
    if( (pos = str.find(repstr)) != std::string::npos ){
        /* %sをchgstrに置換 */
        str.replace(pos, repstr.size(), chgstr);
    }

    return str;
}
