/*!
    @file  C_for_cmd_interpreter.h
    @brief FORコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_for_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_for_cmd_interpreter::C_for_cmd_interpreter(boost::shared_ptr<I_param> param,
                                             boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
}
    

/*! @brief   インタプリタの状態をリセットする。 
 */
void C_for_cmd_interpreter::reset() {
    m_check = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_for_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                        std::vector<std::string>::iterator end) {

    /* FORコマンドのトークン列サイズをチェック */
    bool size_check = ((end - begin) == 3);

    bool for_token_check = false;
    bool val_check = false;

    if (size_check) {
        /* 各要素のイテレータ設定 */
        std::vector<std::string>::iterator for_cmd = begin;
        std::vector<std::string>::iterator val = begin+1;

        /* FORコマンドチェック */
        for_token_check = (*for_cmd == "@FOR");

        /* ループ回数がint型かどうかのチェック */
        C_token_list val_token(val,val+1);
        val_check = (val_token.isInt());
    }

    m_check = (size_check && for_token_check && val_check);

    /* エラーがあった場合、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     for_token_check,
                                     val_check);
    return m_check;
}

/*! @brief   ループ変数初期化コマンドを生成する 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_for_cmd_interpreter::createInitLoopVarCmd(int line_num) {
    C_scenario_elem init_cmd;

    int *loop_var = NULL;

    int *init_val = new int(0);
    getScPtrTablePtr()->addPtr(init_val);

    /* ループ変数初期化用コマンド生成 */
    init_cmd.setDirectSetCmd(loop_var,init_val,line_num);

    return init_cmd;
}


/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_for_cmd_interpreter::createLoopBeginCmd(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end,
                                          int line_num) {

    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;

    std::vector<std::string>::iterator for_cmd = begin;
    std::vector<std::string>::iterator val = begin+1;

    int *loop_var = new int(0);
    getScPtrTablePtr()->addPtr(loop_var);

    C_token_list val_token(val,val+1);
    int *loop_num = new int(val_token.toInt());
    getScPtrTablePtr()->addPtr(loop_num);

    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    sc_elem.setForCmd(loop_var,loop_num,label_pos,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_for_cmd_interpreter::createLoopEndCmd(int line_num) {

    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;

    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    sc_elem.setForEndCmd(label_pos,line_num);

    return sc_elem;
}


/*! @brief   FORコマンドラベル名の取得を行う 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  ラベル名 
 */
std::string C_for_cmd_interpreter::getLabelName(std::vector<std::string>::iterator begin,
                                                std::vector<std::string>::iterator end) {
    if (!m_check) return "";
    std::vector<std::string>::iterator label = begin+2;
    return *label;
}


/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_for_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_chek       コマンドトークン列サイズチェック 
 *  @param   for_token_check FORコマンドトークン存在チェック 
 *  @param   val_check       値のトークン存在チェック 
 */
void C_for_cmd_interpreter::createFormatErrMsg(bool size_check,
                                               bool for_token_check,
                                               bool val_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* トークン列のサイズが不正時にエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::FOR_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* FORトークンにエラーがある場合、エラーメッセージを追加 */
    if (!for_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::FOR_CMD_INTERPRETER_ERR_FOR_CMD_POS);
        addErrMsg(msg);
    }

    /* ループ回数の値トークンのエラーある場合、エラーメッセージを追加 */
    if (!val_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::FOR_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}
