/*! 
    @file  C_jump_cmd_interpreter.cpp 
    @brief JUMPコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_jump_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_jump_cmd_interpreter::
C_jump_cmd_interpreter(boost::shared_ptr<I_param> param,
                       boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    : C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_type = C_jump_cmd_interpreter::NON;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_jump_cmd_interpreter::reset() {
    m_check = false;
    m_type = C_jump_cmd_interpreter::NON;
    clearErrMsg();
}


/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_jump_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                         std::vector<std::string>::iterator end) {

    m_check = false;

    /* JUMPコマンドのイテレータ設定 */
    std::vector<std::string>::iterator jump_token = begin;

    /* JUMPコマンドトークン列のサイズチェック */
    bool size_check_cond = ((end - begin) >= 8);    /* 条件付きJUMPコマンドサイズ */
    bool size_check_no_cond = ((end - begin) == 3); /* 条件なしJUMPコマンドサイズ */

    /* JUMPコマンドのチェック */
    bool jump_token_check = (*jump_token == "@JUMP");

    if (size_check_cond) {
        /* 条件付きJUMPコマンドのフォーマットチェック */

        /* 各要素のイテレータ設定 */
        std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);
        std::vector<std::string>::iterator max_jump_num = end-1;
        std::vector<std::string>::iterator left_parenthetic 
            = findToken(begin,end,std::string("("));
        std::vector<std::string>::iterator right_parenthetic 
            = findLastToken(begin,end,std::string(")"));

        /* カッコのチェック */
        bool left_parenthetic_check = (left_parenthetic != end);
        bool right_parenthetic_check = (right_parenthetic != end);

        /* 条件式前後のサイズチェック */
        bool cond_before_size_check = ((left_parenthetic - begin) == 2);
        bool cond_after_size_check = ((end - right_parenthetic) == 2);

        /* 条件演算子の存在をチェック */
        bool cond_op_check = (end != cond_op);

        /* 最大ジャンプ回数がint型かどうかのチェック */
        C_token_list token_list(max_jump_num,max_jump_num+1);
        bool max_jump_num_check = token_list.isInt();

        bool lhs_token_check = true;
        bool rhs_token_check = true;
        if (cond_op_check) {
            /* 条件演算子の存在する場合 */

            /* 条件式の各要素のイテレータ設定 */
            std::vector<std::string>::iterator cond_lhs_begin = begin+3;
            std::vector<std::string>::iterator cond_lhs_end = cond_op;
            std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
            std::vector<std::string>::iterator cond_rhs_end = end-2;

            C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
            C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

            /* 条件式の左辺、右辺のパラメータが1つ存在することをチェック */
            lhs_token_check = (lhs_token.createParamName(getParamPtr()).size() == 1);
            rhs_token_check = (rhs_token.createParamName(getParamPtr()).size() == 1);
        }

        m_check = (jump_token_check 
                   && right_parenthetic_check && left_parenthetic_check
                   && cond_before_size_check && cond_after_size_check
                   && cond_op_check && max_jump_num_check
                   && lhs_token_check && rhs_token_check);

        /* エラー発生時、エラーメッセージを生成 */
        if (!m_check) createCondFormatErrMsg(jump_token_check,
                                             left_parenthetic_check,
                                             right_parenthetic_check,
                                             cond_op_check,
                                             lhs_token_check,
                                             rhs_token_check,
                                             max_jump_num_check);
    }
    else if (size_check_no_cond) {
        /* 条件なしJUMPコマンドのフォーマットチェック */

        /* 各要素のイテレータ設定 */
        std::vector<std::string>::iterator max_jump_num = begin+2;

        /* 最大ジャンプ回数がint型かどうかのチェック */
        C_token_list token_list(max_jump_num,max_jump_num+1);
        bool max_jump_num_check = token_list.isInt();

        m_check = (jump_token_check && max_jump_num_check);

        /* エラー発生時、エラーメッセージを生成 */
        if (!m_check) createNoCondFormatErrMsg(jump_token_check,
                                               max_jump_num_check);
    }

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatSizeErrMsg(size_check_cond,
                                         size_check_no_cond);

    return m_check;
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_jump_cmd_interpreter::checkCondType(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {

    if (!m_check) return false;

    bool cond_type_check = true;
    bool size_check_cond = ((end - begin) >= 8);
    bool size_check_no_cond = ((end - begin) == 3);

    if (size_check_cond) {
        std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);

        std::vector<std::string>::iterator cond_lhs_begin = begin+3;
        std::vector<std::string>::iterator cond_lhs_end = cond_op;
        std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
        std::vector<std::string>::iterator cond_rhs_end = end-2;

        bool int_type_check = checkCompTypeInt(cond_lhs_begin,cond_lhs_end,
                                               cond_rhs_begin,cond_rhs_end);
        bool uint_type_check = checkCompTypeUnsignedInt(cond_lhs_begin,cond_lhs_end,
                                                        cond_rhs_begin,cond_rhs_end);
        bool double_type_check = checkCompTypeDouble(cond_lhs_begin,cond_lhs_end,
                                                     cond_rhs_begin,cond_rhs_end);
        bool string_type_check = checkCompTypeString(cond_lhs_begin,cond_lhs_end,
                                                     cond_rhs_begin,cond_rhs_end);
        bool string_op_check = (*cond_op == "==" || *cond_op == "!=");

        std::string type_info = createCompTypeInfo(cond_lhs_begin,cond_lhs_end,
                                                   cond_rhs_begin,cond_rhs_end);

        if (size_check_cond && int_type_check) {
            m_type = C_jump_cmd_interpreter::INT;
        }
        else if (size_check_cond && uint_type_check) {
            m_type = C_jump_cmd_interpreter::UNSIGNED_INT;
        }
        else if (size_check_cond && double_type_check) {
            m_type = C_jump_cmd_interpreter::DOUBLE;
        }
        else if (size_check_cond && string_type_check && string_op_check) {
            m_type = C_jump_cmd_interpreter::STRING;
        }
        else {
            cond_type_check = false;

            /* エラーメッセージの生成 */
            createCondTypeErrMsg(int_type_check,
                                 uint_type_check,
                                 double_type_check,
                                 string_type_check,
                                 string_op_check,
                                 type_info);
        }
    }
    else if (size_check_no_cond) {
        m_type = C_jump_cmd_interpreter::NO_COND;
    }
    else {
        cond_type_check = false;
    }

    return cond_type_check;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                  std::vector<std::string>::iterator end,
                                  int line_num) {
    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;

    if (m_type != C_jump_cmd_interpreter::NO_COND) {
        std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);

        std::vector<std::string>::iterator cond_lhs_begin = begin+3;
        std::vector<std::string>::iterator cond_lhs_end = cond_op;
        std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
        std::vector<std::string>::iterator cond_rhs_end = end-2;
        std::vector<std::string>::iterator max_jump_num = end-1;
    
        if (m_type == C_jump_cmd_interpreter::INT) {
            sc_elem = createIntTypeCmd(cond_lhs_begin,cond_lhs_end,
                                       cond_rhs_begin,cond_rhs_end,cond_op,
                                       max_jump_num,line_num);
        }
        else if (m_type == C_jump_cmd_interpreter::UNSIGNED_INT) {
            sc_elem = createUnsignedIntTypeCmd(cond_lhs_begin,cond_lhs_end,
                                               cond_rhs_begin,cond_rhs_end,cond_op,
                                               max_jump_num,line_num);
        }
        else if (m_type == C_jump_cmd_interpreter::DOUBLE) {
            sc_elem = createDoubleTypeCmd(cond_lhs_begin,cond_lhs_end,
                                          cond_rhs_begin,cond_rhs_end,cond_op,
                                          max_jump_num,line_num);
        }
        else if (m_type == C_jump_cmd_interpreter::STRING) {
            sc_elem = createStringTypeCmd(cond_lhs_begin,cond_lhs_end,
                                          cond_rhs_begin,cond_rhs_end,cond_op,
                                          max_jump_num,line_num);
        }
    }
    else if (m_type == C_jump_cmd_interpreter::NO_COND) {
        std::vector<std::string>::iterator max_jump_num = begin+2;

        sc_elem = createNoCondTypeCmd(max_jump_num,line_num);
    }

    return sc_elem;
}

/*! @brief   条件式がInt型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   max_jump_num    最大ジャンプ回数 
 *  @param   line_num        シナリオファイルの行数 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                         std::vector<std::string>::iterator cond_lhs_end,
                                         std::vector<std::string>::iterator cond_rhs_begin,
                                         std::vector<std::string>::iterator cond_rhs_end,
                                         std::vector<std::string>::iterator cond_op,
                                         std::vector<std::string>::iterator max_jump_num,
                                         int line_num) {
    int *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    const std::vector<std::string> lhs_params(lhs_token.createParamName(getParamPtr()));
    const std::vector<std::string> rhs_params(rhs_token.createParamName(getParamPtr()));

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new int(lhs_token.toInt());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new int(rhs_token.toInt());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* ラベル位置用ポインタの取得 */
    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    /* ジャンプ回数カウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* 最大ジャンプ回数用ポインタの取得 */
    C_token_list token_list(max_jump_num,max_jump_num+1);
    int *max_num = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(max_num);

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setJumpCmd(lhs,rhs,op,label_pos,count,max_num,line_num);

    return sc_elem;
}

/*! @brief   条件式がUnsigned Int型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   max_jump_num    最大ジャンプ回数 
 *  @param   line_num        シナリオファイルの行数 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createUnsignedIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                                 std::vector<std::string>::iterator cond_lhs_end,
                                                 std::vector<std::string>::iterator cond_rhs_begin,
                                                 std::vector<std::string>::iterator cond_rhs_end,
                                                 std::vector<std::string>::iterator cond_op,
                                                 std::vector<std::string>::iterator max_jump_num,
                                                 int line_num) {
    unsigned int *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    const std::vector<std::string> lhs_params(lhs_token.createParamName(getParamPtr()));
    const std::vector<std::string> rhs_params(rhs_token.createParamName(getParamPtr()));

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new unsigned int(lhs_token.toUnsignedInt());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new unsigned int(rhs_token.toUnsignedInt());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* ラベル位置用ポインタの取得 */
    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    /* ジャンプ回数カウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* 最大ジャンプ回数用ポインタの取得 */
    C_token_list token_list(max_jump_num,max_jump_num+1);
    int *max_num = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(max_num);

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setJumpCmd(lhs,rhs,op,label_pos,count,max_num,line_num);

    return sc_elem;
}

/*! @brief   条件式がdouble型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   max_jump_num    最大ジャンプ回数 
 *  @param   line_num        シナリオファイルの行数 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createDoubleTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                            std::vector<std::string>::iterator cond_lhs_end,
                                            std::vector<std::string>::iterator cond_rhs_begin,
                                            std::vector<std::string>::iterator cond_rhs_end,
                                            std::vector<std::string>::iterator cond_op,
                                            std::vector<std::string>::iterator max_jump_num,
                                            int line_num) {
    double *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    const std::vector<std::string> lhs_params(lhs_token.createParamName(getParamPtr()));
    const std::vector<std::string> rhs_params(rhs_token.createParamName(getParamPtr()));

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new double(lhs_token.toDouble());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new double(rhs_token.toDouble());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* ラベル位置用ポインタの取得 */
    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    /* ジャンプ回数カウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* 最大ジャンプ回数用ポインタの取得 */
    C_token_list token_list(max_jump_num,max_jump_num+1);
    int *max_num = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(max_num);

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setJumpCmd(lhs,rhs,op,label_pos,count,max_num,line_num);

    return sc_elem;
}

/*! @brief   条件式がString型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   max_jump_num    最大ジャンプ回数 
 *  @param   line_num        シナリオファイルの行数 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createStringTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                            std::vector<std::string>::iterator cond_lhs_end,
                                            std::vector<std::string>::iterator cond_rhs_begin,
                                            std::vector<std::string>::iterator cond_rhs_end,
                                            std::vector<std::string>::iterator cond_op,
                                            std::vector<std::string>::iterator max_jump_num,
                                            int line_num) {
    std::string *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    const std::vector<std::string> lhs_params(lhs_token.createParamName(getParamPtr()));
    const std::vector<std::string> rhs_params(rhs_token.createParamName(getParamPtr()));

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new std::string(lhs_token.toString());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new std::string(rhs_token.toString());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* ラベル位置用ポインタの取得 */
    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    /* ジャンプ回数カウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* 最大ジャンプ回数用ポインタの取得 */
    C_token_list token_list(max_jump_num,max_jump_num+1);
    int *max_num = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(max_num);

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setJumpCmd(lhs,rhs,op,label_pos,count,max_num,line_num);

    return sc_elem;
}

/*! @brief   無条件式のコマンドを生成する 
 *  @param   max_jump_num 最大ジャンプ回数 
 *  @param   line_num     シナリオファイルの行数 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_jump_cmd_interpreter::createNoCondTypeCmd(std::vector<std::string>::iterator max_jump_num,
                                            int line_num) {
    /* ラベル位置用ポインタの取得 */
    int *label_pos = new int(0);
    getScPtrTablePtr()->addPtr(label_pos);

    /* ジャンプ回数カウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* 最大ジャンプ回数用ポインタの取得 */
    C_token_list token_list(max_jump_num,max_jump_num+1);
    int *max_num = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(max_num);

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setJumpCmd(label_pos,count,max_num,line_num);

    return sc_elem;
}

/*! @brief   JUMPコマンドラベル名の取得を行う 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  ラベル名 
 */
std::string C_jump_cmd_interpreter::getLabelName(std::vector<std::string>::iterator begin,
                                                 std::vector<std::string>::iterator end) {
    if (!m_check) return "";
    std::vector<std::string>::iterator label = begin+1;
    return *label;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_jump_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << "\n"
        << "m_type  = " << m_type;

    return oss.str();
}

/*! @brief   コマンドフォーマットサイズエラーメッセージの生成 
 *  @param   size_check_cond     条件ありJUMPコマンドトークン列のサイズチェック結果 
 *  @param   size_check_no_cond  条件なしJUMPコマンドトークン列のサイズチェック結果 
 */
void C_jump_cmd_interpreter::createFormatSizeErrMsg(bool size_check_cond,
                                                    bool size_check_no_cond) {
    
    C_err_msg err_msg;
    std::string msg = "";

    /* JUMPコマンドのトークン列サイズが不正の時、エラーメッセージを追加 */
    if (!size_check_cond && !size_check_no_cond) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }
}

/*! @brief   条件付きJUMPコマンドフォーマットエラーメッセージの生成 
 *  @param   jump_token_check         JUMPコマンドトークン存在チェック 
 *  @param   left_parenthetic_check   "("トークン存在チェック 
 *  @param   right_parenthetic_check  ")"トークン存在チェック 
 *  @param   cond_op_check            条件式の演算子チェック 
 *  @param   cond_lhs_check           条件式の左辺チェック 
 *  @param   cond_rhs_check           条件式の右辺チェック 
 *  @param   max_jump_num_check       最大ジャンプ回数のチェック 
 */
void C_jump_cmd_interpreter::createCondFormatErrMsg(bool jump_token_check,
                                                    bool left_parenthetic_check,
                                                    bool right_parenthetic_check,
                                                    bool cond_op_check,
                                                    bool cond_lhs_check,
                                                    bool cond_rhs_check,
                                                    bool max_jump_num_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* "@JUMP"トークンが不正ときにエラーメッセージを追加 */
    if (!jump_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_JUMP_CMD_POS);
        addErrMsg(msg);
    }

    /* "("トークンが不正ときにエラーメッセージを追加 */
    if (!left_parenthetic_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC);
        addErrMsg(msg);
    }

    /* ")"トークンが不正ときにエラーメッセージを追加 */
    if (!right_parenthetic_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC);
        addErrMsg(msg);
    }

    /* 比較演算子が不正の時にエラーメッセージを追加 */
    if (!cond_op_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_COND_OP);
        addErrMsg(msg);
    }

    /* 条件式左辺が不正の時にエラーメッセージを追加 */
    if (!cond_lhs_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_COND_LEFT);
        addErrMsg(msg);
    }

    /* 条件式右辺が不正の時にエラーメッセージを追加 */
    if (!cond_rhs_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_COND_RIGHT);
        addErrMsg(msg);
    }

    /* max_jump_numが不正の時にエラーメッセージを追加 */
    if (!max_jump_num_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}

/*! @brief   条件なしJUMPコマンドフォーマットエラーメッセージの生成 
 *  @param   jump_token_check         JUMPコマンドトークン存在チェック 
 *  @param   max_jump_num_check       最大ジャンプ回数のチェック 
 */
void C_jump_cmd_interpreter::createNoCondFormatErrMsg(bool jump_token_check,
                                                      bool max_jump_num_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* "@JUMP"トークンが不正ときにエラーメッセージを追加 */
    if (!jump_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_JUMP_CMD_POS);
        addErrMsg(msg);
    }

    /* max_jump_numが不正の時にエラーメッセージを追加 */
    if (!max_jump_num_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}

/*! @brief   型エラーメッセージの生成 
 *  @param   int_type_check      int型のチェック 
 *  @param   uint_type_check     unsigned int型のチェック 
 *  @param   double_type_check   double型のチェック 
 *  @param   string_type_check   string型のチェック 
 *  @param   string_op_check     string型の時の演算子のチェック 
 *  @param   type_info           型情報 
 */
void C_jump_cmd_interpreter::createCondTypeErrMsg(bool int_type_check,
                                                  bool uint_type_check,
                                                  bool double_type_check,
                                                  bool string_type_check,
                                                  bool string_op_check,
                                                  std::string type_info) {
    C_err_msg err_msg;
    std::string msg = "";

    /* 文字列型時の比較演算子が不正の時、エラーメッセージを追加 */
    if (string_type_check && !string_op_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_COND_OP);
        addErrMsg(msg);
    }

    /* 型が不正の時 */
    if (!int_type_check && !uint_type_check && !double_type_check && !string_type_check) {
        msg = err_msg.getErrMsg(C_err_msg::JUMP_CMD_INTERPRETER_ERR_COND_TYPE);
        addErrMsg(msg);
        addErrMsg(type_info);
    }
}
