/*! 
    @file  C_jump_cmd_interpreter.h 
    @brief JUMPコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_JUMP_CMD_INTERPRETER_H
#define C_JUMP_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  JUMPコマンドインタプリタクラス 
  @brief  JUMPコマンドの解釈機能を持つ 
 */
class C_jump_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    enum E_jump_type {
        NON = 0,
        INT,
        UNSIGNED_INT,
        DOUBLE,
        STRING,
        NO_COND
    };

    int m_type;
    bool m_check;

public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_jump_cmd_interpreter(boost::shared_ptr<I_param> param,
                          boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_jump_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkCondType(std::vector<std::string>::iterator begin,
                       std::vector<std::string>::iterator end);

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num);

    /*! @brief   JUMPコマンドラベル名の取得を行う 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  ラベル名 
     */
    std::string getLabelName(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:
    /* private メンバ関数 */

    /*! @brief   条件式がInt型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   max_jump_num    最大ジャンプ回数 
     *  @param   line_num        シナリオファイルの行数 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                     std::vector<std::string>::iterator cond_lhs_end,
                                     std::vector<std::string>::iterator cond_rhs_begin,
                                     std::vector<std::string>::iterator cond_rhs_end,
                                     std::vector<std::string>::iterator cond_op,
                                     std::vector<std::string>::iterator max_jump_num,
                                     int line_num);
    
    /*! @brief   条件式がUnsignedInt型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   max_jump_num    最大ジャンプ回数 
     *  @param   line_num        シナリオファイルの行数 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createUnsignedIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                             std::vector<std::string>::iterator cond_lhs_end,
                                             std::vector<std::string>::iterator cond_rhs_begin,
                                             std::vector<std::string>::iterator cond_rhs_end,
                                             std::vector<std::string>::iterator cond_op,
                                             std::vector<std::string>::iterator max_jump_num,
                                             int line_num);

    /*! @brief   条件式がdouble型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   max_jump_num    最大ジャンプ回数 
     *  @param   line_num        シナリオファイルの行数 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createDoubleTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                        std::vector<std::string>::iterator cond_lhs_end,
                                        std::vector<std::string>::iterator cond_rhs_begin,
                                        std::vector<std::string>::iterator cond_rhs_end,
                                        std::vector<std::string>::iterator cond_op,
                                        std::vector<std::string>::iterator max_jump_num,
                                        int line_num);

    /*! @brief   条件式がString型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   max_jump_num    最大ジャンプ回数 
     *  @param   line_num        シナリオファイルの行数 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createStringTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                        std::vector<std::string>::iterator cond_lhs_end,
                                        std::vector<std::string>::iterator cond_rhs_begin,
                                        std::vector<std::string>::iterator cond_rhs_end,
                                        std::vector<std::string>::iterator cond_op,
                                        std::vector<std::string>::iterator max_jump_num,
                                        int line_num);

    /*! @brief   無条件式のコマンドを生成する 
     *  @param   max_jump_num 最大ジャンプ回数 
     *  @param   line_num     シナリオファイルの行数 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createNoCondTypeCmd(std::vector<std::string>::iterator max_jump_num,
                                        int line_num);

    /*! @brief   コマンドフォーマットサイズエラーメッセージの生成 
     *  @param   size_check_cond     条件ありJUMPコマンドトークン列のサイズチェック結果 
     *  @param   size_check_no_cond  条件なしJUMPコマンドトークン列のサイズチェック結果 
     */
    void createFormatSizeErrMsg(bool size_check_cond,
                                bool size_check_no_cond);
    

    /*! @brief   条件付きJUMPコマンドフォーマットエラーメッセージの生成 
     *  @param   jump_token_check         JUMPコマンドトークン存在チェック 
     *  @param   left_parenthetic_check   "("トークン存在チェック 
     *  @param   right_parenthetic_check  ")"トークン存在チェック 
     *  @param   cond_op_check            条件式の演算子チェック 
     *  @param   cond_lhs_check           条件式の左辺チェック 
     *  @param   cond_rhs_check           条件式の右辺チェック 
     *  @param   max_jump_num_check       最大ジャンプ回数のチェック 
     */
    void createCondFormatErrMsg(bool jump_token_check,
                                bool left_parenthetic_check,
                                bool right_parenthetic_check,
                                bool cond_op_check,
                                bool cond_lhs_check,
                                bool cond_rhs_check,
                                bool max_jump_num_check);

    /*! @brief   条件なしJUMPコマンドフォーマットエラーメッセージの生成 
     *  @param   jump_token_check         JUMPコマンドトークン存在チェック 
     *  @param   max_jump_num_check       最大ジャンプ回数のチェック 
     */
    void createNoCondFormatErrMsg(bool jump_token_check,
                                  bool max_jump_num_check);

    /*! @brief   型エラーメッセージの生成 
     *  @param   int_type_check      int型のチェック 
     *  @param   uint_type_check     unsigned int型のチェック 
     *  @param   double_type_check   double型のチェック 
     *  @param   string_type_check   string型のチェック 
     *  @param   string_op_check     string型の時の演算子のチェック 
     *  @param   type_info           型情報 
     */
    void createCondTypeErrMsg(bool int_type_check,
                              bool uint_type_check,
                              bool double_type_check,
                              bool string_type_check,
                              bool string_op_check,
                              std::string type_info);
};

#endif
