/*! 
    @file  C_labeldef_cmd_interpreter.cpp 
    @brief ラベル定義コマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_labeldef_cmd_interpreter.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_labeldef_cmd_interpreter::
C_labeldef_cmd_interpreter(boost::shared_ptr<I_param> param,
                           boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_labeldef = false;
}
    

/*! @brief   インタプリタの状態をリセットする。 
 */
void C_labeldef_cmd_interpreter::reset() {
    m_check = false;
    m_labeldef = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_labeldef_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                             std::vector<std::string>::iterator end) {
    /* ラベル定義コマンドのトークン列サイズチェック */
    bool size_check = ((end - begin) == 2);

    bool labeldef_token_check = false;

    if (size_check) {
        /* ラベル定義のトークンチェック */
        labeldef_token_check = (*(begin+1) == ":");
    }

    m_check = (size_check && labeldef_token_check);

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     labeldef_token_check);

    return m_check;
}


/*! @brief   トークン列からラベル定義をチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @param   label_map ラベルマップ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_labeldef_cmd_interpreter::
checkLabelDef(std::vector<std::string>::iterator begin,
              std::vector<std::string>::iterator end,
              std::map<std::string,std::pair<size_t,size_t> > &label_def_map) {

    m_labeldef = false;

    if (m_check) {
        /* ２重定義かどうかのチェック */
        std::string label = *begin;
        m_labeldef = (label_def_map.find(label) == label_def_map.end());

        /* エラー発生時、エラーメッセージ生成 */
        if (!m_labeldef) createLabelDefErrMsg();
    }

    return m_labeldef;
}

/*! @brief   トークン列からラベル名を取得する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  ラベル名 
 */
std::string C_labeldef_cmd_interpreter::getLabelName(std::vector<std::string>::iterator begin,
                                                     std::vector<std::string>::iterator end) {
    if (!m_labeldef || !m_check) return "";
    std::vector<std::string>::iterator label_name = begin;
    return *begin;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_labeldef_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check    = " << m_check << "\n"
        << "m_labeldef = " << m_labeldef;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   labeldef_token_check     ":"トークン存在チェック 
 */
void C_labeldef_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                    bool labeldef_token_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* ラベル定義コマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::LABELDEF_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* ":"トークンが不正ときにエラーメッセージを追加 */
    if (!labeldef_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::LABELDEF_CMD_INTERPRETER_ERR_LABELDEF_CMD_POS);
        addErrMsg(msg);
    }
}

/*! @brief   ラベル定義エラーメッセージの生成 
 */
void C_labeldef_cmd_interpreter::createLabelDefErrMsg() {
    C_err_msg err_msg;
    std::string msg = "";

    /* 同一ラベルの再定義時エラー時にエラーメッセージを追加 */
    msg = err_msg.getErrMsg(C_err_msg::LABELDEF_CMD_INTERPRETER_ERR_REDEF_LABEL);
    addErrMsg(msg);
}
