/*! 
    @file  C_param.cpp 
    @brief パラメータクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/
#include "C_param.h"
#include "C_err_msg.h"


/*! @brief 存在確認を行う 
 *  @param   var_name パラメータ名 
 *  @return  true: 存在, false: 存在しない 
 */
bool C_param::exist(const std::string &var_name){
    /* var_nameの存在を検索 */
    return (m_param_map.find(var_name) != m_param_map.end());
}



/*! @brief 参照カウンタ更新 
 *  @param   var_name パラメータ名 
 *  @return  true: 追加成功, false: 追加失敗 
 */
bool C_param::touchParam(const std::string &var_name){
    /* var_nameの存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end()){
        /* パラメータが存在する */
        /* 参照カウンタを更新 */
        it->second.touch();
        return true;
    }
    else{
        /* パラメータが存在しない */
        return false;
    }
}



/*! @brief パラメータの追加 静的パラメータ用 
 *  @param var_name パラメータ名 
 *  @param val_str  パラメータの値の文字列 
 *  @return         true: 追加成功, false: 追加失敗 
 */
bool C_param::addParam(const std::string &var_name, const std::string &var_str){
    /* var_nameの存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);
    /* パラメータを作成 */
    C_param_elem addElem(var_str);

    /* パラメータを検索 */
    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* 見つかったパラメータを削除 */
        m_param_map.erase(it);
        /* 新しいパラメータを追加する */
        m_param_map.insert(std::pair<std::string, C_param_elem>(var_name, addElem));
    }
    else{
        /* パラメータが存在しない */
        /* パラメータを追加 */
        m_param_map.insert(std::pair<std::string, C_param_elem>(var_name, addElem));
    }

    return true;
}



/*! @brief パラメータの追加 シミュレータ内部変数の追加用 
 *  @param var_name パラメータ名 
 *  @param val      int *型の値 
 *  @return         true: 追加成功, false: 追加失敗 
 */
bool C_param::addParam(const std::string &var_name, int *var){
    /* var_nameの存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);
    /* パラメータを作成 */
    C_param_elem addElem("");
    addElem.setPtr(var);

    /* パラメータを検索 */
    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* 見つかったパラメータを削除 */
        m_param_map.erase(it);
        /* 新しいパラメータを追加する */
        m_param_map.insert(std::pair<std::string, C_param_elem>(var_name, addElem));
    }
    else{
        /* パラメータが存在しない */
        /* パラメータを追加 */
        m_param_map.insert(std::pair<std::string, C_param_elem>(var_name, addElem));
    }

    return true;
}



/*! @brief 値の取得 静的パラメータ用 
 *  @param   var_name パラメータ名 
 *  @param   val      値 
 *  @return  true: 取得成功, false: 取得失敗 
 */
bool C_param::getValue(const std::string &var_name,int &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_INT);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_INT);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_INT);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,unsigned int &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_UNSIGNED_INT);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_UNSIGNED_INT);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_UNSIGNED_INT);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,double &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_DOUBLE);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_DOUBLE);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_DOUBLE);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::complex<double> &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_COMPLEX);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_COMPLEX);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_COMPLEX);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::string &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_STRING);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_STRING);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_STRING);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::vector<int> &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_INT_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_INT_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_INT_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::vector<unsigned int> &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_UNSIGNED_INT_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_UNSIGNED_INT_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_UNSIGNED_INT_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::vector<double> &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_DOUBLE_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_DOUBLE_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_DOUBLE_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::vector<std::complex<double> > &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_COMPLEX_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_COMPLEX_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_COMPLEX_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::getValue(const std::string &var_name,std::vector<std::string> &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getValue(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_STRING_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_STRING_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_STRING_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

/*! @brief パラメタの設定 動的パラメータ用 
 *  @param   var_name パラメータ名 
 *  @param   val      値 
 *  @return  true: 取得成功, false: 取得失敗 
 */
bool C_param::setParam(const std::string &var_name,int* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_INT);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_INT);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_INT);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,unsigned int* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_UNSIGNED_INT);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_UNSIGNED_INT);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_UNSIGNED_INT);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,double* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_DOUBLE);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_DOUBLE);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_DOUBLE);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::complex<double>* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_COMPLEX);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_COMPLEX);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_COMPLEX);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::string* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_STRING);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_STRING);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_STRING);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::vector<int>* &val){ 
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_INT_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_INT_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_INT_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::vector<unsigned int>* &val){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_UNSIGNED_INT_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_UNSIGNED_INT_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_UNSIGNED_INT_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::vector<double>* &val){ 
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_DOUBLE_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_DOUBLE_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_DOUBLE_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::vector<std::complex<double> >* &val){ 
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_COMPLEX_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_COMPLEX_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_COMPLEX_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}

bool C_param::setParam(const std::string &var_name,std::vector<std::string>* &val){ 
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem >::iterator ite = m_param_map.find(var_name);
    bool find_result = (ite != m_param_map.end());

    /* 変換結果を取得 */
    bool trans_result = false;
    if (find_result) trans_result = ite->second.getPtr(val);

    /* エラー時の処理 */
    if (!find_result) {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_FIND_NAME);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_FIND_NAME);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_FIND_NAME);
        m_info_msg.push_back(msg);
    }
    else if (!trans_result)  {
        C_err_msg err_msg;
        std::string msg;
        msg = err_msg.getErrMsg(C_err_msg::PARAM_ERR_TRANS_STRING_VEC);
        m_err_msg.push_back(msg);
        msg = err_msg.getWarnMsg(C_err_msg::PARAM_WARN_TRANS_STRING_VEC);
        m_warn_msg.push_back(msg);
        msg = err_msg.getInfoMsg(C_err_msg::PARAM_INFO_TRANS_STRING_VEC);
        m_info_msg.push_back(msg);
    }

    return trans_result;
}



/*! @brief 使用されなかったパラメータの取得 
 *  @return  使用されなかったパラメータ 
 */
std::vector<std::string> C_param::getUnusedParam(){
    /* 使用されなかったパラメータ配列buffer */
    std::vector<std::string> vect;

    /* mapの検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.begin();

    for(; it != m_param_map.end(); ++it){
        /* 未使用チェック */
        if( it->second.isUnused() ){
            /* 未使用のパラメータを配列に格納 */
            vect.push_back( it->first );
        }
    }

    return vect;
}



/*! @brief  静的パラメータかどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: 要素が静的パラメータ, false: 静的パラメータではないorパラメータが存在しない 
 */
bool C_param::isStatic(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isStaticで判別結果を返す */
        return it->second.isStatic();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  動的パラメータかどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: 要素が動的パラメータ, false: 動的パラメータではないorパラメータが存在しない 
 */
bool C_param::isDynamic(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isDynamicで判別結果を返す */
        return it->second.isDynamic();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  シミュレータ内部変数かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: 要素がシミュレータ内部変数, false: シミュレータ内部変数ではないorパラメータが存在しない 
 */
bool C_param::isMonitor(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isMonitorで判別結果を返す */
        return it->second.isMonitor();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  int型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: int型, false: int型でないorパラメータが存在しない 
 */
bool C_param::isInt(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isIntで判別結果を返す */
        return it->second.isInt();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  unsigned int型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: unsigned int型, false: unsigned int型でないorパラメータが存在しない 
 */
bool C_param::isUnsignedInt(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isUnsignedIntで判別結果を返す */
        return it->second.isUnsignedInt();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  double型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: double型, false: double型でないorパラメータが存在しない 
 */
bool C_param::isDouble(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isDoubleで判別結果を返す */
        return it->second.isDouble();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  string型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: string型, false: string型でないorパラメータが存在しない 
 */
bool C_param::isString(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isStringで判別結果を返す */
        return it->second.isString();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  Complex型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: Complex型, false: Complex型でないorパラメータが存在しない 
 */
bool C_param::isComplex(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isComplexで判別結果を返す */
        return it->second.isComplex();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  vector<int>型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: vector<int>型, false: vector<int>型でないorパラメータが存在しない 
 */
bool C_param::isIntVector(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isIntVectorで判別結果を返す */
        return it->second.isIntVector();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  vector<unsigned int>型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: vector<unsigned int>型, false: vector<unsigned int>型でないorパラメータが存在しない 
 */
bool C_param::isUnsignedIntVector(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isUnsignedIntVectorで判別結果を返す */
        return it->second.isUnsignedIntVector();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  vector<double>型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: vector<double>型, false: vector<double>型でないorパラメータが存在しない 
 */
bool C_param::isDoubleVector(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isDoubleVectorで判別結果を返す */
        return it->second.isDoubleVector();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  vector<string>型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: vector<string>型, false: vector<string>型でないorパラメータが存在しない 
 */
bool C_param::isStringVector(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isStringVectorで判別結果を返す */
        return it->second.isStringVector();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  vector<complex>型かどうかを判定 
 *  @param  var_name パラメータ名 
 *  @return true: vector<complex>型, false: vector<complex>型でないorパラメータが存在しない 
 */
bool C_param::isComplexVector(const std::string &var_name){
    /* パラメータ(var_name)の存在を検索 */
    std::map<std::string, C_param_elem>::iterator it = m_param_map.find(var_name);

    if( it != m_param_map.end() ){
        /* パラメータが存在する */
        /* C_param_elem::isComplexVectorで判別結果を返す */
        return it->second.isComplexVector();
    }
    else{
        /* パラメータが存在しない */
        return false; 
    }
}

/*! @brief  エラーメッセージを取得する 
 *  @return エラーメッセージ 
 */
std::vector<std::string> C_param::getErrMsg() {
    return m_err_msg;
}

/*! @brief  ワーニングメッセージを取得する 
 *  @return ワーニングメッセージ 
 */
std::vector<std::string> C_param::getWarnMsg() {
    return m_warn_msg;
}

/*! @brief  インフォメーションメッセージを取得する 
 *  @return インフォメーションメッセージ 
 */
std::vector<std::string> C_param::getInfoMsg() {
    return m_info_msg;
}

/*! @brief  エラーメッセージをクリアする 
 */
void C_param::clearMsg() {
    m_err_msg.clear();
    m_warn_msg.clear();
    m_info_msg.clear();
}

/*! @brief  dumpする 
 *  @param  os 出力ストリーム 
 */
void C_param::dump(std::ostream& os) {
    int cnum = 0;
    std::map<std::string, C_param_elem >::iterator itrParam;
    for (itrParam = m_param_map.begin();
         itrParam != m_param_map.end(); itrParam++) {
        std::string outmsg = (*itrParam).second.dump();
        /* os << ++cnum << ": " << (*itrParam).first << ": " << outmsg << std::endl; */
        /* 先頭番号削除(diffで差分が取りにくいため) */
        os << (*itrParam).first << ": " << outmsg << std::endl;
    }
}

/* @brief インタフェース取得関数 */
boost::shared_ptr<I_param> createI_param(){
    return boost::shared_ptr<I_param>(new C_param());
}
