/*! 
    @file  C_param.h 
    @brief パラメータクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_PARAM_H
#define C_PARAM_H

#include <map>
#include "C_param_elem.h"
#include "C_Util.h"
#include "I_param.h"

/*! 
  パラメータクラス 
  @brief  パラメータを管理する機能を持つ 
 */
class C_param : public I_param {
private:
    /* メンバ変数 */
    std::map<std::string, C_param_elem > m_param_map;  /*!< パラメータマップ */

    std::vector<std::string> m_err_msg;                /*!< エラーメッセージ */
    std::vector<std::string> m_warn_msg;               /*!< ワーニングメッセージ */
    std::vector<std::string> m_info_msg;               /*!< インフォメーションメッセージ */
public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     */
    C_param(){}

    /*! @brief デコンストラクタ 
     */
    virtual ~C_param(){}

    /*! @brief 存在確認を行う 
     *  @param   var_name パラメータ名 
     *  @return  true: 存在, false: 存在しない 
     */
    virtual bool exist(const std::string &var_name);


    /*! @brief 参照カウンタ更新 
     *  @param   var_name パラメータ名 
     *  @return  true: 追加成功, false: 追加失敗 
     */
    virtual bool touchParam(const std::string &var_name);


    /*! @brief パラメータの追加 静的パラメータ用 
     *  @param var_name パラメータ名 
     *  @param val_str  パラメータの値の文字列 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual bool addParam(const std::string &var_name,const std::string &var_str);

    /*! @brief パラメータの追加 シミュレータ内部変数の追加用 
     *  @param var_name パラメータ名 
     *  @param val      int *型の値 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual bool addParam(const std::string &var_name, int *var);

    /*! @brief 値の取得 静的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual bool getValue(const std::string &var_name,int &val);
    virtual bool getValue(const std::string &var_name,unsigned int &val);
    virtual bool getValue(const std::string &var_name,double &val);
    virtual bool getValue(const std::string &var_name,std::complex<double> &val);
    virtual bool getValue(const std::string &var_name,std::string &val);
    virtual bool getValue(const std::string &var_name,std::vector<int> &val);
    virtual bool getValue(const std::string &var_name,std::vector<unsigned int> &val);
    virtual bool getValue(const std::string &var_name,std::vector<double> &val);
    virtual bool getValue(const std::string &var_name,std::vector<std::complex<double> > &val);
    virtual bool getValue(const std::string &var_name,std::vector<std::string> &val);

    /*! @brief パラメタの設定 動的パラメータ用 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual bool setParam(const std::string &var_name,int* &val);
    virtual bool setParam(const std::string &var_name,unsigned int* &val);
    virtual bool setParam(const std::string &var_name,double* &val);
    virtual bool setParam(const std::string &var_name,std::complex<double>* &val);
    virtual bool setParam(const std::string &var_name,std::string* &val);
    virtual bool setParam(const std::string &var_name,std::vector<int>* &val);
    virtual bool setParam(const std::string &var_name,std::vector<unsigned int>* &val);
    virtual bool setParam(const std::string &var_name,std::vector<double>* &val);
    virtual bool setParam(const std::string &var_name,std::vector<std::complex<double> >* &val);
    virtual bool setParam(const std::string &var_name,std::vector<std::string>* &val);

    /*! @brief 使用されなかったパラメータの取得 
     *  @return  使用されなかったパラメータ 
     */
    virtual std::vector<std::string> getUnusedParam();

    /*! @brief  静的パラメータかどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素が静的パラメータ, false: 静的パラメータではないorパラメータが存在しない 
     */
    virtual bool isStatic(const std::string &var_name);

    /*! @brief  動的パラメータかどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素が動的パラメータ, false: 動的パラメータではないorパラメータが存在しない 
     */
    virtual bool isDynamic(const std::string &var_name);

    /*! @brief  シミュレータ内部変数かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: 要素がシミュレータ内部変数, false: シミュレータ内部変数ではないorパラメータが存在しない 
     */
    virtual bool isMonitor(const std::string &var_name);

    /*! @brief  int型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: int型, false: int型でないorパラメータが存在しない 
     */
    virtual bool isInt(const std::string &var_name);
   
    /*! @brief  unsigned int型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: unsigned int型, false: unsigned int型でないorパラメータが存在しない 
     */
    virtual bool isUnsignedInt(const std::string &var_name);

    /*! @brief  double型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: double型, false: double型でないorパラメータが存在しない 
     */
    virtual bool isDouble(const std::string &var_name);

    /*! @brief  string型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: string型, false: string型でないorパラメータが存在しない 
     */
    virtual bool isString(const std::string &var_name);

    /*! @brief  Complex型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: Complex型, false: Complex型でないorパラメータが存在しない 
     */
    virtual bool isComplex(const std::string &var_name);

    /*! @brief  vector<int>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<int>型, false: vector<int>型でないorパラメータが存在しない 
     */
    virtual bool isIntVector(const std::string &var_name);
   
    /*! @brief  vector<unsigned int>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<unsigned int>型, false: vector<unsigned int>型でないorパラメータが存在しない 
     */
    virtual bool isUnsignedIntVector(const std::string &var_name);

    /*! @brief  vector<double>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<double>型, false: vector<double>型でないorパラメータが存在しない 
     */
    virtual bool isDoubleVector(const std::string &var_name);

    /*! @brief  vector<string>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<string>型, false: vector<string>型でないorパラメータが存在しない 
     */
    virtual bool isStringVector(const std::string &var_name);

    /*! @brief  vector<complex>型かどうかを判定 
     *  @param  var_name パラメータ名 
     *  @return true: vector<complex>型, false: vector<complex>型でないorパラメータが存在しない 
     */
    virtual bool isComplexVector(const std::string &var_name);

    /*! @brief  エラーメッセージを取得する 
     *  @return エラーメッセージ 
     */
    virtual std::vector<std::string> getErrMsg();

    /*! @brief  ワーニングメッセージを取得する 
     *  @return ワーニングメッセージ 
     */
    virtual std::vector<std::string> getWarnMsg();

    /*! @brief  インフォメーションメッセージを取得する 
     *  @return インフォメーションメッセージ 
     */
    virtual std::vector<std::string> getInfoMsg();

    /*! @brief  メッセージをクリアする 
     */
    virtual void clearMsg();

    /*! @brief  dumpする 
     *  @param  os 出力ストリーム 
     */
    virtual void dump(std::ostream& os);

private:
    /* private メンバ関数 */
    /*! @brief コピーコンストラクタ 
     */
    C_param(C_param& /*param*/){}

    /*! @brief 代入演算子 
     */
    C_param operator =(C_param &rhs);

/*-------------------------------------------------------*/
/* GEN2 GREEN互換用 */
/*-------------------------------------------------------*/
private:
    inline bool check(std::string name, C_param_wrap::EParamType type, C_param_wrap::EParamType correct_type)
    {
        if (type != correct_type) {
            std::string msg = "GreenWrapper: EParamType mismatch. name=" + name; 
            g_err->appendMsg(msg, ERROR_MSG);
            return false;
        }
        return true;
    }

public:
    /*! @brief 初期化（GEN2 GREEN互換用） 
     * 未サポート 
     */
    virtual inline int init(const std::string& /*filename*/) {assert(0); return 0;};    /* no support */

    /*! @brief ファイル読み込み（GEN2 GREEN互換用） 
     * 未サポート 
     */
    virtual inline int read(const std::string& /*file*/) {assert(0); return 0;};        /* no support */

    /*! @brief 値の取得 静的パラメータ用（GEN2 GREEN互換用） 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @param   ptype    パラメータの型（未使用） 
     *  @param   rad      パラメータの基数（未使用） 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual inline bool getValue(const std::string& name, int* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, unsigned int* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, double* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::DOUBLE);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::complex<double>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::COMPLEX);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::string* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::STRING);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::vector<int>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT_VECTOR);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::vector<unsigned int>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT_VECTOR);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::vector<double>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::DOUBLE_VECTOR);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::vector<std::complex<double> >* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::COMPLEX_VECTOR);
        return getValue(name, *val);
    };
    virtual inline bool getValue(const std::string& name, std::vector<std::string>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::STRING_VECTOR);
        return getValue(name, *val);
    };

    /*! @brief パラメタの設定 動的パラメータ用（GEN2 GREEN互換用） 
     *  @param   var_name パラメータ名 
     *  @param   val      値 
     *  @param   ptype    パラメータの型（未使用） 
     *  @param   rad      パラメータの基数（未使用） 
     *  @return  true: 取得成功, false: 取得失敗 
     */
    virtual inline int setparam(const std::string& name, int* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, unsigned int* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, double* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::DOUBLE);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::complex<double>* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::COMPLEX);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::string* val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::STRING);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::vector<int>*& val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT_VECTOR);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::vector<unsigned int>*& val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::INT_VECTOR);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::vector<double>*& val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::DOUBLE_VECTOR);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::vector<std::complex<double> >*& val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::COMPLEX_VECTOR);
        return setParam(name, val);
    };
    virtual inline int setparam(const std::string& name, std::vector<std::string>*& val, EParamType ptype, int /*rad*/)
    {
        check(name, ptype, C_param_wrap::STRING_VECTOR);
        return setParam(name, val);
    };

    /*! @brief パラメータの追加 静的パラメータ用（GEN2 GREEN互換用） 
     *  @param var_name パラメータ名 
     *  @param val_str  パラメータの値の文字列 
     *  @param mode     （未使用） 
     *  @return         true: 追加成功, false: 追加失敗 
     */
    virtual inline int setvalue( const std::string& var, const std::string& val, EParamMode /*mode*/) { return addParam(var, val);};

    /*! @brief パラメータの追加 シミュレータ内部変数の取得用（GEN2 GREEN互換用） 
     *  未サポート 
     */
    virtual inline SParamElement* getparam( const std::string& /*name*/) {assert(0); return (SParamElement*)NULL;};        /* no support */

    /*! @brief  ベンダー名を返す（GEN2 GREEN互換用） 
     *  @return ベンダー名 
     */
    virtual inline const std::string getVender() { C_Util u; return u.getVender();};

    /*! @brief  時間の文字列を作成する（GEN2 GREEN互換用） 
     *  @return 時間の文字列 
     */
    virtual inline const std::string getMakeTime() { C_Util u; return u.getMakeTime();};

    /*! @brief  dumpをする（GEN2 GREEN互換用） 
     *  @param  os 出力ストリーム 
     */
    virtual inline void param_dump(std::ostream& os) { dump(os); };
};

#endif
