/*! 
    @file  C_param_elem.cpp 
    @brief パラメータ要素クラス  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/
#include <iomanip>
#include <boost/dynamic_bitset.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>
#include <limits.h>

#include "C_param_elem.h"

/*! @brief 参照カウンタ更新 
 */
void C_param_elem::touch(){
    /* INTの最大値より小さい時のみインクリメント */
    if( m_ref_count < INT_MAX-1 )
        m_ref_count++;
}



/*! @brief 値の取得 静的パラメータ用 
 *  @param val      値 
 *  @return         true: 取得成功, false: 取得失敗 
 */
bool C_param_elem::getValue(int &val){
    int ibuf;

    if( m_type == C_param_elem::NON && m_ip == NULL ){
        /* m_ipがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, ibuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::INT;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_ip = new int;
            /* 値の設定 */
            *m_ip = ibuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::INT ){
        /* 既にINT型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_ip;
    touch();
    return true;
}
bool C_param_elem::getValue(unsigned int &val){
    unsigned int uibuf;

    if( m_type == C_param_elem::NON && m_uip == NULL ){
        /* m_uipがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, uibuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::UNSIGNED_INT;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_uip = new unsigned int;
            /* 値の設定 */
            *m_uip = uibuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::UNSIGNED_INT ){
        /* 既にUNSIGNED INT型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_uip;
    touch();
    return true;
}
bool C_param_elem::getValue(double &val){
    double dbuf;

    if( m_type == C_param_elem::NON && m_dp == NULL ){
        /* m_dpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, dbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::DOUBLE;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_dp = new double;
            /* 値の設定 */
            *m_dp = dbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::DOUBLE ){
        /* 既にDOUBLE型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_dp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::complex<double> &val){
    std::complex<double> cbuf;

    if( m_type == C_param_elem::NON && m_cp == NULL ){
        /* m_cpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, cbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::COMPLEX;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_cp = new std::complex<double>;
            /* 値の設定 */
            *m_cp = cbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::COMPLEX ){
        /* 既にCOMPLEX型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_cp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::string &val){
    std::string sbuf;

    if( m_type == C_param_elem::NON && m_sp == NULL ){
        /* m_spがNULLで型が未確定の場合 */
        /* 型の確定 */
        m_type = C_param_elem::STRING;
        m_state= C_param_elem::STATIC;
        /* 領域確保 */
        m_sp = new std::string;
        /* 値の設定 */
        *m_sp = m_var_str;
    }
    else if( m_type != C_param_elem::STRING ){
        /* 既にSTRING型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_sp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::vector<int> &val){
    std::vector<int> ivbuf;

    if( m_type == C_param_elem::NON && m_ivp == NULL ){
        /* m_ivpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( ivbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::INT_VEC;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_ivp = new std::vector<int>;
            /* 値の設定 */
            *m_ivp = ivbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::INT_VEC ){
        /* 既にINT_VEC型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_ivp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::vector<unsigned int> &val){
    std::vector<unsigned int> uivbuf;

    if( m_type == C_param_elem::NON && m_uivp == NULL ){
        /* m_uivpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( uivbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::UNSIGNED_INT_VEC;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_uivp = new std::vector<unsigned int>;
            /* 値の設定 */
            *m_uivp = uivbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::UNSIGNED_INT_VEC ){
        /* 既にUNSIGNED_INT_VEC型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_uivp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::vector<double> &val){
    std::vector<double> dvbuf;

    if( m_type == C_param_elem::NON && m_dvp == NULL ){
        /* m_dvpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( dvbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::DOUBLE_VEC;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_dvp = new std::vector<double>;
            /* 値の設定 */
            *m_dvp = dvbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::DOUBLE_VEC ){
        /* 既にDOUBLE_VEC型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_dvp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::vector<std::complex<double> > &val){
    std::vector<std::complex<double> > cvbuf;

    if( m_type == C_param_elem::NON && m_cvp == NULL ){
        /* m_cvpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( cvbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::COMPLEX_VEC;
            m_state= C_param_elem::STATIC;
            /* 領域確保 */
            m_cvp = new std::vector<std::complex<double> >;
            /* 値の設定 */
            *m_cvp = cvbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::COMPLEX_VEC ){
        /* 既にCOMPLEX_VEC型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_cvp;
    touch();
    return true;
}
bool C_param_elem::getValue(std::vector<std::string> &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(std::string(""), sep);

    if( m_type == C_param_elem::NON && m_svp == NULL ){
        /* m_svpがNULLで型が未確定の場合 */
        /* 型の確定 */
        m_type = C_param_elem::STRING_VEC;
        m_state= C_param_elem::STATIC;
        /* 領域確保 */
        m_svp = new std::vector<std::string>;
        /* m_var_strをトークン分割 */
        tok.assign(m_var_str.begin(), m_var_str.end());
        /* 値の設定 */
        m_svp->assign(tok.begin(), tok.end());
    }
    else if( m_type != C_param_elem::STRING_VEC ){
        /* 既にSTRING_VEC型以外で領域確保している場合 */
        return false;
    }
    /* NULLでない場合は現在の値を返す */
    val = *m_svp;
    touch();
    return true;
}



/*! @brief ポインタ取得関数 
 *  @param val      値 
 *  @return         true: 取得成功, false: 取得失敗 
 */
bool C_param_elem::getPtr(int* &val){
    int ibuf;

    if( m_type == C_param_elem::NON && m_ip == NULL ){
        /* m_ipがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, ibuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::INT;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_ip = new int;
            /* 値の設定 */
            *m_ip = ibuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::INT ){
        /* 既にINT型以外で領域確保している場合 */
        return false;
    }

    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* ポインタを返す */
    val = m_ip;
    touch();
    return true;
}
bool C_param_elem::getPtr(unsigned int* &val){
    unsigned int uibuf;

    if( m_type == C_param_elem::NON && m_uip == NULL ){
        /* m_uipがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, uibuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::UNSIGNED_INT;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_uip = new unsigned int;
            /* 値の設定 */
            *m_uip = uibuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::UNSIGNED_INT ){
        /* 既にUNSIGNED INT型以外で領域確保している場合 */
        return false;
    }

    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* ポインタを返す */
    val = m_uip;
    touch();
    return true;
}
bool C_param_elem::getPtr(double* &val){
    double dbuf;

    if( m_type == C_param_elem::NON && m_dp == NULL ){
        /* m_dpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, dbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::DOUBLE;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_dp = new double;
            /* 値の設定 */
            *m_dp = dbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::DOUBLE ){
        /* 既にDOUBLE型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_dp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::complex<double>* &val){
    std::complex<double> cbuf;

    if( m_type == C_param_elem::NON && m_cp == NULL ){
        /* m_cpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( m_var_str, cbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::COMPLEX;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_cp = new std::complex<double>;
            /* 値の設定 */
            *m_cp = cbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::COMPLEX ){
        /* 既にCOMPLEX型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_cp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::string* &val){
    std::string sbuf;

    if( m_type == C_param_elem::NON && m_sp == NULL ){
        /* m_spがNULLで型が未確定の場合 */
        /* 型の確定 */
        m_type = C_param_elem::STRING;
        m_state= C_param_elem::DYNAMIC;
        /* 領域確保 */
        m_sp = new std::string;
        /* 値の設定 */
        *m_sp = m_var_str;
    }
    else if( m_type != C_param_elem::STRING ){
        /* 既にSTRING型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_sp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::vector<int>* &val){
    std::vector<int> ivbuf;

    if( m_type == C_param_elem::NON && m_ivp == NULL ){
        /* m_ivpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( ivbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::INT_VEC;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_ivp = new std::vector<int>;
            /* 値の設定 */
            *m_ivp = ivbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::INT_VEC ){
        /* 既にINT_VEC型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_ivp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::vector<unsigned int>* &val){
    std::vector<unsigned int> uivbuf;

    if( m_type == C_param_elem::NON && m_uivp == NULL ){
        /* m_uivpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( uivbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::UNSIGNED_INT_VEC;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_uivp = new std::vector<unsigned int>;
            /* 値の設定 */
            *m_uivp = uivbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::UNSIGNED_INT_VEC ){
        /* 既にUNSIGNED_INT_VEC型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_uivp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::vector<double>* &val){
    std::vector<double> dvbuf;

    if( m_type == C_param_elem::NON && m_dvp == NULL ){
        /* m_dvpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( dvbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::DOUBLE_VEC;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_dvp = new std::vector<double>;
            /* 値の設定 */
            *m_dvp = dvbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::DOUBLE_VEC ){
        /* 既にDOUBLE_VEC型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_dvp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::vector<std::complex<double> >* &val){
    std::vector<std::complex<double> > cvbuf;

    if( m_type == C_param_elem::NON && m_cvp == NULL ){
        /* m_cvpがNULLで型が未確定の場合 */
        /* 文字列から値に変換 */
        if( makeValue( cvbuf ) ){
            /* 変換成功 */
            /* 型の確定 */
            m_type = C_param_elem::COMPLEX_VEC;
            m_state= C_param_elem::DYNAMIC;
            /* 領域確保 */
            m_cvp = new std::vector<std::complex<double> >;
            /* 値の設定 */
            *m_cvp = cvbuf;
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    else if( m_type != C_param_elem::COMPLEX_VEC ){
        /* 既にCOMPLEX_VEC型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_cvp;
    touch();
    return true;
}
bool C_param_elem::getPtr(std::vector<std::string>* &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(std::string(""), sep);

    if( m_type == C_param_elem::NON && m_svp == NULL ){
        /* m_svpがNULLで型が未確定の場合 */
        /* 型の確定 */
        m_type = C_param_elem::STRING_VEC;
        m_state= C_param_elem::DYNAMIC;
        /* 領域確保 */
        m_svp = new std::vector<std::string>;
        /* m_var_strをトークン分割 */
        tok.assign(m_var_str.begin(), m_var_str.end());
        /* 値の設定 */
        m_svp->assign(tok.begin(), tok.end());
    }
    else if( m_type != C_param_elem::STRING_VEC ){
        /* 既にSTRING_VEC型以外で領域確保している場合 */
        return false;
    }
    /* 静的パラメータであれば動的に変更する */
    if( m_state == C_param_elem::STATIC )
        m_state = C_param_elem::DYNAMIC;

    /* NULLでない場合は現在のポインタを返す */
    val = m_svp;
    touch();
    return true;
}



/*! @brief ポインタ設定関数(シミュレータ内部変数用) 
 *  @param val      値 
 *  @return         true: 設定成功, false: 設定失敗 
 */
bool C_param_elem::setPtr(int* &val){
    /* m_ipがNULLでない場合、設定失敗 */
    if( m_ip != NULL )  return false;
    /* m_ipにvalを設定 */
    m_ip = val;
    /* INT型にtype設定 */
    m_type = C_param_elem::INT;
    m_state= C_param_elem::MONITOR;
    /* シミュレータ内部変数フラグtrue */
    m_is_sig = true;

    return true;
}



/*! @brief  未参照パラメータかどうかの判定 
 *  @return true: 未参照, false: 参照あり 
 */
bool C_param_elem::isUnused(){
    return (m_ref_count == 0);
}



/*! @brief  静的パラメータかどうかを判定 
 *  @return true: 要素が静的パラメータ, false: 静的パラメータではない 
 */
bool C_param_elem::isStatic(){
    return (m_state == C_param_elem::STATIC);
}
/*! @brief  動的パラメータかどうかを判定 
 *  @return true: 要素が動的パラメータ, false: 動的パラメータではない 
 */
bool C_param_elem::isDynamic(){
    return (m_state == C_param_elem::DYNAMIC);
}
/*! @brief  シミュレータ内部変数かどうかを判定 
 *  @return true: 要素がシミュレータ内部変数, false: シミュレータ内部変数ではない 
 */
bool C_param_elem::isMonitor(){
    return (m_state == C_param_elem::MONITOR);
}



/*! @brief  int型かどうかを判定 
 *  @return true: int型, false: int型でない 
 */
bool C_param_elem::isInt(){
    return (m_type == C_param_elem::INT);
}
/*! @brief  unsigned int型かどうかを判定 
 *  @return true: unsigned int型, false: unsigned int型でない 
 */
bool C_param_elem::isUnsignedInt(){
    return (m_type == C_param_elem::UNSIGNED_INT);
}
/*! @brief  double型かどうかを判定 
 *  @return true: double型, false: double型でない 
 */
bool C_param_elem::isDouble(){
    return (m_type == C_param_elem::DOUBLE);
}
/*! @brief  string型かどうかを判定 
 *  @return true: string型, false: string型でない 
 */
bool C_param_elem::isString(){
    return (m_type == C_param_elem::STRING);
}
/*! @brief  Complex型かどうかを判定 
 *  @return true: Complex型, false: Complex型でない 
 */
bool C_param_elem::isComplex(){
    return (m_type == C_param_elem::COMPLEX);
}
/*! @brief  vector<int>型かどうかを判定 
 *  @return true: vector<int>型, false: vector<int>型でない 
 */
bool C_param_elem::isIntVector(){
    return (m_type == C_param_elem::INT_VEC);
}
/*! @brief  vector<unsigned int>型かどうかを判定 
 *  @return true: vector<unsigned int>型, false: vector<unsigned int>型でない 
 */
bool C_param_elem::isUnsignedIntVector(){
    return (m_type == C_param_elem::UNSIGNED_INT_VEC);
}
/*! @brief  vector<double>型かどうかを判定 
 *  @return true: vector<double>型, false: vector<double>型でない 
 */
bool C_param_elem::isDoubleVector(){
    return (m_type == C_param_elem::DOUBLE_VEC);
}
/*! @brief  vector<string>型かどうかを判定 
 *  @return true: vector<string>型, false: vector<string>型でない 
 */
bool C_param_elem::isStringVector(){
    return (m_type == C_param_elem::STRING_VEC);
}
/*! @brief  vector<complex>型かどうかを判定 
 *  @return true: vector<complex>型, false: vector<complex>型でない 
 */
bool C_param_elem::isComplexVector(){
    return (m_type == C_param_elem::COMPLEX_VEC);
}

template <typename T>
void outputVal(T& value, std::ostream& os) {
    os << std::setprecision(20) << value;
}
template <class T>
struct SOutput {
    std::ostream& m_os;
    bool fst;
    SOutput(std::ostream& os) : m_os(os), fst(true) {;}
    void operator()(T& value) {
        if (!fst) outputVal(", ", m_os);
        else fst = false;
        outputVal(value, m_os);
    }
};
/*! @brief  dumpする 
 *  @return dump内容 
 */
std::string C_param_elem::dump() {
    std::stringstream ret_ss;
    switch (m_type) {
    case INT:
        outputVal(*m_ip, ret_ss); break;
    case UNSIGNED_INT:
        outputVal(*m_uip, ret_ss); break;
    case DOUBLE:
        outputVal(*m_dp, ret_ss); break;
    case STRING:
        outputVal(*m_sp, ret_ss); break;
    case COMPLEX:
        outputVal(*m_cp, ret_ss); break;
    case INT_VEC:
        for_each(m_ivp->begin(), m_ivp->end(), SOutput<int>(ret_ss)); break;
    case UNSIGNED_INT_VEC:
        for_each(m_uivp->begin(), m_uivp->end(), SOutput<unsigned int>(ret_ss)); break;
    case DOUBLE_VEC:
        for_each(m_dvp->begin(), m_dvp->end(), SOutput<double>(ret_ss)); break;
    case STRING_VEC:
        for_each(m_svp->begin(), m_svp->end(), SOutput<std::string>(ret_ss)); break;
    case COMPLEX_VEC:
        for_each(m_cvp->begin(), m_cvp->end(), SOutput<std::complex<double> >(ret_ss)); break;
    default:
        outputVal(m_var_str, ret_ss); break;
    }
    ret_ss << ": (" << convParamState2Str(m_state) << " " <<  convType2Str(m_type) << "): "
           << m_ref_count << ": ";

    return ret_ss.str();
}



/*! @brief 文字列から値に変換 
 *  @param var_str  値の文字列 
 *  @param val      値 
 *  @return         true: 取得成功, false: 取得失敗 
 */
bool C_param_elem::makeValue(std::string& var_str, int &val){
    std::istringstream iss;

    /* istringstreamに文字列を入れる */
    iss.str(var_str);

    /* 文字列 -> INT型変換 */
    if (var_str.substr(0,2).compare("0x") == 0){
        /* hex -> dec変換 */
        /* 16進数で負の数の場合int型に変換失敗するためunsigned intで取得 */
        unsigned int wk_val;
        iss >> std::hex >> wk_val;
        if( !iss.fail() && iss.eof() ){
            try {
                val = static_cast<int>(wk_val);
                return true;
            } catch (std::bad_cast&) {
                return false;
            }
        }
    }
    else if (var_str.substr(0,2).compare("0b") == 0){
        /* bin -> dec変換 */
        return bin2dec(var_str, val);
    }
    else{
        /* dec -> dec変換 */
        iss >> std::dec >> val;
    }
    
    /* 変換チェック(10,16進のみ) */
    if( iss.fail() || !iss.eof() ){
        /* 入出力に失敗 or EOFまで処理出来ていない */
        return false;
    }
    /* 変換成功 */
    return true;
}
bool C_param_elem::makeValue(std::string& var_str, unsigned int &val){    
    std::istringstream uiss;

    /* istringstreamに文字列を入れる */
    uiss.str(var_str);

    /* 文字列 -> UNSIGNED INT型変換 */
    if (var_str.substr(0,2).compare("0x") == 0){
        /* hex -> dec変換 */
        uiss >> std::hex >> val;
    }
    else if (var_str.substr(0,2).compare("0b") == 0){
        /* bin -> dec変換 */
        return bin2dec(var_str, val);
    }
    else{
        /* dec -> dec変換 */
        uiss >> std::dec >> val;
    }
    
    /* 変換チェック(10,16進のみ) */
    if( uiss.fail() || !uiss.eof() ){
        /* 入出力に失敗 or EOFまで処理出来ていない */
        return false;
    }
    else {
        /* 変換成功なので正しい整数が入っているはず 
        ** 先頭に"-"がある場合は負の数と判断する 
        */
        if(var_str.substr(0,1).compare("-") == 0) {
            /* linuxでは、負の数が設定されている場合にuiss >> std::dec >> val; 
            ** を行うと変換に失敗して処理が終了するので、windowsも動作を合わせる。 
            */
            return false;
        }
    }

    /* 変換成功 */
    return true;
}
bool C_param_elem::makeValue(std::string& var_str, double &val){
    try{
        /* 文字列をDOUBLE型に変換 */
        val = boost::lexical_cast<double>( var_str );
    }
    catch(boost::bad_lexical_cast &){
        /* 変換失敗 */
        return false;
    }
    /* 変換成功 */
    return true;
}
bool C_param_elem::makeValue(std::string& var_str, std::complex<double> &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(var_str, sep);
    std::vector<std::string> vbuf;
    double real;
    double imag;
    std::vector<std::string>::iterator it;

    /* トークンの取得 */
    vbuf.assign(tok.begin(), tok.end());

    /* 奇数の場合は変換失敗 */
    if( vbuf.size() & 0x00000001 ) return false;

    /* 全トークンを参照 */
    for(it = vbuf.begin(); it != vbuf.end(); it+=2){
        /* 実部と虚部の変換 */
        if( makeValue( *it, real ) && makeValue( *(it+1), imag )){
            /* 変換成功 */
            val = std::complex<double>(real, imag);
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    return true;
}
bool C_param_elem::makeValue(std::vector<int> &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(m_var_str, sep);
    std::vector<std::string> vbuf;
    std::vector<std::string>::iterator it;
    int ibuf;

    /* トークンの取得 */
    vbuf.assign(tok.begin(), tok.end());

    /* 全トークンを参照 */
    for(it = vbuf.begin(); it != vbuf.end(); ++it){
        /* 文字列から値に変換 */
        if( makeValue( *it, ibuf ) ){
            /* 変換成功 */
            val.push_back(ibuf);
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    return true;
}
bool C_param_elem::makeValue(std::vector<unsigned int> &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(m_var_str, sep);
    std::vector<std::string> vbuf;
    std::vector<std::string>::iterator it;
    unsigned int uibuf;

    /* トークンの取得 */
    vbuf.assign(tok.begin(), tok.end());

    /* 全トークンを参照 */
    for(it = vbuf.begin(); it != vbuf.end(); ++it){
        /* 文字列から値に変換 */
        if( makeValue( *it, uibuf ) ){
            /* 変換成功 */
            val.push_back(uibuf);
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    return true;
}
bool C_param_elem::makeValue(std::vector<double> &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(m_var_str, sep);
    std::vector<std::string> vbuf;
    std::vector<std::string>::iterator it;
    double dbuf;

    /* トークンの取得 */
    vbuf.assign(tok.begin(), tok.end());

    /* 全トークンを参照 */
    for(it = vbuf.begin(); it != vbuf.end(); ++it){
        /* 文字列から値に変換 */
        if( makeValue( *it, dbuf ) ){
            /* 変換成功 */
            val.push_back(dbuf);
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    return true;
}
bool C_param_elem::makeValue(std::vector<std::complex<double> > &val){
    boost::char_separator<char>                     sep(" ");
    boost::tokenizer<boost::char_separator<char> >  tok(m_var_str, sep);
    std::vector<std::string> vbuf;
    double real;
    double imag;
    std::vector<std::string>::iterator it;

    /* トークンの取得 */
    vbuf.assign(tok.begin(), tok.end());

    /* 奇数の場合は変換失敗 */
    if( vbuf.size() & 0x00000001 ) return false;

    /* 全トークンを参照 */
    for(it = vbuf.begin(); it != vbuf.end(); it+=2){
        /* 実部と虚部の変換 */
        if( makeValue( *it, real ) && makeValue( *(it+1), imag ) ){
            /* 変換成功 */
            val.push_back(std::complex<double>(real, imag));
        }
        else{
            /* 変換失敗 */
            return false;
        }
    }
    return true;
}



/*! @brief 2進数の文字列をINT型に変換 
 *  @param var_str 値の文字列 
 *  @param val     値 
 */
bool C_param_elem::bin2dec(std::string& var_str, int &val){
    std::string sbuf = var_str.substr(2);
    std::string::iterator it = sbuf.begin();
    bool binchk = true;

    for(; it != sbuf.end();){
        /* 入力データのチェック */
        if( *it == '_' ){
            /* アンダースコアは取り除く */
            it = sbuf.erase(it);
            continue;
        }
        else if( *it != '0' && *it != '1'){
            /* 0,1以外 */
            binchk = false;
            break;
        }
        ++it;
    }

    if( binchk ){
        /* 0,1の2進数表記 */
        try{
            /* 2進数から10進数に変換 */
            if(sbuf.length() > 32) {
                /* Linuxだと32bitを超える値を変換してもcatctされないので対策を入れる 
                ** 仮の対応：32文字(前0も含む)を超えた場合にfalseを返す 
                */
                return false;
            }
            val = (int)boost::dynamic_bitset<unsigned int>(sbuf).to_ulong();
        }catch(std::overflow_error &){
            /* 変換エラー */
            return false;
        }
        /* 変換成功 */
        return true;
    }
    else{
        /* 0,1以外の文字がある=変換失敗 */
        return false;
    }
}

bool C_param_elem::bin2dec(std::string& var_str, unsigned int &val){
    std::string sbuf = var_str.substr(2);
    std::string::iterator it = sbuf.begin();
    bool binchk = true;

    for(; it != sbuf.end();){
        /* 入力データのチェック */
        if( *it == '_' ){
            /* アンダースコアは取り除く */
            it = sbuf.erase(it);
            continue;
        }
        else if( *it != '0' && *it != '1'){
            /* 0,1以外 */
            binchk = false;
            break;
        }
        ++it;
    }

    if( binchk ){
        /* 0,1の2進数表記 */
        try{
            /* 2進数から10進数に変換 */
            if(sbuf.length() > 32) {
                /* Linuxだと32bitを超える値を変換してもcatctされないので対策を入れる 
                ** 仮の対応：32文字(前0も含む)を超えた場合にfalseを返す 
                */
                return false;
            }
            val = (unsigned int)boost::dynamic_bitset<unsigned int>(sbuf).to_ulong();
        }catch(std::overflow_error &){
            /* 変換エラー */
            return false;
        }
        /* 変換成功 */
        return true;
    }
    else{
        /* 0,1以外の文字がある=変換失敗 */
        return false;
    }
}

/*! @brief データ型種別を名称に変換 
 *  @param param_type データ型種別 
 *  @return           データ型名称 
 */
std::string C_param_elem::convType2Str(int param_type) {
    return (param_type == INT ? "INT" :
           (param_type == UNSIGNED_INT ? "UNSIGNED_INT" :
           (param_type == DOUBLE ? "DOUBLE" :
           (param_type == STRING ? "STRING" :
           (param_type == COMPLEX ? "COMPLEX" :
           (param_type == INT_VEC ? "INT_VEC" :
           (param_type == UNSIGNED_INT_VEC ? "UNSIGNED_INT_VEC" :
           (param_type == DOUBLE_VEC ? "DOUBLE_VEC" :
           (param_type == STRING_VEC ? "STRING_VEC" :
           (param_type == COMPLEX_VEC ? "COMPLEX_VEC" :
           "NO_TYPE"))))))))));
}

/*! @brief パラメータ状態種別を名称に変換 
 *  @param param_type パラメータ状態種別 
 *  @return           パラメータ状態名称 
 */
std::string C_param_elem::convParamState2Str(int param_state) {
    return (param_state == STATIC ? "static" :
           (param_state == DYNAMIC ? "dynamic" :
           (param_state == MONITOR ? "monitor" :
           "no_state")));
}

