/*! 
    @file  C_param_elem.h 
    @brief パラメータ要素クラス  
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_PARAM_ELEM_H
#define C_PARAM_ELEM_H

#include <vector>
#include <string>
#include <complex>

/*! 
  パラメータ要素クラス 
  @brief  パラメータ要素に関するデータを扱うクラス 
 */
class C_param_elem {
private:
    /* クラス内定義 */

    /*! 
      データ型リストの列挙型 
      @brief  パラメータ要素で扱い可能なデータ型のリスト 
    */
    enum E_param_type {
        NON = 0,
        INT,
        UNSIGNED_INT,
        DOUBLE,
        STRING,
        COMPLEX,
        INT_VEC,
        UNSIGNED_INT_VEC,
        DOUBLE_VEC,
        STRING_VEC,
        COMPLEX_VEC
    };

    /*! 
      パラメータ状態の列挙型 
      @brief  パラメータ状態のリスト 
    */
    enum E_param_state {
        NO_STATE = 0,
        STATIC,
        DYNAMIC,
        MONITOR
    };

    /* メンバ変数 */
    int m_state;            /*!< パラメータの状態 */
    int m_type;             /*!< パラメータの型 */
    int m_ref_count;        /*!< 参照カウント */
    std::string m_var_str;  /*!< 値の文字列 */
    bool m_is_sig;          /*!< シミュレータ内部変数判別 */

    /* 動的パラメータ用ポインタ供用型 */
    union {
        int * m_ip;
        unsigned int * m_uip;
        double *m_dp;
        std::string *m_sp;
        std::complex<double> *m_cp;
        std::vector<int> *m_ivp;
        std::vector<unsigned int> *m_uivp;
        std::vector<double> *m_dvp;
        std::vector<std::string> *m_svp;
        std::vector<std::complex<double> > *m_cvp;
    };

public:    

    /*! @brief コンストラクタ 
     *  @param var_str 値の文字列 
     */
    C_param_elem(const std::string &var_str) 
        : m_state(C_param_elem::NO_STATE), m_type(C_param_elem::NON), m_ref_count(0), m_var_str(var_str), m_is_sig(false), m_ip(NULL)
    {}

    /*! @brief デコンストラクタ 
     */
    ~C_param_elem() {
        /* 型別にdelete */
        switch ( m_type ){
            case C_param_elem::INT :
                /* シミュレータ内部変数でない、かつ、ポインタがNULLでない場合delete */
                if( (m_ip != NULL) && !m_is_sig ) delete m_ip;
                break;
            case C_param_elem::UNSIGNED_INT :
                if( (m_uip != NULL) ) delete m_uip;
                break;
            case C_param_elem::DOUBLE :
                if( (m_dp != NULL) ) delete m_dp;
                break;
            case C_param_elem::STRING :
                if( (m_sp != NULL) ) delete m_sp;
                break;
            case C_param_elem::COMPLEX :
                if( (m_cp != NULL) ) delete m_cp;
                break;
            case C_param_elem::INT_VEC :
                if( (m_ivp != NULL) ) delete m_ivp;
                break;
            case C_param_elem::UNSIGNED_INT_VEC :
                if( (m_uivp != NULL) ) delete m_uivp;
                break;
            case C_param_elem::DOUBLE_VEC :
                if( (m_dvp != NULL) ) delete m_dvp;
                break;
            case C_param_elem::STRING_VEC :
                if( (m_svp != NULL) ) delete m_svp;
                break;
            case C_param_elem::COMPLEX_VEC :
                if( (m_cvp != NULL) ) delete m_cvp;
                break;
        }
    }

    /*! @brief 参照カウンタ更新 
     */
    void touch(); 

    /*! @brief 値の取得 静的パラメータ用 
     *  @param val      値 
     *  @return         true: 取得成功, false: 取得失敗 
     */
    bool getValue(int &val);
    bool getValue(unsigned int &val);
    bool getValue(double &val);
    bool getValue(std::complex<double> &val);
    bool getValue(std::string &val);
    bool getValue(std::vector<int> &val);
    bool getValue(std::vector<unsigned int> &val);
    bool getValue(std::vector<double> &val);
    bool getValue(std::vector<std::complex<double> > &val);
    bool getValue(std::vector<std::string> &val);


    /*! @brief ポインタ取得関数 
     *  @param val      値 
     *  @return         true: 取得成功, false: 取得失敗 
     */
    bool getPtr(int* &val);
    bool getPtr(unsigned int* &val);
    bool getPtr(double* &val);
    bool getPtr(std::complex<double>* &val);
    bool getPtr(std::string* &val);
    bool getPtr(std::vector<int>* &val);
    bool getPtr(std::vector<unsigned int>* &val);
    bool getPtr(std::vector<double>* &val);
    bool getPtr(std::vector<std::complex<double> >* &val);
    bool getPtr(std::vector<std::string>* &val);

    /*! @brief ポインタ設定関数(シミュレータ内部変数用) 
     *  @param val      値 
     *  @return         true: 設定成功, false: 設定失敗 
     */
    bool setPtr(int* &val);

    /*! @brief  未参照パラメータかどうかの判定 
     *  @return true: 未参照, false: 参照あり 
     */
    bool isUnused();
 

    /*! @brief  静的パラメータかどうかを判定 
     *  @return true: 要素が静的パラメータ, false: 静的パラメータではない 
     */
    bool isStatic();

    /*! @brief  動的パラメータかどうかを判定 
     *  @return true: 要素が動的パラメータ, false: 動的パラメータではない 
     */
    bool isDynamic();

    /*! @brief  シミュレータ内部変数かどうかを判定 
     *  @return true: 要素がシミュレータ内部変数, false: シミュレータ内部変数ではない 
     */
    bool isMonitor();

    /*! @brief  int型かどうかを判定 
     *  @return true: int型, false: int型でない 
     */
    bool isInt();
    
    /*! @brief  unsigned int型かどうかを判定 
     *  @return true: unsigned int型, false: unsigned int型でない 
     */
    bool isUnsignedInt();

    /*! @brief  double型かどうかを判定 
     *  @return true: double型, false: double型でない 
     */
    bool isDouble();

    /*! @brief  string型かどうかを判定 
     *  @return true: string型, false: string型でない 
     */
    bool isString();

    /*! @brief  Complex型かどうかを判定 
     *  @return true: Complex型, false: Complex型でない 
     */
    bool isComplex();

    /*! @brief  vector<int>型かどうかを判定 
     *  @return true: vector<int>型, false: vector<int>型でない 
     */
    bool isIntVector();
    
    /*! @brief  vector<unsigned int>型かどうかを判定 
     *  @return true: vector<unsigned int>型, false: vector<unsigned int>型でない 
     */
    bool isUnsignedIntVector();

    /*! @brief  vector<double>型かどうかを判定 
     *  @return true: vector<double>型, false: vector<double>型でない 
     */
    bool isDoubleVector();

    /*! @brief  vector<string>型かどうかを判定 
     *  @return true: vector<string>型, false: vector<string>型でない 
     */
    bool isStringVector();

    /*! @brief  vector<complex>型かどうかを判定 
     *  @return true: vector<complex>型, false: vector<complex>型でない 
     */
    bool isComplexVector();

    /*! @brief  dumpする 
     *  @return dump内容 
     */
    std::string dump();

private :
    /*! @brief 文字列から値に変換 
     *  @param var_str  値の文字列 
     *  @param val      値 
     *  @return         true: 取得成功, false: 取得失敗 
     */
    bool makeValue(std::string& var_str, int &val);
    bool makeValue(std::string& var_str, unsigned int &val);
    bool makeValue(std::string& var_str, double &val);
    bool makeValue(std::string& var_str, std::complex<double> &val);
    bool makeValue(std::vector<int> &val);
    bool makeValue(std::vector<unsigned int> &val);
    bool makeValue(std::vector<double> &val);
    bool makeValue(std::vector<std::complex<double> > &val);
    
    /*! @brief 2進数の文字列をINT型に変換 
     *  @param var_str 値の文字列 
     *  @param val     値 
     */
    bool bin2dec(std::string& var_str, int &val);
    bool bin2dec(std::string& var_str, unsigned int &val);

    /*! @brief データ型種別を名称に変換 
     *  @param param_type データ型種別 
     *  @return           データ型名称 
     */
    std::string convType2Str(int param_type);

    /*! @brief パラメータ状態種別を名称に変換 
     *  @param param_type パラメータ状態種別 
     *  @return           パラメータ状態名称 
     */
    std::string convParamState2Str(int param_state);
};

#endif
