/*! 
    @file  C_param_parser.h 
    @brief パラメータファイルをパースするクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_param_parser.h"
#include "C_token_list.h"
#include "C_err_msg.h"

/*! @brief パースしたい文字列のvectorとパラメータクラスを設定します。 
 *  @param filename  ファイル名 
 *  @param param   パラメータクラス 
 */
void C_param_parser::setData(std::string &filename,
                             boost::shared_ptr<I_param> &param) {
    /* parser_base のメンバ変数のクリア */
    reset();

    /* 設定 */
    m_filename = filename;
    m_param = param;
}


/*! @brief パスからファイル名を取り出します。 
 *  @return      true: パース成功, false: パース失敗 
 */
bool C_param_parser::parse() {
    /* トークン生成 */
    if (!createTokens(m_filename)) return false;

    std::vector<std::string>::iterator begin = m_tokens.begin();
    std::vector<std::string>::iterator end = m_tokens.end();
    std::vector<std::string>::iterator code_begin = begin;
    std::vector<std::string>::iterator code_end;
    bool success = true;

    /* 1論理行ずつ処理 */
    while ((code_end = findCodeEndToken(code_begin,end)) != end) {
        bool code_success = true;

        /* code_begin の インデックス計算を行い行番号を取得  */
        size_t param_token_idx = static_cast<size_t>(code_begin-begin);
        size_t line_num = m_token_line_num_table[param_token_idx];

        /* パラメータ名定義終端のチェック */
        std::vector<std::string>::iterator param_def_end;
        code_success &= checkParamDefEnd(code_begin,code_end,line_num,param_def_end);

        std::vector<std::string> param_name;
        std::string val_str;

        /* パラメータ終端の":"チェック */
        if( param_def_end != code_end ){
            /* パラメータ名の処理 */
            code_success &= parseParamToken(code_begin,param_def_end,line_num,param_name);
            /* 値の処理 */
            code_success &= parseValToken(param_def_end+1,code_end,line_num,val_str);
        }
                                 
        /* パラメータ登録 */
        if (code_success) {
            std::vector<std::string>::iterator ite = param_name.begin();
            for (; ite != param_name.end(); ++ite) {
                m_param->addParam(*ite,val_str);
            }
        }
        code_begin = code_end+1;
        success &= code_success;
    }

    /* ファイルの終端をチェック */
    if (code_begin != end) {
        /* エラー(コマンド終端が見つからない) */
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::PARAM_PARSER_ERR_NO_EXIST_CODE_END);

        /* code_begin の インデックス計算を行い行番号を取得  */
        size_t param_token_idx = static_cast<size_t>(code_begin-begin);
        size_t line_num = m_token_line_num_table[param_token_idx];

        /* エラーメッセージの追加 */
        addErrMsg(msg,line_num);
        success = false;
    }

    return success;
}


/*! @brief   パラメータ定義終端(:)を探索する。 
 *  @param   begin 探索対象トークンの先頭を指すイテレータ 
 *  @param   end   探索対象トークンの終端を指すイテレータ 
 *  @return  パラメータ定義終端を指すイテレータ 
 */
std::vector<std::string>::iterator
C_param_parser::findParamDefEndToken(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end) {
    std::string token(":");             /* 探索対象トークン */
    return findToken(begin,end,token);  /* 探索 */
}


/*! @brief   トークン列のパラメータ名定義終端の存在をチェックする 
 *  @param   begin          対象トークンの先頭を指すイテレータ 
 *  @param   end            対象トークンの終端を指すイテレータ 
 *  @param   line_num       beginが存在する行番号 
 *  @param   param_def_end  パラメータ名定義終端を指すイテレータ (成功時) 
 *  @return  true: 成功, false: 失敗 
 */
bool C_param_parser::checkParamDefEnd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      size_t line_num,
                                      std::vector<std::string>::iterator &param_def_end) {
    bool success = true;

    /* パラメータ名定義終端のチェック */
    param_def_end = findParamDefEndToken(begin,end);

    /* param_def_end がほかにあるかどうかをチェック */
    std::vector<std::string>::iterator other_param_def_end
        = findParamDefEndToken(param_def_end+1,end);

    if (param_def_end == end) {
        /* エラー (パラメータ名定義の終端文字が存在しない) */
        C_err_msg err_msg;
        std::string msg 
            = err_msg.getErrMsg(C_err_msg::PARAM_PARSER_ERR_NO_EXIST_PARAM_DEF_END);
        
        /* エラーメッセージの追加 */
        addErrMsg(msg,line_num);
        success &= false;
    }
    else if (other_param_def_end != end) {
        /* 2つ以上パラメータ名定義終端が存在 */

        /* ":"の検索に失敗と同じ処理を行う */
        param_def_end = end;

        C_err_msg err_msg;
        std::string msg 
            = err_msg.getErrMsg(C_err_msg::PARAM_PARSER_ERR_EXIST_OTHER_PARAM_DEF_END);

        /* エラーメッセージの追加 */
        addErrMsg(msg,line_num);
        success &= false;
    }

    return success;
}

/*! @brief   パラメータ名をパースする 
 *  @param   begin    対象トークンの先頭を指すイテレータ 
 *  @param   end      対象トークンの終端を指すイテレータ 
 *  @param   line_num beginが存在する行番号 
 *  @return  true: パース成功, false: パース失敗 
 */
bool C_param_parser::parseParamToken(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end,
                                     size_t line_num,
                                     std::vector<std::string> &param_name) {
    bool success = true;

    C_token_list param_token_list(begin,end);
    param_name = param_token_list.createParamName(m_param);

    /* パラメータ名のエラーチェック */
    if (param_name.empty()) {
        std::vector<std::string> param_token_err = param_token_list.getErrMsg();

        /* エラーメッセージの追加 */
        for (size_t i = 0; i < param_token_err.size(); i++) {
            addErrMsg(param_token_err[i],line_num);
        }
        success &= false;
    }

    return success;
}

/*! @brief   値トークン列をパースする 
 *  @param   begin      対象トークンの先頭を指すイテレータ 
 *  @param   end        対象トークンの終端を指すイテレータ 
 *  @param   line_num   beginが存在する行番号 
 *  @param   val_str    値の文字列 
 *  @return  true: パース成功, false: パース失敗 
 */
bool C_param_parser::parseValToken(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   size_t line_num,
                                   std::string &val_str) {
    bool success = true;

    C_token_list val_token_list(begin,end);
    val_str = val_token_list.createValStr();

    /* 値のエラーチェック */
    if (val_str == "") {
        std::vector<std::string> val_token_err = val_token_list.getErrMsg();
        for (size_t i = 0; i < val_token_err.size(); i++) {
             /* エラーメッセージの追加 */
            addErrMsg(val_token_err[i],line_num);
        }
        success &= false;
    }

    return success;
}
