/*! 
    @file  C_parser_base.cpp 
    @brief パーサ用ベースクラス  
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include <fstream>
#include <sstream>
#include "C_tokenizer.h"
#include "C_parser_base.h"
#include "C_err_msg.h"
#include <boost/format.hpp>

/*! @brief   メンバ変数のリセットを行う 
 */
void C_parser_base::reset() {
    m_filename = "";
    m_lines.clear();
    m_token_line_num_table.clear();
    m_tokens.clear();
    m_err_msgs.clear();
}

const std::vector<std::string> &C_parser_base::getErrMsg() {
    return m_err_msgs;
}

/*! @brief   ファイルの内容からトークンを生成する 
 *  @param   filename ファイル名 
 *  @return  true 成功、false エラー 
 */
bool C_parser_base::createTokens(std::string &filename) {
    std::ifstream ifs(filename.c_str());

    if (!ifs) {
        /* ファイルオープンチェック */
        C_err_msg err_msg;
        std::string msg = filename + ":" + err_msg.getErrMsg(C_err_msg::PARSER_BASE_ERR_OPEN_FILE);
        m_err_msgs.push_back(msg);
        return false;
    }

    std::string line;
    C_tokenizer tokenizer;
    bool        tag_flag = false;
    while (getline(ifs,line)) {
        /* コメント部分を削除 */
        clearComment(line);

        /* 全角文字が含まれているかどうかのチェック */
        if (checkZenkaku(line)) {
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::PARSER_BASE_ERR_ZENAKKU);
            addErrMsg(msg,m_lines.size()+1);
        }

        /* コメントを除くファイルの先頭行は読まない */
        if ( !tag_flag && !line.empty()){
            tag_flag = true;
            /* 空のファイル行を保存 */
            m_lines.push_back("");
            continue;
        }
        /* トークン取得 */
        tokenizer.setLine(line);
        std::vector<std::string> tokens = tokenizer.getTokens();

        /* ファイル行を保存 */
        m_lines.push_back(line);

        /* トークンを保存 */
        m_tokens.insert(m_tokens.end(),tokens.begin(),tokens.end());

        /* トークン番号とファイル行番号を結び付ける */
        m_token_line_num_table.insert(m_token_line_num_table.end(),
                                      tokens.size(), m_lines.size());
    }

    if( m_token_line_num_table.empty() ){
        /* 空ファイルの場合 */
        C_err_msg err_msg;
        std::string msg = filename + ":" + err_msg.getErrMsg(C_err_msg::PARSER_BASE_ERR_EMPTY_FILE);
        m_err_msgs.push_back(msg);
        return false;
    }
    else if(!m_err_msgs.empty()){
        /* エラーメッセージがある場合(全角文字が含まれている) */
        return false;
    }

    return true;
}


/*! @brief   文字列からコメントを削除する 
 *  @param   str 文字列 
 */
void C_parser_base::clearComment(std::string &str) {
    /* コメントを削除 */
    str = str.substr(0,str.find("//"));
}

/*! @brief   全角文字が含まれているかどうかをチェック 
 *  @param   str 文字列 
 *  @return  1 全角あり, 0 全角なし 
 */
bool C_parser_base::checkZenkaku(std::string &str) {
    bool isZenkaku = false;
    for (size_t i = 0; i < str.length(); i++) {
        unsigned char c = str[i];
        isZenkaku |= (c >= 0x80);
    }
    return isZenkaku;
}


/*! @brief   特定のトークンを探索する。 
 *  @param   token 探索対象トークン 
 *  @return  コード終端を指すイテレータ 
 */
std::vector<std::string>::iterator 
C_parser_base::findToken(std::vector<std::string>::iterator begin,
                         std::vector<std::string>::iterator end,
                         const std::string &token) {

    std::vector<std::string>::iterator ite;
    for (ite = begin; ite != end; ++ite) {
        if (*ite == token) break;
    }
    return ite;
}


/*! @brief   コード終端(;)を探索する。 
 *  @return  コード終端を指すイテレータ 
 */
std::vector<std::string>::iterator 
C_parser_base::findCodeEndToken(std::vector<std::string>::iterator begin,
                                std::vector<std::string>::iterator end) {

    std::string end_token(";");
    return findToken(begin,end,end_token);
}

/*! @brief   エラーメッセージを追加する。 
 *  @param   msg メッセージ  
 */
void C_parser_base::addErrMsg(std::string msg, size_t line_num) {
    std::ostringstream oss("");

    oss << m_filename << "(" << line_num << ") : " << msg;

    m_err_msgs.push_back(oss.str());
}

/*! @brief   エラーメッセージを追加する。 
 *  @param   msg メッセージ  
 */
void C_parser_base::addErrMsg(std::vector<std::string> msgs, size_t line_num) {
    std::vector<std::string>::iterator ite = msgs.begin();
    for (; ite != msgs.end(); ++ite) {
        std::ostringstream oss("");

        oss << m_filename << "(" << line_num << ") : " << *ite;

        m_err_msgs.push_back(oss.str());
    }
}
