/*! 
    @file  C_parser_base.h 
    @brief パーサ用ベースクラス  
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_PARSER_BASE_H
#define C_PARSER_BASE_H

#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>
#include "I_param.h"

/*! パーサ用ベースクラス 
 *  @brief パーサの基本的な機能を持つクラス 
 */
class C_parser_base {
protected:
    /* メンバ変数 */
    boost::shared_ptr<I_param> m_param;       /*!< パラメータ */
    std::string m_filename;                   /*!< パース対象ファイル名 */
    std::vector<std::string> m_lines;         /*!< ファイル内容 */
    std::vector<size_t> m_token_line_num_table;  /*!< トークンと行数のテーブル */
    std::vector<std::string> m_tokens;        /*!< トークン列 */
    std::vector<std::string> m_err_msgs;      /*!< エラーメッセージ */
public:
    /* public メンバ関数 */

    /*! @brief デストラクタ 
     */
    virtual ~C_parser_base() {};

    /*! @brief パースしたいファイルとパラメータクラスを設定します。 
     *  @param filename  ファイル名 
     *  @param param     パラメータ 
     */
    virtual void setData(std::string &filename,
                         boost::shared_ptr<I_param> &param) = 0;

    /*! @brief   ファイルを解析します。 
     *  @return  true: パース成功, false: パース失敗 
     */
    virtual bool parse() = 0;

    /*! @brief   エラーメッセージの取得 
     *  @return  エラーメッセージ 
     */
    const std::vector<std::string> &getErrMsg();

protected:
    /* protected メンバ関数 */

    /*! @brief   メンバ変数のリセットを行う 
     */
    void reset();

    /*! @brief   ファイル内容からトークンを生成する。 
     *  @param   filename ファイル名 
     *  @return  true 成功, false 失敗 
     */
    bool createTokens(std::string &filename);

    /*! @brief   特定のトークンを探索する。 
     *  @param   begin 探索対象トークンの先頭を指すイテレータ 
     *  @param   end   探索対象トークンの終端を指すイテレータ 
     *  @param   token 探索対象トークン 
     *  @return  コード終端を指すイテレータ 
     */
    std::vector<std::string>::iterator findToken(std::vector<std::string>::iterator begin,
                                                 std::vector<std::string>::iterator end,
                                                 const std::string &token);

    /*! @brief   コード終端(;)を探索する。 
     *  @param   begin 探索対象トークンの先頭を指すイテレータ 
     *  @param   end   探索対象トークンの終端を指すイテレータ 
     *  @return  コード終端を指すイテレータ 
     */
    std::vector<std::string>::iterator 
    findCodeEndToken(std::vector<std::string>::iterator begin,
                     std::vector<std::string>::iterator end);

    /*! @brief   エラーメッセージを追加する。 
     *  @param   msg メッセージ  
     */
    void addErrMsg(std::string msg, size_t line_num);

    /*! @brief   エラーメッセージを追加する。 
     *  @param   msg メッセージ  
     */
    void addErrMsg(std::vector<std::string> msgs, size_t line_num);

private:

    /*! @brief   文字列からコメント部分を除去する 
     *  @param   str 文字列 
     */
    void clearComment(std::string &str);

    /*! @brief   全角文字が含まれているかどうかをチェック 
     *  @param   str 文字列 
     *  @return  1 全角あり, 0 全角なし 
     */
    bool checkZenkaku(std::string &str);
};

#endif
