/*! 
    @file  C_rset_cmd_interpreter.cpp 
    @brief 相対設定コマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_rset_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_rset_cmd_interpreter::
C_rset_cmd_interpreter(boost::shared_ptr<I_param> param,
                       boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    : C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_type = C_rset_cmd_interpreter::NON;
    m_check = false;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_rset_cmd_interpreter::reset() {
    m_type = C_rset_cmd_interpreter::NON;
    m_check = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool 
C_rset_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                    std::vector<std::string>::iterator end) {

    /* サイズチェック */
    bool size_check = ((end-begin) > 2);

    /* "+=" の存在をチェック */
    std::string plus_equal_token("+=");
    std::vector<std::string>::iterator plus_equal_pos 
        = findToken(begin,end,plus_equal_token);
    bool plus_equal_token_check = (plus_equal_pos != end);

    /* パラメータ名の取得 */
    C_token_list param_tokens(begin,plus_equal_pos);
    std::vector<std::string> params = param_tokens.createParamName(getParamPtr());

    /* パラメータの存在をチェック */
    bool param_check = (!params.empty());
    std::vector<std::string>::iterator ite = params.begin();
    for (; ite != params.end(); ++ite) {
        param_check &= getParamPtr()->exist(*ite);
    }

    /* 値トークンの存在をチェック */
    bool value_check = ((end - plus_equal_pos) > 1);

    /* パラメータと値のvector形式サイズチェック */
    bool vector_size_check = true;
    if (param_check && value_check) {
        for (ite = params.begin(); ite != params.end(); ++ite) {
            vector_size_check &= checkVectorSize(*ite,plus_equal_pos+1,end);
        }
    }

    m_check = (plus_equal_token_check && param_check && value_check && vector_size_check);

    /* エラーの場合、エラーメッセージの生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     plus_equal_token_check,
                                     param_check,
                                     param_tokens.getErrMsg(),
                                     value_check,
                                     vector_size_check);

    return m_check;
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_rset_cmd_interpreter::checkType(std::vector<std::string>::iterator begin,
                                       std::vector<std::string>::iterator end) {
    if (!m_check) return false;

    bool type_check = true;

    /* "+=" トークン場所取得 */
    std::string plus_equal_token("+=");
    std::vector<std::string>::iterator plus_equal_pos = findToken(begin,end,plus_equal_token);

    /* パラメータ生成 */
    C_token_list param_tokens(begin,plus_equal_pos);
    std::vector<std::string> params = param_tokens.createParamName(getParamPtr());
    std::vector<std::string>::iterator ite = params.begin();

    int type = C_rset_cmd_interpreter::NON;
    bool param_type_check = true;
    bool array_type_check = true;
    bool param_state_static = false;
    bool param_state_monitor = false;
    bool param_state_non = false;
    /* 各パラメータの型をチェック */
    for (; ite != params.end(); ++ite) {
        /* 代入先のパラメータ状態チェック */
        if (!checkStateDynamic(*ite)){
            param_state_static  = checkStateStatic(*ite);
            param_state_monitor = checkStateMonitor(*ite);
            param_state_non     = !( param_state_static || param_state_monitor );
            type_check          = false;
        }

        if (checkAssignTypeInt(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::INT;
        }
        else if (checkAssignTypeUnsignedInt(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::UNSIGNED_INT;
        }
        else if (checkAssignTypeDouble(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::DOUBLE;
        }
        else if (checkAssignTypeString(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::STRING;
        }
        else if (checkAssignTypeComplex(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::COMPLEX;
        }
        else if (checkAssignTypeIntVec(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::INT_VEC;
        }
        else if (checkAssignTypeUnsignedIntVec(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::UNSIGNED_INT_VEC;
        }
        else if (checkAssignTypeDoubleVec(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::DOUBLE_VEC;
        }
        else if (checkAssignTypeStringVec(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::STRING_VEC;
        }
        else if (checkAssignTypeComplexVec(*ite,plus_equal_pos+1,end)) {
            type = C_rset_cmd_interpreter::COMPLEX_VEC;
        }
        else {
            /* エラー パラメータと値で型違反 */
            type_check = false;
            param_type_check = false;
        }

        if (ite == params.begin()) m_type = type;

        if (m_type != type) {
            /* エラー 配列要素間で型違反 */
            type_check = false;
            array_type_check = false;
        }
    }

    /* エラーがあった場合、エラーメッセージの作成 */
    if (!type_check) {
        createTypeErrMsg(param_type_check,
                         array_type_check,
                         param_state_static,
                         param_state_monitor,
                         param_state_non);
        m_type = C_rset_cmd_interpreter::NON;
    }

    return type_check;
}


/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素のvector 
 */
std::vector<C_scenario_elem >
C_rset_cmd_interpreter::createCmds(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   int line_num) {
    std::vector<C_scenario_elem> sc_elems;
    if (!m_check) return sc_elems;

    std::string plus_equal_token("+=");
    std::vector<std::string>::iterator plus_equal_pos = findToken(begin,end,plus_equal_token);

    C_token_list param_tokens(begin,plus_equal_pos);
    std::vector<std::string> params = param_tokens.createParamName(getParamPtr());
    std::vector<std::string>::iterator ite = params.begin();

    for (; ite != params.end(); ++ite) {
        if (m_type == C_rset_cmd_interpreter::INT) {
            sc_elems.push_back(createIntTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::UNSIGNED_INT) {
            sc_elems.push_back(createUnsignedIntTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::DOUBLE) {
            sc_elems.push_back(createDoubleTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::STRING) {
            sc_elems.push_back(createStringTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::COMPLEX) {
            sc_elems.push_back(createComplexTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::INT_VEC) {
            sc_elems.push_back(createIntVecTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::UNSIGNED_INT_VEC) {
            sc_elems.push_back(createUnsignedIntVecTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::DOUBLE_VEC) {
            sc_elems.push_back(createDoubleVecTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::STRING_VEC) {
            sc_elems.push_back(createStringVecTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
        else if (m_type == C_rset_cmd_interpreter::COMPLEX_VEC) {
            sc_elems.push_back(createComplexVecTypeCmd(*ite,plus_equal_pos+1,end,line_num));
        }
    }

    return sc_elems;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createIntTypeCmd(const std::string &param_name,
                                         std::vector<std::string>::iterator val_begin,
                                         std::vector<std::string>::iterator val_end,
                                         int line_num) {
    /* パラメータのポインタ取得 */
    int *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    int *val = new int(token_list.toInt());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createUnsignedIntTypeCmd(const std::string &param_name,
                                                 std::vector<std::string>::iterator val_begin,
                                                 std::vector<std::string>::iterator val_end,
                                                 int line_num) {
    /* パラメータのポインタ取得 */
    unsigned int *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    unsigned int *val = new unsigned int(token_list.toUnsignedInt());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createDoubleTypeCmd(const std::string &param_name,
                                            std::vector<std::string>::iterator val_begin,
                                            std::vector<std::string>::iterator val_end,
                                            int line_num) {
    /* パラメータのポインタ取得 */
    double *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    double *val = new double(token_list.toDouble());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createStringTypeCmd(const std::string &param_name,
                                            std::vector<std::string>::iterator val_begin,
                                            std::vector<std::string>::iterator val_end,
                                            int line_num) {
    /* パラメータのポインタ取得 */
    std::string *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    std::string *val = new std::string(*val_begin);

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createComplexTypeCmd(const std::string &param_name,
                                             std::vector<std::string>::iterator val_begin,
                                             std::vector<std::string>::iterator val_end,
                                             int line_num) {
    /* パラメータのポインタ取得 */
    std::complex<double> *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::complex<double> *val = new std::complex<double>(token_list.toComplex());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createIntVecTypeCmd(const std::string &param_name,
                                            std::vector<std::string>::iterator val_begin,
                                            std::vector<std::string>::iterator val_end,
                                            int line_num) {
    /* パラメータのポインタ取得 */
    std::vector<int> *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::vector<int> *val = new std::vector<int>(token_list.toIntVector());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createUnsignedIntVecTypeCmd(const std::string &param_name,
                                                    std::vector<std::string>::iterator val_begin,
                                                    std::vector<std::string>::iterator val_end,
                                                    int line_num) {
    /* パラメータのポインタ取得 */
    std::vector<unsigned int> *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::vector<unsigned int> *val = new std::vector<unsigned int>(token_list.toUnsignedIntVector());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createDoubleVecTypeCmd(const std::string &param_name,
                                               std::vector<std::string>::iterator val_begin,
                                               std::vector<std::string>::iterator val_end,
                                               int line_num) {
    /* パラメータのポインタ取得 */
    std::vector<double> *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::vector<double> *val = new std::vector<double>(token_list.toDoubleVector());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createStringVecTypeCmd(const std::string &param_name,
                                               std::vector<std::string>::iterator val_begin,
                                               std::vector<std::string>::iterator val_end,
                                               int line_num) {
    /* パラメータのポインタ取得 */
    std::vector<std::string> *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::vector<std::string> *val = new std::vector<std::string>(token_list.toStringVector());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   param_name パラメータ名 
 *  @param   val_begin 対象トークンの先頭を指すイテレータ 
 *  @param   val_end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_rset_cmd_interpreter::createComplexVecTypeCmd(const std::string &param_name,
                                                std::vector<std::string>::iterator val_begin,
                                                std::vector<std::string>::iterator val_end,
                                                int line_num) {
    /* パラメータのポインタ取得 */
    std::vector<std::complex<double> > *param;
    getParamPtr()->setParam(param_name,param);

    /* 値のポインタ取得 */
    C_token_list token_list(val_begin,val_end);
    std::vector<std::complex<double> > *val 
        = new std::vector<std::complex<double> >(token_list.toComplexVector());

    /* 値ポインタの登録 */
    getScPtrTablePtr()->addPtr(val);

    /* C_scenario_elem 追加 */
    C_scenario_elem sc_elem;
    sc_elem.setRelativeSetCmd(param,val,line_num);

    return sc_elem;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_rset_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << '\n'
        << "m_type  = " << m_type;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check        サイズチェック 
 *  @param   plus_equal_token_check "="トークンの存在チェック結果 
 *  @param   param_check       パラメータの存在チェック結果 
 *  @param   param_err_msg     パラメータエラーのメッセージ 
 *  @param   value_check       値の存在チェック結果 
 *  @param   vector_size_check パラメータと値のvectorサイズチェック結果 
 */
void C_rset_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                bool plus_equal_token_check,
                                                bool param_check,
                                                std::vector<std::string> param_err_msg,
                                                bool value_check,
                                                bool vector_size_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* "+="トークンが存在しないときにエラーメッセージを追加 */
    if (!plus_equal_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_NO_EXIST_PLUS_EQUAL);
        addErrMsg(msg);
    }

    /* サイズチェックのエラーメッセージを追加 */
    if ((!size_check || !value_check) && plus_equal_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* パラメータ名取得時エラーメッセージを追加 */
    if (plus_equal_token_check) {
        for (size_t i = 0; i < param_err_msg.size(); i++) {
            addErrMsg(param_err_msg[i]);
        }
    }

    /* パラメータ名が存在しないときにエラーメッセージを追加 */
    if (!param_check && plus_equal_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_PARAM_NAME);
        addErrMsg(msg);
    }

    /* vectorサイズは同じではないときエラーメッセージを追加 */
    if (!vector_size_check && plus_equal_token_check && value_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_SIZE_OF_VECTOR);
        addErrMsg(msg);
    }
}

    /*! @brief   型エラーメッセージの生成 
     *  @param   param_type_check  パラメータと値の型違反チェック 
     *  @param   array_type_check  配列要素間の型違反チェック 
     *  @param   param_state_static  パラメータ状態が静的パラメータ 
     *  @param   param_state_monitor パラメータ状態がシミュレータ内部変数 
     *  @param   param_state_non     パラメータ状態が未取得パラメータ 
     */
void C_rset_cmd_interpreter::createTypeErrMsg(bool param_type_check,
                                              bool array_type_check,
                                              bool param_state_static,
                                              bool param_state_monitor,
                                              bool param_state_non) {

    C_err_msg err_msg;
    std::string msg = "";

    /* パラメータと値の型違反時エラーメッセージを追加 */
    if (!param_type_check && !param_state_non && !param_state_monitor && !param_state_static) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_TYPE_OF_PARAM);
        addErrMsg(msg);
    }

    /* 配列要素間の型違反時エラーメッセージを追加 */
    if (!array_type_check && !param_state_non && !param_state_monitor && !param_state_static) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_TYPE_OF_ARRAY);
        addErrMsg(msg);
    }

    /* 静的パラメータに代入時 */
    if (param_state_static) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_STATE_OF_STATIC_PARAM);
        addErrMsg(msg);
    }

    /* シミュレータ内部変数に代入時 */
    if (param_state_monitor) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_STATE_OF_MONITOR_PARAM);
        addErrMsg(msg);
    }

    /* 型未確定パラメータに代入時 */
    if (param_state_non) {
        msg = err_msg.getErrMsg(C_err_msg::RSET_CMD_INTERPRETER_ERR_STATE_OF_NONSTATE_PARAM);
        addErrMsg(msg);
    }
}
