/*! 
    @file  C_rset_cmd_interpreter.h 
    @brief 相対設定コマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_RSET_CMD_INTERPRETER_H
#define C_RSET_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  相対設定コマンドインタプリタクラス 
  @brief  相対設定コマンドの解釈機能を持つ 
 */
class C_rset_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    enum E_rset_type {
        NON = 0,
        INT,
        UNSIGNED_INT,
        DOUBLE,
        STRING,
        COMPLEX,
        INT_VEC,
        UNSIGNED_INT_VEC,
        DOUBLE_VEC,
        STRING_VEC,
        COMPLEX_VEC
    };

    int m_type;
    bool m_check;

public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_rset_cmd_interpreter(boost::shared_ptr<I_param> param,
                           boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_rset_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkType(std::vector<std::string>::iterator begin,
                   std::vector<std::string>::iterator end);

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素のvector 
     */
    std::vector<C_scenario_elem> createCmds(std::vector<std::string>::iterator begin,
                                            std::vector<std::string>::iterator end,
                                            int line_num);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:
    /* private メンバ関数 */

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {return C_scenario_elem();};
    

    C_scenario_elem createIntTypeCmd(const std::string &param_name,
                                     std::vector<std::string>::iterator val_begin,
                                     std::vector<std::string>::iterator val_end,
                                     int line_num);
    C_scenario_elem createUnsignedIntTypeCmd(const std::string &param_name,
                                             std::vector<std::string>::iterator val_begin,
                                             std::vector<std::string>::iterator val_end,
                                             int line_num);
    C_scenario_elem createDoubleTypeCmd(const std::string &param_name,
                                        std::vector<std::string>::iterator val_begin,
                                        std::vector<std::string>::iterator val_end,
                                        int line_num);
    C_scenario_elem createStringTypeCmd(const std::string &param_name,
                                        std::vector<std::string>::iterator val_begin,
                                        std::vector<std::string>::iterator val_end,
                                        int line_num);
    C_scenario_elem createComplexTypeCmd(const std::string &param_name,
                                         std::vector<std::string>::iterator val_begin,
                                         std::vector<std::string>::iterator val_end,
                                         int line_num);
    C_scenario_elem createIntVecTypeCmd(const std::string &param_name,
                                        std::vector<std::string>::iterator val_begin,
                                        std::vector<std::string>::iterator val_end,
                                        int line_num);
    C_scenario_elem createUnsignedIntVecTypeCmd(const std::string &param_name,
                                                std::vector<std::string>::iterator val_begin,
                                                std::vector<std::string>::iterator val_end,
                                                int line_num);
    C_scenario_elem createDoubleVecTypeCmd(const std::string &param_name,
                                           std::vector<std::string>::iterator val_begin,
                                           std::vector<std::string>::iterator val_end,
                                           int line_num);
    C_scenario_elem createStringVecTypeCmd(const std::string &param_name,
                                           std::vector<std::string>::iterator val_begin,
                                           std::vector<std::string>::iterator val_end,
                                           int line_num);
    C_scenario_elem createComplexVecTypeCmd(const std::string &param_name,
                                            std::vector<std::string>::iterator val_begin,
                                            std::vector<std::string>::iterator val_end,
                                            int line_num);

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check             サイズチェック 
     *  @param   plus_equal_token_check "+="トークンの存在チェック結果 
     *  @param   param_check            パラメータの存在チェック結果 
     *  @param   param_err_msg          パラメータエラーのメッセージ 
     *  @param   value_check            値の存在チェック結果 
     *  @param   vector_size_check      パラメータと値のvectorサイズチェック結果 
     */
    void createFormatErrMsg(bool size_check,
                            bool plus_equal_token_check,
                            bool param_check,
                            std::vector<std::string> param_err_msg,
                            bool value_check,
                            bool vector_size_check);

    /*! @brief   型エラーメッセージの生成 
     *  @param   param_type_check  パラメータと値の型違反チェック 
     *  @param   array_type_check  配列要素間の型違反チェック 
     *  @param   param_state_static  パラメータ状態が静的パラメータ 
     *  @param   param_state_monitor パラメータ状態がシミュレータ内部変数 
     *  @param   param_state_non     パラメータ状態が未取得パラメータ 
     */
    void createTypeErrMsg(bool param_type_check,
                          bool array_type_check,
                          bool param_state_static,
                          bool param_state_monitor,
                          bool param_state_non);
};

#endif

