/*! 
    @file  C_sc_cmd_arg.h 
    @brief シナリオ要素の引数 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SC_CMD_ARG_H
#define C_SC_CMD_ARG_H

#include <vector>
#include <string>
#include <complex>
#include <exception>

/*! 
  シナリオ要素の引数用クラス 
  @brief  シナリオ要素の引数を管理する機能をもつ 
 */
class C_sc_cmd_arg {
private:
    /* クラス内定義 */

    /*! 
      データ型リストの列挙型 
      @brief  パラメータ要素で扱い可能なデータ型のリスト 
    */
    enum E_arg_type {
        INT = 0,
        UNSIGNED_INT,
        DOUBLE,
        STRING,
        COMPLEX,
        INT_VEC,
        UNSIGNED_INT_VEC,
        DOUBLE_VEC,
        STRING_VEC,
        COMPLEX_VEC
    };

    /* メンバ変数 */
    int m_type;             /*!< 引数の型 */

    /* 値用のポインタ供用体 */
    union {
        int *m_ip;
        unsigned int *m_uip;
        double *m_dp;
        std::string *m_sp;
        std::complex<double> *m_cp;
        std::vector<int> *m_ivp;
        std::vector<unsigned int> *m_uivp;
        std::vector<double> *m_dvp;
        std::vector<std::string> *m_svp;
        std::vector<std::complex<double> > *m_cvp;
    };

public:

    /*! @brief コンストラクタ 
     *  @param var  値 
     */
    explicit C_sc_cmd_arg(int *val);
    explicit C_sc_cmd_arg(unsigned int *val);
    explicit C_sc_cmd_arg(double *val);
    explicit C_sc_cmd_arg(std::string *val);
    explicit C_sc_cmd_arg(std::complex<double> *val);
    explicit C_sc_cmd_arg(std::vector<int> *val);
    explicit C_sc_cmd_arg(std::vector<unsigned int> *val);
    explicit C_sc_cmd_arg(std::vector<double> *val);
    explicit C_sc_cmd_arg(std::vector<std::string> *val);
    explicit C_sc_cmd_arg(std::vector<std::complex<double> > *val);


    /*! @brief  値取得関数 
     *  @param  val     値 
     *  @return true 成功、false 失敗 
     */
    bool getVal(int &val);
    bool getVal(unsigned int &val);
    bool getVal(double &val);
    bool getVal(std::string &val);
    bool getVal(std::complex<double> &val);
    bool getVal(std::vector<int> &val);
    bool getVal(std::vector<unsigned int> &val);
    bool getVal(std::vector<double> &val);
    bool getVal(std::vector<std::string> &val);
    bool getVal(std::vector<std::complex<double> > &val);

    /*! @brief  値設定関数 
     *  @param  val     値 
     *  @return true 成功、false 失敗 
     */
    bool setVal(int val);

    /*! @brief  値設定関数 
     *  @param  val     値 
     *  @return true 成功、false 失敗 
     */
    bool setVal(unsigned int val);

    /*! @brief  ポインタ設定関数 
     *  @param  val     値 
     *  @return true 成功、false 失敗 
     */
    bool setPtr(int *val);
    
    /*! @brief  ポインタ設定関数 
     *  @param  val     値 
     *  @return true 成功、false 失敗 
     */
    bool setPtr(unsigned int *val);

    /*! @brief   値の代入を行う 
     *  @param   arg 代入値 
     */
    void assignVal(const C_sc_cmd_arg &arg);

    /*! @brief   加算代入を行う 
     *  @param   rhs  加算代入値 
     */
    void addAndAssignVal(const C_sc_cmd_arg &arg);

    /*! @brief   比較演算を行う 
     *  @param   rhs  右辺 
     *  @return  true 比較成功、false 比較失敗 
     */
    bool operator==(const C_sc_cmd_arg &rhs);
    bool operator!=(const C_sc_cmd_arg &rhs);
    bool operator<(const C_sc_cmd_arg &rhs);
    bool operator<=(const C_sc_cmd_arg &rhs);
    bool operator>(const C_sc_cmd_arg &rhs);
    bool operator>=(const C_sc_cmd_arg &rhs);

    /*! @brief   型の判定を行う。 
     *  @return  true 該当の型、false 該当の型ではない 
     */
    bool isInt();
    bool isUnsignedInt();
    bool isDouble();
    bool isString();
    bool isComplex();
    bool isIntVec();
    bool isUnsignedIntVec();
    bool isDoubleVec();
    bool isStringVec();
    bool isComplexVec();

    /*! @breif 内部のメンバ変数を出力(debug) 
     */
    std::string dump();

private:
    /* vector用 相互加算関数 */
    void addForEach(std::vector<int> *vec1, std::vector<int> *vec2);
    void addForEach(std::vector<unsigned int> *vec1, std::vector<unsigned int> *vec2);
    void addForEach(std::vector<double> *vec1, std::vector<double> *vec2);
    void addForEach(std::vector<std::string> *vec1, std::vector<std::string> *vec2);
    void addForEach(std::vector<std::complex<double> > *vec1,
                    std::vector<std::complex<double> > *vec2);

};

/*! 
  シナリオ要素の引数間演算時型エラー用例外クラス 
  @brief  シナリオ要素の引数間で演算時型エラーが起きた時の例外クラス 
 */
class C_sc_cmd_arg_type_error : public std::exception {
public:
    explicit C_sc_cmd_arg_type_error() {}

    virtual ~C_sc_cmd_arg_type_error() throw() {}

    virtual const char* what() const throw() {
        return "C_sc_cmd_arg type error";
    }
};

#endif
