/*! 
    @file  sc_cmd_interpreter.cpp 
    @brief シナリオコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_sc_cmd_interpreter.h"
#include "C_token_list.h"
#include <sstream>
#include <boost/format.hpp> 

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_sc_cmd_interpreter::C_sc_cmd_interpreter(boost::shared_ptr<I_param> param,
                                           boost::shared_ptr<C_sc_ptr_table> sc_ptr_table) 
    : m_param(param), m_sc_ptr_table(sc_ptr_table)
{}

std::vector<std::string> C_sc_cmd_interpreter::getErrMsg() {
    return m_err_msg;
}


/*! @brief   特定のトークンを探索する。 
 *  @param   token 探索対象トークン 
 *  @return  コード終端を指すイテレータ 
 */
std::vector<std::string>::iterator 
C_sc_cmd_interpreter::findToken(std::vector<std::string>::iterator begin,
                                std::vector<std::string>::iterator end,
                                const std::string &token) {

    std::vector<std::string>::iterator ite;
    for (ite = begin; ite != end; ++ite) {
        if (*ite == token) break;
    }
    return ite;
}

/*! @brief   最後の特定のトークンを探索する。 
 *  @param   token 探索対象トークン 
 *  @return  コード終端を指すイテレータ 
 */
std::vector<std::string>::iterator 
C_sc_cmd_interpreter::findLastToken(std::vector<std::string>::iterator begin,
                                    std::vector<std::string>::iterator end,
                                    const std::string &token) {

    std::vector<std::string>::iterator ite;
    std::vector<std::string>::iterator token_pos = end;
    for (ite = begin; ite != end; ++ite) {
        if (*ite == token) token_pos = ite;
    }
    return token_pos;
}


/*! @brief   条件演算子トークンを探索する。 
 *  @param   begin 探索対象トークンの先頭を指すイテレータ 
 *  @param   end   探索対象トークンの終端を指すイテレータ 
 *  @return  コード終端を指すイテレータ 
 */
std::vector<std::string>::iterator 
C_sc_cmd_interpreter::findCondOpToken(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end) {
    std::string equal("==");
    std::string not_equal("!=");
    std::string less(">");
    std::string less_equal(">=");
    std::string greater("<");
    std::string greater_equal("<=");

    std::vector<std::string>::iterator ite;
    ite = findToken(begin,end,equal);
    if (end != ite) return ite;

    ite = findToken(begin,end,not_equal);
    if (end != ite) return ite;

    ite = findToken(begin,end,less);
    if (end != ite) return ite;

    ite = findToken(begin,end,less_equal);
    if (end != ite) return ite;

    ite = findToken(begin,end,greater);
    if (end != ite) return ite;

    ite = findToken(begin,end,greater_equal);
    return ite;
}

/*! @brief   パラメータ状態をチェック 
 *  @param   param パラメータ名 
 *  @return  true 静的パラメータ、false 静的パラメータ以外 
 */
bool C_sc_cmd_interpreter::checkStateStatic(const std::string &param) {
    return (m_param->isStatic(param));
}
/*! @brief   パラメータ状態をチェック 
 *  @param   param パラメータ名 
 *  @return  true 動的パラメータ、false 動的パラメータ以外 
 */
bool C_sc_cmd_interpreter::checkStateDynamic(const std::string &param) {
    return (m_param->isDynamic(param));
}
/*! @brief   パラメータ状態をチェック 
 *  @param   param パラメータ名 
 *  @return  true シミュレータ内部変数、false シミュレータ内部変数以外 
 */
bool C_sc_cmd_interpreter::checkStateMonitor(const std::string &param) {
    return (m_param->isMonitor(param));
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_cmd_interpreter::checkAssignTypeInt(const std::string &param,
                                              std::vector<std::string>::iterator val_begin,
                                              std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isInt(param) && val_tokens.isInt() && (val_end-val_begin) == 1);
}

bool C_sc_cmd_interpreter::checkAssignTypeUnsignedInt(const std::string &param,
                                                      std::vector<std::string>::iterator val_begin,
                                                      std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isUnsignedInt(param) && val_tokens.isUnsignedInt() && (val_end-val_begin) == 1);
}

bool C_sc_cmd_interpreter::checkAssignTypeDouble(const std::string &param,
                                                 std::vector<std::string>::iterator val_begin,
                                                 std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isDouble(param) && val_tokens.isDouble() && (val_end-val_begin) == 1);
}

bool C_sc_cmd_interpreter::checkAssignTypeString(const std::string &param,
                                                 std::vector<std::string>::iterator val_begin,
                                                 std::vector<std::string>::iterator val_end) {
    return (m_param->isDynamic(param) 
            && m_param->isString(param) && (val_end-val_begin) == 1);
}

bool C_sc_cmd_interpreter::checkAssignTypeComplex(const std::string &param,
                                                  std::vector<std::string>::iterator val_begin,
                                                  std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isComplex(param) && val_tokens.isComplex());
}

bool C_sc_cmd_interpreter::checkAssignTypeIntVec(const std::string &param,
                                                 std::vector<std::string>::iterator val_begin,
                                                 std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isIntVector(param) && val_tokens.isIntVector());
}

bool C_sc_cmd_interpreter::checkAssignTypeUnsignedIntVec(const std::string &param,
                                                         std::vector<std::string>::iterator val_begin,
                                                         std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isUnsignedIntVector(param) && val_tokens.isUnsignedIntVector());
}

bool 
C_sc_cmd_interpreter::checkAssignTypeDoubleVec(const std::string &param,
                                               std::vector<std::string>::iterator val_begin,
                                               std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isDoubleVector(param) && val_tokens.isDoubleVector());
}

bool 
C_sc_cmd_interpreter::checkAssignTypeStringVec(const std::string &param,
                                               std::vector<std::string>::iterator val_begin,
                                               std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isStringVector(param) && val_tokens.isStringVector());
}

bool 
C_sc_cmd_interpreter::checkAssignTypeComplexVec(const std::string &param,
                                                std::vector<std::string>::iterator val_begin,
                                                std::vector<std::string>::iterator val_end) {
    C_token_list val_tokens(val_begin,val_end);
    return (m_param->isDynamic(param) 
            && m_param->isComplexVector(param) && val_tokens.isComplexVector());
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   param パラメータ 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_cmd_interpreter::checkVectorSize(const std::string &param,
                                           std::vector<std::string>::iterator val_begin,
                                           std::vector<std::string>::iterator val_end) {
    bool vector_size_check = true;

    if (checkAssignTypeIntVec(param,val_begin,val_end)) {
        C_token_list val_tokens(val_begin,val_end);
        std::vector<int> param_val;
        m_param->getValue(param,param_val);
        vector_size_check = (param_val.size() == val_tokens.toIntVector().size());
    }
    else if (checkAssignTypeUnsignedIntVec(param,val_begin,val_end)) {
        C_token_list val_tokens(val_begin,val_end);
        std::vector<unsigned int> param_val;
        m_param->getValue(param,param_val);
        vector_size_check = (param_val.size() == val_tokens.toUnsignedIntVector().size());
    }
    else if (checkAssignTypeDoubleVec(param,val_begin,val_end)) {
        C_token_list val_tokens(val_begin,val_end);
        std::vector<double> param_val;
        m_param->getValue(param,param_val);
        vector_size_check = (param_val.size() == val_tokens.toDoubleVector().size());
    }
    else if (checkAssignTypeStringVec(param,val_begin,val_end)) {
        C_token_list val_tokens(val_begin,val_end);
        std::vector<std::string> param_val;
        m_param->getValue(param,param_val);
        vector_size_check = (param_val.size() == val_tokens.toStringVector().size());
    }
    else if (checkAssignTypeComplexVec(param,val_begin,val_end)) {
        C_token_list val_tokens(val_begin,val_end);
        std::vector<std::complex<double> > param_val;
        m_param->getValue(param,param_val);
        vector_size_check = (param_val.size() == val_tokens.toComplexVector().size());
    }

    return vector_size_check;
}


/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   lhs_begin 左辺トークン先頭を指すイテレータ 
 *  @param   lhs_end   左辺トークン終端を指すイテレータ 
 *  @param   rhs_begin 右辺トークン先頭を指すイテレータ 
 *  @param   rhs_end   右辺トークン終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_cmd_interpreter::checkCompTypeInt(std::vector<std::string>::iterator lhs_begin,
                                            std::vector<std::string>::iterator lhs_end,
                                            std::vector<std::string>::iterator rhs_begin,
                                            std::vector<std::string>::iterator rhs_end) {
    C_token_list lhs_token(lhs_begin,lhs_end);
    C_token_list rhs_token(rhs_begin,rhs_end);
    std::vector<std::string> lhs_params = lhs_token.createParamName(m_param);
    std::vector<std::string> rhs_params = rhs_token.createParamName(m_param);

    /* パラメータ名が空かどうかのチェック */
    if (lhs_params.empty() || rhs_params.empty()) return false;

    /* どちらもパラメータに属していない場合 */
    if (!m_param->exist(lhs_params[0]) && !m_param->exist(rhs_params[0])) return false;

    /* 右辺、左辺共にint型パラメータの場合 */
    if (m_param->isInt(rhs_params[0]) && m_param->isInt(lhs_params[0])) return true;

    /* 左辺がint型の値、右辺がint型パラメータの場合 */
    if (m_param->isInt(rhs_params[0]) 
        && (lhs_token.isInt() && !m_param->exist(lhs_params[0]))) return true;

    /* 右辺がint型の値、左辺がint型パラメータの場合 */
    if (m_param->isInt(lhs_params[0]) 
        && (rhs_token.isInt() && !m_param->exist(rhs_params[0]))) return true;

    int val;
    /* 右辺、左辺共に静的パラメータかつint型に解釈できる場合 */
    if (m_param->getValue(lhs_params[0],val)
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 左辺がint型の値、右辺が静的パラメータかつint型に解釈できる場合 */
    if ((rhs_token.isInt() && !m_param->exist(rhs_params[0]))
        && m_param->getValue(lhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 右辺がint型の値、左辺が静的パラメータかつint型に解釈できる場合 */
    if ((lhs_token.isInt() && !m_param->exist(lhs_params[0]))
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    return false;
}

bool C_sc_cmd_interpreter::checkCompTypeUnsignedInt(std::vector<std::string>::iterator lhs_begin,
                                                    std::vector<std::string>::iterator lhs_end,
                                                    std::vector<std::string>::iterator rhs_begin,
                                                    std::vector<std::string>::iterator rhs_end) {
    C_token_list lhs_token(lhs_begin,lhs_end);
    C_token_list rhs_token(rhs_begin,rhs_end);
    std::vector<std::string> lhs_params = lhs_token.createParamName(m_param);
    std::vector<std::string> rhs_params = rhs_token.createParamName(m_param);

    /* パラメータ名が空かどうかのチェック */
    if (lhs_params.empty() || rhs_params.empty()) return false;

    /* どちらもパラメータに属していない場合 */
    if (!m_param->exist(lhs_params[0]) && !m_param->exist(rhs_params[0])) return false;

    /* 右辺、左辺共にunsigned int型パラメータの場合 */
    if (m_param->isUnsignedInt(rhs_params[0]) && m_param->isUnsignedInt(lhs_params[0])) return true;

    /* 左辺がunsigned int型の値、右辺がunsigned int型パラメータの場合 */
    if (m_param->isUnsignedInt(rhs_params[0]) 
        && (lhs_token.isUnsignedInt() && !m_param->exist(lhs_params[0]))) return true;

    /* 右辺がunsigned int型の値、左辺がunsigned int型パラメータの場合 */
    if (m_param->isUnsignedInt(lhs_params[0]) 
        && (rhs_token.isUnsignedInt() && !m_param->exist(rhs_params[0]))) return true;

    unsigned int val;
    /* 右辺、左辺共に静的パラメータかつunsigned int型に解釈できる場合 */
    if (m_param->getValue(lhs_params[0],val)
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 左辺がunsigned int型の値、右辺が静的パラメータかつunsigned int型に解釈できる場合 */
    if ((rhs_token.isUnsignedInt() && !m_param->exist(rhs_params[0]))
        && m_param->getValue(lhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 右辺がunsigned int型の値、左辺が静的パラメータかつunsigned int型に解釈できる場合 */
    if ((lhs_token.isUnsignedInt() && !m_param->exist(lhs_params[0]))
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    return false;
}

bool C_sc_cmd_interpreter::checkCompTypeDouble(std::vector<std::string>::iterator lhs_begin,
                                               std::vector<std::string>::iterator lhs_end,
                                               std::vector<std::string>::iterator rhs_begin,
                                               std::vector<std::string>::iterator rhs_end) {
    C_token_list lhs_token(lhs_begin,lhs_end);
    C_token_list rhs_token(rhs_begin,rhs_end);
    std::vector<std::string> lhs_params = lhs_token.createParamName(m_param);
    std::vector<std::string> rhs_params = rhs_token.createParamName(m_param);

    /* パラメータ名が空かどうかのチェック */
    if (lhs_params.empty() || rhs_params.empty()) return false;

    /* どちらもパラメータに属していない場合 */
    if (!m_param->exist(lhs_params[0]) && !m_param->exist(rhs_params[0])) return false;

    /* 右辺、左辺共にdouble型パラメータの場合 */
    if (m_param->isDouble(rhs_params[0]) && m_param->isDouble(lhs_params[0])) return true;

    /* 左辺がdouble型の値、右辺がdouble型パラメータの場合 */
    if (m_param->isDouble(rhs_params[0]) 
        && (lhs_token.isDouble() && !m_param->exist(lhs_params[0]))) return true;

    /* 右辺がdouble型の値、左辺がdouble型パラメータの場合 */
    if (m_param->isDouble(lhs_params[0]) 
        && (rhs_token.isDouble() && !m_param->exist(rhs_params[0]))) return true;

    double val;
    /* 右辺、左辺共に静的パラメータかつdouble型に解釈できる場合 */
    if (m_param->getValue(lhs_params[0],val)
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 左辺がdouble型の値、右辺が静的パラメータかつdouble型に解釈できる場合 */
    if ((rhs_token.isDouble() && !m_param->exist(rhs_params[0]))
        && m_param->getValue(lhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 右辺がdouble型の値、左辺が静的パラメータかつdouble型に解釈できる場合 */
    if ((lhs_token.isDouble() && !m_param->exist(lhs_params[0]))
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    return false;
}

bool C_sc_cmd_interpreter::checkCompTypeString(std::vector<std::string>::iterator lhs_begin,
                                               std::vector<std::string>::iterator lhs_end,
                                               std::vector<std::string>::iterator rhs_begin,
                                               std::vector<std::string>::iterator rhs_end) {
    C_token_list lhs_token(lhs_begin,lhs_end);
    C_token_list rhs_token(rhs_begin,rhs_end);
    std::vector<std::string> lhs_params = lhs_token.createParamName(m_param);
    std::vector<std::string> rhs_params = rhs_token.createParamName(m_param);

    /* パラメータ名が空かどうかのチェック */
    if (lhs_params.empty() || rhs_params.empty()) return false;

    /* どちらもパラメータに属していない場合 */
    if (!m_param->exist(lhs_params[0]) && !m_param->exist(rhs_params[0])) return false;

    /* 右辺、左辺共にstring型パラメータの場合 */
    if (m_param->isString(rhs_params[0]) && m_param->isString(lhs_params[0])) return true;

    /* 左辺がstring型の値、右辺がstring型パラメータの場合 */
    if (m_param->isString(rhs_params[0]) 
        && (lhs_token.isString() && !m_param->exist(lhs_params[0]))) return true;

    /* 右辺がstring型の値、左辺がstring型パラメータの場合 */
    if (m_param->isString(lhs_params[0]) 
        && (rhs_token.isString() && !m_param->exist(rhs_params[0]))) return true;

    std::string val;
    /* 右辺、左辺共に静的パラメータかつdouble型に解釈できる場合 */
    if (m_param->getValue(lhs_params[0],val)
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 左辺がstring型の値、右辺が静的パラメータかつstring型に解釈できる場合 */
    if ((rhs_token.isString() && !m_param->exist(rhs_params[0]))
        && m_param->getValue(lhs_params[0],val)) return true;
    m_param->clearMsg();

    /* 右辺がstring型の値、左辺が静的パラメータかつstring型に解釈できる場合 */
    if ((lhs_token.isString() && !m_param->exist(lhs_params[0]))
        && m_param->getValue(rhs_params[0],val)) return true;
    m_param->clearMsg();

    return false;
}

/*! @brief   トークン列から条件式内の型情報とパラメータ名の情報を生成する 
 *  @param   lhs_begin 左辺トークン先頭を指すイテレータ 
 *  @param   lhs_end   左辺トークン終端を指すイテレータ 
 *  @param   rhs_begin 右辺トークン先頭を指すイテレータ 
 *  @param   rhs_end   右辺トークン終端を指すイテレータ 
 *  @return  型情報 
 */
std::string 
C_sc_cmd_interpreter::createCompTypeInfo(std::vector<std::string>::iterator lhs_begin,
                                         std::vector<std::string>::iterator lhs_end,
                                         std::vector<std::string>::iterator rhs_begin,
                                         std::vector<std::string>::iterator rhs_end) {
    C_token_list lhs_token(lhs_begin,lhs_end);
    C_token_list rhs_token(rhs_begin,rhs_end);
    std::vector<std::string> lhs_params = lhs_token.createParamName(m_param);
    std::vector<std::string> rhs_params = rhs_token.createParamName(m_param);

    std::string lhs_type_info;
    std::string rhs_type_info;
    std::string lhs_state;
    std::string rhs_state;

    if (m_param->exist(lhs_params[0])) {
        lhs_type_info = createParamTypeInfo(lhs_params[0]);
        lhs_state = "parameter";
    }
    else {
        lhs_type_info = createValueTypeInfo(lhs_begin,lhs_end);
        lhs_state = "value";
    }
    if (m_param->exist(rhs_params[0])) {
        rhs_type_info = createParamTypeInfo(rhs_params[0]);
        rhs_state = "parameter";
    }
    else {
        rhs_type_info = createValueTypeInfo(rhs_begin,rhs_end);
        rhs_state = "value";
    }

    std::string fmt = "\"%s\" is %s %s. \"%s\" is %s %s.";

    std::ostringstream oss("");
    oss << boost::format(fmt) 
        % lhs_params[0] % lhs_type_info % lhs_state
        % rhs_params[0] % rhs_type_info % rhs_state;

    return oss.str();
}

/*! @brief   パラメータの型情報を生成する 
 *  @param   param_name パラメータ名 
 *  @return  型情報 
 */
std::string 
C_sc_cmd_interpreter::createParamTypeInfo(std::string param_name) {
    std::string type_info("no_type");

    if (m_param->exist(param_name) && m_param->isInt(param_name)) {
        type_info = "int";
    }
    else if (m_param->exist(param_name) && m_param->isUnsignedInt(param_name)) {
        type_info = "unsigned int";
    }
    else if (m_param->exist(param_name) && m_param->isDouble(param_name)) {
        type_info = "double";
    }
    else if (m_param->exist(param_name) && m_param->isString(param_name)) {
        type_info = "string";
    }
    else if (m_param->exist(param_name) && m_param->isComplex(param_name)) {
        type_info = "complex<double>";
    }
    else if (m_param->exist(param_name) && m_param->isIntVector(param_name)) {
        type_info = "vector<int>";
    }
    else if (m_param->exist(param_name) && m_param->isUnsignedIntVector(param_name)) {
        type_info = "vector<unsigned int>";
    }
    else if (m_param->exist(param_name) && m_param->isDoubleVector(param_name)) {
        type_info = "vector<double>";
    }
    else if (m_param->exist(param_name) && m_param->isStringVector(param_name)) {
        type_info = "vector<string>";
    }
    else if (m_param->exist(param_name) && m_param->isComplexVector(param_name)) {
        type_info = "vector<complex<double> >";
    }

    return type_info;
}

/*! @brief   値の型情報を生成する 
 *  @param   lhs_begin 値トークン先頭を指すイテレータ 
 *  @param   lhs_end   値トークン終端を指すイテレータ 
 *  @return  型情報 
 */
std::string 
C_sc_cmd_interpreter::createValueTypeInfo(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end) {
    std::string type_info("no_type");

    C_token_list val_token(begin,end);
    if (val_token.isInt()) {
        type_info = "int";
    }
    else if (val_token.isUnsignedInt()) {
        type_info = "unsigned int";
    }
    else if (val_token.isDouble()) {
        type_info = "double";
    }
    else if (val_token.isString()) {
        type_info = "string";
    }
    else if (val_token.isComplex()) {
        type_info = "complec<double>";
    }
    else if (val_token.isIntVector()) {
        type_info = "vector<int>";
    }
    else if (val_token.isUnsignedIntVector()) {
        type_info = "vector<unsigned int>";
    }
    else if (val_token.isDoubleVector()) {
        type_info = "vector<double>";
    }
    else if (val_token.isStringVector()) {
        type_info = "vector<string>";
    }
    else if (val_token.isInt()) {
        type_info = "vector<complex<double> >";
    }

    return type_info;
}

/*! @brief   エラーメッセージを追加する 
 *  @param   msg メッセージ 
 */
void C_sc_cmd_interpreter::addErrMsg(std::string msg) {
    m_err_msg.push_back(msg);
}

/*! @brief   エラーメッセージをクリアする 
 */
void C_sc_cmd_interpreter::clearErrMsg() {
    m_err_msg.clear();
}
