/*! 
    @file  sc_cmd_interpreter.h 
    @brief シナリオコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SC_CMD_INTERPRETER_H
#define C_SC_CMD_INTERPRETER_H

#include <vector>
#include <map>
#include <stack>
#include <string>
#include <boost/shared_ptr.hpp>
#include "C_scenario_elem.h"
#include "C_sc_ptr_table.h"
#include "I_param.h"

/*! 
  シナリオコマンドインタプリタ基底クラス 
  @brief  シナリオコマンドを解釈するための基本的機能を持つクラス 
 */
class C_sc_cmd_interpreter {
private:
    /* メンバ変数 */

    /* パラメータ */
    boost::shared_ptr<I_param> m_param;

    /* シナリオコマンド生成時に新規生成するポインタの管理テーブル */
    boost::shared_ptr<C_sc_ptr_table> m_sc_ptr_table;

    /* エラーメッセージテーブル */
    std::vector<std::string> m_err_msg;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_sc_cmd_interpreter(boost::shared_ptr<I_param> param,
                         boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);

    /*! @brief   デストラクタ 
     */
    virtual ~C_sc_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset() = 0;

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end) = 0;

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) = 0;

    /*! @brief   エラーメッセージを取得する 
     *  @return  エラーメッセージ 
     */
    std::vector<std::string> getErrMsg();

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump() = 0;

protected:

    /*! @brief   boost::shared_ptr<I_param> を取得する。 
     *  @return  boost::shared_ptr<I_param>  
     */
    boost::shared_ptr<I_param> getParamPtr() {return m_param;};

    /*! @brief   boost::shared_ptr<C_sc_ptr_table> を取得する。 
     *  @return  boost::shared_ptr<C_sc_ptr_table> 
     */
    boost::shared_ptr<C_sc_ptr_table> getScPtrTablePtr() {return m_sc_ptr_table;};

    /*! @brief   特定のトークンを探索する。 
     *  @param   begin 探索対象トークンの先頭を指すイテレータ 
     *  @param   end   探索対象トークンの終端を指すイテレータ 
     *  @param   token 探索対象トークン 
     *  @return  コード終端を指すイテレータ 
     */
    std::vector<std::string>::iterator findToken(std::vector<std::string>::iterator begin,
                                                 std::vector<std::string>::iterator end,
                                                 const std::string &token);

    /*! @brief   最後の特定のトークンを探索する。  
     *  @param   begin 探索対象トークンの先頭を指すイテレータ  
     *  @param   end   探索対象トークンの終端を指すイテレータ  
     *  @param   token 探索対象トークン  
     *  @return  コード終端を指すイテレータ  
     */
    std::vector<std::string>::iterator findLastToken(std::vector<std::string>::iterator begin,
                                                     std::vector<std::string>::iterator end,
                                                     const std::string &token);

    /*! @brief   条件演算子トークンを探索する。 
     *  @param   begin 探索対象トークンの先頭を指すイテレータ 
     *  @param   end   探索対象トークンの終端を指すイテレータ 
     *  @return  コード終端を指すイテレータ 
     */
    std::vector<std::string>::iterator findCondOpToken(std::vector<std::string>::iterator begin,
                                                       std::vector<std::string>::iterator end);


    /*! @brief   パラメータ状態をチェック 
     *  @param   param パラメータ名 
     *  @return  true 静的パラメータ、false 静的パラメータ以外 
     */
    bool checkStateStatic(const std::string &param);

    /*! @brief   パラメータ状態をチェック 
     *  @param   param パラメータ名 
     *  @return  true 動的パラメータ、false 動的パラメータ以外 
     */
    bool checkStateDynamic(const std::string &param);

    /*! @brief   パラメータ状態をチェック 
     *  @param   param パラメータ名 
     *  @return  true シミュレータ内部変数、false シミュレータ内部変数以外 
     */
    bool checkStateMonitor(const std::string &param);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkAssignTypeInt(const std::string &param,
                            std::vector<std::string>::iterator val_begin,
                            std::vector<std::string>::iterator val_end);
    bool checkAssignTypeUnsignedInt(const std::string &param,
                                    std::vector<std::string>::iterator val_begin,
                                    std::vector<std::string>::iterator val_end);
    bool checkAssignTypeDouble(const std::string &param,
                               std::vector<std::string>::iterator val_begin,
                               std::vector<std::string>::iterator val_end);
    bool checkAssignTypeString(const std::string &param,
                               std::vector<std::string>::iterator val_begin,
                               std::vector<std::string>::iterator val_end);
    bool checkAssignTypeComplex(const std::string &param,
                                std::vector<std::string>::iterator val_begin,
                                std::vector<std::string>::iterator val_end);
    bool checkAssignTypeIntVec(const std::string &param,
                               std::vector<std::string>::iterator val_begin,
                               std::vector<std::string>::iterator val_end);
    bool checkAssignTypeUnsignedIntVec(const std::string &param,
                                       std::vector<std::string>::iterator val_begin,
                                       std::vector<std::string>::iterator val_end);
    bool checkAssignTypeDoubleVec(const std::string &param,
                                  std::vector<std::string>::iterator val_begin,
                                  std::vector<std::string>::iterator val_end);
    bool checkAssignTypeStringVec(const std::string &param,
                                  std::vector<std::string>::iterator val_begin,
                                  std::vector<std::string>::iterator val_end);
    bool checkAssignTypeComplexVec(const std::string &param,
                                   std::vector<std::string>::iterator val_begin,
                                   std::vector<std::string>::iterator val_end);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   param パラメータ 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkVectorSize(const std::string &param,
                         std::vector<std::string>::iterator val_begin,
                         std::vector<std::string>::iterator val_end);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   lhs_begin 左辺トークン先頭を指すイテレータ 
     *  @param   lhs_end   左辺トークン終端を指すイテレータ 
     *  @param   rhs_begin 右辺トークン先頭を指すイテレータ 
     *  @param   rhs_end   右辺トークン終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkCompTypeInt(std::vector<std::string>::iterator lhs_begin,
                          std::vector<std::string>::iterator lhs_end,
                          std::vector<std::string>::iterator rhs_begin,
                          std::vector<std::string>::iterator rhs_end);
    bool checkCompTypeUnsignedInt(std::vector<std::string>::iterator lhs_begin,
                                  std::vector<std::string>::iterator lhs_end,
                                  std::vector<std::string>::iterator rhs_begin,
                                  std::vector<std::string>::iterator rhs_end);
    bool checkCompTypeDouble(std::vector<std::string>::iterator lhs_begin,
                             std::vector<std::string>::iterator lhs_end,
                             std::vector<std::string>::iterator rhs_begin,
                             std::vector<std::string>::iterator rhs_end);
    bool checkCompTypeString(std::vector<std::string>::iterator lhs_begin,
                             std::vector<std::string>::iterator lhs_end,
                             std::vector<std::string>::iterator rhs_begin,
                             std::vector<std::string>::iterator rhs_end);

    /*! @brief   トークン列から条件式内の型情報とパラメータ名の情報を生成する 
     *  @param   lhs_begin 左辺トークン先頭を指すイテレータ 
     *  @param   lhs_end   左辺トークン終端を指すイテレータ 
     *  @param   rhs_begin 右辺トークン先頭を指すイテレータ 
     *  @param   rhs_end   右辺トークン終端を指すイテレータ 
     *  @return  型情報 
     */
    std::string createCompTypeInfo(std::vector<std::string>::iterator lhs_begin,
                                   std::vector<std::string>::iterator lhs_end,
                                   std::vector<std::string>::iterator rhs_begin,
                                   std::vector<std::string>::iterator rhs_end);

    /*! @brief   パラメータの型情報を生成する 
     *  @param   param_name パラメータ名 
     *  @return  型情報 
     */
    std::string createParamTypeInfo(std::string param_name);

    /*! @brief   値の型情報を生成する 
     *  @param   lhs_begin 値トークン先頭を指すイテレータ 
     *  @param   lhs_end   値トークン終端を指すイテレータ 
     *  @return  型情報 
     */
    std::string createValueTypeInfo(std::vector<std::string>::iterator begin,
                                    std::vector<std::string>::iterator end);


    /*! @brief   エラーメッセージを追加する 
     *  @param   msg メッセージ 
     */
    void addErrMsg(std::string msg);

    /*! @brief   エラーメッセージをクリアする 
     */
    void clearErrMsg();
};

#endif

