/*! 
    @file  sc_parser.cpp 
    @brief シナリオファイルパーサクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_sc_parser.h"
#include "C_dset_cmd_interpreter.h"
#include "C_rset_cmd_interpreter.h"
#include "C_tdset_cmd_interpreter.h"
#include "C_trset_cmd_interpreter.h"
#include "C_for_cmd_interpreter.h"
#include "C_wait_cmd_interpreter.h"
#include "C_jump_cmd_interpreter.h"
#include "C_call_cmd_interpreter.h"
#include "C_scdef_cmd_interpreter.h"
#include "C_scend_cmd_interpreter.h"
#include "C_subdef_cmd_interpreter.h"
#include "C_subend_cmd_interpreter.h"
#include "C_labeldef_cmd_interpreter.h"
#include "C_simstop_cmd_interpreter.h"
#include "C_err_msg.h"
#include <set>


/*! @brief パースしたい文字列のvectorとパラメータクラスを設定します。 
 *  @param filename  ファイル名 
 *  @param param   パラメータクラス 
 */
void C_sc_parser::setData(std::string &filename,
                          boost::shared_ptr<I_param> &param) {
    reset();
    m_def_flag = false;
    m_code_table.clear();
    m_sc_map.clear();
    m_sc_time_ptr_map.clear();
    m_time_cmd_map.clear();
    m_sub_def_map.clear();
    m_sub_caller_map.clear();
    m_label_def_map.clear();
    m_label_use_map.clear();

    while (!m_for_label_stack.empty()) m_for_label_stack.pop();
    while (!m_for_end_cmd_stack.empty()) m_for_end_cmd_stack.pop();

    if (m_sc_ptr_table == NULL) 
        m_sc_ptr_table = boost::shared_ptr<C_sc_ptr_table>(new C_sc_ptr_table());

    m_filename = filename;
    m_param = param;
}

/*! @brief    パスからファイル名を取り出します。 
 *  @return   true: パース成功, false: パース失敗 
 */
bool C_sc_parser::parse() {
    /* トークン生成 */
    if (!createTokens(m_filename)) return false;

    std::vector<std::string>::iterator begin = m_tokens.begin();  /* ファイル先頭 */
    std::vector<std::string>::iterator end = m_tokens.end();      /* ファイル終端 */
    std::vector<std::string>::iterator code_begin = begin;        /* 論理行先頭 */
    std::vector<std::string>::iterator code_end;                  /* 論理行終端 */
    bool success = true;                                          /* パース成功・失敗 */
    while ((code_end = findCodeEndToken(code_begin,end)) != end) {

        int line_num = m_token_line_num_table[code_begin-begin];

        if (isLabelDefCmd(code_begin,code_end)) {
            /* ラベル定義終端の探索 */
            std::vector<std::string>::iterator labeldef_end 
                = findToken(code_begin,code_end,std::string(":"));

            /* ラベル定義コマンドの解釈 */
            success &= interpretLabelDefCmd(code_begin,labeldef_end+1,line_num);
            code_end = labeldef_end;
        }
        else if (isTimeRelativeSetCmd(code_begin,code_end)) {
            /* 実行時刻相対設定コマンドの解釈 */
            success &= interpretTimeRelativeSetCmd(code_begin,code_end,line_num);
        }
        else if (isTimeDirectSetCmd(code_begin,code_end)) {
            /* 実行時刻直接設定コマンドの解釈 */
            success &= interpretTimeDirectSetCmd(code_begin,code_end,line_num);
        }
        else if (isDirectSetCmd(code_begin,code_end)) {
            /* パラメータ直接設定コマンドの解釈 */
            success &= interpretDirectSetCmd(code_begin,code_end,line_num);
        }
        else if (isRelativeSetCmd(code_begin,code_end)) {
            /* パラメータ相対設定コマンドの解釈 */
            success &= interpretRelativeSetCmd(code_begin,code_end,line_num);
        }
        else if (isForCmd(code_begin,code_end)) {
            /* FORコマンドの解釈 */
            success &= interpretForCmd(code_begin,code_end,line_num);
        }
        else if (isWaitCmd(code_begin,code_end)) {
            /* WAITコマンドの解釈 */
            success &= interpretWaitCmd(code_begin,code_end,line_num);
        }
        else if (isJumpCmd(code_begin,code_end)) {
            /* JUMPコマンドの解釈 */
            success &= interpretJumpCmd(code_begin,code_end,line_num);
        }
        else if (isCallCmd(code_begin,code_end)) {
            /* CALLコマンドの解釈 */
            success &= interpretCallCmd(code_begin,code_end,line_num);
        }
        else if (isScenarioDefCmd(code_begin,code_end)) {
            /* シナリオ定義コマンドの解釈 */
            success &= interpretScenarioDefCmd(code_begin,code_end,line_num);
        }
        else if (isScenarioEndCmd(code_begin,code_end)) {
            /* シナリオ終端コマンドの解釈 */
            success &= interpretScenarioEndCmd(code_begin,code_end,line_num);
        }
        else if (isSubDefCmd(code_begin,code_end)) {
            /* サブルーチン定義コマンドの解釈 */
            success &= interpretSubDefCmd(code_begin,code_end,line_num);
        }
        else if (isSubEndCmd(code_begin,code_end)) {
            /* サブルーチン終端コマンドの解釈 */
            success &= interpretSubEndCmd(code_begin,code_end,line_num);
        }
        else if (isSimStopCmd(code_begin,code_end)) {
            /* ラベル定義コマンドの解釈 */
            success &= interpretSimStopCmd(code_begin,code_end,line_num);
        }
        else {
            /* エラー */
            success = false;
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_UNDEF_CMD);
            size_t line_num = m_token_line_num_table[(code_begin-begin)];
            addErrMsg(msg,line_num);
        }

        /* イテレータの更新 */
        code_begin = code_end + 1;
    }

    if (code_begin != end) {
        /* エラー (ファイルがセミコロンで終わっていない) */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_NO_EXIST_CODE_END);
        size_t line_num = m_token_line_num_table[(code_begin-begin)];
        addErrMsg(msg,line_num);
    }

    success &= checkUndefSub();
    success &= checkUndefLabel();
    success &= checkLabelPos();
    success &= checkLoop();

    return success;
}

/*! @brief   パース結果を返す関数 
 *  @return  パース結果(シナリオクラスオブジェクト)を返す 
 */
std::vector<C_scenario> C_sc_parser::getResult() {
    std::vector<C_scenario> scenario_table;

    std::map<std::string,size_t>::iterator ite = m_sc_map.begin();

    for (; ite != m_sc_map.end(); ++ite) {
        scenario_table.push_back(createScenario(ite->first));
    }

    return scenario_table;
}

/*! @brief   トークン列がパラメータ直接設定コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isDirectSetCmd(std::vector<std::string>::iterator begin,
                                 std::vector<std::string>::iterator end) {
    std::string token = "=";
    return (end != findToken(begin,end,token));
}

/*! @brief   トークン列がパラメータ相対設定コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isRelativeSetCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end) {
    std::string token = "+=";
    return (end != findToken(begin,end,token));
}

/*! @brief   トークン列がFORコマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isForCmd(std::vector<std::string>::iterator begin,
                           std::vector<std::string>::iterator end) {
    return (*begin == "@FOR");
}

/*! @brief   トークン列がWAITコマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isWaitCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end) {
    return (*begin == "@WAIT");
}

/*! @brief   トークン列がJUMPコマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isJumpCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end) {
    return (*begin == "@JUMP");
}

/*! @brief   トークン列が時刻直接設定コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isTimeDirectSetCmd(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end) {
    return (*begin == "@TIME" || *begin == "@FRAME");
}

/*! @brief   トークン列が時刻相対設定コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isTimeRelativeSetCmd(std::vector<std::string>::iterator begin,
                                       std::vector<std::string>::iterator end) {
    return ((*begin == "@TIME" && (end-begin) > 2 && *(begin+1) == "@TIME")
            || (*begin == "@FRAME" && (end-begin) > 2 && *(begin+1) == "@FRAME")
            || (*begin == "@WAIT" && (end - begin) == 2));
}


/*! @brief   トークン列がCALLコマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isCallCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end) {
    return (*begin == "@CALL");
}


/*! @brief   トークン列がシナリオ定義コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isScenarioDefCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end) {
    return (*begin == "@SCENARIO");
}

/*! @brief   トークン列がシナリオ定義終了コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isScenarioEndCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end) {
    return (*begin == "@END");
}

/*! @brief   トークン列がサブルーチン定義コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isSubDefCmd(std::vector<std::string>::iterator begin,
                              std::vector<std::string>::iterator end) {
    return (*begin == "@SUB");
}

/*! @brief   トークン列がサブルーチン定義終了コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isSubEndCmd(std::vector<std::string>::iterator begin,
                              std::vector<std::string>::iterator end) {
    return (*begin == "@SUBEND");
}

/*! @brief   トークン列がラベル定義コマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isLabelDefCmd(std::vector<std::string>::iterator begin,
                                std::vector<std::string>::iterator end) {
    std::string token = ":";
    return (end != findToken(begin,end,token));
}

/*! @brief   トークン列がSIM_STOPコマンドかどうかを判定する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true 該当コマンド、false 該当コマンドでない 
 */
bool C_sc_parser::isSimStopCmd(std::vector<std::string>::iterator begin,
                               std::vector<std::string>::iterator end) {
    return (*begin == "@SIM_STOP");
}

/*! @brief   トークン列からパラメータ直接設定コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretDirectSetCmd(std::vector<std::string>::iterator begin,
                                        std::vector<std::string>::iterator end,
                                        int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_dset_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool type_check = interpreter.checkType(begin,end);

    if (format_check && type_check && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        std::vector<C_scenario_elem> sc_elems = interpreter.createCmds(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().insert(m_code_table.back().end(),
                                   sc_elems.begin(),
                                   sc_elems.end());
    }
    else if (m_code_table.empty()) {
        ;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!type_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_ASSIGN_TYPE_ERROR);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}


/*! @brief   トークン列からパラメータ相対設定コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretRelativeSetCmd(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end,
                                          int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_rset_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool type_check = interpreter.checkType(begin,end);

    if (format_check && type_check && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        std::vector<C_scenario_elem> sc_elems = interpreter.createCmds(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().insert(m_code_table.back().end(),
                                   sc_elems.begin(),
                                   sc_elems.end()); 
    }
    else if (m_code_table.empty()) {
        ;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!type_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_ASSIGN_TYPE_ERROR);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}


/*! @brief   トークン列から時刻直接設定コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretTimeDirectSetCmd(std::vector<std::string>::iterator begin,
                                            std::vector<std::string>::iterator end,
                                            int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_tdset_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* 時間設定箇所 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* 時間用ポインタのポインタの登録 */
        if (interpreter.isTimeSetCmd(begin,end)) {
            m_time_cmd_map.insert(std::make_pair("TIME",pos));
        }
        else {
            m_time_cmd_map.insert(std::make_pair("FRAME",pos));
        }
     }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列から時刻相対設定コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretTimeRelativeSetCmd(std::vector<std::string>::iterator begin,
                                              std::vector<std::string>::iterator end,
                                              int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_trset_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* 時間設定箇所 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* 時間用ポインタのポインタの登録 */
        if (interpreter.isTimeSetCmd(begin,end)) {
            m_time_cmd_map.insert(std::make_pair("TIME",pos));
        }
        else {
            m_time_cmd_map.insert(std::make_pair("FRAME",pos));
        }
     }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からFORコマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretForCmd(std::vector<std::string>::iterator begin,
                                  std::vector<std::string>::iterator end,
                                  int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_for_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* ループ変数初期化用コマンド生成 */
        C_scenario_elem init_cmd = interpreter.createInitLoopVarCmd(line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(init_cmd);

        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createLoopBeginCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* FORコマンド位置 */
        std::pair<size_t,size_t> cmd_pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* ラベル使用箇所登録 */
        std::string label_name = interpreter.getLabelName(begin,end);
        m_label_use_map.insert(std::make_pair(label_name,cmd_pos));

        /* FORコマンド用ラベルスタックにラベル名を積む */
        m_for_label_stack.push(label_name);

        /* FOR文開始位置に対するラベル名の設定 */
        std::string for_begin_label = "__FOR_BEGIN_" + label_name;

        /* FOR文開始位置ラベル設定 */
        m_label_def_map[for_begin_label] = cmd_pos;

        /* ループ終端用コマンドの生成 */
        C_scenario_elem jump_cmd = interpreter.createLoopEndCmd(line_num);

        /* FOR分終端JUMPコマンドをFORコマンド用コマンドスタックに積む */
        m_for_end_cmd_stack.push(jump_cmd);
    }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からWAITコマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretWaitCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_wait_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool cond_type_check = interpreter.checkCondType(begin,end);

    if (format_check && cond_type_check && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* 時間設定箇所 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* 時間用ポインタのポインタの登録 */
        m_time_cmd_map.insert(std::make_pair("TIME",pos));
    }
    else if (m_code_table.empty()) {
        ;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!cond_type_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_COMP_TYPE_ERROR);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からJUMPコマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretJumpCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_jump_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool cond_type_check = interpreter.checkCondType(begin,end);

    if (format_check && cond_type_check && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* JUMPコマンド位置 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* ラベル使用箇所登録 */
        std::string label_name = interpreter.getLabelName(begin,end);
        m_label_use_map.insert(std::make_pair(label_name,pos));
    }
    else if (m_code_table.empty()) {
        ;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!cond_type_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_COMP_TYPE_ERROR);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からCALLコマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretCallCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_call_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* CALLコマンド位置 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

        /* サブルーチン呼び出し箇所登録 */
        std::string sub_name = interpreter.getSubName(begin,end);
        m_sub_caller_map.insert(std::make_pair(sub_name,pos));
    }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からシナリオ定義コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretScenarioDefCmd(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end,
                                          int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_INSIDE_DEF);
        addErrMsg(msg,line_num);
    }

    C_scdef_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool sc_def_check = interpreter.checkScDef(begin,end,m_sc_map);

    if (format_check && sc_def_check) {
        /* コードテーブルにコードブロックを新規追加 */
        m_code_table.push_back(std::vector<C_scenario_elem>());

        /* シナリオ名取得 */
        std::string sc_name = interpreter.getScName(begin,end);

        /* シナリオ定義マップに登録 */
        m_sc_map[sc_name] = m_code_table.size()-1;

        /* 定義フラグ設定 */
        m_def_flag = true;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!sc_def_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_SC_REDEF);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    if (MULTI_SCENARIO_WARN_FLAG && m_sc_map.size() > 1) {
        /* 複数シナリオ定義時のワーニング */
        C_err_msg err_msg;
        std::string msg = err_msg.getWarnMsg(C_err_msg::SC_PARSER_WARN_MULTI_DEF_SC);
        addErrMsg(msg,line_num);
    }

    return success;
}

/*! @brief   トークン列からシナリオ定義終了コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretScenarioEndCmd(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end,
                                          int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_END);
        addErrMsg(msg,line_num);
    }

    C_scend_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* 定義フラグ設定 */
        m_def_flag = false;
    }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からサブルーチン定義コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretSubDefCmd(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end,
                                     int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_INSIDE_DEF);
        addErrMsg(msg,line_num);
    }

    C_subdef_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool sub_def_check = interpreter.checkSubDef(begin,end,m_sub_def_map);

    if (format_check && sub_def_check) {
        /* コードテーブルにコードブロックを新規追加 */
        m_code_table.push_back(std::vector<C_scenario_elem>());

        /* サブルーチン名取得 */
        std::string sub_name = interpreter.getSubName(begin,end);

        /* サブルーチンの定義位置 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1,0);

        /* サブルーチン定義マップに登録 */
        m_sub_def_map[sub_name] = pos;

        /* 定義フラグ設定 */
        m_def_flag = true;
    }
    else if (!format_check){
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!sub_def_check){
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_SUB_REDEF);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からサブルーチン定義終了コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretSubEndCmd(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end,
                                     int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_END);
        addErrMsg(msg,line_num);
    }

    C_subend_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);

        /* 定義フラグ設定 */
        m_def_flag = false;
    }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からラベル定義コマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretLabelDefCmd(std::vector<std::string>::iterator begin,
                                       std::vector<std::string>::iterator end,
                                       int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_labeldef_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    bool format_check = interpreter.checkFormat(begin,end);
    bool label_def_check = interpreter.checkLabelDef(begin,end,m_label_def_map);

    if (format_check && label_def_check && !m_code_table.empty()) {
        /* ラベル名取得 */
        std::string label_name = interpreter.getLabelName(begin,end);

        /* FOR文ラベルスタックを確認 */
        if (!m_for_end_cmd_stack.empty() && m_for_label_stack.top() == label_name) {
            /* JUMPコマンドを追加 */
            m_code_table.back().push_back(m_for_end_cmd_stack.top());

            /* JUMPコマンド位置 */
            std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size()-1);

            /* ラベル使用位置を登録 */
            std::string for_begin_label = "__FOR_BEGIN_" + label_name;
            m_label_use_map.insert(std::make_pair(for_begin_label,pos));

            /* ラベルスタックからPOP */
            m_for_label_stack.pop();

            /* ループ終端コマンドスタックからPOP */
            m_for_end_cmd_stack.pop();
        }

        /* 現在位置の次の位置を取得 */
        std::pair<size_t,size_t> pos(m_code_table.size()-1, m_code_table.back().size());

        /* ラベル定義マップに登録 */
        m_label_def_map[label_name] = pos;
    }
    else if (m_code_table.empty()) {
        ;
    }
    else if (!format_check) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }
    else if (!label_def_check) {
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_LABEL_REDEF);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}

/*! @brief   トークン列からSIM_STOPコマンドを解釈する 
 *  @param   begin 判定対象トークンの先頭を指すイテレータ 
 *  @param   end   判定対象トークンの終端を指すイテレータ 
 *  @param   line_num  シナリオファイルの行数 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_sc_parser::interpretSimStopCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {
    bool success = true;

    /* 定義中かどうかのチェック */
    if (!m_def_flag) {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_OUTSIDE_DEF_CMD);
        addErrMsg(msg,line_num);
    }

    C_simstop_cmd_interpreter interpreter(m_param,m_sc_ptr_table);

    if (interpreter.checkFormat(begin,end) && !m_code_table.empty()) {
        /* シナリオ要素作成 */
        C_scenario_elem sc_elem = interpreter.createCmd(begin,end,line_num);

        /* コードテーブルに追加 */
        m_code_table.back().push_back(sc_elem);
    }
    else if (m_code_table.empty()) {
        ;
    }
    else {
        /* エラー処理 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_CMD_FORMAT);
        addErrMsg(msg,line_num);
        addErrMsg(interpreter.getErrMsg(),line_num);
    }

    return success;
}



/*! @brief   実行時刻ポインタの更新 
 *  @param   sc_name  シナリオ名 
 *  @return  シナリオ 
 */
C_scenario C_sc_parser::createScenario(std::string sc_name) {

    /* m_code_tableのインデックス */
    size_t table_idx = m_sc_map[sc_name];

    /* 呼び出しているサブルーチンのm_code_tableのインデックスを取得 */
    std::set<size_t> sub_idx_set;
    std::multimap<std::string,std::pair<size_t,size_t> >::iterator ite 
        = m_sub_caller_map.begin();
    for (; ite != m_sub_caller_map.end(); ++ite) {
        std::string sub_name = ite->first;              /* サブルーチン名 */
        std::pair<size_t,size_t> pos = ite->second;     /* 呼び出し位置   */
        if (table_idx == pos.first) {
            /* サブルーチンのm_code_tableインデックスを取得 */
            sub_idx_set.insert(m_sub_def_map[sub_name].first);
        }
    }

    /* サブルーチンが呼び出しているサブルーチンのm_code_tableインデックスを取得 */
    while (1) {
        size_t old_sub_idx_set_size = sub_idx_set.size();
        std::set<size_t>::iterator sub_ite = sub_idx_set.begin();
        for (; sub_ite != sub_idx_set.end(); ++sub_ite) {
            size_t sub_idx = *sub_ite;
            for (ite = m_sub_caller_map.begin() ; ite != m_sub_caller_map.end(); ++ite) {
                std::string sub_name = ite->first;              /* サブルーチン名 */
                std::pair<size_t,size_t> pos = ite->second;     /* 呼び出し位置   */
                if (sub_idx == pos.first) {
                    /* サブルーチンのm_code_tableインデックスを取得 */
                    sub_idx_set.insert(m_sub_def_map[sub_name].first);
                }
            }
        }
        if (old_sub_idx_set_size == sub_idx_set.size()) break;
    }

    /* 1シナリオ分のシナリオ要素テーブルの作成 */
    std::vector<C_scenario_elem> sc_table(m_code_table[table_idx]);

    /* idx_tableの作成 */
    std::vector<size_t> idx_table;
    idx_table.push_back(table_idx);

    /* サブルーチン分のシナリオ要素をテーブルに挿入 */
    std::set<size_t>::iterator idx_pos = sub_idx_set.begin();
    for (; idx_pos != sub_idx_set.end(); ++idx_pos) {
         /* 要素テーブルの最後に挿入 */
         sc_table.insert(sc_table.end(),
                         m_code_table[*idx_pos].begin(),
                         m_code_table[*idx_pos].end());
         /* idx_tableにサブルーチンのインデックスを追加 */
         idx_table.push_back(*idx_pos);
    }

    /* 実行時刻用領域作成 */
    int *time_ptr = new int(0);
    int *frame_ptr = new int(0);
    m_sc_ptr_table->addPtr(time_ptr);
    m_sc_ptr_table->addPtr(frame_ptr);

    /* 実行時刻用ポインタの登録 */
    m_sc_time_ptr_map[sc_name] = std::pair<int*,int*>(frame_ptr,time_ptr);

    /* 実行時刻ポインタの更新 */
    updateTimePtr(sc_name,idx_table,sc_table,frame_ptr,time_ptr);

    /* ラベル位置情報の更新 */
    updateLabelPos(sc_name,idx_table,sc_table);

    /* サブルーチン位置情報の更新 */
    updateSubPos(sc_name,idx_table,sc_table);

    /* LOOPコマンドのループ変数の更新 */
    updateLoopVar(sc_table);

    /* WAITコマンドの時刻カウンタ変数の更新 */
    updateWaitCount(sc_table);

    /* JUMPコマンドのジャンプカウンタ変数の更新 */
    updateJumpCount(sc_table);

    /* トレースフラグの取得 */
    bool trace_flag = false;
    /* DUMP方法をGreenと合わせる */
    /* std::string flag_str; */
    int flag_str;
    if (m_param->getValue("COM_SCENARIO_TRACE",flag_str)) {
        /* trace_flag = (flag_str == "ON"); */
        trace_flag = (flag_str != 0);
    }

    return C_scenario(sc_name,m_lines,sc_table,m_sc_ptr_table,
                      frame_ptr,time_ptr,trace_flag);
}


/*! @brief   未定義サブルーチンが使用されているかどうかをチェックする 
 *  @return  true 成功、false 失敗 
 */
bool C_sc_parser::checkUndefSub() {
    bool success = true;

    std::multimap<std::string,std::pair<size_t,size_t> >::iterator ite
        = m_sub_caller_map.begin();
    for (; ite != m_sub_caller_map.end(); ++ite) {
        std::pair<size_t,size_t> pos = ite->second;
        if (m_sub_def_map.find(ite->first) == m_sub_def_map.end()) {
            /* エラー(未定義サブルーチンの検出) */
            success = false;
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_SUB_UNDEF);
            size_t line_num = m_code_table[pos.first][pos.second].getLineNum();
            addErrMsg(msg,line_num);
        }
    }

    return success;
}

/*! @brief   未定義ラベルが使用されているかどうかをチェックする 
 *  @return  true 成功、false 失敗 
 */
bool C_sc_parser::checkUndefLabel() {
    bool success = true;

    std::multimap<std::string,std::pair<size_t,size_t> >::iterator ite
        = m_label_use_map.begin();
    for (; ite != m_label_use_map.end(); ++ite) {
        std::pair<size_t,size_t> pos = ite->second;
        if (m_label_def_map.find(ite->first) == m_label_def_map.end()) {
            /* エラー(未定義ラベルの検出) */
            success = false;
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_LABEL_UNDEF);
            size_t line_num = m_code_table[pos.first][pos.second].getLineNum();
            addErrMsg(msg,line_num);
        }
    }

    return success;
}

/*! @brief   ラベル位置が正しいかチェックする 
 *  @return  true 成功、false 失敗 
 */
bool C_sc_parser::checkLabelPos() {
    bool success = true;

    std::multimap<std::string,std::pair<size_t,size_t> >::iterator ite
        = m_label_use_map.begin();
    for (; ite != m_label_use_map.end(); ++ite) {
        std::string label_name = ite->first;
        if (m_label_def_map.find(label_name) == m_label_def_map.end()) continue;

        std::pair<size_t,size_t> use_pos = ite->second;
        std::pair<size_t,size_t> def_pos = m_label_def_map[label_name];

        /* 同一シナリオ・サブルーチン上で定義され使用されているかどうかをチェック */
        if (use_pos.first != def_pos.first) {
            /* エラー(使用位置不正ラベルの検出) */
            success = false;
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_ILLEGAL_JUMP);
            size_t line_num = m_code_table[use_pos.first][use_pos.second].getLineNum();
            addErrMsg(msg,line_num);
        }
    }

    return success;
}

/*! @brief   ループが正しく閉じているかをチェックする 
 *  @return  true 成功、false 失敗 
 */
bool C_sc_parser::checkLoop() {
    bool success = true;

    /* ループ用ラベルスタックをチェック */
    if (!m_for_label_stack.empty()) {
        /* 閉じられていないループの検出 */
        success = false;
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SC_PARSER_ERR_NOT_FOUND_LOOP_END);

        std::string label_name = m_for_label_stack.top();
        std::pair<size_t,size_t> use_pos = m_label_use_map.find(label_name)->second;
        size_t line_num = m_code_table[use_pos.first][use_pos.second].getLineNum();
        addErrMsg(msg,line_num);
    }

    return success;
}


/*! @brief   実行時刻ポインタの更新 
 *  @param   sc_name  シナリオ名 
 *  @param   sc_table 更新対象 
 *  @param   frame_ptr フレーム用ポインタ 
 *  @param   time_ptr  tick用ポインタ 
 */
void C_sc_parser::updateTimePtr(std::string &sc_name,
                                std::vector<size_t> &idx_table,
                                std::vector<C_scenario_elem> &sc_table,
                                int *frame_ptr,
                                int *time_ptr) {

    /* 設定箇所からsc_table上のインデックスを計算 */
    std::map<size_t,std::string> time_cmd_map = createIndexNameMap(idx_table,m_time_cmd_map);

    std::map<size_t,std::string>::iterator ite = time_cmd_map.begin();
    for (; ite != time_cmd_map.end(); ++ite) {
        size_t idx = ite->first;               /* インデックス */
        std::string time_kind = ite->second;   /* 時間の種類   */

        if (time_kind == "TIME" && sc_table[idx].isRelativeSetCmd()) {
            sc_table[idx].updateRelativeSetCmdParam(time_ptr);
        }
        else if (time_kind == "TIME" && sc_table[idx].isDirectSetCmd()) {
            sc_table[idx].updateDirectSetCmdParam(time_ptr);
        }
        else if (time_kind == "TIME" && sc_table[idx].isWaitCmd()) {
            sc_table[idx].updateWaitCmdTimePtr(time_ptr);
        }
        else if (time_kind == "FRAME" && sc_table[idx].isRelativeSetCmd()) {
            sc_table[idx].updateRelativeSetCmdParam(frame_ptr);
        }
        else if (time_kind == "FRAME" && sc_table[idx].isDirectSetCmd()) {
            sc_table[idx].updateDirectSetCmdParam(frame_ptr);
        }
    }
}

/*! @brief   ラベル位置情報の更新 
 *  @param   sc_name  シナリオ名 
 *  @param   sc_table 更新対象 
 *  @return  true 成功、false 失敗 
 */
void C_sc_parser::updateLabelPos(std::string &sc_name,
                                 std::vector<size_t> &idx_table,
                                 std::vector<C_scenario_elem> &sc_table) {

    /* 定義箇所からsc_table上のインデックスを計算 */
    std::map<std::string,size_t> label_def_map = createNameIndexMap(idx_table,m_label_def_map);


    /* 使用箇所からsc_table上のインデックスを計算 */
    std::map<size_t,std::string> label_use_map = createIndexNameMap(idx_table,m_label_use_map);

    /* 使用箇所のラベル位置情報ポインタを更新 */
    std::map<size_t,std::string>::iterator ite = label_use_map.begin();
    for (; ite != label_use_map.end(); ++ite) {
        int* label_pos = new int(label_def_map[ite->second]);
        m_sc_ptr_table->addPtr(label_pos);

        if (sc_table[ite->first].isForCmd()) {
            sc_table[ite->first].updateForCmdLabelPos(label_pos);
        }
        else if (sc_table[ite->first].isForEndCmd()) {
            sc_table[ite->first].updateForEndCmdLabelPos(label_pos);
        }
        else if (sc_table[ite->first].isJumpCmd()) {
            sc_table[ite->first].updateJumpCmdLabelPos(label_pos);
        }
    }
}

/*! @brief   サブルーチン位置情報を更新 
 *  @param   sc_name  シナリオ名 
 *  @param   sc_table 更新対象 
 *  @return  true 成功、false 失敗 
 */
void C_sc_parser::updateSubPos(std::string &sc_name,
                               std::vector<size_t> &idx_table,
                               std::vector<C_scenario_elem> &sc_table) {

    /* サブルーチンマップからsc_table上のインデックスを計算 */
    std::map<std::string,size_t> sub_def_map = createNameIndexMap(idx_table,m_sub_def_map);

    /* サブルーチン呼び出し箇所からsc_table上のインデックスを計算 */
    std::map<size_t,std::string> caller_map = createIndexNameMap(idx_table,m_sub_caller_map);

    /* CALLコマンドのサブルーチン位置情報ポインタを更新 */ 
    std::map<size_t,std::string>::iterator ite = caller_map.begin();
    for (; ite != caller_map.end(); ++ite) {
        int* sub_pos = new int(sub_def_map[ite->second]);
        m_sc_ptr_table->addPtr(sub_pos);

        if (sc_table[ite->first].isCallCmd()) {
            sc_table[ite->first].updateCallCmdSubPos(sub_pos);
        }
    }
}

/*! @brief   LOOPコマンドのループ変数を更新する 
 *  @param   sc_table 更新対象 
 */
void C_sc_parser::updateLoopVar(std::vector<C_scenario_elem> &sc_table) {
    std::vector<C_scenario_elem>::iterator ite = sc_table.begin();
    for (; ite != sc_table.end(); ++ite) {
        if (ite->isForCmd()) {
            int *loop_var = new int(0);
            m_sc_ptr_table->addPtr(loop_var);

            /* FORコマンド用ループ変数初期化設定 */
            (ite-1)->updateDirectSetCmdParam(loop_var);

            /* FORコマンド用ループ変数設定 */
            ite->updateForCmdLoopVar(loop_var);
        }
    }
}

/*! @brief   WAITコマンドの時刻カウントを更新する 
 *  @param   sc_table 更新対象 
 */
void C_sc_parser::updateWaitCount(std::vector<C_scenario_elem> &sc_table) {
    std::vector<C_scenario_elem>::iterator ite = sc_table.begin();
    for (; ite != sc_table.end(); ++ite) {
        if (ite->isWaitCmd()) {
            int *time_count = new int(0);
            ite->updateWaitCmdTimeCount(time_count);
            m_sc_ptr_table->addPtr(time_count);
        }
    }
}

/*! @brief   JUMPコマンドのジャンプ回数を更新する 
 *  @param   sc_table 更新対象 
 */
void C_sc_parser::updateJumpCount(std::vector<C_scenario_elem> &sc_table) {
    std::vector<C_scenario_elem>::iterator ite = sc_table.begin();
    for (; ite != sc_table.end(); ++ite) {
        if (ite->isJumpCmd()) {
            int *jump_count = new int(0);
            ite->updateJumpCmdJumpCount(jump_count);
            m_sc_ptr_table->addPtr(jump_count);
        }
    }
}

/*! @brief   位置情報を計算して一つのインデックスと名前のマップを生成する 
 *  @param   idx_table  m_sc_tableのインデックスのテーブル 
 *  @param   pos_map    位置情報のマップ 
 *  @param   インデックスと名前のテーブル 
 */
std::map<size_t,std::string> 
C_sc_parser::createIndexNameMap(std::vector<size_t> &idx_table,
                                std::multimap<std::string,std::pair<size_t,size_t> > &pos_map) {
    std::map<size_t,std::string> idx_map;

    size_t offset = 0;
    std::vector<size_t>::iterator idx_ite = idx_table.begin();
    for (; idx_ite != idx_table.end(); ++idx_ite) {
        std::multimap<std::string,std::pair<size_t,size_t> >::iterator pos_ite= pos_map.begin();
        for (; pos_ite != pos_map.end(); ++pos_ite) {
            std::string name = pos_ite->first;               /* 名前     */
            std::pair<size_t,size_t> pos = pos_ite->second;  /* 位置情報 */
            if (pos.first == *idx_ite) {
                /* インデックスが一致した場合登録 */
                idx_map[pos.second + offset] = name;
            }
        }
        offset += m_code_table[*idx_ite].size();
    }

    return idx_map;
}

/*! @brief   位置情報を計算して一つの名前とインデックスのマップを生成する 
 *  @param   idx_table  m_sc_tableのインデックスのテーブル 
 *  @param   pos_map    位置情報のマップ 
 *  @param   名前とインデックスのマップ 
 */
std::map<std::string,size_t> 
C_sc_parser::createNameIndexMap(std::vector<size_t> &idx_table,
                                std::map<std::string,std::pair<size_t,size_t> > &pos_map) {

    std::map<std::string,size_t> name_map;

    size_t offset = 0;
    std::vector<size_t>::iterator idx_ite = idx_table.begin();
    for (; idx_ite != idx_table.end(); ++idx_ite) {
        std::map<std::string,std::pair<size_t,size_t> >::iterator pos_ite = pos_map.begin();
        for (; pos_ite != pos_map.end(); ++pos_ite) {
            std::string name = pos_ite->first;                 /* 名前     */
            std::pair<size_t,size_t> pos = pos_ite->second;    /* 位置情報 */
            if (pos.first == *idx_ite) {
                /* インデックスが一致した場合登録 */
                name_map[name] = pos.second + offset;
            }
        }
        offset += m_code_table[*idx_ite].size();
    }

    return name_map;
}

