/*! 
    @file  sc_parser.h 
    @brief シナリオファイルパーサクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SC_PARSER_H
#define C_SC_PARSER_H

#include <vector>
#include <map>
#include <stack>
#include <string>
#include <boost/shared_ptr.hpp>
#include "C_parser_base.h"
#include "C_scenario.h"
#include "C_sc_ptr_table.h"
#include "I_param.h"

/*! 
  シナリオファイルパーサクラス 
  @brief  シナリオファイルからシナリオクラスオブジェクト生成する機能を持つ 
 */
class C_sc_parser : public C_parser_base {
private:
    /* メンバ変数 */

    /* 複数シナリオ設定ワーニングスイッチ 
       0 : ワーニングなし、1 : ワーニング 
     */
    const static int MULTI_SCENARIO_WARN_FLAG = 0;

    /* シナリオ、サブルーチン定義中を表すフラグ */
    bool m_def_flag;

    /* コードブロックごとのテーブル */
    std::vector<std::vector<C_scenario_elem> > m_code_table;

    /* シナリオ定義マップ (シナリオ名、m_code_tableのインデックス) */
    std::map<std::string,size_t>  m_sc_map;

    /* シナリオ名と実行時間変数のポインタテーブル 
     * (シナリオ名, (frame,tick) ) */
    std::map<std::string,std::pair<int*,int*> > m_sc_time_ptr_map;

    /* 時間設定箇所のマップ 
     * (m_code_tableのインデックス、m_code_table[i]のインデックス) */
    std::multimap<std::string,std::pair<size_t,size_t> > m_time_cmd_map;

    /* サブルーチン定義マップ (サブルーチン名、m_code_tableのインデックス) */
    std::map<std::string,std::pair<size_t,size_t> > m_sub_def_map;

    /* サブルーチンの呼び出し先のマップ 
     * (サブルーチン名,呼び出し箇所) */
    std::multimap<std::string,std::pair<size_t,size_t> > m_sub_caller_map;

    /* ラベルマップ ラベル名と定義箇所の関係を表す 
     * (ラベル名,定義箇所) */
    std::map<std::string,std::pair<size_t,size_t> > m_label_def_map;

    /* ラベル使用先マップ ラベル名と使用先の関係を表す 
     * (ラベル名,使用箇所) */
    std::multimap<std::string,std::pair<size_t,size_t> > m_label_use_map;

    /* FORコマンド用 ラベルスタック */
    std::stack<std::string> m_for_label_stack;

    /* FORコマンド用 ループ終端用JUMPコマンドスタック */
    std::stack<C_scenario_elem> m_for_end_cmd_stack;

    /* シナリオ用ポインタテーブル(sc_parser内で生成したポインタを管理) */
    boost::shared_ptr<C_sc_ptr_table> m_sc_ptr_table;

public:
    /* public メンバ関数 */

    /*! @brief パースしたい文字列のvectorとパラメータクラスを設定します。 
     *  @param filename  ファイル名 
     *  @param param   パラメータクラス 
     */
    virtual void setData(std::string &filename,
                         boost::shared_ptr<I_param> &param);

    /*! @brief    パスからファイル名を取り出します。 
     *  @return   true: パース成功, false: パース失敗 
     */
    virtual bool parse();

    /*! @brief   パース結果を返す関数 
     *  @return  パース結果(シナリオクラスオブジェクト)を返す 
     */
    std::vector<C_scenario> getResult();
private:
    /* private メンバ関数 */
    /* ここには、C_scenarioクラスのオブジェクト生成に関係する関数を記述 */
    /* 構文解析用の関数を記述 */

    /*! @brief   トークン列がパラメータ直接設定コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isDirectSetCmd(std::vector<std::string>::iterator begin,
                        std::vector<std::string>::iterator end);

    /*! @brief   トークン列がパラメータ相対設定コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isRelativeSetCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end);

    /*! @brief   トークン列がFORコマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isForCmd(std::vector<std::string>::iterator begin,
                  std::vector<std::string>::iterator end);

    /*! @brief   トークン列がWAITコマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isWaitCmd(std::vector<std::string>::iterator begin,
                   std::vector<std::string>::iterator end);

    /*! @brief   トークン列がJUMPコマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isJumpCmd(std::vector<std::string>::iterator begin,
                   std::vector<std::string>::iterator end);

    /*! @brief   トークン列が時刻直接設定コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isTimeDirectSetCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end);

    /*! @brief   トークン列が時刻相対設定コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isTimeRelativeSetCmd(std::vector<std::string>::iterator begin,
                              std::vector<std::string>::iterator end);


    /*! @brief   トークン列がCALLコマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isCallCmd(std::vector<std::string>::iterator begin,
                   std::vector<std::string>::iterator end);


    /*! @brief   トークン列がシナリオ定義コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isScenarioDefCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end);

    /*! @brief   トークン列がシナリオ定義終了コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isScenarioEndCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end);

    /*! @brief   トークン列がサブルーチン定義コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isSubDefCmd(std::vector<std::string>::iterator begin,
                     std::vector<std::string>::iterator end);

    /*! @brief   トークン列がサブルーチン定義終了コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isSubEndCmd(std::vector<std::string>::iterator begin,
                     std::vector<std::string>::iterator end);

    /*! @brief   トークン列がラベル定義コマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isLabelDefCmd(std::vector<std::string>::iterator begin,
                       std::vector<std::string>::iterator end);

    /*! @brief   トークン列がSIM_STOPコマンドかどうかを判定する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true 該当コマンド、false 該当コマンドでない 
     */
    bool isSimStopCmd(std::vector<std::string>::iterator begin,
                      std::vector<std::string>::iterator end);


    /*! @brief   トークン列からパラメータ直接設定コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretDirectSetCmd(std::vector<std::string>::iterator begin,
                               std::vector<std::string>::iterator end,
                               int line_num);

    /*! @brief   トークン列からパラメータ相対設定コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretRelativeSetCmd(std::vector<std::string>::iterator begin,
                                 std::vector<std::string>::iterator end,
                                 int line_num);

    /*! @brief   トークン列から時刻直接設定コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretTimeDirectSetCmd(std::vector<std::string>::iterator begin,
                                   std::vector<std::string>::iterator end,
                                   int line_num);

    /*! @brief   トークン列から時刻相対設定コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretTimeRelativeSetCmd(std::vector<std::string>::iterator begin,
                                     std::vector<std::string>::iterator end,
                                     int line_num);

    /*! @brief   トークン列からFORコマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretForCmd(std::vector<std::string>::iterator begin,
                         std::vector<std::string>::iterator end,
                         int line_num);

    /*! @brief   トークン列からWAITコマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretWaitCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end,
                          int line_num);

    /*! @brief   トークン列からJUMPコマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretJumpCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end,
                          int line_num);

    /*! @brief   トークン列からCALLコマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretCallCmd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end,
                          int line_num);

    /*! @brief   トークン列からシナリオ定義コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretScenarioDefCmd(std::vector<std::string>::iterator begin,
                                 std::vector<std::string>::iterator end,
                                 int line_num);

    /*! @brief   トークン列からシナリオ定義終了コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretScenarioEndCmd(std::vector<std::string>::iterator begin,
                                 std::vector<std::string>::iterator end,
                                 int line_num);

    /*! @brief   トークン列からサブルーチン定義コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretSubDefCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end,
                            int line_num);

    /*! @brief   トークン列からサブルーチン定義終了コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretSubEndCmd(std::vector<std::string>::iterator begin,
                            std::vector<std::string>::iterator end,
                            int line_num);

    /*! @brief   トークン列からラベル定義コマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretLabelDefCmd(std::vector<std::string>::iterator begin,
                              std::vector<std::string>::iterator end,
                              int line_num);

    /*! @brief   トークン列からSIMSTOPコマンドを解釈する 
     *  @param   begin 判定対象トークンの先頭を指すイテレータ 
     *  @param   end   判定対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool interpretSimStopCmd(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end,
                             int line_num);

    /*! @brief   実行時刻ポインタの更新 
     *  @param   sc_name  シナリオ名 
     *  @return  シナリオ 
     */
    C_scenario createScenario(std::string sc_name);

    /*! @brief   未定義サブルーチンが使用されているかどうかをチェックする 
     *  @return  true 成功、false 失敗 
     */
    bool checkUndefSub();

    /*! @brief   未定義ラベルが使用されているかどうかをチェックする 
     *  @return  true 成功、false 失敗 
     */
    bool checkUndefLabel();

    /*! @brief   ラベル位置が正しいかチェックする 
     *  @return  true 成功、false 失敗 
     */
    bool checkLabelPos();

    /*! @brief   ループが正しく閉じているかをチェックする 
     *  @return  true 成功、false 失敗 
     */
    bool checkLoop();

    /*! @brief   実行時刻ポインタの更新 
     *  @param   sc_name  シナリオ名 
     *  @param   sc_table 更新対象 
     *  @param   frame_ptr フレーム用ポインタ 
     *  @param   time_ptr  tick用ポインタ 
     */
    void updateTimePtr(std::string &sc_name,
                       std::vector<size_t> &idx_table,
                       std::vector<C_scenario_elem> &sc_table,
                       int *frame_ptr,
                       int *time_ptr);

    /*! @brief   ラベル位置情報の更新 
     *  @param   sc_name  シナリオ名 
     *  @param   idx_table sc_tableに含まれるm_code_tableのインデックス 
     *  @param   sc_table 更新対象 
     */
    void updateLabelPos(std::string &sc_name,
                        std::vector<size_t> &idx_table,
                        std::vector<C_scenario_elem> &sc_table);

    /*! @brief   サブルーチン位置情報を更新 
     *  @param   sc_name  シナリオ名 
     *  @param   idx_table sc_tableに含まれるm_code_tableのインデックス 
     *  @param   sc_table 更新対象 
     */
    void updateSubPos(std::string &sc_name,
                      std::vector<size_t> &idx_table,
                      std::vector<C_scenario_elem> &sc_table);

    /*! @brief   LOOPコマンドのループ変数を更新する 
     *  @param   sc_table 更新対象 
     */
    void updateLoopVar(std::vector<C_scenario_elem> &sc_table);

    /*! @brief   WAITコマンドの時刻カウントを更新する 
     *  @param   sc_table 更新対象 
     */
    void updateWaitCount(std::vector<C_scenario_elem> &sc_table);

    /*! @brief   JUMPコマンドのジャンプ回数を更新する 
     *  @param   sc_table 更新対象 
     */
    void updateJumpCount(std::vector<C_scenario_elem> &sc_table);

    /*! @brief   位置情報を計算して一つのインデックスと名前のマップを生成する 
     *  @param   idx_table  m_sc_tableのインデックスのテーブル 
     *  @param   pos_map    位置情報のマップ 
     *  @param   インデックスと名前のテーブル 
     */
    std::map<size_t,std::string> 
    createIndexNameMap(std::vector<size_t> &idx_table,
                       std::multimap<std::string,std::pair<size_t,size_t> > &pos_map);


    /*! @brief   位置情報を計算して一つの名前とインデックスのマップを生成する 
     *  @param   idx_table  m_sc_tableのインデックスのテーブル 
     *  @param   pos_map    位置情報のマップ 
     *  @param   名前とインデックスのマップ 
     */
    std::map<std::string,size_t> 
    createNameIndexMap(std::vector<size_t> &idx_table,
                       std::map<std::string,std::pair<size_t,size_t> > &pos_map);
};

#endif

