/*! 
    @file  C_scdef_cmd_interpreter.cpp 
    @brief SCENARIOコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_scdef_cmd_interpreter.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_scdef_cmd_interpreter::
C_scdef_cmd_interpreter(boost::shared_ptr<I_param> param,
                        boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_scdef = false;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_scdef_cmd_interpreter::reset() {
    m_check = false;
    m_scdef = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_scdef_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end) {
    /* SCENARIOコマンドのトークン列サイズチェック */
    bool size_check = ((end - begin) == 2);

    bool scdef_token_check = false;
    
    if (size_check) {
        /* SCENARIOコマンドのチェック */
        scdef_token_check = (*begin == "@SCENARIO");
    }

    m_check = (size_check && scdef_token_check);

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     scdef_token_check);

    return m_check;
}

/*! @brief   トークン列からシナリオ定義をチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @param   sc_map シナリオ名マップ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_scdef_cmd_interpreter::checkScDef(std::vector<std::string>::iterator begin,
                                         std::vector<std::string>::iterator end,
                                         std::map<std::string,size_t> &sc_map) {
    m_scdef = false;
    if (m_check) {
        /* サブルーチン定義がすでにされているかのチェック */
        std::string sc_name = *(begin+1);
        m_scdef = (sc_map.find(sc_name) == sc_map.end());

        /* エラー発生時、エラーメッセージ生成 */
        if (!m_scdef) createScDefErrMsg();
    }

    return m_scdef;
}

/*! @brief   トークン列からシナリオ名を取得する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ名 
 */
std::string C_scdef_cmd_interpreter::getScName(std::vector<std::string>::iterator begin,
                                               std::vector<std::string>::iterator end) {
    if (!m_scdef || !m_check) return "";
    std::vector<std::string>::iterator sc_name = begin+1;
    return *sc_name;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_scdef_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << "\n"
        << "m_scdef = " << m_scdef;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   scdef_token_check        SCENARIOコマンドトークン存在チェック 
 */
void C_scdef_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                 bool scdef_token_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* シナリオ定義コマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SCDEF_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* "@SCENARIO"トークンが不正ときにエラーメッセージを追加 */
    if (!scdef_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SCDEF_CMD_INTERPRETER_ERR_SCDEF_CMD_POS);
        addErrMsg(msg);
    }
}

/*! @brief   シナリオ定義エラーメッセージの生成 
 */
void C_scdef_cmd_interpreter::createScDefErrMsg() {
    C_err_msg err_msg;
    std::string msg = "";

    /* シナリオの再定義時エラー時にエラーメッセージを追加 */
    msg = err_msg.getErrMsg(C_err_msg::SCDEF_CMD_INTERPRETER_ERR_REDEF_SCENARIO);
    addErrMsg(msg);
}
