/*! 
    @file  C_scenario.h 
    @brief シナリオクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_scenario.h"
#include "C_err_msg.h"
#include <sstream>
#include <boost/shared_ptr.hpp>
#include <boost/format.hpp>

/*! @brief コンストラクタ 
 *  @param sc_name      シナリオ名 
 *  @param sc_str_table シナリオファイルの文字列データ 
 *  @param sc_table     シナリオ要素のテーブル 
 *  @param sc_ptr_table シナリオで使用されるポインタのテーブル 
 *  @param frame_ptr    実行時刻frame用ポインタ 
 *  @param tick_ptr     実行時刻tick用ポインタ 
 *  @param trace_flag   トレースオン/オフ 
 */
C_scenario::C_scenario(std::string sc_name,
                       std::vector<std::string> &sc_str_table,
                       std::vector<C_scenario_elem> &sc_table,
                       boost::shared_ptr<C_sc_ptr_table> &sc_ptr_table,
                       int* frame_ptr, int* tick_ptr, bool trace_flag) 
    : m_sc_name(sc_name), m_sc_idx(0), m_sc_str_table(sc_str_table), m_sc_table(sc_table), 
      m_trace_flag(trace_flag)
{
    m_sc_ptr_table = sc_ptr_table; 
    m_next_frame = frame_ptr;
    m_next_tick = tick_ptr;
    m_next_time = boost::shared_ptr<C_SimTime>(new C_SimTime());
    m_next_time->set(*m_next_frame, *m_next_tick);
    m_next_time->normalize();
}

/*! @brief   シナリオ実行関数 
 *  @param   current_time 現在の時間 
 *  @return  true 通常実行, false シナリオ停止 
 */
bool C_scenario::execute(boost::shared_ptr<C_SimTime> &current_time) {

    int next_sc_idx = m_sc_idx;

    if (m_sc_table.empty()) return false;

    if (m_sc_table[m_sc_idx].isEndCmd()) return true;

    while (*m_next_time <= *current_time) {
        C_scenario_elem sc_elem = m_sc_table[m_sc_idx];

        if (*m_next_time < *current_time) {
            /* ワーニング処理 */
            C_err_msg err_msg;
            std::string msg = err_msg.getWarnMsg(C_err_msg::SCENARIO_WARN_NEXT_TIME);
            addMsg(msg,current_time);
        }

        if (m_trace_flag) {
            /* トレース情報の生成と保存 */
            m_trace_info.push_back(createTraceInfo(current_time));
        }

        /* シナリオ要素の実行と次のインデックスの取得 */
        bool exec_result = sc_elem.execute(next_sc_idx);

        /* シナリオ要素からのメッセージ取得 */
        addMsg(sc_elem.getMsgs(),current_time);

        /* 実行コマンドでエラーが発生した場合、停止 */
        if (!exec_result) return false;

        /* 各コマンドの実行後処理 */
        if (sc_elem.isCallCmd()) {
            if (m_sub_stack.size() > STACK_SIZE) {
                /* エラー処理 (スタックオーバーフロー) */
                C_err_msg err_msg;
                std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ERR_SUBSTACK_OVERFLOW);
                addMsg(msg,current_time);
                return false;
            }

            /* サブルーチンのリターン位置をスタックに積む */
            m_sub_stack.push(m_sc_idx+1);
            m_sc_idx = next_sc_idx;
        }
        else if (sc_elem.isSubEndCmd()) {
            if (m_sub_stack.empty()) {
                /* エラー処理 (スタックが空) */
                C_err_msg err_msg;
                std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ERR_SUBSTACK_EMPTY);
                addMsg(msg,current_time);
                return false;
            }
            else {
                /* サブルーチンのリターン位置を取得し、スタックから下ろす */
                m_sc_idx = m_sub_stack.top();
                next_sc_idx = m_sc_idx;
                m_sub_stack.pop();
            }
        }
        else if (sc_elem.isEndCmd()) {
            /* 通常終了 */
            C_err_msg err_msg;
            std::string msg = err_msg.getInfoMsg(C_err_msg::SCENARIO_INFO_END);
            addMsg(msg,current_time);
            return true;
        }
        else if (sc_elem.isStopCmd()) {
            /* STOP 情報出力 */
            C_err_msg err_msg;
            std::string msg = err_msg.getInfoMsg(C_err_msg::SCENARIO_INFO_STOP);
            addMsg(msg,current_time);
            return false;
        }
        else {
            m_sc_idx = next_sc_idx;
        }

        /* 実行時刻をチェック */
        if (m_next_time->getFrame() != *m_next_frame 
            || m_next_time->getTick() != *m_next_tick) {
            /* 実行時刻のアップデート */
            m_next_time->set(*m_next_frame, *m_next_tick);
            m_next_time->normalize();
        }
    }
    return true;
}

/*! @brief   トレース情報の取得 
 *  @return  トレース情報 
 */
std::vector<std::string> C_scenario::getTraceInfo() {
    /* トレース出力OFFの場合は空のvectorを出力 */
    if( !m_trace_flag ) return std::vector<std::string>();

    std::vector<std::string> header;
    std::vector<std::string> footer;

    std::ostringstream oss("");

    header.push_back("=======================================================");
    oss << boost::format("==      scenario begin %s  ") % m_sc_name;
    header.push_back(oss.str());
    header.push_back("=======================================================");

    oss.str("");

    footer.push_back("=======================================================");
    oss << boost::format("==      scenario end %s  ") % m_sc_name;
    footer.push_back(oss.str());
    footer.push_back("=======================================================");

    std::vector<std::string> trace_info(header);
    trace_info.insert(trace_info.end(),m_trace_info.begin(),m_trace_info.end());
    trace_info.insert(trace_info.end(),footer.begin(),footer.end());

    return trace_info;
}


/*! @brief   メッセージの取得 
 *  @return  メッセージ 
 */
std::vector<std::string> C_scenario::getMsg() {
    return m_msgs;
}


/*! @brief   1コマンド分のトレース情報の生成を行う 
 *  @return  1コマンド分のトレース情報 
 */
std::string C_scenario::createTraceInfo(boost::shared_ptr<C_SimTime> &time) {
    std::ostringstream oss("");

    int frame = time->getFrame();
    int tick  = time->getTick();
    int line_num = m_sc_table[m_sc_idx].getLineNum();
    std::string cmd_dump = m_sc_table[m_sc_idx].dump();

    std::string fmt("[frame = %16d, tick = %16d, line_num = %16d]\n%s\n%s");

    oss << boost::format(fmt)
        % frame
        % tick
        % (line_num)
        % m_sc_str_table[line_num-1]
        % cmd_dump;

    return oss.str();
}

/*! @brief   メッセージの追加を行う 
 *  @param   msg  メッセージ 
 *  @param   time SIM時間 
 */
void C_scenario::addMsg(std::string &msg, boost::shared_ptr<C_SimTime> &time) {
    std::ostringstream oss("");

    int frame = time->getFrame();
    int tick  = time->getTick();
    int line_num = m_sc_table[m_sc_idx].getLineNum();

    std::string fmt("[frame = %8d, tick = %8d, line_num = %8d]\n%s\n%s\n");

    oss << boost::format(fmt)
        % frame
        % tick
        % (line_num)
        % m_sc_str_table[line_num-1]
        % msg;

    m_msgs.push_back(oss.str());
}

/*! @brief   メッセージの追加を行う 
 *  @param   msgs メッセージ 
 *  @param   time SIM時間 
 */
void C_scenario::addMsg(std::vector<std::string> msgs, boost::shared_ptr<C_SimTime> &time) {
    std::vector<std::string>::iterator ite = msgs.begin();
    for (; ite != msgs.end(); ++ite) {
        addMsg(*ite,time);
    }
}

/*! @brief   ダンプ関数 
 *  @return  ダンプ 
 */
std::vector<std::string> C_scenario::dump() {
    std::vector<std::string> msg;

    std::vector<C_scenario_elem>::iterator sc_elem_ite = m_sc_table.begin();
    for (; sc_elem_ite != m_sc_table.end(); ++sc_elem_ite) {
        std::ostringstream oss("");
        oss << "sc_idx : " << (sc_elem_ite - m_sc_table.begin());
        msg.push_back(oss.str());
        msg.push_back(sc_elem_ite->dump());
    }

    return msg;
}
