/*! 
    @file  C_scenario.h 
    @brief シナリオクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SCENARIO_H
#define C_SCENARIO_H

#include <string>
#include <vector>
#include <map>
#include <stack>
#include <boost/shared_ptr.hpp>
#include "C_SimTime.h"
#include "C_scenario_elem.h"
#include "C_sc_ptr_table.h"

/*! 
  シナリオクラス 
  @brief  シナリオ実行を行う機能を持つクラス 
 */
class C_scenario {
private:
    /* メンバ変数 */
    static const size_t STACK_SIZE = 1000;


    /* シナリオ名 */
    std::string m_sc_name;

    /* 次の実行時間 */
    boost::shared_ptr<C_SimTime> m_next_time;

    /* 次の実行時間のframe */
    int *m_next_frame;

    /* 次の実行時間のtick */
    int *m_next_tick;

    /* シナリオテーブルインデックス */
    int m_sc_idx;
    
    /* シナリオファイル文字列データ */
    std::vector<std::string> m_sc_str_table;

    /* シナリオ要素テーブル */
    std::vector<C_scenario_elem> m_sc_table;

    /* サブルーチンスタック  
     * リターンインデックスのスタック */
    std::stack<int> m_sub_stack;

    /* トレースフラグ */
    bool m_trace_flag;

    /* トレース情報 */
    std::vector<std::string> m_trace_info;

    /* メッセージ(エラー、ワーニング等) */
    std::vector<std::string> m_msgs;

    /* シナリオで使用するポインタのテーブル */
    boost::shared_ptr<C_sc_ptr_table> m_sc_ptr_table;

public:
    /* public メンバ関数 */

    /*! @brief コンストラクタ 
     *  @param sc_name      シナリオ名 
     *  @param sc_str_table シナリオファイルの文字列データ 
     *  @param sc_table     シナリオ要素のテーブル 
     *  @param sc_ptr_table シナリオで使用されるポインタのテーブル 
     *  @param frame_ptr    実行時刻frame用ポインタ 
     *  @param tick_ptr     実行時刻tick用ポインタ 
     *  @param trace_flag   トレースオン/オフ 
     */
    C_scenario(std::string sc_name,
               std::vector<std::string> &sc_str_table,
               std::vector<C_scenario_elem> &sc_table,
               boost::shared_ptr<C_sc_ptr_table> &sc_ptr_table,
               int* frame_ptr, int* tick_ptr, bool trace_flag=false);

    /*! @brief   シナリオ実行関数 
     *  @param   current_time 現在の時間 
     *  @return  true 通常実行, false シナリオ停止 
     */
    bool execute(boost::shared_ptr<C_SimTime> &current_time);

    /*! @brief   トレース情報の取得 
     *  @return  トレース情報 
     */
    std::vector<std::string> getTraceInfo();

    /*! @brief   エラーメッセージの取得 
     *  @return  エラーメッセージ 
     */
    std::vector<std::string> getMsg();

    /*! @brief   ダンプ関数 
     *  @return  ダンプ 
     */
    std::vector<std::string> dump();

private:
    /* private メンバ関数 (暫定) */

    /*! @brief   トレース情報の生成を行う 
     *  @return  トレース情報 
     */
    std::string createTraceInfo(boost::shared_ptr<C_SimTime> &time);

    /*! @brief   メッセージの追加を行う 
     *  @param   msg  メッセージ 
     *  @param   time SIM時間 
     */
    void addMsg(std::string &msg, boost::shared_ptr<C_SimTime> &time);

    /*! @brief   メッセージの追加を行う 
     *  @param   msgs メッセージ 
     *  @param   time SIM時間 
     */
    void addMsg(std::vector<std::string> msgs, boost::shared_ptr<C_SimTime> &time);

};

#endif
