/*! 
    @file  C_scenario_elem.cpp 
    @brief シナリオ要素クラス  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_scenario_elem.h"
#include "C_err_msg.h"
#include <sstream>
#include <boost/format.hpp>

/*! @brief コンストラクタ 
 */
C_scenario_elem::C_scenario_elem()
    : m_line_num(0), m_type(C_scenario_elem::NON)
{}

/*! @brief   シナリオタイプ設定 
 *  @param   param       パラメータ 
 *  @param   val         値 
 *  @param   line_num    シナリオファイル行数 
 */
template<typename T>
void C_scenario_elem::setDirectSetCmdTmpl(T param,
                                          T val,
                                          int line_num) {
    m_args.clear();
    C_sc_cmd_arg param_arg(param);
    C_sc_cmd_arg val_arg(val);

    m_type = C_scenario_elem::DIRECT_SET;
    m_args.push_back(param_arg);
    m_args.push_back(val_arg);
    m_line_num = line_num;
}


/*! @brief   シナリオタイプ設定 
 *  @param   param       パラメータ 
 *  @param   val         値 
 *  @param   line_num    シナリオファイル行数 
 */
template<typename T>
void C_scenario_elem::setRelativeSetCmdTmpl(T param,
                                            T val,
                                            int line_num) {
    m_args.clear();
    C_sc_cmd_arg param_arg(param);
    C_sc_cmd_arg val_arg(val);

    m_type = C_scenario_elem::RELATIVE_SET;
    m_args.push_back(param_arg);
    m_args.push_back(val_arg);
    m_line_num = line_num;
}

/*! @brief   WAITコマンドの設定 
 *  @param   cond_lhs     条件式の左辺 (パラメータ) 
 *  @param   cond_rhs     条件式の右辺 (パラメータ) 
 *  @param   cond_op      条件式の比較演算子      
 *  @param   time_count   タイムカウント 
 *  @param   timeout_num  タイムアウト時間 
 *  @param   timeout_type タイムアウトタイプ (N or W or E)      
 *  @param   time_ptr     時刻ポインタ 
 *  @param   line_num     シナリオファイル行数 
 */
template<typename T>
void C_scenario_elem::setWaitCmdTmpl(T cond_lhs,
                                     T cond_rhs,
                                     std::string *cond_op,
                                     int *time_count,
                                     int *timeout_time,
                                     std::string *timeout_type,
                                     int *time_ptr,
                                     int line_num) {
    m_args.clear();
    C_sc_cmd_arg cond_lhs_arg(cond_lhs);
    C_sc_cmd_arg cond_rhs_arg(cond_rhs);
    C_sc_cmd_arg cond_op_arg(cond_op);
    C_sc_cmd_arg time_count_arg(time_count);
    C_sc_cmd_arg timeout_time_arg(timeout_time);
    C_sc_cmd_arg timeout_type_arg(timeout_type);
    C_sc_cmd_arg time_ptr_arg(time_ptr);

    m_type = C_scenario_elem::WAIT;
    m_args.push_back(cond_lhs_arg);
    m_args.push_back(cond_rhs_arg);
    m_args.push_back(cond_op_arg);
    m_args.push_back(time_count_arg);
    m_args.push_back(timeout_time_arg);
    m_args.push_back(timeout_type_arg);
    m_args.push_back(time_ptr_arg);
    m_line_num = line_num;
}

/*! @brief   条件JUMPコマンドの設定 
 *  @param   cond_lhs     条件式の左辺 (パラメータ) 
 *  @param   cond_rhs     条件式の右辺 (パラメータ) 
 *  @param   cond_op      条件式の比較演算子      
 *  @param   label_pos    ラベル位置 
 *  @param   jump_count   ジャンプカウント 
 *  @param   max_jump_num 最大ジャンプ回数 
 *  @param   line_num     シナリオファイル行数 
 */
template<typename T>
void C_scenario_elem::setJumpCmdTmpl(T cond_lhs,
                                     T cond_rhs,
                                     std::string *cond_op,
                                     int *label_pos,
                                     int *jump_count,
                                     int *max_jump_num,
                                     int line_num) {
    m_args.clear();
    C_sc_cmd_arg cond_lhs_arg(cond_lhs);
    C_sc_cmd_arg cond_rhs_arg(cond_rhs);
    C_sc_cmd_arg cond_op_arg(cond_op);
    C_sc_cmd_arg label_pos_arg(label_pos);
    C_sc_cmd_arg jump_count_arg(jump_count);
    C_sc_cmd_arg max_jump_num_arg(max_jump_num);

    m_type = C_scenario_elem::JUMP;
    m_args.push_back(label_pos_arg);
    m_args.push_back(jump_count_arg);
    m_args.push_back(max_jump_num_arg);
    m_args.push_back(cond_lhs_arg);
    m_args.push_back(cond_rhs_arg);
    m_args.push_back(cond_op_arg);
    m_line_num = line_num;
}


/*! @brief   シナリオタイプ設定 
 *  @param   param       パラメータ 
 *  @param   val         値 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setDirectSetCmd(int *param,
                                      int *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(unsigned int *param,
                                      unsigned int *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(double *param,
                                      double *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::string *param,
                                      std::string *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::complex<double> *param,
                                      std::complex<double> *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::vector<int> *param,
                                      std::vector<int> *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::vector<unsigned int> *param,
                                      std::vector<unsigned int> *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::vector<double> *param,
                                      std::vector<double> *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::vector<std::string> *param,
                                      std::vector<std::string> *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setDirectSetCmd(std::vector<std::complex<double> > *param,
                                      std::vector<std::complex<double> > *val,
                                      int line_num) {
    setDirectSetCmdTmpl(param,val,line_num);
}

/*! @brief   直接設定コマンドのパラメータの更新 
 *  @param   param       パラメータ 
 */
void C_scenario_elem::updateDirectSetCmdParam(int *param) {
    if (m_type == C_scenario_elem::DIRECT_SET) {
        m_args[0].setPtr(param);
    }
}


/*! @brief   シナリオタイプ設定 
 *  @param   param       パラメータ 
 *  @param   val         値 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setRelativeSetCmd(int *param,
                                        int *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(unsigned int *param,
                                        unsigned int *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(double *param,
                                        double *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::string *param,
                                        std::string *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::complex<double> *param,
                                        std::complex<double> *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::vector<int> *param,
                                        std::vector<int> *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::vector<unsigned int> *param,
                                        std::vector<unsigned int> *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::vector<double> *param,
                                        std::vector<double> *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::vector<std::string> *param,
                                        std::vector<std::string> *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}

void C_scenario_elem::setRelativeSetCmd(std::vector<std::complex<double> > *param,
                                        std::vector<std::complex<double> > *val,
                                        int line_num) {
    setRelativeSetCmdTmpl(param,val,line_num);
}


/*! @brief   相対設定コマンドのパラメータの更新 
 *  @param   param       パラメータ 
 */
void C_scenario_elem::updateRelativeSetCmdParam(int *param) {
    if (m_type == C_scenario_elem::RELATIVE_SET) {
        m_args[0].setPtr(param);
    }
}


/*! @brief   FORコマンドの設定 
 *  @param   loop_var     ループ変数 
 *  @param   loop_num     ループ回数 
 *  @param   label_pos    ラベル位置      
 *  @param   line_num     シナリオファイル行数 
 */
void C_scenario_elem::setForCmd(int *loop_var,
                                int *loop_num,
                                int *label_pos,
                                int line_num) {
    
    m_args.clear();
    C_sc_cmd_arg loop_var_arg(loop_var);
    C_sc_cmd_arg loop_num_arg(loop_num);
    C_sc_cmd_arg label_pos_arg(label_pos);
    
    m_type = C_scenario_elem::FOR;
    m_args.push_back(loop_var_arg);
    m_args.push_back(loop_num_arg);
    m_args.push_back(label_pos_arg);
    m_line_num = line_num;
}

/*! @brief   FORコマンドのループ変数の更新 
 *  @param   loop_var ループ変数 
 */
void C_scenario_elem::updateForCmdLoopVar(int *loop_var) {
    if (m_type == C_scenario_elem::FOR) {
        m_args[0].setPtr(loop_var);
    }
}

/*! @brief   FORコマンドのラベル位置の更新 
 *  @param   label_pos  ラベル位置 
 */
void C_scenario_elem::updateForCmdLabelPos(int *label_pos) {
    if (m_type == C_scenario_elem::FOR) {
        m_args[2].setPtr(label_pos);
    }
}

/*! @brief   FORENDコマンドの設定 
 *  @param   label_pos    ラベル位置      
 *  @param   line_num     シナリオファイル行数 
 */
void C_scenario_elem::setForEndCmd(int *label_pos,
                                   int line_num) {
    m_args.clear();
    C_sc_cmd_arg label_pos_arg(label_pos);
    
    m_type = C_scenario_elem::FOREND;
    m_args.push_back(label_pos_arg);
    m_line_num = line_num;
}

/*! @brief   FORENDコマンドのラベル位置の更新 
 *  @param   label_pos  ラベル位置 
 */
void C_scenario_elem::updateForEndCmdLabelPos(int *label_pos) {
    if (m_type == C_scenario_elem::FOREND) {
        m_args[0].setPtr(label_pos);
    }
}


/*! @brief   WAITコマンドの設定 
 *  @param   cond_lhs     条件式の左辺 (パラメータ) 
 *  @param   cond_rhs     条件式の右辺 (パラメータ) 
 *  @param   cond_op      条件式の比較演算子      
 *  @param   time_count   タイムカウント 
 *  @param   timeout_num  タイムアウト時間 
 *  @param   timeout_type タイムアウトタイプ (N or W or E)      
 *  @param   time_ptr     時刻ポインタ 
 *  @param   line_num     シナリオファイル行数 
 */
void C_scenario_elem::setWaitCmd(int *cond_lhs,
                                 int *cond_rhs,
                                 std::string *cond_op,
                                 int *time_count,
                                 int *timeout_time,
                                 std::string *timeout_type,
                                 int *time_ptr,
                                 int line_num) {
    setWaitCmdTmpl(cond_lhs,cond_rhs,cond_op,
                   time_count,timeout_time,timeout_type,time_ptr,line_num);
}

void C_scenario_elem::setWaitCmd(unsigned int *cond_lhs,
                                 unsigned int *cond_rhs,
                                 std::string *cond_op,
                                 int *time_count,
                                 int *timeout_time,
                                 std::string *timeout_type,
                                 int *time_ptr,
                                 int line_num) {
    setWaitCmdTmpl(cond_lhs,cond_rhs,cond_op,
                   time_count,timeout_time,timeout_type,time_ptr,line_num);
}

void C_scenario_elem::setWaitCmd(double *cond_lhs,
                                 double *cond_rhs,
                                 std::string *cond_op,
                                 int *time_count,
                                 int *timeout_time,
                                 std::string *timeout_type,
                                 int *time_ptr,
                                 int line_num) {
    setWaitCmdTmpl(cond_lhs,cond_rhs,cond_op,
                   time_count,timeout_time,timeout_type,time_ptr,line_num);
}

void C_scenario_elem::setWaitCmd(std::string *cond_lhs,
                                 std::string *cond_rhs,
                                 std::string *cond_op,
                                 int *time_count,
                                 int *timeout_time,
                                 std::string *timeout_type,
                                 int *time_ptr,
                                 int line_num) {
    setWaitCmdTmpl(cond_lhs,cond_rhs,cond_op,
                   time_count,timeout_time,timeout_type,time_ptr,line_num);
}

/*! @brief   Waitコマンドの時刻カウンタ変数の更新 
 *  @param   time_count 時刻カウンタ変数の更新 
 */
void C_scenario_elem::updateWaitCmdTimeCount(int *time_count) {
    if (m_type == C_scenario_elem::WAIT) {
        m_args[3].setPtr(time_count);
    }
}

/*! @brief   Waitコマンドの時刻ポインタの更新 
 *  @param   time_ptr 時刻ポインタ変数の更新 
 */
void C_scenario_elem::updateWaitCmdTimePtr(int *time_ptr) {
    if (m_type == C_scenario_elem::WAIT) {
        m_args[6].setPtr(time_ptr);
    }
}

/*! @brief   条件JUMPコマンドの設定 
 *  @param   cond_lhs     条件式の左辺 (パラメータ) 
 *  @param   cond_rhs     条件式の右辺 (パラメータ) 
 *  @param   cond_op      条件式の比較演算子      
 *  @param   label_pos    ラベル位置 
 *  @param   jump_count   ジャンプカウント 
 *  @param   max_jump_num 最大ジャンプ回数 
 *  @param   line_num     シナリオファイル行数 
 */
void C_scenario_elem::setJumpCmd(int *cond_lhs,
                                 int *cond_rhs,
                                 std::string *cond_op,
                                 int *label_pos,
                                 int *jump_count,
                                 int *max_jump_num,
                                 int line_num) {
    setJumpCmdTmpl(cond_lhs,cond_rhs,cond_op,label_pos,jump_count,max_jump_num,line_num);
}

void C_scenario_elem::setJumpCmd(unsigned int *cond_lhs,
                                 unsigned int *cond_rhs,
                                 std::string *cond_op,
                                 int *label_pos,
                                 int *jump_count,
                                 int *max_jump_num,
                                 int line_num) {
    setJumpCmdTmpl(cond_lhs,cond_rhs,cond_op,label_pos,jump_count,max_jump_num,line_num);
}

void C_scenario_elem::setJumpCmd(double *cond_lhs,
                                 double *cond_rhs,
                                 std::string *cond_op,
                                 int *label_pos,
                                 int *jump_count,
                                 int *max_jump_num,
                                 int line_num) {
    setJumpCmdTmpl(cond_lhs,cond_rhs,cond_op,label_pos,jump_count,max_jump_num,line_num);
}

void C_scenario_elem::setJumpCmd(std::string *cond_lhs,
                                 std::string *cond_rhs,
                                 std::string *cond_op,
                                 int *label_pos,
                                 int *jump_count,
                                 int *max_jump_num,
                                 int line_num) {
    setJumpCmdTmpl(cond_lhs,cond_rhs,cond_op,label_pos,jump_count,max_jump_num,line_num);
}


/*! @brief   無条件JUMPコマンドの設定 
 *  @param   label_pos    ラベル位置 
 *  @param   jump_count   ジャンプカウント 
 *  @param   max_jump_num 最大ジャンプ回数 
 *  @param   line_num     シナリオファイル行数 
 */
void C_scenario_elem::setJumpCmd(int *label_pos,
                                 int *jump_count,
                                 int *max_jump_num,
                                 int line_num) {
    m_args.clear();
    C_sc_cmd_arg label_pos_arg(label_pos);
    C_sc_cmd_arg jump_count_arg(jump_count);
    C_sc_cmd_arg max_jump_num_arg(max_jump_num);
    
    m_type = C_scenario_elem::JUMP;
    m_args.push_back(label_pos_arg);
    m_args.push_back(jump_count_arg);
    m_args.push_back(max_jump_num_arg);
    m_line_num = line_num;
}

/*! @brief   JUMPコマンドのジャンプカウンタ変数の更新 
 *  @param   jump_count ジャンプカウンタ変数 
 */
void C_scenario_elem::updateJumpCmdJumpCount(int *jump_count) {
    if (m_type == C_scenario_elem::JUMP) {
        m_args[1].setPtr(jump_count);
    }
}

/*! @brief   JUMPコマンドのラベル位置の更新 
 *  @param   label_pos  ラベル位置 
 */
void C_scenario_elem::updateJumpCmdLabelPos(int *label_pos) {
    if (m_type == C_scenario_elem::JUMP) {
        m_args[0].setPtr(label_pos);
    }
}

/*! @brief   CALLコマンドの設定 
 *  @param   sub_pos     サブルーチンの先頭位置 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setCallCmd(int *sub_pos,
                                 int line_num) {
    m_args.clear();
    C_sc_cmd_arg sub_pos_arg(sub_pos);
    
    m_type = C_scenario_elem::CALL;
    m_args.push_back(sub_pos_arg);
    m_line_num = line_num;
}

/*! @brief   CALLコマンドのサブルーチン先頭位置の更新 
 *  @param   sub_pos サブルーチン位置 
 */
void C_scenario_elem::updateCallCmdSubPos(int *sub_pos) {
    if (m_type == C_scenario_elem::CALL) {
        m_args[0].setPtr(sub_pos);
    }
}


/*! @brief   シナリオ停止コマンドの設定 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setStopCmd(int line_num) {
    m_args.clear();
    m_type = C_scenario_elem::STOP;
    m_line_num = line_num;
}

/*! @brief   シナリオ終端コマンドの設定 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setEndCmd(int line_num) {
    m_args.clear();
    m_type = C_scenario_elem::END;
    m_line_num = line_num;
}

/*! @brief   サブルーチン終端コマンドの設定 
 *  @param   line_num    シナリオファイル行数 
 */
void C_scenario_elem::setSubEndCmd(int line_num) {
    m_args.clear();
    m_type = C_scenario_elem::SUBEND;
    m_line_num = line_num;
}


/*! @brief   シナリオ要素実行関数 
 *  @param   param        パラメータクラスオブジェクト 
 *  @param   next_cmd_idx 次の実行シナリオコマンドインデックス 
 *  @return  true 通常実行, false エラー 
 */
bool C_scenario_elem::execute(int &next_cmd_idx) {
    bool success = true;

    try {
        switch (m_type) {
        case C_scenario_elem::DIRECT_SET:
            m_args[0].assignVal(m_args[1]);
            next_cmd_idx += 1;
            break;
        case C_scenario_elem::RELATIVE_SET:
            m_args[0].addAndAssignVal(m_args[1]);
            next_cmd_idx += 1;
            break;
        case C_scenario_elem::FOR:
            success = execForCmd(next_cmd_idx);
            break;
        case C_scenario_elem::FOREND:
            success = execForEndCmd(next_cmd_idx);
            break;
        case C_scenario_elem::WAIT:
            success = execWaitCmd(next_cmd_idx);
            break;
        case C_scenario_elem::JUMP:
            success = execJumpCmd(next_cmd_idx);
            break;
        case C_scenario_elem::CALL:
            success = execCallCmd(next_cmd_idx);
            break;
        case C_scenario_elem::STOP:
            next_cmd_idx = -1;
            break;
        case C_scenario_elem::END:
            next_cmd_idx = -1;
            break;
        case C_scenario_elem::SUBEND:
            next_cmd_idx = -1;
            break;
        default:
            /* エラー (未定義コマンドエラー) */
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_UNDEF_CMD);
            m_msgs.push_back(msg);
            return false;
        }

        if (!success && m_msgs.empty()) {
            /* エラー (実行時エラー) */
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_CMD_EXECUTE);
            m_msgs.push_back(msg);
            return false;
        }
        else if( !success ){
            return false;
        }
    }
    catch (std::exception &e) {
        /* エラー処理 (実行時型違反エラー) */
        C_err_msg err_msg;
        std::string msg;
        if (m_type == C_scenario_elem::DIRECT_SET && m_type == C_scenario_elem::RELATIVE_SET) {
            msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_ASSIGN_TYPE);
        }
        else {
            msg= err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_COMP_TYPE);
        }
        m_msgs.push_back(msg);
        return false;
    }
    return true;
}

/*! @brief   シナリオ要素判定関数 
 *  @return  true 該当コマンド, false 該当コマンドではない 
 */
bool C_scenario_elem::isDirectSetCmd() {
    return (m_type == C_scenario_elem::DIRECT_SET);
}

bool C_scenario_elem::isRelativeSetCmd() {
    return (m_type == C_scenario_elem::RELATIVE_SET);
}

bool C_scenario_elem::isForCmd() {
    return (m_type == C_scenario_elem::FOR);
}

bool C_scenario_elem::isForEndCmd() {
    return (m_type == C_scenario_elem::FOREND);
}

bool C_scenario_elem::isWaitCmd() {
    return (m_type == C_scenario_elem::WAIT);
}

bool C_scenario_elem::isJumpCmd() {
    return (m_type == C_scenario_elem::JUMP);
}

bool C_scenario_elem::isCallCmd() {
    return (m_type == C_scenario_elem::CALL);
}

bool C_scenario_elem::isStopCmd() {
    return (m_type == C_scenario_elem::STOP);
}

bool C_scenario_elem::isEndCmd() {
    return (m_type == C_scenario_elem::END);
}

bool C_scenario_elem::isSubEndCmd() {
    return (m_type == C_scenario_elem::SUBEND);
}

/*! @brief   シナリオコマンド設定関数 
 *  @return  シナリオファイル行番号 
 */
int C_scenario_elem::getLineNum() {
    return m_line_num;
}


bool C_scenario_elem::execForCmd(int &next_cmd_idx) {
    /* FORコマンドのm_argsの中身 
     * m_args[0] : ループ変数 
     * m_args[1] : 最大ループ回数 
     * m_args[2] : ループ終了ラベル位置 
     */

    bool success = true;

    /* ループ回数チェック */
    if (m_args[0] < m_args[1]) {
        /* ループ変数インクリメント */
        int loop_var = 0;
        success &= m_args[0].getVal(loop_var);
        success &= m_args[0].setVal(loop_var+1);

        /* 次の実行インデックスを設定 */
        next_cmd_idx++;
    }
    else {
        /* 次の実行インデックスをループ終了ラベル位置に設定 */
        success &= m_args[2].getVal(next_cmd_idx);
    }
    return success;
}

bool C_scenario_elem::execForEndCmd(int &next_cmd_idx) {
    /* FORコマンドのm_argsの中身 
     * m_args[0] : ループ先頭位置 
     */

    bool success = true;

    /* 次の実行インデックスをループ先頭ラベル位置に設定 */
    success &= m_args[0].getVal(next_cmd_idx);

    return success;
}

bool C_scenario_elem::execWaitCmd(int &next_cmd_idx) {
    /* WAITコマンドのm_argsの中身 
     * m_args[0] : 条件式左辺 
     * m_args[1] : 条件式右辺 
     * m_args[2] : 比較演算子          (string型) 
     * m_args[3] : 時間のカウント      (int型) 
     * m_args[4] : 最大待ち時間        (int型) 
     * m_args[5] : タイムアウト時処理  (string型) 
     * m_args[6] : 時刻                (int型) 
     */

    bool success = true;

    /* wait時間のチェック */
    if (m_args[3] >= m_args[4]) {
        /* timeout */
        std::string timeout_type;
        success &= m_args[5].getVal(timeout_type);
        if (timeout_type == "E") {
            /* エラー処理 */
            C_err_msg err_msg;
            std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_WAIT_TIME_OVER);
            m_msgs.push_back(msg);
            return false;
        }
        else if (timeout_type == "W") {
            /* ワーニング処理 */
            C_err_msg err_msg;
            std::string msg = err_msg.getWarnMsg(C_err_msg::SCENARIO_ELEM_WARN_WAIT_TIME_OVER);
            m_msgs.push_back(msg);
        }
        /* 次の実行インデックスを設定 */
        next_cmd_idx++;

        /* wait時間を0に戻す */
        int wait_count = 0;
        success &= m_args[3].setVal(wait_count);

        return success;
    }

    /* 比較演算子の取得 */
    std::string cond_op;
    success &= m_args[2].getVal(cond_op);

    /* 比較の実施 */
    if (compairArgs(m_args[0],m_args[1],cond_op)) {
        /* 次の実行インデックスを設定 */
        next_cmd_idx++;

        /* wait時間を0に戻す */
        int wait_count = 0;
        success &= m_args[3].setVal(wait_count);
    }
    else {
        /* wait時間をカウントアップ */
        int wait_count = 0;
        success &= m_args[3].getVal(wait_count);
        success &= m_args[3].setVal(wait_count+1);

        /* 時刻をカウントアップする */
        int time = 0;
        success &= m_args[6].getVal(time);
        success &= m_args[6].setVal(time+1);
    }


    return success;
}

bool C_scenario_elem::execJumpCmd(int &next_cmd_idx) {
    /* JUMPコマンドのm_argsの中身 
     * m_args[0] : ラベル位置       (int型) 
     * m_args[1] : ジャンプ回数     (int型) 
     * m_args[2] : 最大ジャンプ回数 (int型) 
     * m_args[3] : 条件式左辺 
     * m_args[4] : 条件式右辺 
     * m_args[5] : 比較演算子       (string型) 
     */

    bool success = true;

    /* 最大ジャンプ回数のチェック */
    if (m_args[1] >= m_args[2]) {
        /* エラー処理 (最大ジャンプ回数オーバー) */
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::SCENARIO_ELEM_ERR_JUMP_COUNT_OVER);
        m_msgs.push_back(msg);
        return false;
    }

    /* 条件式が設定されている場合 */
    if (m_args.size() == 6) {
        /* 比較演算子の取得 */
        std::string cond_op;
        success &= m_args[5].getVal(cond_op);

        /* 比較の実施 (比較が失敗した場合) */
        if (!compairArgs(m_args[3],m_args[4],cond_op)) {
            /* 次の実行インデックスを設定 */
            next_cmd_idx++;
            return success;
        }
    }

    /* 次の実行インデックスをラベル位置に設定 */
    int label_pos = 0;
    success &= m_args[0].getVal(label_pos);
    next_cmd_idx = label_pos;

    /* ジャンプ回数をカウントアップ */
    int jump_count = 0;
    success &= m_args[1].getVal(jump_count);
    success &= m_args[1].setVal(jump_count+1);

    return success;
}

bool C_scenario_elem::execCallCmd(int &next_cmd_idx) {
    /* CALLコマンドのm_argsの中身 
     * m_args[0] : サブルーチン先頭位置 (int型) 
     */

    bool success = true;

    int sub_pos = 0;
    success &= m_args[0].getVal(sub_pos);
    next_cmd_idx = sub_pos;

    return success;
}

bool C_scenario_elem::compairArgs(C_sc_cmd_arg &cond_lhs,
                                  C_sc_cmd_arg &cond_rhs,
                                  std::string &cond_op) {

    /* 比較処理 */
    bool comp_flag;

    if (cond_op == "==") {
        comp_flag = (cond_lhs == cond_rhs);
    }
    else if (cond_op == "!=") {
        comp_flag = (cond_lhs != cond_rhs);
    }
    else if (cond_op == "<") {
        comp_flag = (cond_lhs < cond_rhs);
    }
    else if (cond_op == "<=") {
        comp_flag = (cond_lhs <= cond_rhs);
    }
    else if (cond_op == ">") {
        comp_flag = (cond_lhs > cond_rhs);
    }
    else if (cond_op == ">=") {
        comp_flag = (cond_lhs >= cond_rhs);
    }
    else {
        comp_flag = false;
    }

    return comp_flag;
}

/*! @brief   メッセージの取得 
 *  @return  メッセージ 
 */
std::vector<std::string> &C_scenario_elem::getMsgs() {
    return m_msgs;
}

/*! @brief   ダンプを取得する 
 *  @return  内部メッセージ 
 */
std::string C_scenario_elem::dump() {
    std::string str;

    switch (m_type) {
    case C_scenario_elem::DIRECT_SET:
        str = dumpDirectSetCmd();
        break;
    case C_scenario_elem::RELATIVE_SET:
        str = dumpRelativeSetCmd();
        break;
    case C_scenario_elem::FOR:
        str = dumpForCmd();
        break;
    case C_scenario_elem::FOREND:
        str = dumpForEndCmd();
        break;
    case C_scenario_elem::WAIT:
        str = dumpWaitCmd();
        break;
    case C_scenario_elem::JUMP:
        str = dumpJumpCmd();
        break;
    case C_scenario_elem::CALL:
        str = dumpCallCmd();
        break;
    case C_scenario_elem::STOP:
        str = dumpStopCmd();
        break;
    case C_scenario_elem::END:
        str = dumpEndCmd();
        break;
    case C_scenario_elem::SUBEND:
        str = dumpSubEndCmd();
        break;
    default:
        ;
    }
    return str;
}

/*! @brief   コマンド別ダンプ関数 
 *  @return  ダンプ 
 */
std::string C_scenario_elem::dumpDirectSetCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("DIRECT_SET_CMD\n(param: %s)\n(value: %s)\n") 
        % m_args[0].dump() 
        % m_args[1].dump()
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpRelativeSetCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("RELATIVE_SET_CMD\n(param: %s)\n(value: %s)\n") 
        % m_args[0].dump() 
        % m_args[1].dump()
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpForCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("FOR_CMD\n(loop_var: %s)\n(loop_num: %s)\n(label_pos %s)\n") 
        % m_args[0].dump() 
        % m_args[1].dump()
        % m_args[2].dump()
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpForEndCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("FOREND_CMD\n(label_pos %s)\n") 
        % m_args[0].dump() 
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpWaitCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("WAIT_CMD\n(cond_lhs: %s)\n(cond_rhs: %s)\n(cond_op: %s)\n") 
        % m_args[0].dump() 
        % m_args[1].dump()
        % m_args[2].dump()
        << boost::format("(time_count: %s)\n(timeout_time: %s)\n(timeout_type: %s)\n") 
        % m_args[3].dump() 
        % m_args[4].dump()
        % m_args[5].dump()
        << boost::format("(time_ptr: %s)\n") 
        % m_args[6].dump() 
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpJumpCmd() {
    std::ostringstream oss("");

    if (m_args.size() == 6)
        oss << "-------------------------------\n"
            << boost::format("JUMP_CMD\n(cond_lhs: %s)\n(cond_rhs: %s)\n(cond_op: %s)\n") 
            % m_args[3].dump() 
            % m_args[4].dump()
            % m_args[5].dump()
            << boost::format("(label_pos: %s)\n(jump_count: %s)\n(max_jump_num: %s)\n") 
            % m_args[0].dump() 
            % m_args[1].dump()
            % m_args[2].dump()
            << "-------------------------------\n";
    else {
        oss << "-------------------------------\n"
            << boost::format("JUMP_CMD\n(label_pos: %s)\n(jump_count: %s)\n(max_jump_num: %s)\n") 
            % m_args[0].dump() 
            % m_args[1].dump()
            % m_args[2].dump()
            << "-------------------------------\n";
    }
    return oss.str();
}

std::string C_scenario_elem::dumpCallCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("CALL_CMD\n(sub_pos: %s)\n") 
        % m_args[0].dump() 
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpStopCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("STOP_CMD\n") 
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpEndCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("END_CMD\n") 
        << "-------------------------------\n";
    return oss.str();
}

std::string C_scenario_elem::dumpSubEndCmd() {
    std::ostringstream oss("");
    oss << "-------------------------------\n"
        << boost::format("SUBEND_CMD\n") 
        << "-------------------------------\n";
    return oss.str();
}
