/*! 
    @file  C_scenario_elem.h 
    @brief シナリオ要素クラス  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SCENARIO_ELEM_H
#define C_SCENARIO_ELEM_H

#include <vector>
#include <string>
#include <complex>
#include "C_sc_cmd_arg.h"

/*! 
  シナリオ要素クラス 
  @brief  シナリオ要素を扱うクラス 
 */
class C_scenario_elem {
private:
    /* クラス内定義 */
    enum E_code_type {
        NON = 0,
        DIRECT_SET,
        RELATIVE_SET,
        FOR,
        FOREND,
        WAIT,
        JUMP,
        CALL,
        STOP,
        END,
        SUBEND
    };

    /* メンバ変数 */
    int m_line_num;                     /*!< シナリオファイルの実行行数 */
    int m_type;                         /*!< シナリオ要素種別 */
    std::vector<C_sc_cmd_arg> m_args;   /*!< コマンド引数     */ 
    std::vector<std::string> m_msgs;    /*!< メッセージ       */
public:

    /*! @brief コンストラクタ 
     */
    C_scenario_elem();

    /*! @brief   シナリオタイプ設定 
     *  @param   param       パラメータ 
     *  @param   val         値 
     *  @param   line_num    シナリオファイル行数 
     */
    void setDirectSetCmd(int *param,
                         int *val,
                         int line_num);
    void setDirectSetCmd(unsigned int *param,
                         unsigned int *val,
                         int line_num);
    void setDirectSetCmd(double *param,
                         double *val,
                         int line_num);
    void setDirectSetCmd(std::string *param,
                         std::string *val,
                         int line_num);
    void setDirectSetCmd(std::complex<double> *param,
                         std::complex<double> *val,
                         int line_num);
    void setDirectSetCmd(std::vector<int> *param,
                         std::vector<int> *val,
                         int line_num);
    void setDirectSetCmd(std::vector<unsigned int> *param,
                         std::vector<unsigned int> *val,
                         int line_num);
    void setDirectSetCmd(std::vector<double> *param,
                         std::vector<double> *val,
                         int line_num);
    void setDirectSetCmd(std::vector<std::string> *param,
                         std::vector<std::string> *val,
                         int line_num);
    void setDirectSetCmd(std::vector<std::complex<double> > *param,
                         std::vector<std::complex<double> > *val,
                         int line_num);

    /*! @brief   直接設定コマンドのパラメータの更新 
     *  @param   param       パラメータ 
     */
    void updateDirectSetCmdParam(int *param);

    /*! @brief   シナリオタイプ設定 
     *  @param   param       パラメータ 
     *  @param   val         値 
     *  @param   line_num    シナリオファイル行数 
     */
    void setRelativeSetCmd(int *param,
                         int *val,
                         int line_num);
    void setRelativeSetCmd(unsigned int *param,
                           unsigned int *val,
                           int line_num);
    void setRelativeSetCmd(double *param,
                           double *val,
                           int line_num);
    void setRelativeSetCmd(std::string *param,
                           std::string *val,
                           int line_num);
    void setRelativeSetCmd(std::complex<double> *param,
                           std::complex<double> *val,
                           int line_num);
    void setRelativeSetCmd(std::vector<int> *param,
                           std::vector<int> *val,
                           int line_num);
    void setRelativeSetCmd(std::vector<unsigned int> *param,
                           std::vector<unsigned int> *val,
                           int line_num);
    void setRelativeSetCmd(std::vector<double> *param,
                           std::vector<double> *val,
                           int line_num);
    void setRelativeSetCmd(std::vector<std::string> *param,
                           std::vector<std::string> *val,
                           int line_num);
    void setRelativeSetCmd(std::vector<std::complex<double> > *param,
                           std::vector<std::complex<double> > *val,
                           int line_num);

    /*! @brief   相対設定コマンドのパラメータの更新 
     *  @param   param       パラメータ 
     */
    void updateRelativeSetCmdParam(int *param);

    /*! @brief   FORコマンドの設定 
     *  @param   loop_var     ループ変数 
     *  @param   loop_num     ループ回数 
     *  @param   label_pos    ラベル位置      
     *  @param   line_num     シナリオファイル行数 
     */
    void setForCmd(int *loop_var,
                   int *loop_num,
                   int *label_pos,
                   int line_num);

    /*! @brief   FORコマンドのループ変数の更新 
     *  @param   loop_var ループ変数 
     */
    void updateForCmdLoopVar(int *loop_var);

    /*! @brief   FORコマンドのラベル位置の更新 
     *  @param   label_pos ラベル位置 
     */
    void updateForCmdLabelPos(int *label_pos);

    /*! @brief   FORENDコマンドの設定 
     *  @param   label_pos    ラベル位置      
     *  @param   line_num     シナリオファイル行数 
     */
    void setForEndCmd(int *label_pos,
                      int line_num);

    /*! @brief   FORENDコマンドのラベル位置の更新 
     *  @param   label_pos ラベル位置 
     */
    void updateForEndCmdLabelPos(int *label_pos);


    /*! @brief   WAITコマンドの設定 
     *  @param   cond_lhs     条件式の左辺 (パラメータ) 
     *  @param   cond_rhs     条件式の右辺 (パラメータ) 
     *  @param   cond_op      条件式の比較演算子      
     *  @param   time_count   タイムカウント 
     *  @param   timeout_num  タイムアウト時間 
     *  @param   timeout_type タイムアウトタイプ (N or W or E) 
     *  @param   time_ptr     時刻ポインタ 
     *  @param   line_num     シナリオファイル行数 
     */
    void setWaitCmd(int *cond_lhs,
                    int *cond_rhs,
                    std::string *cond_op,
                    int *time_count,
                    int *timeout_time,
                    std::string *timeout_type,
                    int *time_ptr,
                    int line_num);
    void setWaitCmd(unsigned int *cond_lhs,
                    unsigned int *cond_rhs,
                    std::string *cond_op,
                    int *time_count,
                    int *timeout_time,
                    std::string *timeout_type,
                    int *time_ptr,
                    int line_num);
    void setWaitCmd(double *cond_lhs,
                    double *cond_rhs,
                    std::string *cond_op,
                    int *time_count,
                    int *timeout_time,
                    std::string *timeout_type,
                    int *time_ptr,
                    int line_num);
    void setWaitCmd(std::string *cond_lhs,
                    std::string *cond_rhs,
                    std::string *cond_op,
                    int *time_count,
                    int *timeout_time,
                    std::string *timeout_type,
                    int *time_ptr,
                    int line_num);

    /*! @brief   Waitコマンドの時刻カウンタ変数の更新 
     *  @param   time_count 時刻カウンタ変数の更新 
     */
    void updateWaitCmdTimeCount(int *time_count);

    /*! @brief   Waitコマンドの時刻ポインタの更新 
     *  @param   time_ptr 時刻ポインタ変数の更新 
     */
    void updateWaitCmdTimePtr(int *time_ptr);

    /*! @brief   条件JUMPコマンドの設定 
     *  @param   cond_lhs     条件式の左辺 (パラメータ) 
     *  @param   cond_rhs     条件式の右辺 (パラメータ) 
     *  @param   cond_op      条件式の比較演算子      
     *  @param   label_pos    ラベル位置 
     *  @param   jump_count   ジャンプカウント 
     *  @param   max_jump_num 最大ジャンプ回数 
     *  @param   line_num     シナリオファイル行数 
     */
    void setJumpCmd(int *cond_lhs,
                    int *cond_rhs,
                    std::string *cond_op,
                    int *label_pos,
                    int *jump_count,
                    int *max_jump_num,
                    int line_num);
    void setJumpCmd(unsigned int *cond_lhs,
                    unsigned int *cond_rhs,
                    std::string *cond_op,
                    int *label_pos,
                    int *jump_count,
                    int *max_jump_num,
                    int line_num);
    void setJumpCmd(double *cond_lhs,
                    double *cond_rhs,
                    std::string *cond_op,
                    int *label_pos,
                    int *jump_count,
                    int *max_jump_num,
                    int line_num);
    void setJumpCmd(std::string *cond_lhs,
                    std::string *cond_rhs,
                    std::string *cond_op,
                    int *label_pos,
                    int *jump_count,
                    int *max_jump_num,
                    int line_num);

    /*! @brief   無条件JUMPコマンドの設定 
     *  @param   label_pos    ラベル位置 
     *  @param   jump_count   ジャンプカウント 
     *  @param   max_jump_num 最大ジャンプ回数 
     *  @param   line_num     シナリオファイル行数 
     */
    void setJumpCmd(int *label_pos,
                    int *jump_count,
                    int *max_jump_num,
                    int line_num);

    /*! @brief   JUMPコマンドのジャンプカウンタ変数の更新 
     *  @param   jump_count ジャンプカウンタ変数 
     */
    void updateJumpCmdJumpCount(int *jump_count);

    /*! @brief   JUMPコマンドのラベル位置の更新 
     *  @param   label_pos  ラベル位置 
     */
    void updateJumpCmdLabelPos(int *label_pos);


    /*! @brief   CALLコマンドの設定 
     *  @param   sub_pos     サブルーチンの先頭位置 
     *  @param   line_num    シナリオファイル行数 
     */
    void setCallCmd(int *sub_pos,
                    int line_num);

    /*! @brief   SUBコマンドのサブルーチン先頭位置の更新 
     *  @param   sub_pos  サブルーチン位置 
     */
    void updateCallCmdSubPos(int *sub_pos);

    /*! @brief   シナリオ停止コマンドの設定 
     *  @param   line_num    シナリオファイル行数 
     */
    void setStopCmd(int line_num);

    /*! @brief   シナリオ終端コマンドの設定 
     *  @param   line_num    シナリオファイル行数 
     */
    void setEndCmd(int line_num);

    /*! @brief   サブルーチン終端コマンドの設定 
     *  @param   line_num    シナリオファイル行数 
     */
    void setSubEndCmd(int line_num);


    /*! @brief   シナリオ要素実行関数 
     *  @param   param        パラメータクラスオブジェクト 
     *  @param   next_cmd_idx 次の実行シナリオコマンドインデックス 
     *  @return  true 通常実行, false エラー 
     */
    bool execute(int &next_cmd_idx);

    /*! @brief   シナリオ要素判定関数 
     *  @return  true 該当コマンド, false 該当コマンドではない 
     */
    bool isDirectSetCmd();
    bool isRelativeSetCmd();
    bool isForCmd();
    bool isForEndCmd();
    bool isWaitCmd();
    bool isJumpCmd();
    bool isCallCmd();
    bool isStopCmd();
    bool isEndCmd();
    bool isSubEndCmd();

    /*! @brief   シナリオコマンド設定関数 
     *  @return  シナリオファイル行番号 
     */
    int getLineNum();

    /*! @brief   メッセージの取得 
     *  @return  メッセージ 
     */
    std::vector<std::string> &getMsgs();

    /*! @brief   ダンプを取得する 
     *  @return  内部メッセージ 
     */
    std::string dump();

private:
    bool execDirectSetCmd(int &next_cmd_idx);
    bool execRelativeSetCmd(int &next_cmd_idx);
    bool execForCmd(int &next_cmd_idx);
    bool execForEndCmd(int &next_cmd_idx);
    bool execWaitCmd(int &next_cmd_idx);
    bool execJumpCmd(int &next_cmd_idx);
    bool execCallCmd(int &next_cmd_idx);

    bool compairArgs(C_sc_cmd_arg &cond_lhs,
                     C_sc_cmd_arg &cond_rhs,
                     std::string &cond_op);

    template<typename T>
    void setDirectSetCmdTmpl(T param,
                             T val,
                             int line_num);

    template<typename T>
    void setRelativeSetCmdTmpl(T param,
                               T val,
                               int line_num);


    template<typename T>
    void setWaitCmdTmpl(T cond_lhs,
                        T cond_rhs,
                        std::string *cond_op,
                        int *time_count,
                        int *timeout_time,
                        std::string *timeout_type,
                        int *time_ptr,
                        int line_num);

    template<typename T>
    void setJumpCmdTmpl(T cond_lhs,
                        T cond_rhs,
                        std::string *cond_op,
                        int *label_pos,
                        int *jump_count,
                        int *max_jump_num,
                        int line_num);

    /*! @brief   コマンド別ダンプ関数 
     *  @return  ダンプ 
     */
    std::string dumpDirectSetCmd();
    std::string dumpRelativeSetCmd();
    std::string dumpForCmd();
    std::string dumpForEndCmd();
    std::string dumpWaitCmd();
    std::string dumpJumpCmd();
    std::string dumpCallCmd();
    std::string dumpStopCmd();
    std::string dumpEndCmd();
    std::string dumpSubEndCmd();

};

#endif

