/*! 
    @file  C_subdef_cmd_interpreter.cpp 
    @brief SUBコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_subdef_cmd_interpreter.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_subdef_cmd_interpreter::
C_subdef_cmd_interpreter(boost::shared_ptr<I_param> param,
                         boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_subdef = false;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_subdef_cmd_interpreter::reset() {
    m_check = false;
    m_subdef = false;
    clearErrMsg();
}


/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_subdef_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {

    /* SUBコマンドのトークン列サイズチェック */
    std::vector<std::string>::iterator subdef = begin;
    bool size_check ((end - begin) == 2);

    bool subdef_token_check = false;

    if (size_check) {
        /* SUBコマンドのチェック */
        subdef_token_check = (*subdef == "@SUB");
    }

    m_check = (size_check && subdef_token_check);

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     subdef_token_check);

    return m_check;
}

/*! @brief   トークン列からサブルーチン定義をチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @param   sub_map サブルーチンマップ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_subdef_cmd_interpreter::
checkSubDef(std::vector<std::string>::iterator begin,
            std::vector<std::string>::iterator end,
            std::map<std::string,std::pair<size_t,size_t> > &sub_map) {

    m_subdef = false;
    if (m_check) {
        /* シナリオ定義がすでにされているかのチェック */
        std::vector<std::string>::iterator sub_name = begin+1;
        m_subdef = (sub_map.find(*sub_name) == sub_map.end());

        /* エラー発生時、エラーメッセージ生成 */
        if (!m_subdef) createSubDefErrMsg();
    }

    return m_subdef;
}

/*! @brief   トークン列からサブルーチン名を取得する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  サブルーチン名 
 */
std::string C_subdef_cmd_interpreter::getSubName(std::vector<std::string>::iterator begin,
                                                 std::vector<std::string>::iterator end) {
    if (!m_subdef || !m_check) return "";
    std::vector<std::string>::iterator sub_name = begin+1;
    return *sub_name;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_subdef_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check  = " << m_check << "\n"
        << "m_subdef = " << m_subdef;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   subdef_token_check         SCENARIOコマンドトークン存在チェック 
 */
void C_subdef_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                  bool subdef_token_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* サブルーチン定義コマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SUBDEF_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* "@SUB"トークンが不正ときにエラーメッセージを追加 */
    if (!subdef_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SUBDEF_CMD_INTERPRETER_ERR_SUBDEF_CMD_POS);
        addErrMsg(msg);
    }
}

/*! @brief   サブルーチン定義エラーメッセージの生成 
 */
void C_subdef_cmd_interpreter::createSubDefErrMsg() {
    C_err_msg err_msg;
    std::string msg = "";

    /* 同一サブルーチンの再定義時エラー時にエラーメッセージを追加 */
    msg = err_msg.getErrMsg(C_err_msg::SUBDEF_CMD_INTERPRETER_ERR_REDEF_SUB);
    addErrMsg(msg);
}
