/*! 
    @file  C_subend_cmd_interpreter.cpp 
    @brief SUBENDコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_subend_cmd_interpreter.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_subend_cmd_interpreter::
C_subend_cmd_interpreter(boost::shared_ptr<I_param> param,
                         boost::shared_ptr<C_sc_ptr_table> sc_ptr_table)
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
}
    

/*! @brief   インタプリタの状態をリセットする。 
 */
void C_subend_cmd_interpreter::reset() {
    m_check = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_subend_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {
    std::vector<std::string>::iterator subend = begin;

    /* SUBENDコマンドのトークン列サイズチェック */
    bool size_check ((end - begin) == 1);

    bool subend_token_check = false;
    if (size_check) {
        /* SUBENDコマンドのチェック */
        subend_token_check = (*subend == "@SUBEND");
    }

    m_check = (size_check && subend_token_check);

    /* エラー発生時、エラーメッセージを生成 */
    if (!m_check) createFormatErrMsg(size_check,
                                     subend_token_check);

    return m_check;
}


/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_subend_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                                    std::vector<std::string>::iterator end,
                                                    int line_num) {
    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;
    
    sc_elem.setSubEndCmd(line_num);
    return sc_elem;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_subend_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   subend_token_check       SUBENDコマンドトークン存在チェック 
 */
void C_subend_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                  bool subend_token_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* SUBENDコマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SUBEND_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* "@SUBEND"トークンが不正ときにエラーメッセージを追加 */
    if (!subend_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::SUBEND_CMD_INTERPRETER_ERR_SUBEND_CMD_POS);
        addErrMsg(msg);
    }
}
