/*! 
    @file  C_tdset_cmd_interpreter.cpp 
    @brief 実行時刻直接設定コマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_tdset_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_tdset_cmd_interpreter::
C_tdset_cmd_interpreter(boost::shared_ptr<I_param> param,
                        boost::shared_ptr<C_sc_ptr_table> sc_ptr_table) 
    : C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_type = C_tdset_cmd_interpreter::NON;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_tdset_cmd_interpreter::reset() {
    m_check = false;
    m_type = C_tdset_cmd_interpreter::NON;
    clearErrMsg();
}


/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_tdset_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end) {
    std::vector<std::string>::iterator time_var = begin;
    std::vector<std::string>::iterator frame_var = begin;
    std::vector<std::string>::iterator val = begin+1;

    /* コマンドトークン列のサイズチェック */
    bool size_check = ((end - begin) == 2);

    bool time_token_check = false;
    bool frame_token_check = false;
    bool val_check = false;

    if (size_check) {
        /* 時刻変数のチェック */
        time_token_check = (*time_var == "@TIME");
        frame_token_check = (*frame_var == "@FRAME");

        /* 設定時刻がint型かどうかのチェック */
        C_token_list val_token(val,val+1);
        val_check = (val_token.isInt());

        /* 時刻変数の種類に応じてタイプを設定 */
        if (time_token_check) m_type = C_tdset_cmd_interpreter::TIME;
        else if (frame_token_check) m_type = C_tdset_cmd_interpreter::FRAME;
    }

    m_check = (size_check && (time_token_check || frame_token_check) && val_check);

    /* エラーだった場合に、エラーメッセージ生成 */
    if (!m_check) {
        createFormatErrMsg(size_check,
                           time_token_check,
                           frame_token_check,
                           val_check);
        m_type = C_tdset_cmd_interpreter::NON;
    }

    return m_check;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_tdset_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                                   std::vector<std::string>::iterator end,
                                                   int line_num) {
    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;

    std::vector<std::string>::iterator time_var = begin;
    std::vector<std::string>::iterator frame_var = begin;
    std::vector<std::string>::iterator num = begin+1;

    C_token_list num_token(num,num+1);
    int *val = new int(num_token.toInt());
    getScPtrTablePtr()->addPtr(val);

    int *time_var_ptr = NULL;
    if (m_type == C_tdset_cmd_interpreter::TIME) {
        sc_elem.setDirectSetCmd(time_var_ptr,val,line_num);
    }
    else if (m_type == C_tdset_cmd_interpreter::FRAME) {
        sc_elem.setDirectSetCmd(time_var_ptr,val,line_num);
    }

    return sc_elem;
}

/*! @brief   @TIMEへの設定コマンドかどうかを返す。 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  1 @TIMEへの設定コマンド、0 @FRAMEへの設定コマンド 
 */
bool C_tdset_cmd_interpreter::isTimeSetCmd(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {
    checkFormat(begin,end);
    bool result = (m_type == C_tdset_cmd_interpreter::TIME);
    reset();
    return result;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_tdset_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << "\n"
        << "m_type  = " << m_type;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check        トークン列のサイズチェック結果 
 *  @param   time_token_check  TIMEトークンの存在チェック結果 
 *  @param   frame_token_check FRAMEトークンの存在チェック結果 
 *  @param   val_check         値のチェック結果 
 */
void C_tdset_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                 bool time_token_check,
                                                 bool frame_token_check,
                                                 bool val_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* トークン列のサイズが不正時にエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::TDSET_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* @TIME,@FRAMEトークンにエラーがある場合、エラーメッセージを追加 */
    if (!time_token_check && !frame_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::TDSET_CMD_INTERPRETER_ERR_TIME_VAR_POS);
        addErrMsg(msg);
    }

    /* 実行時刻にエラーある場合、エラーメッセージを追加 */
    if (!val_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::TDSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}
