/*! 
    @file  C_token_list.cpp 
    @brief トークンリストクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include <sstream>
#include <bitset>
#include <boost/dynamic_bitset.hpp>
#include <boost/lexical_cast.hpp>
#include "C_token_list.h"
#include "C_err_msg.h"

/*! @brief   コンストラクタ 
 *  @param   begin 先頭イテレータ 
 *  @param   end   終端イテレータ 
 */
C_token_list::C_token_list(std::vector<std::string>::iterator begin,
                           std::vector<std::string>::iterator end)
    : m_begin(begin), m_end(end)
{
    createList();
}


/*! @brief   先頭イテレータを返す 
 *  @return  先頭イテレータ 
 */
std::vector<std::string>::iterator C_token_list::begin() {
    return m_begin;
}

/*! @brief   終端イテレータを返す 
 *  @return  終端イテレータ 
 */
std::vector<std::string>::iterator C_token_list::end() {
    return m_end;
}

/*! @brief   与えられた範囲のトークン列からパラメータ名を生成 
 *  @return  文字列のvector 
 */
std::vector<std::string> C_token_list::createParamName(boost::shared_ptr<I_param> param) {

    std::vector<std::string> param_names;
    if ((m_list.end() - m_list.begin()) == 1) {
        /* 単一パラメータ */
        param_names.push_back(*m_list.begin());
    }
    else if (isArrayParam(param)) {
        /* 配列パラメータ */
        param_names = toArrayParam(param);
    }
    else {
        /* エラー */
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::TOKEN_LIST_ERR_PARAM_NAME);
        m_err_msg.push_back(msg);
    }

    /* パラメータのエラーメッセージ等をクリア */
    param->clearMsg();

    return param_names;
}

/*! @brief   与えられた範囲のトークン列から値の文字列を生成 
 *  @return  値の文字列 
 */
std::string C_token_list::createValStr() {

    std::string val_str("");

    if ((m_list.end() - m_list.begin()) == 1) {
        /* 単一の値 */
        val_str = *m_list.begin();
    }
    else if (isVectorVal()) {
        /* vector形式である */
        val_str = toVectroValStr();
    }
    else {
        /* エラー処理 */
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::TOKEN_LIST_ERR_VAL_STR);
        m_err_msg.push_back(msg);
    }
    return val_str;
}


/*! @brief   トークン列が整数の文字列であるかの判定関数 
 *  @return  true: 整数, false: 整数でない 
 */
bool C_token_list::isInt() {
    return ((m_list.end() - m_list.begin()) == 1 && TokenIsInt(*m_list.begin()));
}

/*! @brief   トークン列が整数の文字列であるかの判定関数 
 *  @return  true: 整数, false: 整数でない 
 */
bool C_token_list::isUnsignedInt() {
    return ((m_list.end() - m_list.begin()) == 1 && TokenIsUnsignedInt(*m_list.begin()));
}

/*! @brief    トークン列が実数の文字列であるかの判定関数 
 *  @return   true: 実数, false: 実数でない 
 */
bool C_token_list::isDouble() {
    return ((m_list.end() - m_list.begin()) == 1 && TokenIsDouble(*m_list.begin()));
}

/*! @brief    トークン列が文字列であるかの判定関数 
 *  @return   true: 文字列、 false: 文字列ない 
 */
bool C_token_list::isString() {
    return ((m_list.end() - m_list.begin()) == 1);
}

/*! @brief   トークン列が複素数であるかの判定関数 
 *  @return  true:  複素数, false: 複素数でない 
 */
bool C_token_list::isComplex() {

    bool is_complex = false;

    /* vector形式であるかどうかをチェック */
    if (isVectorVal()) {
        /* vector形式の文字列 */
        std::vector<std::string> val_table = toVectroVal();

        /* 2項目かつそれらがdouble型かどうかチェック */
        is_complex = (val_table.size() == 2
                      && TokenIsDouble( *val_table.begin() ) 
                      && TokenIsDouble( *(val_table.begin()+1) ));
    }
    return is_complex;
}


/*! @brief   トークン列が整数のvectorであるかの判定関数 
 *  @return  true: 整数のvector, false: 整数のvectorではない 
 */
bool C_token_list::isIntVector() {
    bool is_int_vec = true;

    /* vector形式であるかどうかをチェック */
    if (isVectorVal()) {
        /* vector形式の文字列 */
        std::vector<std::string> val_table = toVectroVal();
        std::vector<std::string>::iterator ite = val_table.begin();
        for (; ite != val_table.end(); ++ite) {
            /* 全項目がint型かどうかチェック */
            is_int_vec &= TokenIsInt(*ite);
        }
    }
    else {
        is_int_vec = false;
    }

    return is_int_vec;
}

/*! @brief   トークン列が整数のvectorであるかの判定関数 
 *  @return  true: 整数のvector, false: 整数のvectorではない 
 */
bool C_token_list::isUnsignedIntVector() {
    bool is_uint_vec = true;

    /* vector形式であるかどうかをチェック */
    if (isVectorVal()) {
        /* vector形式の文字列 */
        std::vector<std::string> val_table = toVectroVal();
        std::vector<std::string>::iterator ite = val_table.begin();
        for (; ite != val_table.end(); ++ite) {
            /* 全項目がunsigned int型かどうかチェック */
            is_uint_vec &= TokenIsUnsignedInt(*ite);
        }
    }
    else {
        is_uint_vec = false;
    }

    return is_uint_vec;
}

/*! @brief   トークン列が実数のvectorであるかの判定関数 
 *  @return  true: 実数値のvector, false: 実数のvector 
 */
bool C_token_list::isDoubleVector() {
    bool is_double_vec = true;

    /* vector形式であるかどうかをチェック */
    if (isVectorVal()) {
        /* vector形式の文字列 */
        std::vector<std::string> val_table = toVectroVal();
        std::vector<std::string>::iterator ite = val_table.begin();
        for (; ite != val_table.end(); ++ite) {
            /* 全項目がdouble型かどうかチェック */
            is_double_vec &= TokenIsDouble(*ite);
        }
    }
    else {
        is_double_vec = false;
    }

    return is_double_vec;
}


/*! @brief   トークン列が文字列のvectorであるかの判定関数 
 *  @param   begin  対象トークンの先頭を指すイテレータ 
 *  @param   end    対象トークンの終端を指すイテレータ 
 *  @return  true: 文字列のvector, false: 数値以外も含まれる 
 */
bool C_token_list::isStringVector() {
    return isVectorVal();
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @param   begin  対象トークンの先頭を指すイテレータ 
 *  @param   end    対象トークンの終端を指すイテレータ 
 *  @return  true: 数値, false: 数値以外も含まれる 
 */
bool C_token_list::isComplexVector() {
    bool is_complex_vec = true;

    /* vector形式であるかどうかをチェック */
    if (isVectorVal()) {
        /* vector形式の文字列 */
        std::vector<std::string> val_table = toVectroVal();

        /* 偶数のチェック & サイズチェック */
        if ((val_table.size() % 2) == 0 && val_table.size() > 2) {
            std::vector<std::string>::iterator ite = val_table.begin();

            /* 全項目がcomplex型かどうかチェック */
            for (; ite != val_table.end(); ite+=2) {
                is_complex_vec &= (TokenIsDouble(*ite) && TokenIsDouble(*(ite+1)));
            }
        }
        else is_complex_vec = false;
    }
    else is_complex_vec = false;

    return is_complex_vec;
}

/*! @brief   トークン列が整数であるかの判定関数 
 *  @return  変換後の値 
 */
int C_token_list::toInt() {
    return TokenToInt(*m_list.begin());
}

/*! @brief   トークン列が整数であるかの判定関数 
 *  @return  変換後の値 
 */
unsigned int C_token_list::toUnsignedInt() {
    return TokenToUnsignedInt(*m_list.begin());
}

/*! @brief   トークン列が実数であるかの判定関数 
 *  @return  変換後の値 
 */
double C_token_list::toDouble() {
    return TokenToDouble(*m_list.begin());
}

/*! @brief   トークン列が文字列であるかの判定関数 
 *  @return  変換後の値 
 */
std::string C_token_list::toString() {
    return *m_list.begin();
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::complex<double> C_token_list::toComplex() {
    return std::complex<double>(TokenToDouble( *m_list.begin() ),
                                TokenToDouble( *(m_list.begin()+2) ));
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::vector<int> C_token_list::toIntVector() {
    std::vector<int> int_vec;
    std::vector<std::string> val_table = toVectroVal();
    std::vector<std::string>::iterator ite = val_table.begin();
    for (; ite != val_table.end(); ++ite) {
        int_vec.push_back(TokenToInt(*ite));
    }
    return int_vec;
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::vector<unsigned int> C_token_list::toUnsignedIntVector() {
    std::vector<unsigned int> uint_vec;
    std::vector<std::string> val_table = toVectroVal();
    std::vector<std::string>::iterator ite = val_table.begin();
    for (; ite != val_table.end(); ++ite) {
        uint_vec.push_back(TokenToUnsignedInt(*ite));
    }
    return uint_vec;
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::vector<double> C_token_list::toDoubleVector() {
    std::vector<double> double_vec;
    std::vector<std::string> val_table = toVectroVal();
    std::vector<std::string>::iterator ite = val_table.begin();
    for (; ite != val_table.end(); ++ite) {
        double_vec.push_back(TokenToDouble(*ite));
    }
    return double_vec;
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::vector<std::complex<double> > C_token_list::toComplexVector() {
    std::vector<std::complex<double> > complex_vec;
    std::vector<std::string> val_table = toVectroVal();
    std::vector<std::string>::iterator ite = val_table.begin();
    for (; ite != val_table.end(); ite+=2) {
        complex_vec.push_back(std::complex<double>(TokenToDouble( *ite ),
                                                   TokenToDouble( *(ite+1) )));
    }
    return complex_vec;
}

/*! @brief   トークン列を複素数のvectorであるかの判定関数 
 *  @return  変換後の値 
 */
std::vector<std::string> C_token_list::toStringVector() {
    return toVectroVal();
}


/**************************************************************** 
 * privateメンバ関数 
 ***************************************************************/

/*! @brief   トークン列を内部list表現に変換する。 
 */
void C_token_list::createList() {
    std::string val("");

    int state = 0;
    std::vector<std::string>::iterator ite = m_begin;
    for (; ite != m_end; ++ite) {
        if (*ite == "," || *ite == "[" || *ite == "]") {
            if (val != "") m_list.push_back(val);
            m_list.push_back(*ite);
            val = "";
        }
        else if (*ite == "(" || *ite == ")") {
            val += *ite;
            state = 1;
        }
        else if (state == 1) {
            /* 前のトークンが"("or")"だった場合 */
            val += *ite;
            state = 1;
        }
        else if (state == 0) {
            /* 前のトークンが"("or")"以外だった場合 */
            if (val != "") m_list.push_back(val);
            val = *ite;
        }
    }
    if (val != "") m_list.push_back(val);
}

/*! @brief   トークン列に配列パラメータが含まれているかどうかの確認 
 *  @param   param パラメータ 
 *  @return  true: 配列パラメータ, false: 配列パラメータではない 
 */
bool C_token_list::isArrayParam(boost::shared_ptr<I_param> param) {

    int state = 0;
    bool is_array = false;

    std::vector<std::string>::iterator ite;
    for (ite = m_list.begin(); ite != m_list.end(); ++ite) {
        if (state == 0 && *ite != "[" && *ite != "]") {
            /* 配列名 */
            state = 1;
            is_array = false;
        }
        else if (state == 1 && *ite == "[") {
            /* 右括弧 */
            state = 2;
            is_array = false;
        }
        else if (state == 2 && *ite != "[" && *ite != "]" && isArrayIndex(param,*ite)) {
            /* 配列インデックス */
            state = 3;
            is_array = false;
        }
        else if (state == 3 && *ite == "]") {
            /* 左括弧 */
            state = 1;
            is_array = true;
        }
        else {
            /* error処理 */
            is_array = false;
            break;
        }
    }

    return is_array;
}

/*! @brief   配列パラメータ名への変換 
 *  @param   param パラメータ 
 *  @return  パラメータ名の展開 
 */
std::vector<std::string> C_token_list::toArrayParam(boost::shared_ptr<I_param> param) {
    std::vector<std::string> result;
    std::vector<std::string>::iterator ite = m_list.begin();

    /* 配列名の設定 */
    result.push_back(*ite);
    ite += 2;

    while (ite != m_list.end()) {
        if (*ite == "[" || *ite == "]") {
            ++ite;
            continue;
        }

        bool param_is_int = false;
        bool val_is_int = false;
        bool is_at = false;
        bool is_range = false;
        int int_val;

        bool param_is_int_end = false;
        bool val_is_int_end = false;
        int int_val_end;
        std::string::size_type pos;

        /* パラメータのチェックとインデックスの値の取得 */
        if (ite->at(0) == '@') {
            param_is_int = param->getValue(ite->substr(1),int_val);
            val_is_int = TokenIsInt(ite->substr(1));
            if (!param_is_int && val_is_int) int_val = TokenToInt(ite->substr(1));
            is_at = true;
        }
        else if( (pos = ite->find("-")) != std::string::npos ){
            /* 開始位置取得 */
            param_is_int = param->getValue(ite->substr(0,pos),int_val);
            val_is_int = TokenIsInt(ite->substr(0,pos));
            if (!param_is_int && val_is_int) int_val = TokenToInt(ite->substr(0,pos));
            /* 終了位置取得 */
            param_is_int_end = param->getValue(ite->substr(pos+1),int_val_end);
            val_is_int_end = TokenIsInt(ite->substr(pos+1));
            if (!param_is_int_end && val_is_int_end) int_val_end = TokenToInt(ite->substr(pos+1));
            is_range = true;
        }
        else {
            param_is_int = param->getValue(*ite,int_val);
            val_is_int = TokenIsInt(*ite);
            if (!param_is_int && val_is_int) int_val = TokenToInt(*ite);
            is_at = false;
        }

        /* 変換 */
        if (is_at && (param_is_int || val_is_int)) {
            /* @付int型パラメータ or @付整数 */
            /* 配列の展開 */
            std::vector<std::string> new_result; /* 展開後の配列 */
            for (size_t i = 0; i < result.size(); i++) {
                for (int j = 0; j < int_val; j++) {
                    std::string idx_str = boost::lexical_cast<std::string>(j);
                    new_result.push_back(result[i] + "[" + idx_str + "]");
                }
            }
            result = new_result;
        }
        else if ( is_range && (param_is_int || val_is_int) && (param_is_int_end || val_is_int_end) ){
            /* 範囲指定配列 */
            if ( int_val_end < int_val ){
                /* 開始位置が終了位置より大きい場合はエラー */
                C_err_msg err_msg;
                std::string msg = err_msg.getErrMsg(C_err_msg::TOKEN_LIST_ERR_ARRAY_RANGE);
                m_err_msg.push_back(msg);
                /* 空のパラメータ名を返す */
                result.clear();
                return result;
            }
            std::vector<std::string> new_result; /* 展開後の配列 */
            for (size_t i = 0; i < result.size(); i++) {
                for (int j = int_val; j <= int_val_end; j++) {
                    std::string idx_str = boost::lexical_cast<std::string>(j);
                    new_result.push_back(result[i] + "[" + idx_str + "]");
                }
            }
            result = new_result;
        }
        else if (param_is_int || val_is_int) {
            /* パラメータ or 整数 */
            /* インデックス追加 */
            std::ostringstream oss("");
            for (size_t i = 0; i < result.size(); i++) {
                std::string idx_str = boost::lexical_cast<std::string>(int_val);
                result[i] += "[" + idx_str + "]";
            }
        }

        /* イテレータインクリメント */
        ++ite;
    }

    return result;
}

/*! @brief   トークンが配列インデックスかどうかを判定 
 *  @param   param パラメータ 
 *  @param   token 対象トークン 
 *  @return  true: 配列インデックス, false: 配列インデックスでない 
 */
bool C_token_list::isArrayIndex(boost::shared_ptr<I_param> param,
                                const std::string &token) {
    bool is_index = false;

    std::string::size_type pos;
    int val;
    if (token[0] == '@'
        && (param->getValue(token.substr(1),val) || TokenIsInt(token.substr(1)))) {
        /* @付整数 */
        is_index = true;
    }
    else if (param->getValue(token,val) || TokenIsInt(token)) {
        /* 整数 */
        is_index = true;
    }
    else if ( (pos = token.find("-")) != std::string::npos ) {
        /* 範囲指定整数 */
        std::string val_str_begin = token.substr(0, pos);
        std::string val_str_end   = token.substr(pos+1);

        bool is_int_begin = ( param->getValue(val_str_begin,val) || TokenIsInt(val_str_begin) );
        bool is_int_end   = ( param->getValue(val_str_end,val)   || TokenIsInt(val_str_end) );

        /* 開始と終了が整数であればtrue */
        is_index = is_int_begin & is_int_end;
    }
    else {
        /* エラー */
        C_err_msg err_msg;
        std::string msg = err_msg.getErrMsg(C_err_msg::TOKEN_LIST_ERR_ARRAY_IDX);
        m_err_msg.push_back(msg);
    }
    return is_index;
}


/*! @brief   与えられた範囲のトークン列がvector形式かどうかを判定 
 *  @return  true 該当形式、false 該当形式ではない 
 */
bool C_token_list::isVectorVal() {

    int state = 0;
    bool is_vector = false;

    std::vector<std::string>::iterator ite;
    for (ite = m_list.begin(); ite != m_list.end(); ++ite) {
        if (state == 0 && *ite != ",") {
            /* 値 */
            state = 1;
            is_vector = true;
        }
        else if (state == 1 && *ite == ",") {
            /* カンマ */
            state = 0;
            is_vector = false;
        }
        else {
            /* error処理 */
            is_vector = false;
            break;
        }
    }

    if ((m_list.end() - m_list.begin()) == 1) {
        /* トークン数が1 */
        is_vector = false;
    }

    return is_vector;
}


/*! @brief   与えられた範囲のトークン列を値の文字列に変換 
 *  @return  値の文字列 
 */
std::string C_token_list::toVectroValStr() {

    std::string var_str;
    std::vector<std::string>::iterator ite;
    for (ite = m_list.begin(); ite != m_list.end(); ++ite) {
        if (*ite != ",") {
            /* 値 */
            var_str += *ite;
        }
        else if (*ite == ",") {
            /* カンマ */
            var_str += " ";
        }
    }

    return var_str;
}

/*! @brief   与えられた範囲のトークン列を値の文字列に変換 
 *  @return  値の文字列vector 
 */
std::vector<std::string> C_token_list::toVectroVal() {

    std::vector<std::string> var_table;
    std::vector<std::string>::iterator ite;
    for (ite = m_list.begin(); ite != m_list.end(); ++ite) {
        if (*ite != ",") {
            /* 値 */
            var_table.push_back(*ite);
        }
    }
    return var_table;
}

/*! @brief   トークンが整数の文字列であるかの判定関数 
 *  @param   token トークン 
 *  @return  true: 整数, false: 整数でない 
 */
bool C_token_list::TokenIsInt(const std::string &token) {

/* #6238 */
    bool is_int = false;
    int  check_val = 0;
    std::istringstream iss(token);
    if (token.find("0x") == 0) {
        /* 先頭に0xがあった場合16進数として処理 */
        /* 16進数で負の数の場合int型に変換失敗するためunsigned int型で取得 */
        unsigned int wk_val;
        iss >> std::hex >> wk_val;
        if( !iss.fail() && iss.eof() ){
            try {
                check_val = static_cast<int>(wk_val);
                is_int = true;
            } catch (std::bad_cast&) {
                is_int = false;
            }
        }
    }
    else if (token.find("0b") == 0) {
        /* 先頭に0bがあった場合2進数として処理 */
        std::string wk_token = token;
        is_int = bin2dec(wk_token, check_val);
    }
    else {
        /* 数字以外の文字が見つからなければ */
        iss >> std::dec >> check_val;
        if( !iss.fail() && iss.eof() ){
            is_int = true;
        }
    }

    return is_int;
}

/*! @brief   トークンが整数の文字列であるかの判定関数 
 *  @param   token トークン 
 *  @return  true: 整数, false: 整数でない 
 */
bool C_token_list::TokenIsUnsignedInt(const std::string &token) {
/* #6238 */
    bool is_uint = false;
    unsigned int check_val = 0;
    std::istringstream uiss(token);
    if (token.find("0x") == 0) {
        /* 先頭に0xがあった場合16進数として処理 */
        uiss >> std::hex >> check_val;
    }
    else if (token.find("0b") == 0) {
        /* 先頭に0bがあった場合2進数として処理 */
        std::string wk_token = token;
        is_uint = bin2dec(wk_token, check_val);
    }
    else {
        /* 数字以外の文字が見つからなければ */
        if (token.find_first_not_of("0123456789") == std::string::npos) {
            uiss >> std::dec >> check_val;
        }
    }
    
    if( !uiss.fail() && uiss.eof() ){
        is_uint = true;
    }
    return is_uint;
}

/*! @brief    トークンが実数の文字列であるかの判定関数 
 *  @param    token トークン 
 *  @return   true: 実数, false: 実数でない 
 */
bool C_token_list::TokenIsDouble(const std::string &token) {
    bool is_double = true;
    double val;
    try {
        val = boost::lexical_cast<double>(token);
    }
    catch (boost::bad_lexical_cast e) {
        is_double = false;
    }
    return is_double;
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @param   token  トークン 
 *  @return  変換後の値 
 */
int C_token_list::TokenToInt(const std::string &token) {
    int val;
    if (token.find("0x") == 0) {
        /* 先頭に0xがあった場合16進数として処理 */
        std::istringstream iss(token);
        /* 16進数で負の数の場合int型に変換失敗するためunsigned int型で取得 */
        unsigned int wk_val;
        iss >> std::hex >> wk_val;
        if( !iss.fail() && iss.eof() ){
            /* 変換に成功したらint型にキャスト */
            val = static_cast<int>(wk_val);
        }
    }
    else if (token.find("0b") == 0) {
        /* 先頭に0bがあった場合2進数として処理 */
        std::bitset<32> binary(getBinStr(token));
        val = static_cast<int>(binary.to_ulong());
    }
    else {
        /* 数字以外の文字が見つからなければ */
        val = boost::lexical_cast<int>(token);
    }

    return val;

}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @param   token  トークン 
 *  @return  変換後の値 
 */
unsigned int C_token_list::TokenToUnsignedInt(const std::string &token) {

    unsigned int val;
    if (token.find("0x") == 0) {
        /* 先頭に0xがあった場合16進数として処理 */
        std::istringstream uiss(token);
        uiss >> std::hex >> val;
    }
    else if (token.find("0b") == 0) {
        /* 先頭に0bがあった場合2進数として処理 */
        std::bitset<32> binary(getBinStr(token));
        val = static_cast<unsigned int>(binary.to_ulong());
    }
    else {
        /* 数字以外の文字が見つからなければ */
        val = boost::lexical_cast<unsigned int>(token);
    }

    return val;
}

/*! @brief   トークン列が複素数のvectorであるかの判定関数 
 *  @param   token  トークン 
 *  @return  変換後の値 
 */
double C_token_list::TokenToDouble(const std::string &token) {
    return boost::lexical_cast<double>(token);
}


/*! @brief   エラーメッセージ取得関数 
 *  @return  エラーメッセージ 
 */
std::vector<std::string> C_token_list::getErrMsg() {
    return m_err_msg;
}

/*! @brief 2進数の文字列を取得する 
 *  @param var_str 値の文字列 
 *  @return  取得後の値 
 */
std::string C_token_list::getBinStr(const std::string& var_str){
    std::string sbuf = var_str.substr(2);
    std::string::iterator it = sbuf.begin();

    for(; it != sbuf.end();){
        if( *it == '_' ){
            /* アンダースコアは取り除く */
            it = sbuf.erase(it);
        } else {
            ++it;
        }
    }

    return sbuf;
}

/*! @brief 2進数の文字列をINT型に変換 
 *  @param var_str 値の文字列 
 *  @param val     値 
 */
bool C_token_list::bin2dec(std::string& var_str, int &val){
    std::string sbuf = var_str.substr(2);
    std::string::iterator it = sbuf.begin();
    bool binchk = true;

    for(; it != sbuf.end();){
        /* 入力データのチェック */
        if( *it == '_' ){
            /* アンダースコアは取り除く */
            it = sbuf.erase(it);
            continue;
        }
        else if( *it != '0' && *it != '1'){
            /* 0,1以外 */
            binchk = false;
            break;
        }
        ++it;
    }

    if( binchk ){
        /* 0,1の2進数表記 */
        try{
            /* 2進数から10進数に変換 */
            if(sbuf.length() > 32) {
                /* Linuxだと32bitを超える値を変換してもcatctされないので対策を入れる 
                ** 仮の対応：32文字(前0も含む)を超えた場合にfalseを返す 
                */
                return false;
            }
            val = (int)boost::dynamic_bitset<unsigned int>(sbuf).to_ulong();
        }catch(std::overflow_error &){
            /* 変換エラー */
            return false;
        }
        /* 変換成功 */
        return true;
    }
    else{
        /* 0,1以外の文字がある=変換失敗 */
        return false;
    }
}

bool C_token_list::bin2dec(std::string& var_str, unsigned int &val){
    std::string sbuf = var_str.substr(2);
    std::string::iterator it = sbuf.begin();
    bool binchk = true;

    for(; it != sbuf.end();){
        /* 入力データのチェック */
        if( *it == '_' ){
            /* アンダースコアは取り除く */
            it = sbuf.erase(it);
            continue;
        }
        else if( *it != '0' && *it != '1'){
            /* 0,1以外 */
            binchk = false;
            break;
        }
        ++it;
    }

    if( binchk ){
        /* 0,1の2進数表記 */
        try{
            /* 2進数から10進数に変換 */
            if(sbuf.length() > 32) {
                /* Linuxだと32bitを超える値を変換してもcatctされないので対策を入れる 
                ** 仮の対応：32文字(前0も含む)を超えた場合にfalseを返す 
                */
                return false;
            }
            val = (unsigned int)boost::dynamic_bitset<unsigned int>(sbuf).to_ulong();
        }catch(std::overflow_error &){
            /* 変換エラー */
            return false;
        }
        /* 変換成功 */
        return true;
    }
    else{
        /* 0,1以外の文字がある=変換失敗 */
        return false;
    }
}

