/*! 
    @file  C_token_list.h 
    @brief トークンリストクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_TOKEN_LIST_H
#define C_TOKEN_LIST_H

#include <vector>
#include <string>
#include <complex>
#include <boost/shared_ptr.hpp>
#include "I_param.h"

/*! トークンリストクラス 
 *  @brief トークン列の判定、変換機能をもつ 
 */
class C_token_list {
protected:
    /* メンバ変数 */
    std::vector<std::string>::iterator m_begin; 
    std::vector<std::string>::iterator m_end;

    std::vector<std::string> m_list;

    std::vector<std::string> m_err_msg;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   begin 先頭イテレータ 
     *  @param   end   終端イテレータ 
     */
    C_token_list(std::vector<std::string>::iterator begin,
                 std::vector<std::string>::iterator end);

    /*! @brief   先頭イテレータを返す 
     *  @return  先頭イテレータ 
     */
    std::vector<std::string>::iterator begin();

    /*! @brief   終端イテレータを返す 
     *  @return  終端イテレータ 
     */
    std::vector<std::string>::iterator end();

    /*! @brief   与えられた範囲のトークン列からパラメータ名を生成 
     *  @param   param パラメータ 
     *  @return  文字列のvector 
     */
    std::vector<std::string> createParamName(boost::shared_ptr<I_param> param);

    /*! @brief   与えられた範囲のトークン列から値の文字列を生成 
     *  @return  値の文字列 
     */
    std::string createValStr();

    /*! @brief   トークン列が整数の文字列であるかの判定関数 
     *  @return  true: 整数, false: 整数でない 
     */
    bool isInt();
    
    /*! @brief   トークン列が整数の文字列であるかの判定関数 
     *  @return  true: 整数, false: 整数でない 
     */
    bool isUnsignedInt();

    /*! @brief    トークン列が実数の文字列であるかの判定関数 
     *  @return   true: 実数, false: 実数でない 
     */
    bool isDouble();

    /*! @brief    トークン列が文字列であるかの判定関数 
     *  @return   true: 文字列、 false: 文字列ない 
     */
    bool isString();

    /*! @brief   トークン列が複素数であるかの判定関数 
     *  @return  true:  複素数, false: 複素数でない 
     */
    bool isComplex();

    /*! @brief   トークン列が整数のvectorであるかの判定関数 
     *  @return  true: 整数のvector, false: 整数のvectorではない 
     */
    bool isIntVector();
    
    /*! @brief   トークン列が整数のvectorであるかの判定関数 
     *  @return  true: 整数のvector, false: 整数のvectorではない 
     */
    bool isUnsignedIntVector();

    /*! @brief   トークン列が実数のvectorであるかの判定関数 
     *  @return  true: 実数値のvector, false: 実数のvector 
     */
    bool isDoubleVector();


    /*! @brief   トークン列が文字列のvectorであるかの判定関数 
     *  @return  true: 文字列のvector, false: 数値以外も含まれる 
     */
    bool isStringVector();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  true: 数値, false: 数値以外も含まれる 
     */
    bool isComplexVector();


    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    int toInt();
    
    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    unsigned int toUnsignedInt();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    double toDouble();

    /*! @brief   トークン列が文字列であるかの判定関数 
     *  @return  変換後の値 
     */
    std::string toString();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::complex<double> toComplex();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::vector<int> toIntVector();
    
    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::vector<unsigned int> toUnsignedIntVector();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::vector<double> toDoubleVector();

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::vector<std::complex<double> > toComplexVector();

    /*! @brief   トークン列を複素数のvectorであるかの判定関数 
     *  @return  変換後の値 
     */
    std::vector<std::string> toStringVector();

    /*! @brief   エラーメッセージ取得関数 
     *  @return  エラーメッセージ 
     */
    std::vector<std::string> getErrMsg();
private:

    /*! @brief   トークン列を内部list表現に変換する。 
     */
    void createList();

    /*! @brief   トークン列に配列パラメータが含まれているかどうかの確認 
     *  @param   param パラメータ 
     *  @return  true: 配列パラメータ, false: 配列パラメータではない 
     */
    bool isArrayParam(boost::shared_ptr<I_param> param);
    
    /*! @brief   配列パラメータ名への変換 
     *  @param   param パラメータ 
     *  @return  パラメータ名の展開 
     */
    std::vector<std::string> toArrayParam(boost::shared_ptr<I_param> param);

    /*! @brief   トークンが配列インデックスかどうかを判定 
     *  @param   param パラメータ 
     *  @param   token 対象トークン 
     *  @return  true: 配列インデックス, false: 配列インデックスでない 
     */
    bool isArrayIndex(boost::shared_ptr<I_param> param,
                      const std::string &token);

    /*! @brief   与えられた範囲のトークン列がvector形式かどうかを判定 
     *  @return  true 該当形式、false 該当形式ではない 
     */
    bool isVectorVal();


    /*! @brief   与えられた範囲のトークン列を値の文字列に変換 
     *  @return  値の文字列 
     */
    std::string toVectroValStr();


    /*! @brief   与えられた範囲のトークン列を値の文字列に変換 
     *  @return  値の文字列vector 
     */
    std::vector<std::string> toVectroVal();

    /*! @brief   トークン列が整数の文字列であるかの判定関数 
     *  @param   token  トークン 
     *  @return  true: 整数, false: 整数でない 
     */
    bool TokenIsInt(const std::string &token);
    
    /*! @brief   トークン列が整数の文字列であるかの判定関数 
     *  @param   token  トークン 
     *  @return  true: 整数, false: 整数でない 
     */
    bool TokenIsUnsignedInt(const std::string &token);

    /*! @brief    トークン列が実数の文字列であるかの判定関数 
     *  @param   token  トークン 
     *  @return   true: 実数, false: 実数でない 
     */
    bool TokenIsDouble(const std::string &token);


    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @param   token  トークン 
     *  @return  変換後の値 
     */
    int TokenToInt(const std::string &token);
    
    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @param   token  トークン 
     *  @return  変換後の値 
     */
    unsigned int TokenToUnsignedInt(const std::string &token);

    /*! @brief   トークン列が複素数のvectorであるかの判定関数 
     *  @param   token  トークン 
     *  @return  変換後の値 
     */
    double TokenToDouble(const std::string &token);

    /*! @brief 2進数の文字列を取得する 
     *  @param var_str 値の文字列 
     *  @return  取得後の値 
     */
    std::string getBinStr(const std::string& var_str);
    
    /*! @brief 2進数の文字列をINT型に変換 
     *  @param var_str 値の文字列 
     *  @param val     値 
     */
    bool bin2dec(std::string& var_str, int &val);
    bool bin2dec(std::string& var_str, unsigned int &val);
};

#endif

