/*! 
    @file  C_tokenizer.h 
    @brief トークン作成機クラス  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_TOKENIZER_H
#define C_TOKENIZER_H

#include <boost/tokenizer.hpp>
#include <vector>
#include <string>

/*! 
  トークン作成機クラス 
  @brief  文字列からトークンを作成する機能を持つクラス 
 */
class C_tokenizer {

private:
    /* クラス内定義 */
    /*! 
      セパレータ (関数オブジェクト用クラス) 
      @brief  boost/tokenizerに分割基準を与える関数オブジェクト用クラス 
    */
    class separator {
    private:
        /* private メンバ変数 */
    public:
        /*! @brief メンバ変数のリセットを実施。 
         */
        void reset() {};

        /* 分割判定処理関数 */
        template <typename Iterator, typename Token>
        bool operator() (Iterator& next, Iterator& end, Token& tok);

    private:
        /* privateメンバ関数 */

        /*! @brief 指数の判別 */
        template <typename Iterator, typename Token>
        bool checkExponential(Iterator& next, Iterator& end, Token& tok);
    };

    /* メンバ変数 */
    std::string m_str;                          /* 分割前文字列 */
    boost::tokenizer<separator> m_tokenizer;    /* boost::tokenizer */
public:
    /*! @brief       コンストラクタ 
     */
    C_tokenizer() : m_str(""), m_tokenizer(m_str) {};

    /*! @brief       分割したい文字列を設定する 
     *  @param lines トークン分割したい文字列 
     */
    void setLine(std::string &lines);

    /*! @brief       分割後のトークンを返す 
     *  @return      tokenのvectorを返す 
     */
    std::vector<std::string> getTokens();

};

/*! @brief       分割の判定を行う。 
 *  @param next  文字列探索箇所を指すイテレータ 
 *  @param end   文字列終端を指すイテレータ 
 *  @return      true: 分割することを示す、false: 分割終了を示す 
 */

template <typename Iterator, typename Token>
bool C_tokenizer::separator::operator() (Iterator& next, Iterator& end, Token& tok) {
    tok = Token();

    /* 文字列終端の場合 */
    if (next == end) {
        return false;
    }

    /* 文字列探索 & 分割判定 */
    int state = 0;
    while (next != end) {
        switch (*next) {
        case ' ': case '\t': case '\n': case '\r':
            /* ' ','\t','\n'文字の場合の処理 */
            if (tok.size() == 0) {
                ++next;
                break;
            }
            return true;
        case '[': case ']': case ':': case ',': case ';': case '(': case ')':
            /* 上記の文字の場合の処理 */
            if (tok.size() == 0) {
                tok = *next;
                ++next;
            }
            return true;
        case '+':
            /* 上記の文字の場合の処理 */
            if ( checkExponential(next, end, tok) ){
                /* 指数の場合 */
                tok += *next;
                ++next;
                state = 0;
                break;
            }
            else if (state == 0 && tok.size() != 0) {
                /* stateが異なり、tokに文字列がある場合 */
                return true;
            }
            tok += *next;
            ++next;
            state = 1;
            break;
        case '=': case '!': case '<': case '>':
            /* 上記の文字の場合の処理 */
            if (state == 0 && tok.size() != 0) {
                /* stateが異なり、tokに文字列がある場合 */
                return true;
            }
            tok += *next;
            ++next;
            state = 1;
            break;
        default:
            /* 上記以外の文字の場合 */
            if (state == 1 && tok.size() != 0) {
                /* stateが異なり、tokに文字列がある場合 */
                return true;
            }
            tok += *next;
            ++next;
            state = 0;
            break;
        }
    }

    if (tok.size() != 0) return true;

    return false;
}

#endif
