/*! 
    @file  C_trset_cmd_interpreter.cpp 
    @brief 実行時刻相対設定コマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_trset_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_trset_cmd_interpreter::
C_trset_cmd_interpreter(boost::shared_ptr<I_param> param,
                        boost::shared_ptr<C_sc_ptr_table> sc_ptr_table) 
    : C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_check = false;
    m_type = C_trset_cmd_interpreter::NON;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_trset_cmd_interpreter::reset() {
    m_check = false;
    m_type = C_trset_cmd_interpreter::NON;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_trset_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                          std::vector<std::string>::iterator end) {

    bool size_check_wait = ((end - begin) == 2); /* WAITコマンド形式時サイズチェック */
    bool size_check_time = ((end - begin) == 4); /* TIME使用時形式のサイズチェック   */

    if (size_check_time) {
        /* TIME使用時形式のフォーマットチェック */

        /* 各要素のイテレータを設定 */
        std::vector<std::string>::iterator time_var1 = begin;
        std::vector<std::string>::iterator time_var2 = begin+1;
        std::vector<std::string>::iterator frame_var1 = begin;
        std::vector<std::string>::iterator frame_var2 = begin+1;
        std::vector<std::string>::iterator plus = begin+2;
        std::vector<std::string>::iterator val = begin+3;

        /* 時刻変数をチェック */
        bool time_token_check1 = (*time_var1 == "@TIME");
        bool time_token_check2 = (*time_var2 == "@TIME");
        bool frame_token_check1 = (*frame_var1 == "@FRAME");
        bool frame_token_check2 = (*frame_var2 == "@FRAME");

        /* "+"の存在をチェック */
        bool plus_token_check = (*plus == "+");

        /* 設定時刻がint型かどうかのチェック */
        C_token_list val_token(val,val+1);
        bool val_check = (val_token.isInt());

        /* 時刻変数に応じたタイプ設定 */
        if (time_token_check1) m_type = C_trset_cmd_interpreter::TIME;
        else if (frame_token_check1) m_type = C_trset_cmd_interpreter::FRAME;

        m_check = (((time_token_check1 && time_token_check2) 
                    || (frame_token_check1 && frame_token_check2))
                   && plus_token_check && val_check);

        /* エラーだった場合、エラーメッセージ生成 */
        if (!m_check) createTimeFormatErrMsg((time_token_check1 && time_token_check2),
                                             (frame_token_check1 && frame_token_check2),
                                             plus_token_check,
                                             val_check);
    }
    else if (size_check_wait) {
        /* WAITコマンド形式のフォーマットチェック */

        /* 各要素のイテレータを設定 */
        std::vector<std::string>::iterator wait_cmd = begin;
        std::vector<std::string>::iterator val = begin+1;

        /* WAITコマンドをチェック */
        bool wait_token_check = (*wait_cmd == "@WAIT");

        /* 設定時刻がint型かどうかのチェック */
        C_token_list val_token(val,val+1);
        bool val_check = (val_token.isInt());

        m_check = (wait_token_check && val_check);

        /* タイプ設定 */
        m_type = C_trset_cmd_interpreter::TIME;

        /* エラーだった場合、エラーメッセージ生成 */
        if (!m_check) createWaitFormatErrMsg(wait_token_check,
                                             val_check);
    }
    else {
        m_check = false;
        createFormatErrMsg(size_check_time,
                           size_check_wait);
    }

    if (!m_check) m_type = C_trset_cmd_interpreter::NON;

    return m_check;
}

/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_trset_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                                   std::vector<std::string>::iterator end,
                                                   int line_num) {
    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;

    bool size_check_wait = ((end - begin) == 2); /* WAITコマンド形式時サイズチェック */

    std::vector<std::string>::iterator num;
    if (size_check_wait) num = begin+1;
    else num = begin+3;

    C_token_list num_token(num,num+1);
    int *val = new int(num_token.toInt());
    getScPtrTablePtr()->addPtr(val);

    int *time_var = NULL;
    if (m_type == C_trset_cmd_interpreter::TIME) {
        sc_elem.setRelativeSetCmd(time_var,val,line_num);
    }
    else if (m_type == C_trset_cmd_interpreter::FRAME) {
        sc_elem.setRelativeSetCmd(time_var,val,line_num);
    }

    return sc_elem;
}

/*! @brief   @TIMEへの設定コマンドかどうかを返す。 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  1 @TIMEへの設定コマンド、0 @FRAMEへの設定コマンド 
 */
bool C_trset_cmd_interpreter::isTimeSetCmd(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {
    checkFormat(begin,end);
    bool result = (m_type == C_trset_cmd_interpreter::TIME);
    reset();
    return result;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_trset_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << "\n"
        << "m_type  = " << m_type;

    return oss.str();
}

/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check_time  トークン列のサイズチェック結果 (TIME型フォーマットでの結果) 
 *  @param   size_check_wait  トークン列のサイズチェック結果 (WAIT型フォーマットでの結果) 
 */
void C_trset_cmd_interpreter::createFormatErrMsg(bool size_check_time,
                                                 bool size_check_wait) {
    C_err_msg err_msg;
    std::string msg = "";

    /* トークン列のサイズが不正時にエラーメッセージを追加 */
    if (!size_check_time || !size_check_wait) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }
}

/*! @brief   TIME型フォーマットエラーメッセージの生成 
 *  @param   time_token_check  TIMEトークンの存在チェック結果 
 *  @param   frame_token_check FRAMEトークンの存在チェック結果 
 *  @param   plus_token_check  "+"トークンの存在チェック結果 
 *  @param   val_check         値のチェック結果 
 */
void C_trset_cmd_interpreter::createTimeFormatErrMsg(bool time_token_check,
                                                     bool frame_token_check,
                                                     bool plus_token_check,
                                                     bool val_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* @TIME,@FRAMEトークンにエラーがある場合、エラーメッセージを追加 */
    if (!time_token_check && !frame_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_TIME_VAR_POS);
        addErrMsg(msg);
    }

    /* "+"トークンにエラーがある場合、エラーメッセージを追加 */
    if (!plus_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_PLUS_POS);
        addErrMsg(msg);
    }

    /* 実行時刻にエラーある場合、エラーメッセージを追加 */
    if (!val_check) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}

/*! @brief   WAIT型フォーマットエラーメッセージの生成 
 *  @param   wait_check        WAITトークンの存在チェック結果 
 *  @param   val_check         値のチェック結果 
 */
void C_trset_cmd_interpreter::createWaitFormatErrMsg(bool wait_token_check,
                                                     bool val_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* @WAITトークンにエラーがある場合、エラーメッセージを追加 */
    if (!wait_token_check) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_WAIT_CMD_POS);
        addErrMsg(msg);
    }

    /* 実行時刻にエラーある場合、エラーメッセージを追加 */
    if (!val_check) {
        msg = err_msg.getErrMsg(C_err_msg::TRSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }
}
