/*! 
    @file  C_wait_cmd_interpreter.cpp 
    @brief WAITコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_wait_cmd_interpreter.h"
#include "C_token_list.h"
#include "C_err_msg.h"
#include <sstream>

/*! @brief   コンストラクタ 
 *  @param   param パラメータ 
 *  @param   sc_ptr_talbe  ポインタテーブル 
 */
C_wait_cmd_interpreter::
C_wait_cmd_interpreter(boost::shared_ptr<I_param> param,
                       boost::shared_ptr<C_sc_ptr_table> sc_ptr_table) 
    :C_sc_cmd_interpreter(param,sc_ptr_table)
{
    m_type = C_wait_cmd_interpreter::NON;
    m_check = false;
}
    
/*! @brief   インタプリタの状態をリセットする。 
 */
void C_wait_cmd_interpreter::reset() {
    m_type = C_wait_cmd_interpreter::NON;
    m_check = false;
    clearErrMsg();
}

/*! @brief   トークン列からコマンドフォーマットをチェックする 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  true エラーなし、false エラーあり 
 */
bool C_wait_cmd_interpreter::checkFormat(std::vector<std::string>::iterator begin,
                                         std::vector<std::string>::iterator end) {

    m_check = false;

    /* WAITコマンドトークン列のサイズチェック */
    bool size_check = ((end - begin) >= 8);

    if (size_check) {
        /* 各要素のイテレータを設定 */
        std::vector<std::string>::iterator wait_token = begin;
        std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);
        std::vector<std::string>::iterator timeout_time = end-2;
        std::vector<std::string>::iterator timeout_type = end-1;
        std::vector<std::string>::iterator left_parenthetic 
            = findToken(begin,end,std::string("("));
        std::vector<std::string>::iterator right_parenthetic 
            = findLastToken(begin,end,std::string(")"));

        /* WAITコマンドのチェック */
        bool wait_token_check = (*wait_token == "@WAIT");

        /* カッコのチェック */
        bool left_parenthetic_check = (left_parenthetic != end);
        bool right_parenthetic_check = (right_parenthetic != end);

        /* 条件式前後のサイズチェック */
        bool cond_before_size_check = ((left_parenthetic - begin) == 1);
        bool cond_after_size_check = ((end - right_parenthetic) == 3);

        /* 条件演算子のチェック */
        bool cond_op_check = (cond_op != end);

        /* タイムアウト時刻がint型どうかをチェック */
        C_token_list token_list(timeout_time,timeout_time+1);
        bool timeout_time_check = token_list.isInt();

        /* タイムアウト時の処理指定のチェック */
        std::string type = *timeout_type;
        bool timeout_type_check = (type == "N" || type == "W" || type == "E");

        bool cond_lhs_check = true;
        bool cond_rhs_check = true;
        if (cond_op_check) {
            /* 条件演算子があった場合 */

            /* 条件式の左辺、右辺の各要素のイテレータを設定 */
            std::vector<std::string>::iterator cond_lhs_begin = begin+2;
            std::vector<std::string>::iterator cond_lhs_end = cond_op;
            std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
            std::vector<std::string>::iterator cond_rhs_end = end-3;

            C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
            C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

            /* 左辺、右辺がパラメータ名1つであるかどうかのチェック */
            cond_lhs_check = (lhs_token.createParamName(getParamPtr()).size() == 1);
            cond_rhs_check = (rhs_token.createParamName(getParamPtr()).size() == 1);
        }

        m_check = (size_check && wait_token_check 
                   && right_parenthetic_check && left_parenthetic_check
                   && cond_before_size_check && cond_after_size_check
                   && cond_op_check && timeout_time_check && timeout_type_check
                   && cond_lhs_check && cond_rhs_check);

        /* エラーが発生した場合、エラーメッセージを生成 */
        if (!m_check) createFormatErrMsg(size_check,
                                         wait_token_check,
                                         left_parenthetic_check,
                                         right_parenthetic_check,
                                         cond_op_check,
                                         cond_lhs_check,
                                         cond_rhs_check,
                                         timeout_time_check,
                                         timeout_type_check);
    }

    /* エラーメッセージを生成 */
    if (!size_check) createFormatErrMsg(size_check,
                                        false,false,false,false,false,false,false,false);

    return m_check;
}


/*! @brief   トークン列からコマンドフォーマットをチェックする  
 *  @param   begin 対象トークンの先頭を指すイテレータ  
 *  @param   end   対象トークンの終端を指すイテレータ  
 *  @return  true エラーなし、false エラーあり  
 */
bool C_wait_cmd_interpreter::checkCondType(std::vector<std::string>::iterator begin,
                                           std::vector<std::string>::iterator end) {
    if (!m_check) return false;

    std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);

    std::vector<std::string>::iterator cond_lhs_begin = begin+2;
    std::vector<std::string>::iterator cond_lhs_end = cond_op;
    std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
    std::vector<std::string>::iterator cond_rhs_end = end-3;

    bool int_type_check = checkCompTypeInt(cond_lhs_begin,cond_lhs_end,
                                           cond_rhs_begin,cond_rhs_end);
    bool uint_type_check = checkCompTypeUnsignedInt(cond_lhs_begin,cond_lhs_end,
                                                    cond_rhs_begin,cond_rhs_end);
    bool double_type_check = checkCompTypeDouble(cond_lhs_begin,cond_lhs_end,
                                                 cond_rhs_begin,cond_rhs_end);
    bool string_type_check = checkCompTypeString(cond_lhs_begin,cond_lhs_end,
                                                 cond_rhs_begin,cond_rhs_end);
    bool string_op_check = (*cond_op == "==" || *cond_op == "!=");

    std::string type_info = createCompTypeInfo(cond_lhs_begin,cond_lhs_end,
                                               cond_rhs_begin,cond_rhs_end);

    if (int_type_check) {
        m_type = C_wait_cmd_interpreter::INT;
    }
    else if (uint_type_check) {
        m_type = C_wait_cmd_interpreter::UNSIGNED_INT;
    }
    else if (double_type_check) {
        m_type = C_wait_cmd_interpreter::DOUBLE;
    }
    else if (string_type_check && string_op_check) {
        m_type = C_wait_cmd_interpreter::STRING;
    }
    else {
        /* エラーメッセージの生成 */
        createCondTypeErrMsg(int_type_check,
                             uint_type_check,
                             double_type_check,
                             string_type_check,
                             string_op_check,
                             type_info);
        return false;
    }
    return true;
}


/*! @brief   シナリオ要素を生成する 
 *  @param   begin 対象トークンの先頭を指すイテレータ 
 *  @param   end   対象トークンの終端を指すイテレータ 
 *  @return  シナリオ要素 
 */
C_scenario_elem C_wait_cmd_interpreter::createCmd(std::vector<std::string>::iterator begin,
                                                  std::vector<std::string>::iterator end,
                                                  int line_num) {
    C_scenario_elem sc_elem;
    if (!m_check) return sc_elem;
    
    std::vector<std::string>::iterator cond_op = findCondOpToken(begin,end);
    std::vector<std::string>::iterator timeout_time = end-2;
    std::vector<std::string>::iterator timeout_type = end-1;

    std::vector<std::string>::iterator cond_lhs_begin = begin+2;
    std::vector<std::string>::iterator cond_lhs_end = cond_op;
    std::vector<std::string>::iterator cond_rhs_begin = cond_op+1;
    std::vector<std::string>::iterator cond_rhs_end = end-3;

    if (m_type == C_wait_cmd_interpreter::INT) {
        sc_elem = createIntTypeCmd(cond_lhs_begin,cond_lhs_end,
                                   cond_rhs_begin,cond_rhs_end,cond_op,
                                   timeout_time,timeout_type,line_num);
    }
    else if (m_type == C_wait_cmd_interpreter::UNSIGNED_INT) {
        sc_elem = createUnsignedIntTypeCmd(cond_lhs_begin,cond_lhs_end,
                                           cond_rhs_begin,cond_rhs_end,cond_op,
                                           timeout_time,timeout_type,line_num);
    }
    else if (m_type == C_wait_cmd_interpreter::DOUBLE) {
        sc_elem = createDoubleTypeCmd(cond_lhs_begin,cond_lhs_end,
                                      cond_rhs_begin,cond_rhs_end,cond_op,
                                      timeout_time,timeout_type,line_num);
    }
    else if (m_type == C_wait_cmd_interpreter::STRING) {
        sc_elem = createStringTypeCmd(cond_lhs_begin,cond_lhs_end,
                                      cond_rhs_begin,cond_rhs_end,cond_op,
                                      timeout_time,timeout_type,line_num);
    }

    return sc_elem;
}

/*! @brief   条件式がint型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   timeout_time    タイムアウト時間 
 *  @param   timeout_type    タイムアウト時処理タイプ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_wait_cmd_interpreter::createIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                         std::vector<std::string>::iterator cond_lhs_end,
                                         std::vector<std::string>::iterator cond_rhs_begin,
                                         std::vector<std::string>::iterator cond_rhs_end,
                                         std::vector<std::string>::iterator cond_op,
                                         std::vector<std::string>::iterator timeout_time,
                                         std::vector<std::string>::iterator timeout_type,
                                         int line_num) {
    int *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    std::vector<std::string> lhs_params = lhs_token.createParamName(getParamPtr());
    std::vector<std::string> rhs_params = rhs_token.createParamName(getParamPtr());

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new int(lhs_token.toInt());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new int(rhs_token.toInt());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* タイムアウトカウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* タイムアウト時間用ポインタの取得 */
    C_token_list token_list(timeout_time,timeout_time+1);
    int *time = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(time);

    /* タイムアウト時の処理タイプポインタの取得 */
    std::string *type = new std::string(*timeout_type);
    getScPtrTablePtr()->addPtr(type);

    /* 時刻用ポインタの取得 */
    int *time_ptr = NULL;

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setWaitCmd(lhs,rhs,op,count,time,type,time_ptr,line_num);

    return sc_elem;
}

/*! @brief   条件式がunsigned int型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   timeout_time    タイムアウト時間 
 *  @param   timeout_type    タイムアウト時処理タイプ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_wait_cmd_interpreter::createUnsignedIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                                 std::vector<std::string>::iterator cond_lhs_end,
                                                 std::vector<std::string>::iterator cond_rhs_begin,
                                                 std::vector<std::string>::iterator cond_rhs_end,
                                                 std::vector<std::string>::iterator cond_op,
                                                 std::vector<std::string>::iterator timeout_time,
                                                 std::vector<std::string>::iterator timeout_type,
                                                 int line_num) {
    unsigned int *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    std::vector<std::string> lhs_params = lhs_token.createParamName(getParamPtr());
    std::vector<std::string> rhs_params = rhs_token.createParamName(getParamPtr());

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new unsigned int(lhs_token.toUnsignedInt());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new unsigned int(rhs_token.toUnsignedInt());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* タイムアウトカウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* タイムアウト時間用ポインタの取得 */
    C_token_list token_list(timeout_time,timeout_time+1);
    int *time = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(time);

    /* タイムアウト時の処理タイプポインタの取得 */
    std::string *type = new std::string(*timeout_type);
    getScPtrTablePtr()->addPtr(type);

    /* 時刻用ポインタの取得 */
    int *time_ptr = NULL;

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setWaitCmd(lhs,rhs,op,count,time,type,time_ptr,line_num);

    return sc_elem;
}

/*! @brief   条件式がint型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   timeout_time    タイムアウト時間 
 *  @param   timeout_type    タイムアウト時処理タイプ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_wait_cmd_interpreter::createDoubleTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                            std::vector<std::string>::iterator cond_lhs_end,
                                            std::vector<std::string>::iterator cond_rhs_begin,
                                            std::vector<std::string>::iterator cond_rhs_end,
                                            std::vector<std::string>::iterator cond_op,
                                            std::vector<std::string>::iterator timeout_time,
                                            std::vector<std::string>::iterator timeout_type,
                                            int line_num) {
    double *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    std::vector<std::string> lhs_params = lhs_token.createParamName(getParamPtr());
    std::vector<std::string> rhs_params = rhs_token.createParamName(getParamPtr());

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new double(lhs_token.toDouble());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new double(rhs_token.toDouble());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* タイムアウトカウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* タイムアウト時間用ポインタの取得 */
    C_token_list token_list(timeout_time,timeout_time+1);
    int *time = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(time);

    /* タイムアウト時の処理タイプポインタの取得 */
    std::string *type = new std::string(*timeout_type);
    getScPtrTablePtr()->addPtr(type);

    /* 時刻用ポインタの取得 */
    int *time_ptr = NULL;

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setWaitCmd(lhs,rhs,op,count,time,type,time_ptr,line_num);

    return sc_elem;
}

/*! @brief   条件式がint型のコマンドを生成する 
 *  @param   cond_lhs_begin  条件式左辺先頭 
 *  @param   cond_lhs_end    条件式左辺終端 
 *  @param   cond_rhs_begin  条件式右辺先頭 
 *  @param   cond_rhs_end    条件式右辺終端 
 *  @param   cond_op         条件式比較演算子 
 *  @param   timeout_time    タイムアウト時間 
 *  @param   timeout_type    タイムアウト時処理タイプ 
 *  @return  シナリオ要素 
 */
C_scenario_elem 
C_wait_cmd_interpreter::createStringTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                            std::vector<std::string>::iterator cond_lhs_end,
                                            std::vector<std::string>::iterator cond_rhs_begin,
                                            std::vector<std::string>::iterator cond_rhs_end,
                                            std::vector<std::string>::iterator cond_op,
                                            std::vector<std::string>::iterator timeout_time,
                                            std::vector<std::string>::iterator timeout_type,
                                            int line_num) {
    std::string *lhs, *rhs;
    C_token_list lhs_token(cond_lhs_begin,cond_lhs_end);
    C_token_list rhs_token(cond_rhs_begin,cond_rhs_end);

    std::vector<std::string> lhs_params = lhs_token.createParamName(getParamPtr());
    std::vector<std::string> rhs_params = rhs_token.createParamName(getParamPtr());

    /* 条件式左辺の処理 */
    if (getParamPtr()->exist(lhs_params[0])) {
        getParamPtr()->setParam(lhs_params[0],lhs);
    }
    else {
        lhs = new std::string(lhs_token.toString());
        getScPtrTablePtr()->addPtr(lhs);
    }

    /* 条件式右辺の処理 */
    if (getParamPtr()->exist(rhs_params[0])) {
        getParamPtr()->setParam(rhs_params[0],rhs);
    }
    else {
        rhs = new std::string(rhs_token.toString());
        getScPtrTablePtr()->addPtr(rhs);
    }

    /* 条件演算子ポインタの取得 */
    std::string *op = new std::string(*cond_op);
    getScPtrTablePtr()->addPtr(op);

    /* タイムアウトカウント用ポインタの取得 */
    int *count = new int(0);
    getScPtrTablePtr()->addPtr(count);

    /* タイムアウト時間用ポインタの取得 */
    C_token_list token_list(timeout_time,timeout_time+1);
    int *time = new int(token_list.toInt());
    getScPtrTablePtr()->addPtr(time);

    /* タイムアウト時の処理タイプポインタの取得 */
    std::string *type = new std::string(*timeout_type);
    getScPtrTablePtr()->addPtr(type);

    /* 時刻用ポインタの取得 */
    int *time_ptr = NULL;

    /* C_scenario_elem の登録 */
    C_scenario_elem sc_elem;
    sc_elem.setWaitCmd(lhs,rhs,op,count,time,type,time_ptr,line_num);

    return sc_elem;
}

/*! @brief   内部情報のダンプ 
 *  @return  内部情報 
 */
std::string C_wait_cmd_interpreter::dump() {
    std::ostringstream oss("");

    oss << "m_check = " << m_check << "\n"
        << "m_type  = " << m_type;

    return oss.str();
}


/*! @brief   フォーマットエラーメッセージの生成 
 *  @param   size_check               コマンドトークン列サイズチェック 
 *  @param   wait_token_check         WAITコマンドトークン存在チェック 
 *  @param   left_parenthetic_check   "("トークン存在チェック 
 *  @param   right_parenthetic_check  ")"トークン存在チェック 
 *  @param   cond_op_check            条件式の演算子チェック 
 *  @param   cond_lhs_check           条件式の左辺チェック 
 *  @param   cond_rhs_check           条件式の右辺チェック 
 *  @param   timeout_time_check       タイムアウト時刻のチェック 
 *  @param   timeout_type_check       タイムアウトタイプのチェック 
 */
void C_wait_cmd_interpreter::createFormatErrMsg(bool size_check,
                                                bool wait_token_check,
                                                bool left_parenthetic_check,
                                                bool right_parenthetic_check,
                                                bool cond_op_check,
                                                bool cond_lhs_check,
                                                bool cond_rhs_check,
                                                bool timeout_time_check,
                                                bool timeout_type_check) {
    C_err_msg err_msg;
    std::string msg = "";

    /* WAITコマンドのトークン列サイズが不正のときにエラーメッセージを追加 */
    if (!size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_ARGS_NUM);
        addErrMsg(msg);
    }

    /* "@WAIT"トークンが不正ときにエラーメッセージを追加 */
    if (!wait_token_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_WAIT_CMD_POS);
        addErrMsg(msg);
    }

    /* "("トークンが不正ときにエラーメッセージを追加 */
    if (!left_parenthetic_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC);
        addErrMsg(msg);
    }

    /* ")"トークンが不正ときにエラーメッセージを追加 */
    if (!right_parenthetic_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC);
        addErrMsg(msg);
    }

    /* 比較演算子が不正の時にエラーメッセージを追加 */
    if (!cond_op_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_COND_OP);
        addErrMsg(msg);
    }

    /* 条件式左辺が不正の時にエラーメッセージを追加 */
    if (!cond_lhs_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_COND_LEFT);
        addErrMsg(msg);
    }

    /* 条件式右辺が不正の時にエラーメッセージを追加 */
    if (!cond_rhs_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_COND_RIGHT);
        addErrMsg(msg);
    }

    /* timeout_timeが不正の時にエラーメッセージを追加 */
    if (!timeout_time_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_NO_INTEGER_ARG);
        addErrMsg(msg);
    }

    /* timeout_typeが不正の時にエラーメッセージを追加 */
    if (!timeout_type_check && size_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_TIMEOUT_TYPE);
        addErrMsg(msg);
    }
}

/*! @brief   型エラーメッセージの生成 
 *  @param   int_type_check      int型のチェック 
 *  @param   uint_type_check     unsigned int型のチェック 
 *  @param   double_type_check   double型のチェック 
 *  @param   string_type_check   string型のチェック 
 *  @param   string_op_check     string型の時の演算子のチェック 
 *  @param   type_info           型情報 
 */
void C_wait_cmd_interpreter::createCondTypeErrMsg(bool int_type_check,
                                                  bool uint_type_check,
                                                  bool double_type_check,
                                                  bool string_type_check,
                                                  bool string_op_check,
                                                  std::string type_info) {
    C_err_msg err_msg;
    std::string msg = "";

    /* 文字列型時の比較演算子が不正の時、エラーメッセージを追加 */
    if (string_type_check && !string_op_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_COND_OP);
        addErrMsg(msg);
    }

    /* 型が不正の時 */
    if (!int_type_check && !uint_type_check && !double_type_check && !string_type_check) {
        msg = err_msg.getErrMsg(C_err_msg::WAIT_CMD_INTERPRETER_ERR_COND_TYPE);
        addErrMsg(msg);
        addErrMsg(type_info);
    }
}
