/*! 
    @file  C_wait_cmd_interpreter.h 
    @brief WAITコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_WAIT_CMD_INTERPRETER_H
#define C_WAIT_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  WAITコマンドインタプリタクラス 
  @brief  WAITコマンドの解釈機能を持つ 
 */
class C_wait_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    enum E_wait_type {
        NON = 0,
        INT,
        UNSIGNED_INT,
        DOUBLE,
        STRING
    };

    int m_type;
    bool m_check;

public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ  
     *  @param   param パラメータ  
     *  @param   sc_ptr_talbe  ポインタテーブル  
     */
    C_wait_cmd_interpreter(boost::shared_ptr<I_param> param,
                           boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_wait_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ  
     *  @param   end   対象トークンの終端を指すイテレータ  
     *  @return  true エラーなし、false エラーあり  
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ  
     *  @return  true エラーなし、false エラーあり  
     */
    bool checkCondType(std::vector<std::string>::iterator begin,
                       std::vector<std::string>::iterator end);

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:
    /* private メンバ関数 */

    /*! @brief   条件式がint型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   timeout_time    タイムアウト時間 
     *  @param   timeout_type    タイムアウト時処理タイプ 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                     std::vector<std::string>::iterator cond_lhs_end,
                                     std::vector<std::string>::iterator cond_rhs_begin,
                                     std::vector<std::string>::iterator cond_rhs_end,
                                     std::vector<std::string>::iterator cond_op,
                                     std::vector<std::string>::iterator timeout_time,
                                     std::vector<std::string>::iterator timeout_type,
                                     int line_num);
    
    /*! @brief   条件式がunsigned int型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   timeout_time    タイムアウト時間 
     *  @param   timeout_type    タイムアウト時処理タイプ 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createUnsignedIntTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                             std::vector<std::string>::iterator cond_lhs_end,
                                             std::vector<std::string>::iterator cond_rhs_begin,
                                             std::vector<std::string>::iterator cond_rhs_end,
                                             std::vector<std::string>::iterator cond_op,
                                             std::vector<std::string>::iterator timeout_time,
                                             std::vector<std::string>::iterator timeout_type,
                                             int line_num);

    /*! @brief   条件式がdouble型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   timeout_time    タイムアウト時間 
     *  @param   timeout_type    タイムアウト時処理タイプ 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createDoubleTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                        std::vector<std::string>::iterator cond_lhs_end,
                                        std::vector<std::string>::iterator cond_rhs_begin,
                                        std::vector<std::string>::iterator cond_rhs_end,
                                        std::vector<std::string>::iterator cond_op,
                                        std::vector<std::string>::iterator timeout_time,
                                        std::vector<std::string>::iterator timeout_type,
                                        int line_num);

    /*! @brief   条件式がstring型のコマンドを生成する 
     *  @param   cond_lhs_begin  条件式左辺先頭 
     *  @param   cond_lhs_end    条件式左辺終端 
     *  @param   cond_rhs_begin  条件式右辺先頭 
     *  @param   cond_rhs_end    条件式右辺終端 
     *  @param   cond_op         条件式比較演算子 
     *  @param   timeout_time    タイムアウト時間 
     *  @param   timeout_type    タイムアウト時処理タイプ 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createStringTypeCmd(std::vector<std::string>::iterator cond_lhs_begin,
                                        std::vector<std::string>::iterator cond_lhs_end,
                                        std::vector<std::string>::iterator cond_rhs_begin,
                                        std::vector<std::string>::iterator cond_rhs_end,
                                        std::vector<std::string>::iterator cond_op,
                                        std::vector<std::string>::iterator timeout_time,
                                        std::vector<std::string>::iterator timeout_type,
                                        int line_num);

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check               コマンドトークン列サイズチェック 
     *  @param   wait_token_check         WAITコマンドトークン存在チェック 
     *  @param   left_parenthetic_check   "("トークン存在チェック 
     *  @param   right_parenthetic_check  ")"トークン存在チェック 
     *  @param   cond_op_check            条件式の演算子チェック 
     *  @param   cond_lhs_check           条件式の左辺チェック 
     *  @param   cond_rhs_check           条件式の右辺チェック 
     *  @param   timeout_time_check       タイムアウト時刻のチェック 
     *  @param   timeout_type_check       タイムアウトタイプのチェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool wait_token_check,
                            bool left_parenthetic_check,
                            bool right_parenthetic_check,
                            bool cond_op_check,
                            bool cond_lhs_check,
                            bool cond_rhs_check,
                            bool timeout_time_check,
                            bool timeout_type_check);
    
    /*! @brief   型エラーメッセージの生成 
     *  @param   int_type_check      int型のチェック 
     *  @param   uint_type_check     unsigned int型のチェック 
     *  @param   double_type_check   double型のチェック 
     *  @param   string_type_check   string型のチェック 
     *  @param   string_op_check     string型の時の演算子のチェック 
     *  @param   type_info           型情報 
     */
    void createCondTypeErrMsg(bool int_type_check,
                              bool uint_type_check,
                              bool double_type_check,
                              bool string_type_check,
                              bool string_op_check,
                              std::string type_info);
};

#endif

