/*! 
    @file  Error.cpp 
    @brief エラー管理クラス(秘匿化部分)  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_ErrMng.h"
#include <vector>
#include <map>
#include <string>
#include <iostream>

namespace COMMON_FUJITSU {
  #include "checkLicense.h"
}

class MsgInfo {
public:
    MsgInfo(const std::string& msg, ERROR_SEVERITY sev) { 
        m_msg = msg ; 
        m_sev = sev ;
    }
    ~MsgInfo() { ; }

    const std::string& getMsg() { return m_msg ; }
    ERROR_SEVERITY     getSev() { return m_sev ; }
private:
    std::string    m_msg ;
    ERROR_SEVERITY m_sev ;
} ; /* class MsgInfo end */

class ErrorMng : public ErrorMng_wrap {
public:
    ErrorMng() : ErrorMng_wrap(), m_term_flag(false), m_max_sev(INFO_MSG) {
        COMMON_FUJITSU::checkLicense() ; /* ライセンスチェック */

        /* シナリオ解釈時のエラーメッセージ登録処理 */
        makeMsgSc() ;
    }
    ~ErrorMng() { 
        terminal() ;
    }
    virtual const std::string getVender() {
        return std::string("FUJITSU") ;
    }
    virtual const std::string getMakeTime() {
        return std::string(__DATE__) + std::string(" ") + std::string(__TIME__) ;
    }
  
    void appendMsg(const std::string& msg, ERROR_SEVERITY sev = ERROR_MSG) {
        m_max_sev = std::max(sev, m_max_sev) ;
        std::string str_sv ;
        switch (sev) {
        case INFO_MSG:    str_sv = std::string("Info    : ") ; break ;
        case WARNING_MSG: str_sv = std::string("Warning : ") ; break ;
        case ERROR_MSG:   str_sv = std::string("Error   : ") ; break ;
        default : std::cerr << "Error   : ERROR_SEVERITY ignore type" << std::endl ; break ;
        } /* switch end */
        if (m_os1) *m_os1 << str_sv << msg << std::endl ;
        if (m_os2) *m_os2 << str_sv << msg << std::endl ;
    }

    void terminal() {
        if (!m_term_flag) {
            m_term_flag = true ;
            std::string str_sv ;
            switch (m_max_sev) {
            case INFO_MSG:    str_sv = "Information End." ; break ;
            case WARNING_MSG: str_sv = "Warning End."     ; break ;
            case ERROR_MSG:   str_sv = "Error End."       ; break ;
            default : std::cerr << "Error   : ERROR_SEVERITY ignore type" << std::endl ; break ;
            } /* switch end */
            if (m_os1) *m_os1 << str_sv << std::endl ;
            if (m_os2) *m_os2 << str_sv << std::endl ;

            switch (m_max_sev) {
            case INFO_MSG:    exit(0) ;
            case WARNING_MSG: exit(4) ;
            case ERROR_MSG:   exit(8) ;
            default : std::cerr << "Error   : ERROR_SEVERITY ignore type" << std::endl ; break ;
            } /* switch end */
        }
    }
  
    void checkStatus() {
        if (m_max_sev == ERROR_MSG) exit(0) ;
    }

    void setOstreamOne(std::ostream* os) { m_os1 = os ; }

    void setOstreamTwo(std::ostream* os) { m_os2 = os ; }

    /* ================================================= 
       以降の関数は、シナリオ解釈クラスSc_Ctrl専用 
       ================================================= */
    void appendMsgSc(      int                       msg_no, 
                           const std::string&              file_name, 
                           int                       line_no, 
                           const std::vector<std::string>& add_str_vec) {
        std::stringstream msg_ss ;

        /* メッセージ番号を検索 */
        std::map<int, MsgInfo>::iterator map_itr = m_scenario_msg_map.find(msg_no) ;
        if (map_itr != m_scenario_msg_map.end()) {
            /* メッセージ番号があるなら */

            /* 出力するメッセージと、セビリティを取得する */
            msg_ss << file_name << " [" << line_no << "] : " << map_itr->second.getMsg() ;
            m_max_sev = std::max(map_itr->second.getSev(), m_max_sev) ;
        } else {
            std::cerr << "Error   : Ignore Scenario_Msg_No" << std::endl ; 
        }

        /* 追加文字列があれば取得する */
        if (!add_str_vec.empty()) {
            msg_ss << " : " << add_str_vec.at(0) ;
        }

        /* メッセージ出力 */
        if (m_os1) *m_os1 << msg_ss.str() << std::endl ;
        if (m_os2) *m_os2 << msg_ss.str() << std::endl ;
    }

private:
    /*! シナリオ解釈用メッセージおよび、セビリティの登録関数 
     */
    void makeMsgSc() {
        /* シミュレーションエラーメッセージの登録 */
        m_scenario_msg_map.insert(std::make_pair(1, 
                                                 MsgInfo("## ERROR: The scenario file doesn't exist. ##",                 ERROR_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(2, 
                                                 MsgInfo("## ERROR: The command is wrong. ##",                            ERROR_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(3, 
                                                 MsgInfo("## ERROR: The number of arguments of the command is wrong. ##", ERROR_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(4, 
                                                 MsgInfo("## ERROR: The type of arguments of the command is wrong. ##",   ERROR_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(5, 
                                                 MsgInfo("## ERROR: The label name is not found. ##",                     ERROR_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(6, 
                                                 MsgInfo("## ERROR: The path is not found. ##",                     ERROR_MSG))) ;

        /* 検証エラーメッセージの登録 */
        m_scenario_msg_map.insert(std::make_pair(51, 
                                                 MsgInfo("## WARNING: non WAIT_FIXED command ##",        WARNING_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(52, 
                                                 MsgInfo("## ERROR: non ACK ##",                         INFO_MSG))) ;
        m_scenario_msg_map.insert(std::make_pair(53, 
                                                 MsgInfo("## ERROR: Expected value comparison error ##", INFO_MSG))) ;

        /* シナリオ定義ファイルのエラーメッセージ  */
        m_scenario_msg_map.insert(std::make_pair(70, 
                                                 MsgInfo("## ERROR: The format is wrong. ##", ERROR_MSG))) ;

    }
  

private:
    bool                   m_term_flag ;
    ERROR_SEVERITY         m_max_sev ;
    std::map<int, MsgInfo> m_scenario_msg_map ;
} ;

boost::shared_ptr<ErrorMng_wrap> 
createErrorMng() {
    return boost::shared_ptr<ErrorMng_wrap>(new ErrorMng()) ;
}
